#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif

#ifndef _IDATE_
#define _IDATE_
/*******************************************************************************
* FILE NAME: idate.hpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IDate                                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>

/*  The "struct blocks" must be placed before SOM enablement test  */
#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM

struct _CDATE;

#endif
#endif

struct ICnrDate {
  unsigned char  day;
  unsigned char  month;
  unsigned short year;
};


#pragma pack(4)

class ostream;


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif



class IString;

class IDate : public IBase {
public:
/*-------------------------- Day/Month/Year Declarations ---------------------*/
typedef enum {
  Monday = 0,
  Tuesday,
  Wednesday,
  Thursday,
  Friday,
  Saturday,
  Sunday
  } DayOfWeek;

typedef enum {
  January = 1,
  February,
  March,
  April,
  May,
  June,
  July,
  August,
  September,
  October,
  November,
  December
  } Month;

typedef enum {
  yy,
  yyyy
  } YearFormat;

/*-------------------------- Constructors ------------------------------------*/
  IDate       ( );

  IDate       ( Month aMonth,
                int   aDay,
                int   aYear );

  IDate       ( int   aDay,
                Month aMonth,
                int   aYear );

  IDate       ( int   aYear,
                int   aDay );

  IDate       ( const IDate&    aDate );

  IDate       ( unsigned long   julianDayNumber );

  IDate       ( const ICnrDate& cnrDate );

/*-------------------------- Current Date ------------------------------------*/
static IDate
  today       ( );

/*-------------------------- Diagnostics -------------------------------------*/
IString
  asString    ( YearFormat  yearFmt = yy ) const,
  asString    ( const char *fmt ) const;

friend ostream
 &operator << ( ostream     &aStream,
                const IDate &aDate );

/*-------------------------- Day Queries -------------------------------------*/
DayOfWeek
  dayOfWeek   ( ) const;

IString
  dayName     ( ) const;

int
  dayOfMonth  ( ) const,
  dayOfYear   ( ) const;

/*-------------------------- Month Queries -----------------------------------*/
Month
  monthOfYear ( ) const;

IString
  monthName   ( ) const;

/*-------------------------- Year Queries ------------------------------------*/
int
  year        ( ) const;

/*-------------------------- Date Queries ------------------------------------*/
unsigned long
  julianDate  ( ) const;

ICnrDate
  asICnrDate  ( ) const;

/*-------------------------- Comparisons -------------------------------------*/
Boolean
  operator == ( const IDate &aDate ) const,
  operator != ( const IDate &aDate ) const,
  operator <  ( const IDate &aDate ) const,
  operator <= ( const IDate &aDate ) const,
  operator >  ( const IDate &aDate ) const,
  operator >= ( const IDate &aDate ) const;

/*------------------------- Manipulations ------------------------------------*/
IDate
  operator +  ( int numDays ) const,
  operator -  ( int numDays ) const;

IDate
 &operator += ( int numDays ),
 &operator -= ( int numDays );

long
  operator -  ( const IDate &aDate ) const;

/*------------------------- General Date Queries -----------------------------*/
static IString
  dayName     ( DayOfWeek aDay   ),
  monthName   ( Month     aMonth );

static int
  daysInMonth ( Month aMonth,
                int   aYear ),
  daysInYear  ( int   aYear );

/*------------------------- Validation ---------------------------------------*/
static Boolean
  isLeapYear  ( int   aYear ),
  isValid     ( Month aMonth,
                int   aDay,
                int   aYear ),
  isValid     ( int   aDay,
                Month aMonth,
                int   aYear ),
  isValid     ( int   aYear,
                int   aDay );

protected:
/*------------------------- Initialize ---------------------------------------*/
IDate
 &initialize  ( Month aMonth,
                int   aDay,
                int   aYear );

private:
/*--------------------------------- Private ----------------------------------*/
#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM
public:

  IDate       ( const _CDATE &cDate );

_CDATE
  asCDATE     ( ) const;

private:

#endif
#endif

unsigned long
  julian;


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)

#ifdef __MVS__

#pragma SOMReleaseOrder( \
/*   1 */       IDate(Month,int,int),\
/*   2 */       IDate(int,Month,int),\
/*   3 */       IDate(int,int),\
/*   4 */       IDate(unsigned long),\
/*   5 */    /* IDate(const _CDATE&), OS/2 speecific NOT Supported on MVS */ \
/*   6 */       today(),\
/*   7 */       asString(YearFormat) const,\
/*   8 */       asString(const char*) const,\
/*   9 */       dayOfWeek() const,\
/*  10 */       dayName() const,\
/*  11 */       dayOfMonth() const,\
/*  12 */       dayOfYear() const,\
/*  13 */       monthOfYear() const,\
/*  14 */       monthName() const,\
/*  15 */       year() const,\
/*  16 */       julianDate() const,\
/*  17 */   /*  asCDATE() const, OS/2 speecific NOT Supported on MVS */ \
/*  18 */       operator==(const IDate&) const,\
/*  19 */       operator!=(const IDate&) const,\
/*  20 */       operator<(const IDate&) const,\
/*  21 */       operator<=(const IDate&) const,\
/*  22 */       operator>(const IDate&) const,\
/*  23 */       operator>=(const IDate&) const,\
/*  24 */       operator+(int) const,\
/*  25 */       operator-(int) const,\
/*  26 */       operator+=(int),\
/*  27 */       operator-=(int),\
/*  28 */       operator-(const IDate&) const,\
/*  29 */       dayName(DayOfWeek),\
/*  30 */       monthName(Month),\
/*  31 */       daysInMonth(Month,int),\
/*  32 */       daysInYear(int),\
/*  33 */       isLeapYear(int),\
/*  34 */       isValid(Month,int,int),\
/*  35 */       isValid(int,Month,int),\
/*  36 */       isValid(int,int),\
/*  37 */       initialize(Month,int,int),\
/*  38 */       asICnrDate() const,\
/*  39 */       IDate(const ICnrDate&))

#else

#pragma SOMReleaseOrder( \
/*   1 */       IDate(Month,int,int),\
/*   2 */       IDate(int,Month,int),\
/*   3 */       IDate(int,int),\
/*   4 */       IDate(unsigned long),\
/*   5 */       IDate(const _CDATE&), \
/*   6 */       today(),\
/*   7 */       asString(YearFormat) const,\
/*   8 */       asString(const char*) const,\
/*   9 */       dayOfWeek() const,\
/*  10 */       dayName() const,\
/*  11 */       dayOfMonth() const,\
/*  12 */       dayOfYear() const,\
/*  13 */       monthOfYear() const,\
/*  14 */       monthName() const,\
/*  15 */       year() const,\
/*  16 */       julianDate() const,\
/*  17 */       asCDATE() const,  \
/*  18 */       operator==(const IDate&) const,\
/*  19 */       operator!=(const IDate&) const,\
/*  20 */       operator<(const IDate&) const,\
/*  21 */       operator<=(const IDate&) const,\
/*  22 */       operator>(const IDate&) const,\
/*  23 */       operator>=(const IDate&) const,\
/*  24 */       operator+(int) const,\
/*  25 */       operator-(int) const,\
/*  26 */       operator+=(int),\
/*  27 */       operator-=(int),\
/*  28 */       operator-(const IDate&) const,\
/*  29 */       dayName(DayOfWeek),\
/*  30 */       monthName(Month),\
/*  31 */       daysInMonth(Month,int),\
/*  32 */       daysInYear(int),\
/*  33 */       isLeapYear(int),\
/*  34 */       isValid(Month,int,int),\
/*  35 */       isValid(int,Month,int),\
/*  36 */       isValid(int,int),\
/*  37 */       initialize(Month,int,int),\
/*  38 */       asICnrDate() const,\
/*  39 */       IDate(const ICnrDate&))

#endif   // for release order pragma

#endif    // SOM_ENABLED & RRBC_LIB


}; // IDate


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#pragma pack()

#ifndef I_NO_INLINES
  #include <idate.inl>
#endif

#endif /* _IDATE_ */

#ifdef __MVS__
                   #pragma checkout(resume)
#endif

