/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iikbhsh.h>
#include <iiexc.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IAKeyBagAsHshTableOps)
#endif

// ---------------------------
// IKeyBagAsHshTableCursorImpl
// ---------------------------

// public members

IKeyBagAsHshTableCursorImpl::
IKeyBagAsHshTableCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivNode (0)
{
}

IKeyBagAsHshTableCursorImpl::
IKeyBagAsHshTableCursorImpl
  (IKeyBagAsHshTableCursorImpl const& cursor)
: ICursorImpl (cursor), ivNode (cursor.ivNode)
{
}

IKeyBagAsHshTableCursorImpl::
~IKeyBagAsHshTableCursorImpl ()
{
}

ICursorImpl*
IKeyBagAsHshTableCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
IKeyBagAsHshTableCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}


// ---------------------
// IKeyBagAsHshTableImpl
// ---------------------

char const*
IKeyBagAsHshTableImpl::cvClassName = "KeyBagAsHshTable";

// public members

IKeyBagAsHshTableImpl::
IKeyBagAsHshTableImpl (Ops& ops, INumber numberOfElements)
: IAKeyBagImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable (0), ivNumberOfBuckets (0), ivNumberOfElements (0)
{ Initialize (numberOfElements);
}

IKeyBagAsHshTableImpl::
IKeyBagAsHshTableImpl
  (Ops& ops, IKeyBagAsHshTableImpl const& collection)
: IAKeyBagImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable (0), ivNumberOfBuckets (0), ivNumberOfElements (0)
{ Initialize (collection.ivNumberOfBuckets);
  Self::AddAllFrom (collection);
}

IKeyBagAsHshTableImpl::
~IKeyBagAsHshTableImpl ()
{
  if (ivTable != 0) {
    Self::RemoveAll ();
    DeleteNodes (ivTable);
  }
}

IBoolean
IKeyBagAsHshTableImpl::
Add (void const* element, ICursorImpl& cursor)
{ if (ivNumberOfBuckets * 2 < Self::NumberOfElements ()) {
    GrowBy (1);
  }

  unsigned long hashValue =
    HashKey (element, ivNumberOfBuckets, kElement) % ivNumberOfBuckets;

  IASSERT (hashValue < ivNumberOfBuckets);

  Node* node = (Node*) CreateNode (element);

  //
  // link element in bucket
  //
  node->ivNext = ivTable [hashValue];
  ivTable [hashValue] = node;

  ivNumberOfElements++;

  //
  // Let cursor point to newly added element
  //
  BucketOf (cursor) = hashValue;
  NodeOf (cursor) = node;

  return true;
}

void
IKeyBagAsHshTableImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& kb = (Self const&)collection;
    // ivDummyCursor may not be initialized yet
    Cursor dummyCursor (*this);
    for (INumber i = 0; i < kb.ivNumberOfBuckets; i++) {
      Node* node = kb.ivTable [i];
      while (node != 0) {
        //
        // New element is added at the beginning of the bucket.
        //
        Self::Add (ElementOf (node), dummyCursor);
        node = node->ivNext;
      }
    }
  }
  else
    Inherited::AddAllFrom (collection);
}

IBoolean
IKeyBagAsHshTableImpl::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateElementWithKey (Key (element), cursor);

  if (isContained)
    Self::ReplaceAt (cursor, element);
  else
    Self::Add (element, cursor);

  return ! isContained;
}

IBoolean
IKeyBagAsHshTableImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ if (Self::SetToFirst (CursorOf (*this)))
    while ((*applFunc) (Self::ElementAt (CursorOf (*this)), addArg) &&
           Self::SetToNext (CursorOf (*this)));

  return (NodeOf (CursorOf (*this)) == 0);
}

void*
IKeyBagAsHshTableImpl::
Any () const
{ Self::SetToFirst (CursorOf (*this));
  return Self::ElementAt (CursorOf (*this));
}

IBoolean
IKeyBagAsHshTableImpl::
CheckCursor (ICursorImpl const& cursor) const
{
  {
    for (Self::SetToFirst (CursorOf (*this));
         NodeOf (CursorOf (*this)) != NodeOf (cursor);
         Self::SetToNext (CursorOf (*this)));

    return (NodeOf (CursorOf (*this)) != 0);
  }
}

IBoolean
IKeyBagAsHshTableImpl::
CheckReplacement (ICursorImpl const& cursor, void const* element) const
{ return EqualKeys (Self::ElementAt (cursor), element, kElementElement);
}

IACollectionImpl*
IKeyBagAsHshTableImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

IBoolean
IKeyBagAsHshTableImpl::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ if (Self::ClassName () == collection.ClassName ()) {
    if (this != &collection) {
      Self const& kb = (Self const&)collection;
      ICursorImpl& cursor = (CursorOf (kb));
      if (kb.Self::SetToFirst (cursor))
        while (Self::ContainsElementWithKey
                 (Key (kb.Self::ElementAt (cursor)))
               && kb.Self::SetToNext (cursor));

      return (NodeOf (cursor) == 0);
    }
    else
      return true;
  }
  else
    return Inherited::ContainsAllKeysFrom (collection);
}

void
IKeyBagAsHshTableImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

IKeyBagAsHshTableImpl::Cursor*
IKeyBagAsHshTableImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

IKeyBagAsHshTableImpl::MngCursor*
IKeyBagAsHshTableImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*(Self*)this));
}

void*
IKeyBagAsHshTableImpl::
ElementWithKey (void const* key) const
{ ICursorImpl& cursor = CursorOf (*this);
  ICHECK_ContainsKey (ClassName (), "ElementWithKey ()")
  return Self::ElementAt (cursor);
}

IBoolean
IKeyBagAsHshTableImpl::
LocateElementWithKey (void const* key, ICursorImpl& cursor) const
{ BucketOf (cursor) =
    HashKey (key, ivNumberOfBuckets, kKey) % ivNumberOfBuckets;
  Node*& node = (NodeOf (cursor) = ivTable [BucketOf (cursor)]);

  while (! (node == 0 ||
            EqualKeys (ElementOf (node), key, kElementKey))) {
    //
    // while element available and key not equal to argument key,
    // try next one
    //
    node = node->ivNext;
  }

  return (node != 0);
}

IBoolean
IKeyBagAsHshTableImpl::
LocateNextElementWithKey (void const* key, ICursorImpl& cursor) const
{

  Node*& node = NodeOf (cursor);

  IASSERT (node != 0);

  do {
    node = node->ivNext;
  } while (! (node == 0 ||
              EqualKeys (Self::ElementAt (cursor), key, kElementKey)));

  return (node != 0);
}

IBoolean
IKeyBagAsHshTableImpl::
LocateOrAddElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateElementWithKey (Key (element), cursor);

  if (! isContained)
     Self::Add (element, cursor);

  return isContained;
}

INumber
IKeyBagAsHshTableImpl::
NumberOfDifferentKeys () const
{ INumber result = 0;

  for (Self::SetToFirst (CursorOf (*this));
       NodeOf (CursorOf (*this)) != 0;
       Self::SetToNextWithDifferentKey (CursorOf (*this))) {
    result++;
  }

  return result;
}

INumber
IKeyBagAsHshTableImpl::
NumberOfElementsWithKey (void const* key) const
{ INumber n = 0;

  if (Self::LocateElementWithKey (key, CursorOf (*this))) {
    n = 1;
    while (Self::LocateNextElementWithKey (key, CursorOf (*this)))
      n++;
  }

  return n;
}

INumber
IKeyBagAsHshTableImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  Node *current,
       *next;
  for (INumber i = 0; i < ivNumberOfBuckets; i++) {
    current = ivTable [i];
    while (current != 0) {
      next = current->ivNext;
      DeleteNode (current);
      current = next;
    }
    ivTable [i] = 0;
  }
  ivNumberOfElements = 0;


  return n;
}

INumber
IKeyBagAsHshTableImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ Cursor& current = (Cursor&) CursorOf (*this);
  Cursor cache (*this);
  INumber removed = 0;
  for (Self::SetToFirst (current), Self::SetToFirst (cache);
       NodeOf (current) != 0;
       current = cache) {
    Self::SetToNext (cache);
    if ((*predFunc) (Self::ElementAt (current), addArg)) {
      Self::RemoveAt (current);
      removed++;
    }
  }
  return removed;
}

INumber
IKeyBagAsHshTableImpl::
RemoveAllElementsWithKey (void const* key)
{ Cursor& current = (Cursor&) CursorOf (*this);
  Cursor cache (*this);
  IBoolean removed = 0;

  unsigned long hashValue =
    HashKey (key, ivNumberOfBuckets, kKey) % ivNumberOfBuckets;

  IASSERT (hashValue < ivNumberOfBuckets);

  BucketOf (current) = hashValue;
  NodeOf (current) = ivTable [BucketOf (current)];

  if (NodeOf (current) != 0) {
    cache = current;
    do {
      Self::SetToNext (cache);
      if (EqualKeys (Self::ElementAt (current), key, kElementKey)) {
        Self::RemoveAt (current);
        removed++;
      }
      current = cache;
    }
    while (NodeOf (current) != 0 && BucketOf (current) == hashValue);
  }

  return removed;
}

void
IKeyBagAsHshTableImpl::
RemoveAt (ICursorImpl& cursor)
{ IASSERT (NodeOf (cursor) != 0);

  Node* nodeOfCursor = NodeOf (cursor);
  INumber bucketOfCursor = BucketOf (cursor);


  if (nodeOfCursor == ivTable [bucketOfCursor])
    ivTable [bucketOfCursor] = nodeOfCursor->ivNext;
  else {
    Node* previous = ivTable [bucketOfCursor];
    while (previous->ivNext != nodeOfCursor) {
      //
      // find previous of element
      //
      previous = previous->ivNext;
      IASSERT (previous != 0);
    }

    previous->ivNext = nodeOfCursor->ivNext;
  }

  DeleteNode (nodeOfCursor);
  ivNumberOfElements--;

  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

IBoolean
IKeyBagAsHshTableImpl::
RemoveElementWithKey (void const* key)
{ IBoolean isContained =
    Self::LocateElementWithKey (key, CursorOf (*this));

  if (isContained) {
    Self::RemoveAt (CursorOf (*this));
  }

  return isContained;
}

IBoolean
IKeyBagAsHshTableImpl::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateElementWithKey (Key (element), cursor);

  if (isContained)
    Self::ReplaceAt (cursor, element);

  return isContained;
}

IBoolean
IKeyBagAsHshTableImpl::
SetToFirst (ICursorImpl& cursor) const
{ BucketOf (cursor) = 0;

  if (Self::IsEmpty()) {
    //
    // setToFirst will return very fast on an empty hashtable
    //
    NodeOf (cursor) = 0;
  }
  else {
    //
    // find first non-empty bucket
    //
    while (ivTable [BucketOf (cursor)] == 0 &&
           BucketOf (cursor) < ivNumberOfBuckets - 1) {
      BucketOf (cursor)++;
    }

    NodeOf (cursor) = ivTable [BucketOf (cursor)];
  }

  return (NodeOf (cursor) != 0);
}

IBoolean
IKeyBagAsHshTableImpl::
SetToNext (ICursorImpl& cursor) const
{

  Node*& node = NodeOf (cursor);

  IASSERT (node != 0);

  node = node->ivNext;
  if (node == 0) {
    //
    // end of the bucket,
    // try to find new non-empty bucket
    //
    while (BucketOf (cursor) < ivNumberOfBuckets - 1) {
      BucketOf (cursor)++;
      if (ivTable [BucketOf (cursor)] != 0) {
        node = ivTable [BucketOf (cursor)];
        break;
      }
    }
  }

  return (node != 0);
}

IBoolean
IKeyBagAsHshTableImpl::
SetToNextWithDifferentKey (ICursorImpl& cursor) const
{ void const* key = Key (Self::ElementAt (cursor));

  do {
    Self::SetToNext (cursor);
  }
  while (NodeOf (cursor) != 0 &&
         EqualKeys (Self::ElementAt (cursor), key, kElementKey));

  return (NodeOf (cursor) != 0);
}

void
IKeyBagAsHshTableImpl::
operator= (IKeyBagAsHshTableImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    CopyFrom (collection);
  }
}

TStream&
IKeyBagAsHshTableImpl::
operator<<= (TStream& fromWhere)
{
  return fromWhere;
}

TStream&
IKeyBagAsHshTableImpl::
operator>>= (TStream& toWhere) const
{
  return toWhere;
}

// private members

void
IKeyBagAsHshTableImpl::
CopyFrom (IKeyBagAsHshTableImpl const& collection)
{ IASSERT (Self::IsEmpty ());

  if (ivNumberOfBuckets == collection.ivNumberOfBuckets) {
    //
    // copy contents of hash table
    //
    for (INumber i = 0; i < ivNumberOfBuckets; i++) {
      Node* node;
      Node* thatNode;

      ivTable [i] = 0;

      thatNode = collection.ivTable [i];
      while (thatNode != 0) {

        ITRY {
          node = (Node*) CreateNode (ElementOf (thatNode));
        }
        IENDTRY

        ICATCH (IOutOfCollectionMemoryImpl exc) {
          for (++i; i < ivNumberOfBuckets; i++) {
            //
            // leave table in consistent state.
            //
            ivTable [i] = 0;
          }
          Self::RemoveAll();
          IRETHROW (exc);
        }
        IENDCATCH

        node->ivNext = ivTable [i];
        ivTable [i] = node;

        thatNode = thatNode->ivNext;
      }
    }
    ivNumberOfElements = collection.NumberOfElements ();
  }
  else {
    //
    // add per element,
    // hash value has to be computed for every element
    //
    Self::AddAllFrom (collection);
  }
}

INumber
IKeyBagAsHshTableImpl::
FindNearestPrime (INumber n)
{ n |= 0x01L;

  for (INumber i = 3; i * i <= n; i += 2) {
    if (n % i == 0) {
      n += 2;
      i = 1;
    }
  }
  return n;
}

void
IKeyBagAsHshTableImpl::
GrowBy (INumber)
{ INumber oldNumberOfBuckets = ivNumberOfBuckets;
  Node** oldTable = ivTable;

  ITRY {
    Initialize (ivNumberOfBuckets * 2);
  }
  IENDTRY

  ICATCH (IOutOfCollectionMemoryImpl exc) {
    ivTable = oldTable;
    ivNumberOfBuckets = oldNumberOfBuckets;
    IRETHROW (exc);
  }
  IENDCATCH

  for (INumber i = 0; i < oldNumberOfBuckets; i++) {
    while (oldTable [i] != 0) {
      Node* node = oldTable [i];
      oldTable [i] = node->ivNext;
      unsigned long hashValue =
        HashKey (ElementOf (node), ivNumberOfBuckets, kElement)
          % ivNumberOfBuckets;
      node->ivNext = ivTable [hashValue];
      ivTable [hashValue] = node;

      ivNumberOfElements++;
    }
  }

  DeleteNodes (oldTable);
}

void
IKeyBagAsHshTableImpl::
Initialize (INumber numberOfBuckets)
{ if (numberOfBuckets == 0)
    ivNumberOfBuckets = 1;
  else
    ivNumberOfBuckets = FindNearestPrime (numberOfBuckets);

  ivNumberOfElements = 0;
  ivTable = 0; // in case CreateNodes () fails
  ivTable = (Node**) CheckPointer (CreateNodes (ivNumberOfBuckets));

  for (INumber i = 0; i < ivNumberOfBuckets; i++) {
    ivTable [i] = 0;
  }
}

void
IKeyBagAsHshTableImpl::
SetInBetween (MngCursor& cursor) const
{
}

#pragma info (restore)
#pragma pack ()
