/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiseqtab.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IASequenceAsTableOps)
#endif

// --------------------------
// ISequenceAsTableCursorImpl
// --------------------------

// public members

ISequenceAsTableCursorImpl::
ISequenceAsTableCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivIndex (0)
{
}

ISequenceAsTableCursorImpl::
ISequenceAsTableCursorImpl
  (ISequenceAsTableCursorImpl const& cursor)
: ICursorImpl (cursor), ivIndex (cursor.ivIndex)
{
}

ISequenceAsTableCursorImpl::
~ISequenceAsTableCursorImpl ()
{
}

ICursorImpl*
ISequenceAsTableCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsTableCursorImpl::
Copy (ICursorImpl const&  cursor)
{ operator= ((Self const&)cursor);
}


// --------------------
// ISequenceAsTableImpl
// --------------------

char const*
ISequenceAsTableImpl::cvClassName = "SequenceAsTable";

// public members

ISequenceAsTableImpl::
ISequenceAsTableImpl (Ops& ops, INumber numberOfElements)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ if (numberOfElements == 0)
    numberOfElements++;
  Initialize (numberOfElements);
}

ISequenceAsTableImpl::
ISequenceAsTableImpl
  (Ops& ops, ISequenceAsTableImpl const& collection)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ Initialize (collection.ivTable.ivAllocatedElements);
  Self::AddAllFrom (collection);
}

ISequenceAsTableImpl::
~ISequenceAsTableImpl ()
{
  Self::RemoveAll ();
  DeleteNodes (ivTable.ivNodes);
}

void
ISequenceAsTableImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& seq = (Self const&)collection;
    if (! ivTable.CanAdd (seq.ivTable.ivNumberOfElements))
      GrowBy (seq.ivTable.ivNumberOfElements);

    CopyFrom (seq.ivTable);
  }
  else
    Inherited::AddAllFrom (collection);
}

void
ISequenceAsTableImpl::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  ivTable.DecMod (ivTable.ivFirst);

  ConstructNode (ivTable.NodeAtIndex (ivTable.ivFirst), element);
  ivTable.ivNumberOfElements++;
  IndexOf (cursor) = ivTable.ivFirst;
}

void
ISequenceAsTableImpl::
AddAsLast (void const* element, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  ivTable.IncMod (ivTable.ivLast);

  ConstructNode (ivTable.NodeAtIndex (ivTable.ivLast), element);
  ivTable.ivNumberOfElements++;
  IndexOf (cursor) = ivTable.ivLast;
}

void
ISequenceAsTableImpl::
AddAsNext (void const* element, ICursorImpl& cursor)
{
  { if (IndexOf (cursor) == ivTable.ivLast)
      Self::AddAsLast (element, cursor);
    else {
      Self::SetToNext (cursor);
      Self::AddAsPrevious (element, cursor);
    }
  }
}

void
ISequenceAsTableImpl::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{ INumber& index = IndexOf (cursor);


  if (! ivTable.CanAdd (1)) {
    index = ivTable.Pos (index);
    GrowBy (1);
  }

  if (ivTable.ivFirst == index) {
    ivTable.DecMod (ivTable.ivFirst);
    index = ivTable.ivFirst;
  }
  else if (ivTable.ivFirst <= ivTable.ivLast &&
           (ivTable.ivFirst == 1 ||
            (ivTable.ivLast < ivTable.ivAllocatedElements &&
             index - ivTable.ivFirst > ivTable.ivLast - index))) {
    BlockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else if (ivTable.ivFirst > ivTable.ivLast &&
           index <= ivTable.ivLast) {
    BlockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else {
    BlockLeft (ivTable.ivFirst - 1, index - ivTable.ivFirst);
    ivTable.ivFirst--;
    index--;
  }

  ivTable.ivNumberOfElements++;
  ConstructNode (ivTable.NodeAtIndex (index), element);
}

void
ISequenceAsTableImpl::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ if (position == Self::NumberOfElements () + 1)
    Self::AddAsLast (element, cursor);
  else {
    Self::SetToPosition (position, cursor);
    Self::AddAsPrevious (element, cursor);
  }
}

IBoolean
ISequenceAsTableImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < Self::NumberOfElements (); i++) {
    if (! (*applFunc) (ElementOf (ivTable.NodeAtIndex (index)), addArg))
      return false;
    ivTable.IncMod (index);
  }
  return true;
}

IBoolean
ISequenceAsTableImpl::
CheckCursor (ICursorImpl const& cursor) const
{
  {
    INumber const& index = ((Cursor const&)cursor).ivIndex;
    if (ivTable.ivNumberOfElements == 0)
      return false;
    else if (ivTable.ivFirst <= ivTable.ivLast)
      return ivTable.ivFirst <= index && index <= ivTable.ivLast;
    else
      return (ivTable.ivFirst <= index &&
                index <= ivTable.ivAllocatedElements) ||
              index <= ivTable.ivLast;
  }
}

IACollectionImpl*
ISequenceAsTableImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
ISequenceAsTableImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

ISequenceAsTableImpl::Cursor*
ISequenceAsTableImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

ISequenceAsTableImpl::MngCursor*
ISequenceAsTableImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

IBoolean
ISequenceAsTableImpl::
IsConsistent () const
{ INumber f = ivTable.ivFirst,
          l = ivTable.ivLast,
          n = ivTable.ivNumberOfElements,
          a = ivTable.ivAllocatedElements;
  return n % a == (l >= f ? l - f + 1 : l + a - f + 1) % a;
}

INumber
ISequenceAsTableImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  if (n != 0) {
    if (ivTable.ivFirst <= ivTable.ivLast) {
      DestroyNodes (ivTable.NodeAtIndex (ivTable.ivFirst),
                    ivTable.ivNumberOfElements);
    }
    else {
      DestroyNodes (ivTable.NodeAtIndex (ivTable.ivFirst),
                    ivTable.ivAllocatedElements - ivTable.ivFirst + 1);
      DestroyNodes (ivTable.ivNodes, ivTable.ivLast);
    }


    ivTable.ivFirst = 1;
    ivTable.ivLast = ivTable.ivAllocatedElements;
    ivTable.ivNumberOfElements = 0;
  }
  return n;
}


INumber
ISequenceAsTableImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber index = ivTable.ivFirst,
          current = ivTable.ivFirst;
  INumber removed = 0;

  for (INumber i = Self::NumberOfElements (); i != 0; i--) {
    if ((*predFunc) (ElementOf (ivTable.NodeAtIndex (current)),
                     addArg)) {
      removed++;
    }
    else {
      if (index != current)  {
        Assign (ElementOf (ivTable.NodeAtIndex (index)),
                ElementOf (ivTable.NodeAtIndex (current)));
      }
      ivTable.IncMod (index);
    }
    ivTable.IncMod (current);
  }

  if (removed != 0) {
    if (index <= ivTable.ivLast) {
      DestroyNodes (ivTable.NodeAtIndex (index), removed);
    }
    else {
      DestroyNodes (ivTable.NodeAtIndex (index),
                    ivTable.ivAllocatedElements - index + 1);
      DestroyNodes (ivTable.NodeAtIndex (1),
                    removed -
                      (ivTable.ivAllocatedElements - index + 1));
    }
  }

  ivTable.ivLast = ivTable.DecMod (index);
  ivTable.ivNumberOfElements -= removed;

  return removed;
}

void
ISequenceAsTableImpl::
RemoveAtPosition (IPosition position)
{ Self::SetToPosition (position, CursorOf (*this));
  RemoveAtIndex (IndexOf (CursorOf (*this)));
}

void
ISequenceAsTableImpl::
Reverse ()
{ if (Self::NumberOfElements () > 1) {
    INumber count = Self::NumberOfElements () / 2;
    INumber forward = ivTable.ivFirst,
            backward = ivTable.ivLast;
    while (count--) {
      Swap (forward, backward);
    }
  }

}

void
ISequenceAsTableImpl::
operator= (ISequenceAsTableImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    if (! ivTable.CanAdd (collection.ivTable.ivNumberOfElements))
      GrowBy (collection.ivTable.ivNumberOfElements);
    CopyFrom (collection.ivTable);
  }
}

TStream&
ISequenceAsTableImpl::
operator<<= (TStream& fromWhere)
{
  return fromWhere;
}

TStream&
ISequenceAsTableImpl::
operator>>= (TStream& toWhere) const
{
  return toWhere;
}

// private members

void
ISequenceAsTableImpl::
GrowBy (INumber n)
{

  n += ivTable.ivNumberOfElements;
  ITableImpl table = ivTable;

  INumber newAllocatedElements = ivTable.ivAllocatedElements;
  while (newAllocatedElements < n)
    newAllocatedElements *= 2;

  Initialize (newAllocatedElements);
  CopyFrom (table);

  if (table.ivFirst <= table.ivLast) {
    DestroyNodes (table.NodeAtIndex (table.ivFirst),
                  table.ivNumberOfElements);
  }
  else {
    DestroyNodes (table.NodeAtIndex (table.ivFirst),
                  table.ivAllocatedElements - table.ivFirst + 1);
    DestroyNodes (table.NodeAtIndex (1), table.ivLast);
  }
  DeleteNodes (table.ivNodes);
}

void
ISequenceAsTableImpl::
Initialize (INumber n)
{ ivTable.ivNodes = CreateNodes (n, ivTable.ivNodeSize);
  ivTable.ivFirst = 1;
  ivTable.ivLast = ivTable.ivAllocatedElements = n;
  ivTable.ivNumberOfElements = 0;
}

void
ISequenceAsTableImpl::
Quicksort (ICompFunc compFunc, void* addArg, INumber from, INumber to)
{ INumber i = from, j = to;
  ivTable.IncMod (i);
  for (;;) {
    while (i != j &&
           ((*compFunc) (ElementOf (ivTable.NodeAtIndex (i)),
                         ElementOf (ivTable.NodeAtIndex (from)),
                         addArg) <= 0))
      ivTable.IncMod (i);
    while (i != j &&
           ((*compFunc) (ElementOf (ivTable.NodeAtIndex (j)),
                         ElementOf (ivTable.NodeAtIndex (from)),
                         addArg) >= 0))
      ivTable.DecMod (j);
    if (i == j)
      break;
    Swap (i, j);
    ivTable.IncMod (i);
    if (i != j)
      ivTable.DecMod (j);
  }
  if ((*compFunc) (ElementOf (ivTable.NodeAtIndex (i)),
                   ElementOf (ivTable.NodeAtIndex (from)),
                   addArg) > 0)
    ivTable.DecMod (i);
  j = i;
  if (i != from) {
    Swap (from, i);
    ivTable.DecMod (i);
  }
  if (j != to)
    ivTable.IncMod (j);
  if (from != i)
    Quicksort (compFunc, addArg, from, i);
  if (j != to)
    Quicksort (compFunc, addArg, j, to);
}

void
ISequenceAsTableImpl::
RemoveAtIndex (INumber index)
{ DestroyNodes (ivTable.NodeAtIndex (index));
  if (ivTable.ivFirst == index) {
    ivTable.IncMod (ivTable.ivFirst);
  }
  else if (ivTable.ivLast == index) {
    ivTable.DecMod (ivTable.ivLast);
  }
  else if ((ivTable.ivFirst <= ivTable.ivLast &&
            index - ivTable.ivFirst > ivTable.ivLast - index) ||
           (ivTable.ivFirst > ivTable.ivLast &&
            index <= ivTable.ivLast)) {

    BlockLeft(index, ivTable.ivLast - index);
    ivTable.ivLast--;
  }
  else {
    BlockRight (ivTable.ivFirst + 1, index - ivTable.ivFirst);
    ivTable.ivFirst++;
    ivTable.IncMod (index);
  }
  ivTable.ivNumberOfElements--;
}


void
ISequenceAsTableImpl::
SetInBetween (MngCursor& cursor) const
{
}

#pragma info (nopar, nouse)

void
ISequenceAsTableImpl::
UpdateMngCursorsForAssign
  (INumber targetIndex, INumber sourceIndex) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForCopyFrom (INumber fromIndex, INumber toIndex) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForGrowBy () const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForPredRemove
  (INumber removedIndex, INumber prevIndex) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForRemoveOf (INumber index) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForShiftLeftOf (INumber index) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForShiftRightOf (INumber index) const
{
}

void
ISequenceAsTableImpl::
UpdateMngCursorsForSwap (INumber index1, INumber index2) const
{
}

#pragma info (restore)

#pragma info (restore)
#pragma pack ()
