/*******************************************************************************
* FILE NAME: imemstrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*     IMemoryStream                                                            *                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
* ADDITONAL COPYRIGHTS:                                                        *
*  Copyright (C) 1996, Taligent, Inc. All rights reserved.                     *
*  Copyright (C) 1996, Lotus Development Corporation. All Rights Reserved.     *
*                                                                              *
*                                                                              *
*******************************************************************************/
#include <imemstrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMEMSTRM_CPP_
  #include <ipagetun.h>
#endif


static unsigned long kMemoryInitSize = 1024;

IMemoryStream::IMemoryStream( const void* memptr, size_t  size)
: fBuffer(0), fPosition(size), fCapacity(0)
{
   if (size < kMemoryInitSize)
      fCapacity = kMemoryInitSize;
   else
      fCapacity = size;
#ifndef __DEBUG_ALLOC__
   fBuffer = (char*)::operator new(fCapacity);
#else
   fBuffer = (char*)::operator new(fCapacity, __FILE__, __LINE__);
#endif
   memcpy( fBuffer, memptr, (int)size);

}

IMemoryStream::IMemoryStream( )
: fBuffer(0), fPosition(0), fCapacity(kMemoryInitSize)
{
#ifndef __DEBUG_ALLOC__
   fBuffer = (char*)::operator new(fCapacity);
#else
   fBuffer = (char*)::operator new(fCapacity, __FILE__, __LINE__);
#endif
}



IMemoryStream::~IMemoryStream()
{
   delete fBuffer;
}

//IMemoryStream&
//IMemoryStream::operator=(const IMemoryStream& copy)
//{
//    IBaseStream::operator=(copy);
//    fBuffer = copy.fBuffer;
//    fPosition = copy.fPosition;
//    fCapacity = copy.fCapacity;
//    return *this;
//}

IBaseStream::Position
IMemoryStream::position() const
{
    return (IBaseStream::Position)fPosition;
}

IBaseStream::Position
IMemoryStream::physicalEndOfStream() const
{
  return fCapacity;
}

IMemoryStream&
IMemoryStream::seek(IBaseStream::Position position)
{
  fPosition = position;
  return *this;
}

IMemoryStream&
IMemoryStream::setPhysicalEndOfStream(IBaseStream::Position endOfStream)
{
    if ( endOfStream != fCapacity)
      {
        fCapacity = endOfStream;
        if ( fPosition > fCapacity )
          fPosition = fCapacity;

#ifndef __DEBUG_ALLOC__
        char* tmp = (char*)::operator new(fCapacity);
#else
        char* tmp = (char*)::operator new(fCapacity, __FILE__, __LINE__);
#endif
        memcpy(tmp, fBuffer, fPosition);
        delete fBuffer;
        fBuffer = tmp;
      }
    return *this;
}

void
IMemoryStream::handleWriteBufferFull (const void* source, size_t sourceSize)
{
    // Second argument is the size of the elements being written (1 == 1 byte).
    if ((fPosition + sourceSize) > fCapacity)
      {
        fCapacity *= 2;
#ifndef __DEBUG_ALLOC__
        char* tmp = (char*)::operator new(fCapacity);
#else
        char* tmp = (char*)::operator new(fCapacity, __FILE__, __LINE__);
#endif
        memcpy(tmp, fBuffer, fPosition);
        delete fBuffer;
        fBuffer = tmp;
      }

    memcpy(&fBuffer[fPosition], source, (int)sourceSize);
    fPosition += sourceSize;
}

void
IMemoryStream::handleReadBufferEmpty (void* target, size_t targetSize)
{
    // Second argument is the size of the elements being read (1 == 1 byte).
    memcpy(target, &fBuffer[fPosition], (int)targetSize);
    fPosition += targetSize;
}


const char *
IMemoryStream:: buffer()
{
  return fBuffer;
}
