/* NOSHIP */
#ifndef _INLSUTIL_
#define _INLSUTIL_
/*******************************************************************************
* FILE NAME: inlsutil.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*    NLS utility classes and functions                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1996             *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ibase.hpp>

#ifdef IC_NLS
  #include <wchar.h>
#endif

#include <limits.h>
#include <stdio.h>

class IWBuffer {
public:
  // Constructors and destructor ----------------------------------------

  IWBuffer(const char *str);
  IWBuffer(const char *buf, unsigned blen);
  ~IWBuffer(void);

  // Searching functions ------------------------------------------------

  unsigned positionOf(IWBuffer &wBuf,
                      unsigned startNum=1) const;
  unsigned positionOfAnyOf(IWBuffer &wBuf,
                           unsigned startNum=1) const;
  unsigned positionOfAnyBut(IWBuffer &wBuf,
                            unsigned startNum=1) const;
  unsigned lastPositionOf(IWBuffer &wBuf,
                          unsigned startNum=UINT_MAX) const;
  unsigned lastPositionOfAnyOf(IWBuffer &wBuf,
                               unsigned startNum=UINT_MAX) const;
  unsigned lastPositionOfAnyBut(IWBuffer &wBuf,
                                unsigned startNum=UINT_MAX) const;

  // Editing functions --------------------------------------------------

  void reverse() const;

  // Queries ------------------------------------------------------------

  // - return the number of characters stored inside IWBuffer
  //
  size_t charNum(void) const { return charnum; }

  // - reference to the 'index' th location of the wide character
  //   buffer (i.e. 1-based)
  //
  wchar_t &operator [] (unsigned index) {
#ifdef IC_DEVELOP
    IASSERT((index != 0) && (index <= charnum));
#endif
    return *(wptr+index-1);
  }

  wchar_t &operator [] (signed index) {
#ifdef IC_DEVELOP
    IASSERT((index > 0) && (index <= charnum));
#endif
    return *(wptr+index-1);
  }

  // Conversion operators -----------------------------------------------

  // - provide read and write access to the wide character buffer
  //
  operator wchar_t *(void) const { return wptr; }

private:

  wchar_t *wptr;
  size_t   charnum;

  int hasWChar(wchar_t wCh) const;

  // No implementation for these
  IWBuffer(const IWBuffer &wBuf);
  void operator=(const IWBuffer &wBuf);
};

unsigned findCharNum(const char *buf, unsigned blen, unsigned pos);
unsigned findNextCharNum(const char *buf, unsigned blen, unsigned pos);
unsigned findBytePos(const char *buf, unsigned blen, unsigned num);
unsigned findCurPos(const char *buf, unsigned blen, unsigned pos);
unsigned findNextPos(const char *buf, unsigned blen, unsigned pos);
#ifdef IC_MVS400
unsigned redundantSOSI(const char *buf);
#endif

#ifdef IC_HAS_RESTARTABLE
  size_t cvtMB2WC(const char *pMB, unsigned maxLen, wchar_t *pWC,
                  mbstate_t *ps);
#else
  size_t cvtMB2WC(const char *pMB, unsigned maxLen, wchar_t *pWC);
#endif

size_t cvtMBS2WCS(const char *pMBS, unsigned mblen, wchar_t *pWCS);
size_t cvtWCS2MBS(const wchar_t *pWCS, unsigned numWChar, char *pMBS,
                  unsigned mblen=UINT_MAX, int alwaysPad=0,
                  char padChar=' ');
size_t cvtWCS2MBS(const IWBuffer& wBuf, char *pMBS,
                  unsigned mblen=UINT_MAX, int alwaysPad=0,
                  char padChar=' ');

size_t insWCS2WCS(wchar_t *pwcSrc, int  wcSrcLen,
                  wchar_t *pwcIns, int  wcInsLen,
                  int wcIndex,     wchar_t *pwcDest);

int isAllDigits(const char *pStr, unsigned len);
int isAllHexDigits(const char *pStr, unsigned len);
unsigned indexThatIsNot(const char *pStr, unsigned len, char ch);
char *getSubString(const char *sBuf, unsigned sLen,
                   unsigned startPos, char padCharacter,
                   char *tBuf, unsigned tLen);
unsigned getRemainString(const char *sBuf, unsigned sLen,
                         unsigned startPos, char padCharacter,
                         char *tBuf);

#endif
