/*******************************************************************************
* FILE NAME: inotifev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in inotifev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <inotifev.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTIFEV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| INotificationEvent::INotificationEvent                                       |
|                                                                              |
------------------------------------------------------------------------------*/
INotificationEvent::INotificationEvent  ( const INotificationId&   id,
                      INotifier&         notifier,
                      Boolean            hasChanged,
                      const IEventData&  eventData,
                      const IEventData&  observerData)
                 : fEvtId (id),
                   fEvtNotifier(&notifier),
                   fAttrChanged(hasChanged),
                   fEvtData(eventData),
                   fObsData(observerData),
                   fExtendedNotificationData(0)
{}

INotificationEvent::INotificationEvent  (const INotificationEvent& event)
{
  this->fEvtId = event.fEvtId;
  this->fEvtNotifier = event.fEvtNotifier;
  this->fAttrChanged = event.fAttrChanged;
  this->fEvtData = event.fEvtData;
  this->fObsData = event.fObsData;
  // Note if extendedNotificationData implemented, copy it here
  this->fExtendedNotificationData = 0;

}

INotificationEvent::~INotificationEvent ( )
{
}

INotificationEvent& INotificationEvent::operator=( const INotificationEvent& event)
{
  if(this!=&event)
  {
    this->fEvtId = event.fEvtId;
    this->fEvtNotifier = event.fEvtNotifier;
    this->fAttrChanged = event.fAttrChanged;
    this->fEvtData = event.fEvtData;
    this->fObsData = event.fObsData;
    // Note if extendedNotificationData implemented, copy it here
    this->fExtendedNotificationData = 0;
  }
  return *this;
}
