/*******************************************************************************
* FILE NAME: iplatfrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains a class that sets static variables that determine       *
*   the runtime operating system (platform)                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

/* Same priority as ibase.cpp.  This must be the first! */
#pragma priority ( -2147482624)

extern "C"
{
  #include <iwindefs.h>
}

#include <iplatfrm.hpp>


#ifdef IC_WIN
  const Boolean IPlatform::fgWin32 = true;
  const Boolean IPlatform::fgWinNT = IPlatform::fgWin32 &&
                                     (GetVersion() < 0x80000000ul);
  const Boolean IPlatform::fgNTNewShell = IPlatform::fgWinNT &&
                                          (LOBYTE(LOWORD(GetVersion())) == 4);
  const Boolean IPlatform::fgWin32S = IPlatform::fgWin32 &&
                                      (!(IPlatform::fgWinNT) &&
                                      (LOBYTE(LOWORD(GetVersion()))<4));
  const Boolean IPlatform::fgWin95 = (!(IPlatform::fgWinNT) &&
                                      !(IPlatform::fgWin32S)) &&
                                      IPlatform::fgWin32;
#else
  const Boolean IPlatform::fgWin32 = 0;
  const Boolean IPlatform::fgWinNT = 0;
  const Boolean IPlatform::fgNTNewShell = 0;
  const Boolean IPlatform::fgWin32S = 0;
  const Boolean IPlatform::fgWin95 = 0;
#endif

/*------------------------------------------------------------------------------
| IPlatform::isWin32                                                           |
------------------------------------------------------------------------------*/
Boolean IPlatform :: isWin32()
{
   return IPlatform::fgWin32;
}

/*------------------------------------------------------------------------------
| IPlatform::isWinNT                                                           |
------------------------------------------------------------------------------*/
Boolean IPlatform :: isWinNT()
{
   return IPlatform::fgWinNT;
}
/*------------------------------------------------------------------------------
| IPlatform::isNTNewShell                                                      |
------------------------------------------------------------------------------*/
Boolean IPlatform :: isNTNewShell()
{
   return IPlatform::fgNTNewShell;
}
/*------------------------------------------------------------------------------
| IPlatform::isWin32S                                                          |
------------------------------------------------------------------------------*/
Boolean IPlatform :: isWin32S()
{
   return IPlatform::fgWin32S;
}
/*------------------------------------------------------------------------------
| IPlatform::isWin95                                                           |
------------------------------------------------------------------------------*/
Boolean IPlatform :: isWin95()
{
   return IPlatform::fgWin95;
}
/*------------------------------------------------------------------------------
| IPlatform::numProcessors()                                                   |
------------------------------------------------------------------------------*/
int IPlatform :: numProcessors()
{
   _SYSTEM_INFO sysInfo;
   GetSystemInfo(&sysInfo);
   return sysInfo.dwNumberOfProcessors;
}
