/*******************************************************************************
* FILE NAME: istparse.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in istparse.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #include <ctype.h>
  }

#include <istring.hpp>
#include <istrtest.hpp>
#include <istpdata.hpp>
#include <istparse.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTPARSE_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IStringParser::~IStringParser                                                |
------------------------------------------------------------------------------*/
IStringParser :: ~IStringParser ()
{
  if (fData)
    fData->removeRef();
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, IString token)                             |
|                                                                              |
| Create an IStringParser object from a parse string and a token               |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                                  IString &token )
{
  return(IStringParser(text) >> token);
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, const IString pattern)                     |
|                                                                              |
| Create an IStringParser object from a parse string and a pattern             |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                            const IString &pattern )
{
  return(IStringParser(text) >> pattern);
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, const char *pattern)                       |
|                                                                              |
| Create an IStringParser object from a parse string and a pattern             |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                            const char    *pattern )
{
  return(IStringParser(text) >> pattern);
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, char pattern)                              |
|                                                                              |
| Create an IStringParser object from a parse string and a pattern             |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                            char           pattern )
{
  return(IStringParser(text) >> pattern);
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, unsigned long position)                    |
|                                                                              |
| Create an IStringParser object from a parse string and a position            |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                            unsigned long  position )
{
  return(IStringParser(text) >> position);
}

/*------------------------------------------------------------------------------
| operator >> (const IString &text, int position)                              |
|                                                                              |
| Create an IStringParser object from a parse string and a position            |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString &text,
                            int            position )
{
  return(IStringParser(text) >> position);
}

/*------------------------------------------------------------------------------
| operator << (const IString &text, unsigned long position)                    |
|                                                                              |
| Create an IStringParser object from a parse string and a position            |
------------------------------------------------------------------------------*/
IStringParser operator << ( const IString &text,
                            unsigned long  position )
{
  return(IStringParser(text) << position);
}

/*------------------------------------------------------------------------------
| operator << (const IString &text, const IStringText &test)                   |
|                                                                              |
| Create an IStringParser object from a parse string and a string test         |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString     &text,
                            const IStringTest &test )
{
  return(IStringParser(text) >> test);
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (IString &token)                                  |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( IString &token )
{
  fData->processToken(token);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (const IString &pattern)                          |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( const IString &pattern )
{
  fData->processPattern(pattern);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (const char *pattern)                             |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( const char *pattern )
{
  fData->processPattern(pattern);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (char pattern)                                    |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( char pattern )
{
  fData->processPattern(pattern);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (const IStringTest &test)                         |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( const IStringTest &test )
{
  fData->processTest(test);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (int delta)                                       |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( int delta )
{
  fData->changePosition(delta);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator >> (unsigned long delta)                             |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> ( unsigned long delta )
{
  fData->changePosition(delta);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator << (unsigned long position)                          |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator << ( unsigned long position )
{
  fData->setPosition(position);
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::SkipWords::SkipWords                                          |
------------------------------------------------------------------------------*/
IStringParser :: SkipWords :: SkipWords ( unsigned long numberOfWords )
               : fNumSkip ( numberOfWords )
{
}

/*------------------------------------------------------------------------------
| IStringParser::SkipWords::numberOfWords                                      |
------------------------------------------------------------------------------*/
unsigned long IStringParser :: SkipWords :: numberOfWords () const
{
  return fNumSkip;
}

/*------------------------------------------------------------------------------
| operator << (const IString &text, IStringParser::Command command)            |
|                                                                              |
| Create an IStringParser object from a parse string and a command             |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString          &text,
                            IStringParser::Command  command )
{
  return(IStringParser(text) >> command);
}

/*------------------------------------------------------------------------------
| operator << (const IString &text,                                            |
|              const IStringParser::SkipWords &skipObject)                     |
|                                                                              |
| Create an IStringParser object from a parse string and a skip object         |
------------------------------------------------------------------------------*/
IStringParser operator >> ( const IString                  &text,
                            const IStringParser::SkipWords &skipObject )
{
  return(IStringParser(text) >> skipObject );
}


/*------------------------------------------------------------------------------
| IStringParser::operator << (Command command)                                 |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> (Command command)
{
  switch (command)
  {
     case reset:
          fData->setPosition(1);
          break;
     case skipWord:
          fData->processSkip();
          break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::operator << (const SkipWords &skipObject)                     |
------------------------------------------------------------------------------*/
IStringParser& IStringParser :: operator >> (const SkipWords &skipObject)
{
  unsigned long skipCnt = skipObject.numberOfWords();
  while ( skipCnt-- > 0 )
    fData->processSkip();
  return *this;
}

/*------------------------------------------------------------------------------
| IStringParser::IStringParser                                                 |
------------------------------------------------------------------------------*/
IStringParser :: IStringParser ( const IString &text )
{
  fData = new IStringParserData( text );
}

/*------------------------------------------------------------------------------
| IStringParser::IStringParser (copy constructor)                              |
------------------------------------------------------------------------------*/
IStringParser :: IStringParser ( const IStringParser &parser )
{
  fData = parser.fData;
  fData->addRef();
}

