#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif
/* NOSHIP */

#ifndef _ISTPDATA_
#define _ISTPDATA_
/*******************************************************************************
* FILE NAME: istpdata.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IStringParserData                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <istring.hpp>

/*----------------------------------------------------------------------------*/
/* Define token pointer type                                                  */
/*----------------------------------------------------------------------------*/
typedef IString* ITokenPointer;

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

class IStringParserData : public IVBase {
typedef IVBase
  Inherited;
/*******************************************************************************
* Objects of this class are used to store and manage the data for parsing      *
* strings.                                                                     *
*******************************************************************************/
public:
/*------------------------- Constructor and Destructor -------------------------
| The constructor accepts the text string to be parsed and initialized the     |
| parse history information.  The parse history information is used to track   |
| what parsing has been performed (but may not be the final results).          |
|                                                                              |
| The destructor is used to delete possible parse history information.         |
------------------------------------------------------------------------------*/
  IStringParserData ( const IString &text );

 ~IStringParserData ();

/*--------------------------------- Parsing ------------------------------------
| These methods are used to perform the actual string parsing and to store a   |
| record of the actions performed.  These methods are intended to be called    |
| only by IStringParse objects.                                                |
------------------------------------------------------------------------------*/
IStringParserData
  &processToken   ( IString &token, Boolean reparse = false ),
  &processPattern ( const IString &pattern ),
  &processPattern ( const char *pattern ),
  &processPattern ( char pattern ),
  &processSkip    ( Boolean reparse = false ),
  &processTest    ( const IStringTest &test ),
  &setPosition    ( unsigned long position ),
  &changePosition ( int delta ),
  &changePosition ( unsigned long delta );

/*------------------------------ Implementation --------------------------------
| The following methods are used to implement common actions performed during  |
| string parsing.                                                              |
------------------------------------------------------------------------------*/
IStringParserData
  &reparseLastToken (),
  &reparseTokens    ( unsigned long startPos, unsigned long stopPos),
  &saveToken        ( IString *token ),
  &clearSavedTokens ();

/*---------------------------- Reference Counting ------------------------------
| The following functions manage the buffer reference count:                   |
|   addRef    - Increments the usage count.                                    |
|   removeRef - Decrements the usage count and deletes the buffer when the     |
|               usage count goes to 0.                                         |
------------------------------------------------------------------------------*/
void
  addRef    ( ),
  removeRef ( );

/*--------------------------------- Private ----------------------------------*/
private:

IString
  parseText;
unsigned long
  currentPosition,
  patternPosition,
  patternLength,
  tokenCount,
  tokenArraySize;
IString
 *lastToken;
Boolean
  lastSkip;
ITokenPointer
  *usedTokens;
unsigned long
  refs;


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)

#pragma SOMReleaseOrder( \
/*   1 */       IStringParserData(const IString&),\
/*   2 */       processToken(IString&,IBase::Boolean),\
/*   3 */       processPattern(const IString&),\
/*   4 */       processPattern(const char*),\
/*   5 */       processPattern(char),\
/*   6 */       processSkip(IBase::Boolean),\
/*   7 */       processTest(const IStringTest&),\
/*   8 */       setPosition(unsigned long),\
/*   9 */       changePosition(int),\
/*  10 */       changePosition(unsigned long),\
/*  11 */       reparseLastToken(),\
/*  12 */       reparseTokens(unsigned long,unsigned long),\
/*  13 */       saveToken(IString*),\
/*  14 */       clearSavedTokens(),\
/*  15 */       addRef(),\
/*  16 */       removeRef())

#endif    // SOM_ENABLED & RRBC_LIB


};


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif /* _ISTPDATA_ */


#ifdef __MVS__
                   #pragma checkout(resume)
#endif

