#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif

#ifndef _ISTRING_
#define _ISTRING_
/*******************************************************************************
* FILE NAME: istring.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IString                                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#if !defined( _IBASE_ )
  #include <ibase.hpp>
#endif

#include <istrenum.hpp>

#pragma pack(4)

extern "C"
  {
  #include <limits.h>
  }

class ostream;
class istream;


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif


class IStringTest;
class IBuffer;

class IString : public IBase {
public:
/*------------------------- Constructors -------------------------------------*/
  IString           ( );
  IString           ( const IString &aString );

  IString           ( int  anInt );
  IString           ( unsigned  anUnsigned );
  IString           ( long  aLong );
  IString           ( unsigned long  anUnsignedLong );
#ifdef _LONG_LONG
  IString           ( long long  aLongLong );
  IString           ( unsigned long long  anUnsignedLongLong );
#endif
  IString           ( short  aShort );
  IString           ( unsigned short  anUnsignedShort );
  IString           ( double  aDouble );

  IString           ( char  aChar );
  IString           ( unsigned char  anUnsignedChar );
  IString           ( signed char  aSignedChar);

  IString           ( const char  *pChar );
  IString           ( const unsigned char  *pUnsignedChar );
  IString           ( const signed char  *pSignedChar );

  IString           ( const void *pBuffer1,
                      unsigned    lenBuffer1,
                      char        padCharacter = ' ' );
  IString           ( const void *pBuffer1,
                      unsigned     lenBuffer1,
                      const void  *pBuffer2,
                      unsigned     lenBuffer2,
                      char         padCharacter = ' ' );
  IString           ( const void *pBuffer1,
                      unsigned     lenBuffer1,
                      const void  *pBuffer2,
                      unsigned     lenBuffer2,
                      const void  *pBuffer3,
                      unsigned     lenBuffer3,
                      char         padCharacter = ' ' );

  ~IString          ( );

/*------------------------- Diagnostics --------------------------------------*/
IString
  asString          ( ) const,
  asDebugInfo       ( ) const;

friend ostream
 &operator <<       ( ostream       &aStream,
                      const IString &aString );

/*------------------------- Stream Input -------------------------------------*/
friend istream
 &operator >>       ( istream &aStream,
                      IString &aString );

static IString
  lineFrom          ( istream &aStream,
                      char     delim = '\n' );

/*------------------------- Testing ------------------------------------------*/
Boolean
  isAlphanumeric    ( ) const,
  isAlphabetic      ( ) const,
  isASCII           ( ) const,
  isBinaryDigits    ( ) const,
  isControl         ( ) const,
  isDigits          ( ) const,
  isGraphics        ( ) const,
  isHexDigits       ( ) const,
  isLowerCase       ( ) const,
  isPrintable       ( ) const,
  isPunctuation     ( ) const,
  isUpperCase       ( ) const,
  isWhiteSpace      ( ) const;

/*------------------------- NLS Testing --------------------------------------*/
Boolean
  isMBCS            ( ) const,
  isDBCS            ( ) const,
  isSBCS            ( ) const,
  isValidMBCS       ( ) const,
  isValidDBCS       ( ) const,
  includesMBCS      ( ) const,
  includesDBCS      ( ) const,
  includesSBCS      ( ) const;


/*------------------------- Pattern Matching ---------------------------------*/
Boolean
  isLike            ( const IString &aPattern,
                      char           zeroOrMore = '*',
                      char           anyChar    = '?' ) const,
  isLike            ( const char    *pPattern,
                      char           zeroOrMore = '*',
                      char           anyChar    = '?' ) const,

  isAbbreviationFor ( const IString &fullString,
                      unsigned       minAbbrevLength = 0 ) const,
  isAbbreviationFor ( const char    *pFullString,
                      unsigned       minAbbrevLength = 0 ) const,

  includes          ( const IString     &aString ) const,
  includes          ( const char        *pString ) const,
  includes          ( char               aChar   ) const,
  includes          ( const IStringTest &aTest   ) const;

/*------------------------- Comparisons --------------------------------------*/
friend Boolean
  operator ==      ( const IString &string1,
                     const IString &string2  ),
  operator ==      ( const IString &string1,
                     const char    *pString2 ),
  operator ==      ( const char    *pString1,
                     const IString &string2  ),

  operator !=      ( const IString &string1,
                     const IString &string2  ),
  operator !=      ( const IString &string1,
                     const char    *pString2 ),
  operator !=      ( const char    *pString1,
                     const IString &string2  ),

  operator <       ( const IString &string1,
                     const IString &string2  ),
  operator <       ( const IString &string1,
                     const char    *pString2 ),
  operator <       ( const char    *pString1,
                     const IString &string2  ),

  operator <=      ( const IString &string1,
                     const IString &string2  ),
  operator <=      ( const IString &string1,
                     const char    *pString2 ),
  operator <=      ( const char    *pString1,
                     const IString &string2  ),

  operator >       ( const IString &string1,
                     const IString &string2  ),
  operator >       ( const IString &string1,
                     const char    *pString2 ),
  operator >       ( const char    *pString1,
                     const IString &string2  ),

  operator >=      ( const IString &string1,
                     const IString &string2  ),
  operator >=      ( const IString &string1,
                     const char    *pString2 ),
  operator >=      ( const char    *pString1,
                     const IString &string2  );

/*------------------------- Type Conversions ---------------------------------*/
  operator char*          ( ) const;
  operator unsigned char* ( ) const;
  operator signed char*   ( ) const;


long
  asInt                   ( ) const;

unsigned long
  asUnsigned              ( ) const;

#ifdef _LONG_LONG
long long
  asLongLong              ( ) const;

unsigned long long
  asUnsignedLongLong      ( ) const;
#endif

double
  asDouble                ( ) const;

/*------------------------- Binary Conversions -------------------------------*/
IString
 &b2c              ( ),
 &b2d              ( ),
 &b2x              ( );

static IString
  b2c              ( const IString &aString ),
  b2d              ( const IString &aString ),
  b2x              ( const IString &aString );

/*------------------------- Character Conversions ----------------------------*/
IString
 &c2b              ( ),
 &c2d              ( ),
 &c2x              ( );

static IString
  c2b              ( const IString &aString ),
  c2d              ( const IString &aString ),
  c2x              ( const IString &aString );

/*------------------------- Numeric Conversions ------------------------------*/
IString
 &d2b              ( ),
 &d2c              ( ),
 &d2x              ( );

static IString
  d2b              ( const IString &aString ),
  d2c              ( const IString &aString ),
  d2x              ( const IString &aString );

/*------------------------- Hex Conversions ----------------------------------*/
IString
 &x2b              ( ),
 &x2c              ( ),
 &x2d              ( );

static IString
  x2b              ( const IString &aString ),
  x2c              ( const IString &aString ),
  x2d              ( const IString &aString );


/*------------------------- Manipulations ------------------------------------*/
IString
 &operator =       ( const IString &aString ),
  operator ~       (                        ) const,
  operator +       ( const IString &aString ) const,
  operator +       ( const char    *pString ) const,
 &operator +=      ( const IString &aString ),
 &operator +=      ( const char    *pString ),
  operator &       ( const IString &aString ) const,
  operator &       ( const char    *pString ) const,
 &operator &=      ( const IString &aString ),
 &operator &=      ( const char    *pString ),
  operator |       ( const IString &aString ) const,
  operator |       ( const char    *pString ) const,
 &operator |=      ( const IString &aString ),
 &operator |=      ( const char    *pString ),
  operator ^       ( const IString &aString ) const,
  operator ^       ( const char    *pString ) const,
 &operator ^=      ( const IString &aString ),
 &operator ^=      ( const char    *pString );

friend IString
  operator +       ( const char    *pString,
                     const IString &aString ),
  operator &       ( const char    *pString,
                     const IString &aString ),
  operator |       ( const char    *pString,
                     const IString &aString ),
  operator ^       ( const char    *pString,
                     const IString &aString );

/*------------------------- Queries ------------------------------------------*/
unsigned
  size              ( ) const,
  length            ( ) const;

IString
  subString         ( unsigned startPos ) const,
  subString         ( unsigned startPos,
                      unsigned length,
                      char     padCharacter = ' ' ) const;

char
 &operator []       ( unsigned index );

const char
 &operator []       ( unsigned index ) const;

char
 &operator []       ( signed index );

const char
 &operator []       ( signed index ) const;

char
 &operator []       ( unsigned long index );

const char
 &operator []       ( unsigned long index ) const;

char
 &operator []       ( signed long index );

const char
 &operator []       ( signed long index ) const;

IStringEnum::CharType
  charType          ( unsigned index ) const;

/*------------------------- Forward Searches ---------------------------------*/
unsigned
  indexOf           ( const IString     &aString,
                      unsigned           startPos = 1 ) const,
  indexOf           ( const char        *pString,
                      unsigned           startPos = 1 ) const,
  indexOf           ( char              aCharacter,
                      unsigned           startPos = 1 ) const,
  indexOf           ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,

  indexOfAnyBut     ( const IString     &validChars,
                      unsigned           startPos = 1 ) const,
  indexOfAnyBut     ( const char        *pValidChars,
                      unsigned           startPos = 1 ) const,
  indexOfAnyBut     ( char               validChar,
                     unsigned           startPos = 1 ) const,
  indexOfAnyBut     ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,

  indexOfAnyOf      ( const IString     &searchChars,
                      unsigned           startPos = 1 ) const,
  indexOfAnyOf      ( const char        *pSearchChars,
                      unsigned           startPos = 1 ) const,
  indexOfAnyOf      ( char               searchChar,
                      unsigned           startPos = 1 ) const,
  indexOfAnyOf      ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,

  occurrencesOf     ( const IString     &aString,
                      unsigned           startPos = 1 ) const,
  occurrencesOf     ( const char        *pString,
                      unsigned           startPos = 1 ) const,
  occurrencesOf     ( char              aCharacter,
                      unsigned           startPos = 1 ) const,
  occurrencesOf     ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const;


/*------------------------- Reverse Searches ---------------------------------*/
unsigned
  lastIndexOf       ( const IString &aString,
                      unsigned       startPos = (unsigned)UINT_MAX ) const,
  lastIndexOf       ( const char    *pString,
                      unsigned       startPos = (unsigned)UINT_MAX ) const,
  lastIndexOf       ( char          aCharacter,
                      unsigned       startPos = (unsigned)UINT_MAX ) const,
  lastIndexOf       ( const IStringTest &aTest,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,

  lastIndexOfAnyBut ( const IString     &validChars,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyBut ( const char        *pValidChars,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyBut ( char               validChar,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyBut ( const IStringTest &aTest,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,

  lastIndexOfAnyOf  ( const IString     &searchChars,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyOf  ( const char        *pSearchChars,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyOf  ( char               searchChar,
                      unsigned           startPos = (unsigned)UINT_MAX ) const,
  lastIndexOfAnyOf  ( const IStringTest &aTest,
                      unsigned           startPos = (unsigned)UINT_MAX ) const;

/*------------------------- Editing ------------------------------------------*/
IString
 &center            ( unsigned length,
                      char     padCharacter = ' ' ),

 &change            ( const IString &inputString,
                      const IString &outputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
 &change            ( const IString &inputString,
                      const char    *pOutputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
 &change            ( const char    *pInputString,
                      const IString &outputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
 &change            ( const char    *pInputString,
                      const char    *pOutputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),

 &copy              ( unsigned numCopies ),

 &insert            ( const IString &aString,
                      unsigned       index = 0,
                      char           padCharacter = ' ' ),
 &insert            ( const char    *pString,
                      unsigned       index = 0,
                      char           padCharacter = ' ' ),

 &leftJustify       ( unsigned length,
                      char     padCharacter = ' ' ),

 &lowerCase         ( ),

 &overlayWith       ( const IString &aString,
                      unsigned       index        = 1,
                      char           padCharacter = ' ' ),
 &overlayWith       ( const char    *pString,
                      unsigned       index        = 1,
                      char           padCharacter = ' ' ),

 &remove            ( unsigned startPos ),
 &remove            ( unsigned startPos,
                      unsigned numChars ),

 &reverse           ( ),

 &rightJustify      ( unsigned length,
                      char     padCharacter = ' ' ),

 &strip             ( ),
 &strip             ( char               aCharacter ),
 &strip             ( const IString     &aString ),
 &strip             ( const char        *pString ),
 &strip             ( const IStringTest &aTest ),

 &stripLeading      ( ),
 &stripLeading      ( char               aCharacter ),
 &stripLeading      ( const IString     &aString ),
 &stripLeading      ( const char        *pString ),
 &stripLeading      ( const IStringTest &aTest ),

 &stripTrailing     ( ),
 &stripTrailing     ( char               aCharacter ),
 &stripTrailing     ( const IString     &aString ),
 &stripTrailing     ( const char        *pString ),
 &stripTrailing     ( const IStringTest &aTest ),

 &translate         ( const IString &inputChars,
                      const IString &outputChars,
                      char           padCharacter = ' ' ),
 &translate         ( const IString &inputChars,
                      const char    *pOutputChars,
                      char           padCharacter = ' ' ),
 &translate         ( const char    *pInputChars,
                      const IString &outputChars,
                      char           padCharacter = ' ' ),
 &translate         ( const char    *pInputChars,
                      const char    *pOutputChars,
                      char           padCharacter = ' ' ),

 &upperCase         ( );

static IString
  center            ( const IString &aString,
                      unsigned       length,
                      char           padCharacter = ' ' ),

  change            ( const IString &aString,
                      const IString &inputString,
                      const IString &outputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
  change            ( const IString &aString,
                      const IString &inputString,
                      const char    *pOutputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
  change            ( const IString &aString,
                      const char    *pInputString,
                      const IString &outputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),
  change            ( const IString &aString,
                      const char    *pInputString,
                      const char    *pOutputString,
                      unsigned       startPos = 1,
                      unsigned       numChanges = (unsigned)UINT_MAX ),

  copy              ( const IString &aString,
                      unsigned       numCopies ),

  insert            ( const IString &aString,
                      const IString &anInsert,
                      unsigned       index = 0,
                      char           padCharacter = ' ' ),
  insert            ( const IString &aString,
                      const char    *pInsert,
                      unsigned       index = 0,
                      char           padCharacter = ' ' ),

  leftJustify       ( const IString &aString,
                      unsigned       length,
                      char           padCharacter = ' ' ),

  lowerCase         ( const IString &aString ),

  overlayWith       ( const IString &aString,
                      const IString &anOverlay,
                      unsigned       index        = 1,
                      char           padCharacter = ' ' ),
  overlayWith       ( const IString &aString,
                      const char    *pOverlay,
                      unsigned       index        = 1,
                      char           padCharacter = ' ' ),

  remove            ( const IString &aString,
                      unsigned       startPos ),
  remove            ( const IString &aString,
                      unsigned       startPos,
                      unsigned       numChars ),

  reverse           ( const IString &aString ),

  rightJustify      ( const IString &aString,
                      unsigned       length,
                      char           padCharacter = ' ' ),

  stripBlanks       ( const IString &aString ),
  strip             ( const IString &aString,
                      char           aChar ),
  strip             ( const IString &aString,
                      const IString &aStringOfChars ),
  strip             ( const IString &aString,
                      const char    *pStringOfChars ),
  strip             ( const IString &aString,
                      const IStringTest &aTest ),

  stripLeadingBlanks ( const IString &aString ),
  stripLeading      ( const IString &aString,
                      char           aChar ),
  stripLeading      ( const IString &aString,
                      const IString &aStringOfChars ),
  stripLeading      ( const IString &aString,
                      const char    *pStringOfChars ),
  stripLeading      ( const IString &aString,
                      const IStringTest &aTest ),

  stripTrailingBlanks ( const IString &aString ),
  stripTrailing     ( const IString &aString,
                      char           aChar ),
  stripTrailing     ( const IString &aString,
                      const IString &aStringOfChars ),
  stripTrailing     ( const IString &aString,
                      const char    *pStringOfChars ),
  stripTrailing     ( const IString &aString,
                      const IStringTest &aTest ),

  translate         ( const IString &aString,
                      const IString &inputChars,
                      const IString &outputChars,
                      char           padCharacter = ' ' ),
  translate         ( const IString &aString,
                      const IString &inputChars,
                      const char    *pOutputChars,
                      char           padCharacter = ' ' ),
  translate         ( const IString &aString,
                      const char    *pInputChars,
                      const IString &outputChars,
                      char           padCharacter = ' ' ),
  translate         ( const IString &aString,
                      const char    *pInputChars,
                      const char    *pOutputChars,
                      char           padCharacter = ' ' ),

  upperCase         ( const IString &aString );

/*------------------------- Word Operations ----------------------------------*/
IString
 &removeWords       ( unsigned firstWord ),
 &removeWords       ( unsigned firstWord,
                      unsigned numWords ),

 &space             ( unsigned numSpaces = 1,
                      char     spaceChar = ' ' ),

  word              ( unsigned  wordNumber ) const,

  words             ( unsigned  firstWord ) const,
  words             ( unsigned  firstWord,
                      unsigned  numWords  ) const;

unsigned
  indexOfPhrase     ( const IString &wordString,
                      unsigned       startWord = 1 ) const,

  indexOfWord       ( unsigned wordNumber ) const,

  lengthOfWord      ( unsigned wordNumber ) const,

  numWords          ( ) const,

  wordIndexOfPhrase ( const IString &aPhrase,
                      unsigned       startWord = 1 ) const;

static IString
  space             ( const IString &aString,
                      unsigned       numSpaces = 1,
                      char           spaceChar = ' ' ),

  removeWords       ( const IString &aString,
                      unsigned       startWord ),
  removeWords       ( const IString &aString,
                      unsigned       startWord,
                      unsigned       numWords  );
#ifdef IC_NLS
static void     enableInternationalization(Boolean enable = true),
                disableInternationalization();

static Boolean  isInternationalized();
#endif

protected:
/*------------------------- Utility Data -------------------------------------*/
static const char
 IC_IMPORTB * const null,
 IC_IMPORTB * const zero,
 IC_IMPORTB * const maxLong;

/*-------------------------- To Get at the nullBuffer ------------------------*/
static const char * nullBuf();


/*-------------------------- Initialize --------------------------------------*/
IString
 &setBuffer         ( IBuffer *ibuff ),

 &initBuffer        ( const void *p1,
                      unsigned    len1,
                      const void *p2      = 0,
                      unsigned    len2    = 0,
                      const void *p3      = 0,
                      unsigned    len3    = 0,
                      char        padChar = 0 ),
 &initBuffer        ( long  aLong ),
 &initBuffer        ( unsigned long  anUnsignedLong ),
#ifdef _LONG_LONG
 &initBuffer        ( long long  aLongLong ),
 &initBuffer        ( unsigned long long  anUnsignedLongLong ),
#endif
 &initBuffer        ( double  aDouble );


/*-------------------------- Protected Queries -------------------------------*/
char
 *data              ( ) const;

IBuffer
 *buffer            ( ) const;

static unsigned
  lengthOf          ( const char *pChar );

static char
 *defaultBuffer     ( );


/*------------------------- Protected Searches/Testing -----------------------*/
typedef enum {
  charIndex,
  wordIndex
  } IndexType;

unsigned
  indexOfWord       ( unsigned wordNumber,
                      unsigned startPos,
                      unsigned numWords ) const,

  occurrencesOf     ( const char *pSearchString,
                      unsigned    searchLen,
                      unsigned    startPos ) const,


  findPhrase        ( const IString &aPhrase,
                      unsigned       startWord,
                      IndexType      charOrWord ) const;

Boolean
  isLike            ( const char *pPattern,
                      unsigned    patternLen,
                      char        zeroOrMore,
                      char        anyChar ) const,

  isAbbrevFor       ( const char *pFullString,
                      unsigned    fullLen,
                      unsigned    minLen ) const;

/*------------------------- Protected Editing --------------------------------*/
IString
 &change            ( const char *pPattern,
                      unsigned    patternLen,
                      const char *pReplacement,
                      unsigned    replacementLen,
                      unsigned    startPos,
                      unsigned    numChanges ),

 &insert            ( const char *pInsert,
                      unsigned    insertLen,
                      unsigned    startPos,
                      char        padCharacter ),

 &overlayWith       ( const char *pOverlay,
                      unsigned    overlayLen,
                      unsigned    index,
                      char        padCharacter ),

 &strip             ( const char             *pChar,
                      unsigned                len,
                      IStringEnum::StripMode  mode ),

 &strip             ( const IStringTest      &aTest,
                      IStringEnum::StripMode  mode ),

 &translate         ( const char *pInputChars,
                      unsigned    inputLen,
                      const char *pOutputChars,
                      unsigned    outputLen,
                      char        padCharacter );


/*------------------------- Bit Operations -----------------------------------*/
typedef enum {
  bitAnd,
  bitOr,
  bitExclusiveOr
#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifndef __DIGRAPHS__
  ,
  and = bitAnd,
  or  = bitOr,
  exclusiveOr = bitExclusiveOr
#endif
#endif
  } BitOperator;

IString
 &applyBitOp        ( const char *pArg,
                      unsigned    argLen,
                      BitOperator op );


private:
/*----------------------------- Private --------------------------------------*/
  IString           ( IBuffer *pBuffer );
void
  binaryMath        ( unsigned char newDigit ),
  decimalMath       ( unsigned char newDigit );
IString
 &prepareToChange   ( );
char
 *pBuffer;

static void
  setDefaultBuffer  ( char * );

static char
  *nullBuffer;

friend class IBuffer;

#ifdef IC_NLS
typedef enum {                     /*  I18N states definition   */
  i18nOff = 0,
  i18nOn
  } I18NStates;

static I18NStates
   fgI18NState;
#endif




#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)

#ifdef _LONG_LONG
#pragma SOMReleaseOrder( \
/*   1 */       IString(int),\
/*   2 */       IString(unsigned int),\
/*   3 */       IString(long),\
/*   4 */       IString(unsigned long),\
/*   5 */       IString(short),\
/*   6 */       IString(unsigned short),\
/*   7 */       IString(double),\
/*   8 */       IString(char),\
/*   9 */       IString(unsigned char),\
/*  10 */       IString(signed char),\
/*  11 */       IString(const char*),\
/*  12 */       IString(const unsigned char*),\
/*  13 */       IString(const signed char*),\
/*  14 */       IString(const void*,unsigned int,char),\
/*  15 */       IString(const void*,unsigned int,   \
                        const void*,unsigned int,char),\
/*  16 */       IString(const void*,unsigned int,const void*,  \
                        unsigned int,const void*,unsigned int,char),\
/*  17 */       asString() const,\
/*  18 */       asDebugInfo() const,\
/*  19 */       lineFrom(istream&,char),\
/*  20 */       isAlphanumeric() const,\
/*  21 */       isAlphabetic() const,\
/*  22 */       isASCII() const,\
/*  23 */       isBinaryDigits() const,\
/*  24 */       isControl() const,\
/*  25 */       isDigits() const,\
/*  26 */       isGraphics() const,\
/*  27 */       isHexDigits() const,\
/*  28 */       isLowerCase() const,\
/*  29 */       isPrintable() const,\
/*  30 */       isPunctuation() const,\
/*  31 */       isUpperCase() const,\
/*  32 */       isWhiteSpace() const,\
/*  33 */       isMBCS() const,\
/*  34 */       isDBCS() const,\
/*  35 */       isSBCS() const,\
/*  36 */       isValidMBCS() const,\
/*  37 */       isValidDBCS() const,\
/*  38 */       includesMBCS() const,\
/*  39 */       includesDBCS() const,\
/*  40 */       includesSBCS() const,\
/*  41 */       isLike(const IString&,char,char) const,\
/*  42 */       isLike(const char*,char,char) const,\
/*  43 */       isAbbreviationFor(const IString&,unsigned int) const,\
/*  44 */       isAbbreviationFor(const char*,unsigned int) const,\
/*  45 */       includes(const IString&) const,\
/*  46 */       includes(const char*) const,\
/*  47 */       includes(char) const,\
/*  48 */       includes(const IStringTest&) const,\
/*  49 */       operator char*() const,\
/*  50 */       operator unsigned char*() const,\
/*  51 */       operator signed char*() const,\
/*  52 */       asInt() const,\
/*  53 */       asUnsigned() const,\
/*  54 */       asDouble() const,\
/*  55 */       b2c(),\
/*  56 */       b2d(),\
/*  57 */       b2x(),\
/*  58 */       b2c(const IString&),\
/*  59 */       b2d(const IString&),\
/*  60 */       b2x(const IString&),\
/*  61 */       c2b(),\
/*  62 */       c2d(),\
/*  63 */       c2x(),\
/*  64 */       c2b(const IString&),\
/*  65 */       c2d(const IString&),\
/*  66 */       c2x(const IString&),\
/*  67 */       d2b(),\
/*  68 */       d2c(),\
/*  69 */       d2x(),\
/*  70 */       d2b(const IString&),\
/*  71 */       d2c(const IString&),\
/*  72 */       d2x(const IString&),\
/*  73 */       x2b(),\
/*  74 */       x2c(),\
/*  75 */       x2d(),\
/*  76 */       x2b(const IString&),\
/*  77 */       x2c(const IString&),\
/*  78 */       x2d(const IString&),\
/*  79 */       operator=(const IString&),\
/*  80 */       operator~() const,\
/*  81 */       operator+(const IString&) const,\
/*  82 */       operator+(const char*) const,\
/*  83 */       operator+=(const IString&),\
/*  84 */       operator+=(const char*),\
/*  85 */       operator&(const IString&) const,\
/*  86 */       operator&(const char*) const,\
/*  87 */       operator&=(const IString&),\
/*  88 */       operator&=(const char*),\
/*  89 */       operator|(const IString&) const,\
/*  90 */       operator|(const char*) const,\
/*  91 */       operator|=(const IString&),\
/*  92 */       operator|=(const char*),\
/*  93 */       operator^(const IString&) const,\
/*  94 */       operator^(const char*) const,\
/*  95 */       operator^=(const IString&),\
/*  96 */       operator^=(const char*),\
/*  97 */       size() const,\
/*  98 */       length() const,\
/*  99 */       subString(unsigned int) const,\
/* 100 */       subString(unsigned int,unsigned int,char) const,\
/* 101 */       operator[](unsigned int),\
/* 102 */       operator[](unsigned int) const,\
/* 103 */       charType(unsigned int) const,\
/* 104 */       indexOf(const IString&,unsigned int) const,\
/* 105 */       indexOf(const char*,unsigned int) const,\
/* 106 */       indexOf(char,unsigned int) const,\
/* 107 */       indexOf(const IStringTest&,unsigned int) const,\
/* 108 */       indexOfAnyBut(const IString&,unsigned int) const,\
/* 109 */       indexOfAnyBut(const char*,unsigned int) const,\
/* 110 */       indexOfAnyBut(char,unsigned int) const,\
/* 111 */       indexOfAnyBut(const IStringTest&,unsigned int) const,\
/* 112 */       indexOfAnyOf(const IString&,unsigned int) const,\
/* 113 */       indexOfAnyOf(const char*,unsigned int) const,\
/* 114 */       indexOfAnyOf(char,unsigned int) const,\
/* 115 */       indexOfAnyOf(const IStringTest&,unsigned int) const,\
/* 116 */       occurrencesOf(const IString&,unsigned int) const,\
/* 117 */       occurrencesOf(const char*,unsigned int) const,\
/* 118 */       occurrencesOf(char,unsigned int) const,\
/* 119 */       occurrencesOf(const IStringTest&,unsigned int) const,\
/* 120 */       lastIndexOf(const IString&,unsigned int) const,\
/* 121 */       lastIndexOf(const char*,unsigned int) const,\
/* 122 */       lastIndexOf(char,unsigned int) const,\
/* 123 */       lastIndexOf(const IStringTest&,unsigned int) const,\
/* 124 */       lastIndexOfAnyBut(const IString&,unsigned int) const,\
/* 125 */       lastIndexOfAnyBut(const char*,unsigned int) const,\
/* 126 */       lastIndexOfAnyBut(char,unsigned int) const,\
/* 127 */       lastIndexOfAnyBut(const IStringTest&,unsigned int) const,\
/* 128 */       lastIndexOfAnyOf(const IString&,unsigned int) const,\
/* 129 */       lastIndexOfAnyOf(const char*,unsigned int) const,\
/* 130 */       lastIndexOfAnyOf(char,unsigned int) const,\
/* 131 */       lastIndexOfAnyOf(const IStringTest&,unsigned int) const,\
/* 132 */       center(unsigned int,char),\
/* 133 */       change(const IString&,const IString&,  \
                       unsigned int,unsigned int),\
/* 134 */       change(const IString&,const char*,   \
                       unsigned int,unsigned int),\
/* 135 */       change(const char*,const IString&,  \
                       unsigned int,unsigned int),\
/* 136 */       change(const char*,const char*,unsigned int,unsigned int),\
/* 137 */       copy(unsigned int),\
/* 138 */       insert(const IString&,unsigned int,char),\
/* 139 */       insert(const char*,unsigned int,char),\
/* 140 */       leftJustify(unsigned int,char),\
/* 141 */       lowerCase(),\
/* 142 */       overlayWith(const IString&,unsigned int,char),\
/* 143 */       overlayWith(const char*,unsigned int,char),\
/* 144 */       remove(unsigned int),\
/* 145 */       remove(unsigned int,unsigned int),\
/* 146 */       reverse(),\
/* 147 */       rightJustify(unsigned int,char),\
/* 148 */       strip(),\
/* 149 */       strip(char),\
/* 150 */       strip(const IString&),\
/* 151 */       strip(const char*),\
/* 152 */       strip(const IStringTest&),\
/* 153 */       stripLeading(),\
/* 154 */       stripLeading(char),\
/* 155 */       stripLeading(const IString&),\
/* 156 */       stripLeading(const char*),\
/* 157 */       stripLeading(const IStringTest&),\
/* 158 */       stripTrailing(),\
/* 159 */       stripTrailing(char),\
/* 160 */       stripTrailing(const IString&),\
/* 161 */       stripTrailing(const char*),\
/* 162 */       stripTrailing(const IStringTest&),\
/* 163 */       translate(const IString&,const IString&,char),\
/* 164 */       translate(const IString&,const char*,char),\
/* 165 */       translate(const char*,const IString&,char),\
/* 166 */       translate(const char*,const char*,char),\
/* 167 */       upperCase(),\
/* 168 */       center(const IString&,unsigned int,char),\
/* 169 */       change(const IString&,const IString&, \
                       const IString&, unsigned int,unsigned int),\
/* 170 */       change(const IString&,const IString&,  \
                       const char*,unsigned int,unsigned int),\
/* 171 */       change(const IString&,const char*,  \
                       const IString&,unsigned int,unsigned int),\
/* 172 */       change(const IString&,const char*,  \
                       const char*,unsigned int,unsigned int),\
/* 173 */       copy(const IString&,unsigned int),\
/* 174 */       insert(const IString&,const IString&,unsigned int,char),\
/* 175 */       insert(const IString&,const char*,unsigned int,char),\
/* 176 */       leftJustify(const IString&,unsigned int,char),\
/* 177 */       lowerCase(const IString&),\
/* 178 */       overlayWith(const IString&,const IString&,unsigned int,char),\
/* 179 */       overlayWith(const IString&,const char*,unsigned int,char),\
/* 180 */       remove(const IString&,unsigned int),\
/* 181 */       remove(const IString&,unsigned int,unsigned int),\
/* 182 */       reverse(const IString&),\
/* 183 */       rightJustify(const IString&,unsigned int,char),\
/* 184 */       stripBlanks(const IString&),\
/* 185 */       strip(const IString&,char),\
/* 186 */       strip(const IString&,const IString&),\
/* 187 */       strip(const IString&,const char*),\
/* 188 */       strip(const IString&,const IStringTest&),\
/* 189 */       stripLeadingBlanks(const IString&),\
/* 190 */       stripLeading(const IString&,char),\
/* 191 */       stripLeading(const IString&,const IString&),\
/* 192 */       stripLeading(const IString&,const char*),\
/* 193 */       stripLeading(const IString&,const IStringTest&),\
/* 194 */       stripTrailingBlanks(const IString&),\
/* 195 */       stripTrailing(const IString&,char),\
/* 196 */       stripTrailing(const IString&,const IString&),\
/* 197 */       stripTrailing(const IString&,const char*),\
/* 198 */       stripTrailing(const IString&,const IStringTest&),\
/* 199 */       translate(const IString&,const IString&,const IString&,char),\
/* 200 */       translate(const IString&,const IString&,const char*,char),\
/* 201 */       translate(const IString&,const char*,const IString&,char),\
/* 202 */       translate(const IString&,const char*,const char*,char),\
/* 203 */       upperCase(const IString&),\
/* 204 */       removeWords(unsigned int),\
/* 205 */       removeWords(unsigned int,unsigned int),\
/* 206 */       space(unsigned int,char),\
/* 207 */       word(unsigned int) const,\
/* 208 */       words(unsigned int) const,\
/* 209 */       words(unsigned int,unsigned int) const,\
/* 210 */       indexOfPhrase(const IString&,unsigned int) const,\
/* 211 */       indexOfWord(unsigned int) const,\
/* 212 */       lengthOfWord(unsigned int) const,\
/* 213 */       numWords() const,\
/* 214 */       wordIndexOfPhrase(const IString&,unsigned int) const,\
/* 215 */       space(const IString&,unsigned int,char),\
/* 216 */       removeWords(const IString&,unsigned int),\
/* 217 */       removeWords(const IString&,unsigned int,unsigned int),\
/* 218 */       null,\
/* 219 */       zero,\
/* 220 */       maxLong,\
/* 221 */       nullBuffer,\
/* 222 */       setBuffer(IBuffer*),\
/* 223 */       initBuffer(const void*,unsigned int,const void*, \
                           unsigned int,const void*,unsigned int,char),\
/* 224 */       initBuffer(long),\
/* 225 */       initBuffer(unsigned long),\
/* 226 */       initBuffer(double),\
/* 227 */       data() const,\
/* 228 */       buffer() const,\
/* 229 */       lengthOf(const char*),\
/* 230 */       defaultBuffer(),\
/* 231 */       indexOfWord(unsigned int,unsigned int,unsigned int) const,\
/* 232 */       occurrencesOf(const char*,unsigned int,unsigned int) const,\
/* 233 */       findPhrase(const IString&,unsigned int,IndexType) const,\
/* 234 */       isLike(const char*,unsigned int,char,char) const,\
/* 235 */       isAbbrevFor(const char*,unsigned int,unsigned int) const,\
/* 236 */       change(const char*,unsigned int,const char*, \
                       unsigned int,unsigned int,unsigned int),\
/* 237 */       insert(const char*,unsigned int,unsigned int,char),\
/* 238 */       overlayWith(const char*,unsigned int,unsigned int,char),\
/* 239 */       strip(const char*,unsigned int,IStringEnum::StripMode),\
/* 240 */       strip(const IStringTest&,IStringEnum::StripMode),\
/* 241 */       translate(const char*,unsigned int,  \
                          const char*,unsigned int,char),\
/* 242 */       applyBitOp(const char*,unsigned int,BitOperator),\
/* 243 */       IString(IBuffer*),\
/* 244 */       binaryMath(unsigned char),\
/* 245 */       decimalMath(unsigned char),\
/* 246 */       prepareToChange(),\
/* 247 */       setDefaultBuffer(char*),\
/* 248 */       operator[](signed int),\
/* 249 */       operator[](signed int) const,\
/* 250 */       operator[](unsigned long),\
/* 251 */       operator[](unsigned long) const,\
/* 252 */       operator[](signed long),\
/* 253 */       operator[](signed long) const,\
/* 254 */       IString(long long),\
/* 255 */       IString(unsigned long long),\
/* 256 */       asLongLong() const,\
/* 257 */       asUnsignedLongLong() const,\
/* 258 */       initBuffer(long long),\
/* 259 */       initBuffer(unsigned long long),\
/* 260 */       enableInternationalization(Boolean),\
/* 261 */       disableInternationalization(),\
/* 262 */       isInternationalized(),\
/* 263 */       fgI18NState,\
/* 264 */       nullBuf() )

#else

#pragma SOMReleaseOrder( \
/*   1 */       IString(int),\
/*   2 */       IString(unsigned int),\
/*   3 */       IString(long),\
/*   4 */       IString(unsigned long),\
/*   5 */       IString(short),\
/*   6 */       IString(unsigned short),\
/*   7 */       IString(double),\
/*   8 */       IString(char),\
/*   9 */       IString(unsigned char),\
/*  10 */       IString(signed char),\
/*  11 */       IString(const char*),\
/*  12 */       IString(const unsigned char*),\
/*  13 */       IString(const signed char*),\
/*  14 */       IString(const void*,unsigned int,char),\
/*  15 */       IString(const void*,unsigned int,   \
                        const void*,unsigned int,char),\
/*  16 */       IString(const void*,unsigned int,const void*,  \
                        unsigned int,const void*,unsigned int,char),\
/*  17 */       asString() const,\
/*  18 */       asDebugInfo() const,\
/*  19 */       lineFrom(istream&,char),\
/*  20 */       isAlphanumeric() const,\
/*  21 */       isAlphabetic() const,\
/*  22 */       isASCII() const,\
/*  23 */       isBinaryDigits() const,\
/*  24 */       isControl() const,\
/*  25 */       isDigits() const,\
/*  26 */       isGraphics() const,\
/*  27 */       isHexDigits() const,\
/*  28 */       isLowerCase() const,\
/*  29 */       isPrintable() const,\
/*  30 */       isPunctuation() const,\
/*  31 */       isUpperCase() const,\
/*  32 */       isWhiteSpace() const,\
/*  33 */       isMBCS() const,\
/*  34 */       isDBCS() const,\
/*  35 */       isSBCS() const,\
/*  36 */       isValidMBCS() const,\
/*  37 */       isValidDBCS() const,\
/*  38 */       includesMBCS() const,\
/*  39 */       includesDBCS() const,\
/*  40 */       includesSBCS() const,\
/*  41 */       isLike(const IString&,char,char) const,\
/*  42 */       isLike(const char*,char,char) const,\
/*  43 */       isAbbreviationFor(const IString&,unsigned int) const,\
/*  44 */       isAbbreviationFor(const char*,unsigned int) const,\
/*  45 */       includes(const IString&) const,\
/*  46 */       includes(const char*) const,\
/*  47 */       includes(char) const,\
/*  48 */       includes(const IStringTest&) const,\
/*  49 */       operator char*() const,\
/*  50 */       operator unsigned char*() const,\
/*  51 */       operator signed char*() const,\
/*  52 */       asInt() const,\
/*  53 */       asUnsigned() const,\
/*  54 */       asDouble() const,\
/*  55 */       b2c(),\
/*  56 */       b2d(),\
/*  57 */       b2x(),\
/*  58 */       b2c(const IString&),\
/*  59 */       b2d(const IString&),\
/*  60 */       b2x(const IString&),\
/*  61 */       c2b(),\
/*  62 */       c2d(),\
/*  63 */       c2x(),\
/*  64 */       c2b(const IString&),\
/*  65 */       c2d(const IString&),\
/*  66 */       c2x(const IString&),\
/*  67 */       d2b(),\
/*  68 */       d2c(),\
/*  69 */       d2x(),\
/*  70 */       d2b(const IString&),\
/*  71 */       d2c(const IString&),\
/*  72 */       d2x(const IString&),\
/*  73 */       x2b(),\
/*  74 */       x2c(),\
/*  75 */       x2d(),\
/*  76 */       x2b(const IString&),\
/*  77 */       x2c(const IString&),\
/*  78 */       x2d(const IString&),\
/*  79 */       operator=(const IString&),\
/*  80 */       operator~() const,\
/*  81 */       operator+(const IString&) const,\
/*  82 */       operator+(const char*) const,\
/*  83 */       operator+=(const IString&),\
/*  84 */       operator+=(const char*),\
/*  85 */       operator&(const IString&) const,\
/*  86 */       operator&(const char*) const,\
/*  87 */       operator&=(const IString&),\
/*  88 */       operator&=(const char*),\
/*  89 */       operator|(const IString&) const,\
/*  90 */       operator|(const char*) const,\
/*  91 */       operator|=(const IString&),\
/*  92 */       operator|=(const char*),\
/*  93 */       operator^(const IString&) const,\
/*  94 */       operator^(const char*) const,\
/*  95 */       operator^=(const IString&),\
/*  96 */       operator^=(const char*),\
/*  97 */       size() const,\
/*  98 */       length() const,\
/*  99 */       subString(unsigned int) const,\
/* 100 */       subString(unsigned int,unsigned int,char) const,\
/* 101 */       operator[](unsigned int),\
/* 102 */       operator[](unsigned int) const,\
/* 103 */       charType(unsigned int) const,\
/* 104 */       indexOf(const IString&,unsigned int) const,\
/* 105 */       indexOf(const char*,unsigned int) const,\
/* 106 */       indexOf(char,unsigned int) const,\
/* 107 */       indexOf(const IStringTest&,unsigned int) const,\
/* 108 */       indexOfAnyBut(const IString&,unsigned int) const,\
/* 109 */       indexOfAnyBut(const char*,unsigned int) const,\
/* 110 */       indexOfAnyBut(char,unsigned int) const,\
/* 111 */       indexOfAnyBut(const IStringTest&,unsigned int) const,\
/* 112 */       indexOfAnyOf(const IString&,unsigned int) const,\
/* 113 */       indexOfAnyOf(const char*,unsigned int) const,\
/* 114 */       indexOfAnyOf(char,unsigned int) const,\
/* 115 */       indexOfAnyOf(const IStringTest&,unsigned int) const,\
/* 116 */       occurrencesOf(const IString&,unsigned int) const,\
/* 117 */       occurrencesOf(const char*,unsigned int) const,\
/* 118 */       occurrencesOf(char,unsigned int) const,\
/* 119 */       occurrencesOf(const IStringTest&,unsigned int) const,\
/* 120 */       lastIndexOf(const IString&,unsigned int) const,\
/* 121 */       lastIndexOf(const char*,unsigned int) const,\
/* 122 */       lastIndexOf(char,unsigned int) const,\
/* 123 */       lastIndexOf(const IStringTest&,unsigned int) const,\
/* 124 */       lastIndexOfAnyBut(const IString&,unsigned int) const,\
/* 125 */       lastIndexOfAnyBut(const char*,unsigned int) const,\
/* 126 */       lastIndexOfAnyBut(char,unsigned int) const,\
/* 127 */       lastIndexOfAnyBut(const IStringTest&,unsigned int) const,\
/* 128 */       lastIndexOfAnyOf(const IString&,unsigned int) const,\
/* 129 */       lastIndexOfAnyOf(const char*,unsigned int) const,\
/* 130 */       lastIndexOfAnyOf(char,unsigned int) const,\
/* 131 */       lastIndexOfAnyOf(const IStringTest&,unsigned int) const,\
/* 132 */       center(unsigned int,char),\
/* 133 */       change(const IString&,const IString&,  \
                       unsigned int,unsigned int),\
/* 134 */       change(const IString&,const char*,   \
                       unsigned int,unsigned int),\
/* 135 */       change(const char*,const IString&,  \
                       unsigned int,unsigned int),\
/* 136 */       change(const char*,const char*,unsigned int,unsigned int),\
/* 137 */       copy(unsigned int),\
/* 138 */       insert(const IString&,unsigned int,char),\
/* 139 */       insert(const char*,unsigned int,char),\
/* 140 */       leftJustify(unsigned int,char),\
/* 141 */       lowerCase(),\
/* 142 */       overlayWith(const IString&,unsigned int,char),\
/* 143 */       overlayWith(const char*,unsigned int,char),\
/* 144 */       remove(unsigned int),\
/* 145 */       remove(unsigned int,unsigned int),\
/* 146 */       reverse(),\
/* 147 */       rightJustify(unsigned int,char),\
/* 148 */       strip(),\
/* 149 */       strip(char),\
/* 150 */       strip(const IString&),\
/* 151 */       strip(const char*),\
/* 152 */       strip(const IStringTest&),\
/* 153 */       stripLeading(),\
/* 154 */       stripLeading(char),\
/* 155 */       stripLeading(const IString&),\
/* 156 */       stripLeading(const char*),\
/* 157 */       stripLeading(const IStringTest&),\
/* 158 */       stripTrailing(),\
/* 159 */       stripTrailing(char),\
/* 160 */       stripTrailing(const IString&),\
/* 161 */       stripTrailing(const char*),\
/* 162 */       stripTrailing(const IStringTest&),\
/* 163 */       translate(const IString&,const IString&,char),\
/* 164 */       translate(const IString&,const char*,char),\
/* 165 */       translate(const char*,const IString&,char),\
/* 166 */       translate(const char*,const char*,char),\
/* 167 */       upperCase(),\
/* 168 */       center(const IString&,unsigned int,char),\
/* 169 */       change(const IString&,const IString&, \
                       const IString&, unsigned int,unsigned int),\
/* 170 */       change(const IString&,const IString&,  \
                       const char*,unsigned int,unsigned int),\
/* 171 */       change(const IString&,const char*,  \
                       const IString&,unsigned int,unsigned int),\
/* 172 */       change(const IString&,const char*,  \
                       const char*,unsigned int,unsigned int),\
/* 173 */       copy(const IString&,unsigned int),\
/* 174 */       insert(const IString&,const IString&,unsigned int,char),\
/* 175 */       insert(const IString&,const char*,unsigned int,char),\
/* 176 */       leftJustify(const IString&,unsigned int,char),\
/* 177 */       lowerCase(const IString&),\
/* 178 */       overlayWith(const IString&,const IString&,unsigned int,char),\
/* 179 */       overlayWith(const IString&,const char*,unsigned int,char),\
/* 180 */       remove(const IString&,unsigned int),\
/* 181 */       remove(const IString&,unsigned int,unsigned int),\
/* 182 */       reverse(const IString&),\
/* 183 */       rightJustify(const IString&,unsigned int,char),\
/* 184 */       stripBlanks(const IString&),\
/* 185 */       strip(const IString&,char),\
/* 186 */       strip(const IString&,const IString&),\
/* 187 */       strip(const IString&,const char*),\
/* 188 */       strip(const IString&,const IStringTest&),\
/* 189 */       stripLeadingBlanks(const IString&),\
/* 190 */       stripLeading(const IString&,char),\
/* 191 */       stripLeading(const IString&,const IString&),\
/* 192 */       stripLeading(const IString&,const char*),\
/* 193 */       stripLeading(const IString&,const IStringTest&),\
/* 194 */       stripTrailingBlanks(const IString&),\
/* 195 */       stripTrailing(const IString&,char),\
/* 196 */       stripTrailing(const IString&,const IString&),\
/* 197 */       stripTrailing(const IString&,const char*),\
/* 198 */       stripTrailing(const IString&,const IStringTest&),\
/* 199 */       translate(const IString&,const IString&,const IString&,char),\
/* 200 */       translate(const IString&,const IString&,const char*,char),\
/* 201 */       translate(const IString&,const char*,const IString&,char),\
/* 202 */       translate(const IString&,const char*,const char*,char),\
/* 203 */       upperCase(const IString&),\
/* 204 */       removeWords(unsigned int),\
/* 205 */       removeWords(unsigned int,unsigned int),\
/* 206 */       space(unsigned int,char),\
/* 207 */       word(unsigned int) const,\
/* 208 */       words(unsigned int) const,\
/* 209 */       words(unsigned int,unsigned int) const,\
/* 210 */       indexOfPhrase(const IString&,unsigned int) const,\
/* 211 */       indexOfWord(unsigned int) const,\
/* 212 */       lengthOfWord(unsigned int) const,\
/* 213 */       numWords() const,\
/* 214 */       wordIndexOfPhrase(const IString&,unsigned int) const,\
/* 215 */       space(const IString&,unsigned int,char),\
/* 216 */       removeWords(const IString&,unsigned int),\
/* 217 */       removeWords(const IString&,unsigned int,unsigned int),\
/* 218 */       null,\
/* 219 */       zero,\
/* 220 */       maxLong,\
/* 221 */       nullBuffer,\
/* 222 */       setBuffer(IBuffer*),\
/* 223 */       initBuffer(const void*,unsigned int,const void*, \
                           unsigned int,const void*,unsigned int,char),\
/* 224 */       initBuffer(long),\
/* 225 */       initBuffer(unsigned long),\
/* 226 */       initBuffer(double),\
/* 227 */       data() const,\
/* 228 */       buffer() const,\
/* 229 */       lengthOf(const char*),\
/* 230 */       defaultBuffer(),\
/* 231 */       indexOfWord(unsigned int,unsigned int,unsigned int) const,\
/* 232 */       occurrencesOf(const char*,unsigned int,unsigned int) const,\
/* 233 */       findPhrase(const IString&,unsigned int,IndexType) const,\
/* 234 */       isLike(const char*,unsigned int,char,char) const,\
/* 235 */       isAbbrevFor(const char*,unsigned int,unsigned int) const,\
/* 236 */       change(const char*,unsigned int,const char*, \
                       unsigned int,unsigned int,unsigned int),\
/* 237 */       insert(const char*,unsigned int,unsigned int,char),\
/* 238 */       overlayWith(const char*,unsigned int,unsigned int,char),\
/* 239 */       strip(const char*,unsigned int,IStringEnum::StripMode),\
/* 240 */       strip(const IStringTest&,IStringEnum::StripMode),\
/* 241 */       translate(const char*,unsigned int,  \
                          const char*,unsigned int,char),\
/* 242 */       applyBitOp(const char*,unsigned int,BitOperator),\
/* 243 */       IString(IBuffer*),\
/* 244 */       binaryMath(unsigned char),\
/* 245 */       decimalMath(unsigned char),\
/* 246 */       prepareToChange(),\
/* 247 */       setDefaultBuffer(char*),\
/* 248 */       operator[](signed int),\
/* 249 */       operator[](signed int) const,\
/* 250 */       operator[](unsigned long),\
/* 251 */       operator[](unsigned long) const,\
/* 252 */       operator[](signed long),\
/* 253 */       operator[](signed long) const,\
/* 254 */       enableInternationalization(Boolean),\
/* 255 */       disableInternationalization(),\
/* 256 */       isInternationalized(),\
/* 257 */       fgI18NState,\
/* 258 */       nullBuf() )
#endif



#endif    // SOM_ENABLED & RRBC_LIB


}; // class IString


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#pragma pack()

#ifndef I_NO_INLINES
  #include <istring.inl>
#endif

#endif /* _ISTRING_ */


#ifdef __MVS__
                   #pragma checkout(resume)
#endif

