#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif

#ifndef _ITRACE_
#define _ITRACE_
/*******************************************************************************
* FILE NAME: itrace.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ITrace                                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>

#pragma pack(4)


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

class IString;

class ITrace : public IVBase {
typedef IVBase
  Inherited;

friend class ITraceSetup;

public:
/*------------------------- Constructors -------------------------------------*/
  ITrace                   ( const char* traceName =0,
                             long        lineNumber=0 );

 ~ITrace                   ( );

/*-------------------------- Output/Location ---------------------------------*/
static void
  write                    ( const IString& text ),
  write                    ( const char*    text );

enum Destination           {
  queue,
  standardError,
  standardOutput,
  file
  };

static ITrace::Destination
  traceDestination         ( );

static void
  writeToQueue             ( ),
  writeToStandardError     ( ),
  writeToStandardOutput    ( ),
  writeToFile              ( );

/*------------------------- Enable/Disable -----------------------------------*/
static void
  enableTrace              ( ),
  disableTrace             ( );

static Boolean
  isTraceEnabled           ( );

/*------------------------- Formatting ---------------------------------------*/
static void
  enableWriteLineNumber    ( ),
  disableWriteLineNumber   ( ),
  enableWritePrefix        ( ),
  disableWritePrefix       ( );

static Boolean
  isWriteLineNumberEnabled ( ),
  isWritePrefixEnabled     ( );


protected:
/*------------------------- Thread ID ----------------------------------------*/
static unsigned long
  threadId                 ( );

/*------------------------- Protected Output Operations ----------------------*/
static void
  writeString              ( char*          text   ),
  writeFormattedString     ( const IString& string,
                             char*          marker );

private:
/*------------------------- Private ------------------------------------------*/

enum State {
  uninitialized=1,
  on=2,
  writeLineNumber=4,
  writePrefix=8
  };
char
 *pszClTraceName;
static int
  iClState;
static ITrace::Destination
  iClTraceLocation;
static unsigned long
  remainingStack ( );


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)

#pragma SOMReleaseOrder( \
/*   1 */       ITrace(const char*,long),\
/*   2 */       write(const IString&),\
/*   3 */       write(const char*),\
/*   4 */       traceDestination(),\
/*   5 */       writeToQueue(),\
/*   6 */       writeToStandardError(),\
/*   7 */       writeToStandardOutput(),\
/*   8 */       enableTrace(),\
/*   9 */       disableTrace(),\
/*  10 */       isTraceEnabled(),\
/*  11 */       enableWriteLineNumber(),\
/*  12 */       disableWriteLineNumber(),\
/*  13 */       enableWritePrefix(),\
/*  14 */       disableWritePrefix(),\
/*  15 */       isWriteLineNumberEnabled(),\
/*  16 */       isWritePrefixEnabled(),\
/*  17 */       threadId(),\
/*  18 */       writeString(char*),\
/*  19 */       writeFormattedString(const IString&,char*),\
/*  20 */       iClState,\
/*  21 */       iClTraceLocation,\
/*  22 */       remainingStack(),\
/*  23 */       writeToFile())

#endif    // SOM_ENABLED & RRBC_LIB


};


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif


#ifndef __FUNCTION__
  #define __FUNCTION__ "f"
#endif

#ifdef IC_TRACE_ALL
   #define IMODTRACE_ALL(modname)  ITrace trc(modname, __LINE__ )
   #define IFUNCTRACE_ALL()        ITrace trc(__FUNCTION__, __LINE__ )
   #define ITRACE_ALL(p1)          ITrace::write(p1)
   #ifndef IC_TRACE_DEVELOP
     #define IC_TRACE_DEVELOP
   #endif
#else
   #define IMODTRACE_ALL(modname)
   #define IFUNCTRACE_ALL()
   #define ITRACE_ALL(p1)
#endif

#ifdef IC_TRACE_DEVELOP
   #define IMODTRACE_DEVELOP(modname) ITrace trc(modname,  __LINE__ )
   #define IFUNCTRACE_DEVELOP()       ITrace trc(__FUNCTION__, __LINE__ )
   #define ITRACE_DEVELOP(p1)         ITrace::write(p1)
   #ifndef IC_TRACE_RUNTIME
     #define IC_TRACE_RUNTIME
   #endif
#else
   #define IMODTRACE_DEVELOP(modname)
   #define IFUNCTRACE_DEVELOP()
   #define ITRACE_DEVELOP(p1)
#endif

#ifdef IC_TRACE_RUNTIME
   #define IMODTRACE_RUNTIME(modname)  ITrace trc(modname, __LINE__ )
   #define IFUNCTRACE_RUNTIME()        ITrace trc(__FUNCTION__, __LINE__ )
   #define ITRACE_RUNTIME(p1)          ITrace::write(p1)
#else
   #define IMODTRACE_RUNTIME(modname)
   #define IFUNCTRACE_RUNTIME()
   #define ITRACE_RUNTIME(p1)
#endif

#pragma pack()

#ifndef I_NO_INLINES
  #include <itrace.inl>
#endif

#endif /* _ITRACE_ */


#ifdef __MVS__
                   #pragma checkout(resume)
#endif

