/*******************************************************************************
* FILE NAME: itypeint.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*     Definition of the functions found in itypeint.hpp                        *                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Taligent.    *
*   (C) Copyright Taligent, Inc. 1996 - All rights reserved.                   *
*                                                                              *
*******************************************************************************/
#include <imetatyp.hpp>         // note: this must come before itypeint.hpp
#include <itypeint.hpp>
#include <string.h>
#include <ikeyset.h>
#include <ityptabm.hpp>
#include <iexcept.hpp>
#include <icconsta.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITYPEINT_CPP_
  #include <ipagetun.h>
#endif

//////////////////////////////////////////////////////////////////////////////
// Dictionary mapping class names to TGB functions

IKeySet<ITypeTableMap, IString>*    gKnownTypes = 0;

//////////////////////////////////////////////////////////////////////////////

ITypeInternal* ITypeInternal::GetType(const IString& className)
{
    if (gKnownTypes == 0 || !gKnownTypes->containsElementWithKey(className))
       ITHROWLIBRARYERROR(IC_STREAM_TYPE_MISSING,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);

    IMetaTypeInfo* (*tgbFunction)(ITypeHelper*) = (IMetaTypeInfo* (*)(ITypeHelper*))
    gKnownTypes->elementWithKey(className).GetData();

    return ((*tgbFunction)((ITypeHelper*)0))->GetTypeInternal();
}

#include <iostream.h>
void ITypeInternal::AddType(const IString& className,
                            const IMetaTypeInfo* (*tgbFunction)(ITypeHelper*))
{
    if (gKnownTypes == 0) gKnownTypes = new IKeySet<ITypeTableMap, IString>;
    if (gKnownTypes->containsElementWithKey(className)) {
       ITHROWLIBRARYERROR(IC_STREAM_TYPE_ALREADY_ADDED,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    }
    ITypeTableMap element(className, (void*)tgbFunction);
    gKnownTypes->add(element);
//    cout << "Adding type: " << className << endl;
}

ITypeInternal::ITypeInternal(const IString& className,
                const IMetaTypeInfo* (*tgbFunction)(ITypeHelper* helper),
                void* (*tgdFunction)(const void* obj))
:   fTypeInfo(0),
    fLastBaseClass(0),
    fLastBaseAdjustment(0),
    fTGBFunction(tgbFunction),
    fTGDFunction(tgdFunction),
    fClassName(className)
{
    char* name = new char [className.size() + 1];
    strcpy(name, className);
    fTypeInfo = new IMetaTypeInfo(name);
    fTypeInfo->SetTypeInternal(this);
}

ITypeInternal::ITypeInternal()
:   fTypeInfo(0),
    fLastBaseClass(0),
    fLastBaseAdjustment(0),
    fTGBFunction(0),
    fTGDFunction(0),
    fClassName()
{
}

ITypeInternal::ITypeInternal(const ITypeInternal& other)
:   fTypeInfo(other.fTypeInfo),
    fLastBaseClass(other.fLastBaseClass),
    fLastBaseAdjustment(other.fLastBaseAdjustment),
    fTGBFunction(other.fTGBFunction),
    fTGDFunction(other.fTGDFunction),
    fClassName(other.fClassName)
{
}

ITypeInternal::~ITypeInternal()
{
    delete fTypeInfo;
}

ITypeInternal& ITypeInternal::operator=(const ITypeInternal& other)
{
    if (this != &other) {
        fTypeInfo = other.fTypeInfo;
        fLastBaseClass = other.fLastBaseClass;
        fLastBaseAdjustment = other.fLastBaseAdjustment;
        fTGBFunction = other.fTGBFunction;
        fTGDFunction = other.fTGDFunction;
        fClassName = other.fClassName;
    }
    return *this;
}

IBoolean ITypeInternal::operator==(const ITypeInternal& other) const
{
    return (GetClassName() == other.GetClassName());
}

IBoolean ITypeInternal::operator!=(const ITypeInternal& other) const
{
    return (GetClassName() != other.GetClassName());
}

IMetaTypeInfo* ITypeInternal::GetTypeInfo() const
{
    return fTypeInfo;
}

const IMetaTypeInfo* ITypeInternal::CallTGBFunction(ITypeHelper* helper) const
{
    return (*fTGBFunction)(helper);
}

void* ITypeInternal::CallTGDFunction(const void* obj) const
{
    return (*fTGDFunction)(obj);
}

const IString& ITypeInternal::GetClassName() const
{
    return fClassName;
}

ITypeInternal* ITypeInternal::GetLastBaseClass(long& adjustment) const
{
    adjustment = fLastBaseAdjustment;
    return fLastBaseClass;
}

void ITypeInternal::SetLastBaseClass(ITypeInternal* baseTypeInternal, long adjustment)
{
    fLastBaseClass = baseTypeInternal;
    fLastBaseAdjustment = adjustment;
}
