/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswContMenu
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    CAswContMenu contains the container menu functions required for 
 *    in-place object server support of shared menus and shared accelerators.
 *    This class inherits from the CAswContMenuBase class. The protected
 *    methods in CAswContMenu are available to users who need to subclass
 *    CAswContMenu for special menu handling. 
 *
 *    For regular menu handling you allocate a CAswContMenu object and set
 *    the shared container menu information with CAswContMenu::SetMenuInfo.
 *    You can overload the GetMenuInfo method if you would like to have
 *    Aswan ask for the current menu information. 
 *
 *****************************************************************************/

#ifndef ASCNTMEN_H
#define ASCNTMEN_H
#include <aspack.h>
const UINT ASWAN_MAX_CONTAINER_MENUS = 30;
class ASWPUBCLASS CAswContMenu : public CAswMenuBase
{
public:
   ASWPUBMETHOD CAswContMenu( PASWCONTROL pAswControl );
   ASWPUBMETHOD ~CAswContMenu( void ); 
   CAswResult ASWPUBMETHOD Setup( void ); 
   CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE );
   void ASWPUBMETHOD SetMenuInfo( HMENU             hMenu,
                                  UINT              nCount, 
                                  PASWSHAREDMENU  pAswanSharedMenu ); 
   virtual void ASWPUBMETHOD GetMenuInfo( PHMENU             phMenu, 
                                          LPUINT             pnCount,
                                          PASWSHAREDMENU     pAswanSharedMenu );
   inline void SetRegularMenu( HMENU hMenuRegular ) 
   {
      m_hMenuRegular = hMenuRegular;
   }
   virtual HMENU GetRegularMenu( void );
protected:
	virtual CAswResult ASWPUBMETHOD InsertMenus( HMENU                 hmenuShared,
		                                          LPOLEMENUGROUPWIDTHS  lpMenuWidths );
   virtual CAswResult ASWPUBMETHOD SetMenu( HMENU      hmenuShared, 
                                            HOLEMENU   holemenu, 
	                                         HWND       hwndActiveObject );
	virtual CAswResult ASWPUBMETHOD RemoveMenus( HMENU hmenuShared );
   void ASWPUBMETHOD SetMenuForContainer( HWND  hWnd, 
                                          HMENU hMenu, 
                                          HMENU hMenuWindow );
private:
#ifdef ASWANDEBUG
   void ShowMenus(HMENU hmenu, LPSTR szDesc=NULL);
#endif 
	HMENU m_hMenu;
	ASWSHAREDMENU m_stSharedMenuArray[ASWAN_MAX_CONTAINER_MENUS];
	UINT m_nCount;
	HMENU m_hMenuRegular;
	HMENU m_hWindowPopup;
	HACCEL m_hAccel;
	UINT m_cAccelEntries;
	HWND m_hObjectsMenuWnd;
   BOOL  m_fSetup;
};
#include <aspop.h>
#endif   
