/* NOSHIP */
/*****************************************************************************
 *
 * ascombas.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    Declarations for Aswan Base Classes, ie CAswCOMBase and CAswInterfaceBase.
 *
 *    CAswCOMBase is a class definition which is used to provide COM semantics
 *    to any Aswan class by  inheriting from it.  There are several rules you
 *    must follow to get this support; see the document titled Aswan Base
 *    Classes, by Noah Mendelsohn and Mark Colan, in the Aswan Discussion
 *    database for more info.
 *
 *    CAswInterfaceBase is a class definition used for wrappering Aswan versions
 *    of OLE interface classes to provide them with an IUnknown implementation.
 *    The classes are generated automatically by this file.
 *
 *****************************************************************************/

#ifndef ASCOMBAS_H
#define ASCOMBAS_H
#include <aspack.h>
typedef LPUNKNOWN ASWPTR * LPLPUNKNOWN;
class ASWPUBCLASS CAswRefCount : public CAswMemoryBase
{
public:
   inline CAswRefCount( void )     
   {
      m_refcount = 0;
   }
   inline ULONG AddRef( void )     
   {
      return ++m_refcount;
   }
   inline ULONG Release( void )    
   {
      return --m_refcount;
   }
   inline void Set( ULONG x )      
   {
      m_refcount = x;
   }
   inline ULONG Get( void ) const  
   {
      return m_refcount;
   }
private:
   CAswRefCount( const CAswRefCount & SourceObject );
   void operator= ( const CAswRefCount & SourceObject );
   ULONG m_refcount;                    
};
class ASWPUBCLASS CAswHashedIID : public CAswMemoryBase
{
public:
#if defined(USE_CAP)
   BYTE Hash( REFIID riid ) const;
   CAswHashedIID( void );
   CAswHashedIID( REFIID riid ); 
   void operator= ( const CAswHashedIID &source );
   void operator= ( REFIID riid );
   BOOL operator== ( const CAswHashedIID &source ) const;
   BOOL operator== ( REFIID riid ) const;
   BOOL operator!= ( const CAswHashedIID &source ) const;
   BOOL operator!= (REFIID riid) const;
#else
   inline BYTE Hash( REFIID riid ) const 
   {
      LPULONG up = (LPULONG) ((LPVOID) &riid);  
      ULONG hashcode = (*up) ^ (*(up+1)) ^ (*(up+2)) ^ (*(up+3));
      hashcode = hashcode ^ (hashcode >> 16);
      return ( (BYTE) (hashcode ^ (hashcode >> 8) ) );
   }
   inline CAswHashedIID( void )
   {
   }
   inline CAswHashedIID( REFIID riid ) 
   {
      m_hashcode = Hash( riid );    
   }
   inline void operator= ( const CAswHashedIID &source ) 
   {
      m_hashcode = source.m_hashcode;
   }
   inline void operator= ( REFIID riid ) 
   {
      m_hashcode = Hash( riid );   
   }
   inline BOOL operator== ( const CAswHashedIID &source ) const 
   {
      return m_hashcode == source.m_hashcode;
   }
   inline BOOL operator== ( REFIID riid ) const  
   {
      return ( m_hashcode == Hash( riid ) );
   }
   inline BOOL operator!= ( const CAswHashedIID &source ) const 
   {
      return !(*this == source);
   }
   inline BOOL operator!= (REFIID riid) const    
   {
      return ! (*this == riid);
   }
#endif
private:
   BYTE m_hashcode;      
   CAswHashedIID( const CAswHashedIID &source );
};
typedef CAswHashedIID ASWPTR *PASWHASHENTRY;
class ASWPUBCLASS CAswIIDMapEntry : public CAswMemoryBase
{
public:
   inline CAswIIDMapEntry( void );
   inline IID GetIID( void ) const
   {
      return ( m_iid );
   }
   inline void SetIID( IID iid )
   {
      m_iid = iid;
   }
   inline LONG GetOffset( void ) const
   {
      return ( m_lOffset );
   }
   inline void SetOffset( LONG lOffset )
   {
      m_lOffset = lOffset;
   }
private:
   CAswIIDMapEntry( const CAswIIDMapEntry & SourceObject );
   void operator= ( const CAswIIDMapEntry & SourceObject );
   IID  m_iid;         
   LONG m_lOffset;     
};
#pragma warning(disable:4705)
inline CAswIIDMapEntry::CAswIIDMapEntry( void )
{
}
#pragma warning(default:4705)
typedef CAswIIDMapEntry ASWPTR *PASWIIDMAPENTRY;
class ASWPUBCLASS CAswIIDMapState : public CAswDebugBase
{
public:
   inline CAswIIDMapState( PASWIIDMAPENTRY pIIDMapEntry,
                           PASWHASHENTRY   pHashMap, 
                           const int       nMaxEntries 
   #if defined( ASWANDEBUG )
                           ,
                           LPSTR           pszClassname, 
                           LPSTR           pszFilename, 
                           int nLine )
   #else
                           )
   #endif
      : CAswDebugBase( NULL, FALSE )  
   {
      m_pIIDMapEntry = pIIDMapEntry; 
      m_pHashMap     = pHashMap;
      m_nMaxEntries  = nMaxEntries; 
      m_nCurEntries  = 0;
   #ifdef ASWANDEBUG
      m_nReqEntries  = 0;
      m_pszClassname = pszClassname;
      m_pszFilename  = pszFilename;
      m_nLine        = nLine;
   #endif
      ASWSETCLASSNAME("CAswIIDMapState");
      DebugSetSetup( TRUE );
      DebugSetNeedsShutdown( FALSE );
   }
   inline int GetMaxEntries( void ) const
   {
      return m_nMaxEntries;
   }
   inline PASWIIDMAPENTRY GetMap( void ) const
   {
      return m_pIIDMapEntry;
   }
   inline PASWHASHENTRY GetHashMap( void ) const
   {
      return m_pHashMap;
   }
   inline int GetCurEntries( void ) const
   {
      return( (m_nCurEntries < 0) ? -m_nCurEntries : m_nCurEntries);
   }
   inline void SetCurEntries(int count)
   {
      ASWMETHOD("SetCurEntries",(CH_CLASS_CASWIIDMAPSTATE));
#ifdef ASWANDEBUG
      int nOldEntries = m_nCurEntries;
#endif
      ASWASSERT( count > 0 );
      if ( m_nCurEntries <= 0 )
      {
         count = -count;               
      }
      m_nCurEntries = count;
      ASWPRINTF("count=%d, m_nCurEntries was %d, now %d (%d:%s)",
         (ASWSTR,count,nOldEntries,m_nCurEntries,GetCurEntries(),MapState()));
      ASWEXITX();
   }
   #if defined(ASWANDEBUG) || defined(ASWANASSERT)
   inline void IncrementRequiredEntries(void)
   {
      m_nReqEntries++;
   }
   void ASWPUBMETHOD CheckMapEntries( void ) const;
   #endif 
   inline BOOL MapIsValid( void ) const
   {
      ASWMETHOD("MapIsValid",(CH_CLASS_CASWIIDMAPSTATE));
      ASWPRINTF("m_nCurEntries=%d: %s",(ASWSTR, m_nCurEntries, MapState() ));
      ASWEXITX();
      return(PrivateMapIsValid());
   }
   inline void SetMapValid( void )
   {
      ASWMETHOD("SetMapValid",(CH_CLASS_CASWIIDMAPSTATE));
      m_nCurEntries = (m_nCurEntries < 0) ? (-m_nCurEntries) : m_nCurEntries;
      ASWPRINTF("m_nCurEntries=%d: %s",(ASWSTR, m_nCurEntries, MapState() ));
      ASWEXITX();
   }
   inline void SetMapInvalid( void )
   {
      ASWMETHOD("SetMapInvalid",(CH_CLASS_CASWIIDMAPSTATE));
      m_nCurEntries = MapIsValid() ? (-m_nCurEntries) : m_nCurEntries;
      ASWPRINTF("m_nCurEntries=%d: %s",(ASWSTR, m_nCurEntries, MapState() ));
      ASWEXITX();
   }
private:
   inline BOOL PrivateMapIsValid(void) const
   {
      return(m_nCurEntries>0);
   }
   CAswIIDMapState( const CAswIIDMapState & SourceObject );
   void operator= ( const CAswIIDMapState & SourceObject );
   #ifdef ASWANDEBUG
   inline char *MapState(void) const
   {
      return ( (PrivateMapIsValid()) ? "valid" : "invalid" );
   }
   #endif 
   PASWIIDMAPENTRY m_pIIDMapEntry;
   PASWHASHENTRY   m_pHashMap;
   int             m_nMaxEntries;         
   int             m_nCurEntries;
   #if defined(ASWANDEBUG) || defined(ASWANASSERT)
   int             m_nReqEntries;   
   LPSTR           m_pszClassname;  
   LPSTR           m_pszFilename;   
   int             m_nLine;         
   #endif 
};
typedef CAswIIDMapState ASWPTR *PMAPSTATE;
#define ASWAN_DECLARATIONS(n)\
static CAswIIDMapEntry NEAR AswanInterfaceMap[n];\
static CAswHashedIID   NEAR AswanHashMap[n];\
static CAswIIDMapState NEAR InterfaceMapState
#ifdef ASWANDEBUG
   #define ASWAN_INTERFACES_DEBUG(classname) ,#classname,__FILE__,__LINE__
#else
   #define ASWAN_INTERFACES_DEBUG(classname) 
#endif
#define ASWAN_INTERFACES(classname,n)\
CAswIIDMapEntry NEAR classname::AswanInterfaceMap[n];\
CAswHashedIID   NEAR classname::AswanHashMap[n];\
CAswIIDMapState NEAR classname::InterfaceMapState(classname::AswanInterfaceMap,\
classname::AswanHashMap,n \
ASWAN_INTERFACES_DEBUG(classname))
#define ASWAN_INIT_MAP &(this->InterfaceMapState)
#ifdef  ASWANASSERT
#define ASWAN_CHECK_MAP() if (GetMapState()) GetMapState()->CheckMapEntries()
#else
#define ASWAN_CHECK_MAP()
#endif
class ASWPUBCLASS CAswIIDMap : public CAswBase
{
public:
   inline CAswIIDMap( PASWCONTROL pControl, PMAPSTATE pMapState ) 
      : CAswBase( pControl ) 
   {
      m_pMapState = pMapState;
   }
   void AddEntry( REFIID riid, LPUNKNOWN pUnknown );
   LPUNKNOWN Lookup(REFIID riid);
   inline PMAPSTATE GetMapState( void ) const
   {
#if defined(ASWANASSERT)
      SyncControl();
#endif
      return m_pMapState;
   }
#if defined(ASWANASSERT)
   void ASWPUBMETHOD SyncControl( void ) const;
#endif
private:
   PMAPSTATE m_pMapState;
};
union ASWPUBCLASS PointerOrArray
{
   LPUNKNOWN lpUnknown;
   LPLPUNKNOWN lplpUnknown;
   inline PointerOrArray()
   {
   }
};
class ASWPUBCLASS CAswCOMBase : public CAswIIDMap, public IUnknown
{
public:
   ASWPUBMETHOD CAswCOMBase(PASWCONTROL pControl,
                            PMAPSTATE   pMapState,
                            LPUNKNOWN   pControllingObject = NULL );
   inline CAswCOMBase::~CAswCOMBase()
   {
      ASW_STANDARD_DESTRUCTOR_IMPL( CAswCOMBase, (CH_COMBASE_PRIMITIVES) );
   }
   virtual HRESULT ASWPUBOLEMETHOD STDMETHODCALLTYPE QueryInterface(REFIID riid,
                                                          LPVOID FAR * ppvObj );
   virtual ULONG ASWPUBOLEMETHOD STDMETHODCALLTYPE AddRef ();
   virtual ULONG ASWPUBOLEMETHOD STDMETHODCALLTYPE Release ();
   virtual CAswResult ASWPUBMETHOD Setup(void);
   virtual CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE );
   virtual void ASWPUBMETHOD ObjectReleased();
   CAswResult ASWPUBMETHOD SetControlledObject( LPUNKNOWN pControlledObject );
   CAswResult ASWPUBMETHOD SetControlledObjectArray( int         count,
                                        LPLPUNKNOWN pControlledObjectArray );
   inline LPUNKNOWN GetControllingObject()
   {
      ASWGLOBALASSERT_WPTROKSIZE( m_pControllingObject ); 
      return( m_pControllingObject );
   }
private:
   inline void SetRefcount(ULONG x)      
   {
      m_RefCount.Set(x);
   }
   inline ULONG GetRefcount(void) const  
   {
      return(m_RefCount.Get());
   }
   inline ULONG QuickAddRef(void)
   {
      ASWMETHOD("QuickAddRef",(CH_COMBASE_IUNKNOWN));
      ASWENTRY();
      ULONG r = m_RefCount.AddRef();
      ASWEXITF(r,"this=%p: returns %lu",(ASWSTR, this, r));
   }
   ULONG QuickRelease(void);
   inline void ReleaseControlledObject( void )
   {
      ASWMETHOD("ReleaseControlledObject",(CH_COMBASE_PRIMITIVES));
      ASWENTRY();
      m_ControlledObject.lpUnknown->Release();
      m_ControlledObject.lpUnknown = NULL;
      ASWEXIT();
   }
   void ReleaseControlledObjectArray(void);
   LPUNKNOWN m_pControllingObject;     
   UINT           m_cControlledObjects; 
   PointerOrArray m_ControlledObject;   
   CAswRefCount m_RefCount;            
};
class ASWPUBCLASS CAswInterfaceBase : public CAswCtlBase
{
public:
   ASWPUBMETHOD CAswInterfaceBase( PASWCONTROL pControl,
                                   PASWCOMBASE pCOMBase,
                                   LPUNKNOWN   lpUnknown,
                                   REFIID      riid1 );
   ASWPUBMETHOD CAswInterfaceBase( PASWCONTROL pControl,
                                   PASWCOMBASE pCOMBase,
                                   LPUNKNOWN   lpUnknown,
                                   REFIID      riid1,
                                   REFIID      riid2 );
   ASWPUBMETHOD CAswInterfaceBase( PASWCONTROL pControl,
                                   PASWCOMBASE pCOMBase,
                                   LPUNKNOWN   lpUnknown,
                                   REFIID      riid1,
                                   REFIID      riid2,
                                   REFIID      riid3 );
   ASWPUBMETHOD CAswInterfaceBase::~CAswInterfaceBase( void );
#ifdef ASWANDEBUG
   virtual LPCSTR ASWPUBMETHOD GetInstanceIDString( void ) const;
#endif
   CAswResult ASWPUBMETHOD Setup( void );
   CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE );
   inline LPUNKNOWN GetControllingObject(void) const
   {
      ASWGLOBALASSERT_WPTROKSIZE( m_pControllingObject ); 
      return(m_pControllingObject);
   }
private:
   LPUNKNOWN   m_pControllingObject;
#ifdef ASWANDEBUG
   PASWCOMBASE m_pCOMBase;
#endif
   CAswInterfaceBase( const CAswInterfaceBase & SourceObject );
   void operator= ( const CAswInterfaceBase & SourceObject );
};
#ifdef MAKEDLL
#define DECLARE_ASWAN_INTERFACE1(INTERFACE_NAME)\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase);\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#define DECLARE_ASWAN_INTERFACE2(INTERFACE_NAME, INTERITED_INTERFACE)\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase);\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#define DECLARE_ASWAN_INTERFACE3(INTERFACE_NAME, INHERITED1, INHERITED2)\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase);\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#else
#define DECLARE_ASWAN_INTERFACE1(INTERFACE_NAME)\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
inline CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase)\
   : CAswInterfaceBase(pControl, pCOMBase, this, IID_##INTERFACE_NAME)\
{\
   ASWSETCLASSNAME("CAsw" #INTERFACE_NAME);\
}\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#define DECLARE_ASWAN_INTERFACE2(INTERFACE_NAME, INTERITED_INTERFACE)\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
inline CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase)\
   : CAswInterfaceBase(pControl, pCOMBase, this,\
      IID_##INTERFACE_NAME, IID_##INTERITED_INTERFACE)\
{\
   ASWSETCLASSNAME("CAsw" #INTERFACE_NAME);\
}\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#define DECLARE_ASWAN_INTERFACE3(INTERFACE_NAME, INHERITED1, INHERITED2)\
\
class ASWPUBCLASS CAsw##INTERFACE_NAME : public INTERFACE_NAME, public CAswInterfaceBase\
{public:\
inline CAsw##INTERFACE_NAME(PASWCONTROL pControl, PASWCOMBASE pCOMBase)\
   : CAswInterfaceBase(pControl, pCOMBase, this,\
      IID_##INTERFACE_NAME, IID_##INHERITED1, IID_##INHERITED2)\
{\
   ASWSETCLASSNAME("CAsw" #INTERFACE_NAME);\
}\
STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj);\
STDMETHOD_(ULONG,AddRef)();\
STDMETHOD_(ULONG,Release)();\
};
#endif 
#undef INTERFACE                       
#define INTERFACE(INTERFACE_NAME,l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8)\
DECLARE_ASWAN_INTERFACE1(INTERFACE_NAME)
#define INTERFACE2(INTERFACE_NAME, INTERITED_INTERFACE,l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8)\
DECLARE_ASWAN_INTERFACE2(INTERFACE_NAME,INTERITED_INTERFACE)
#define INTERFACE3(INTERFACE_NAME, INHERITED1, INHERITED2,l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8)\
DECLARE_ASWAN_INTERFACE3(INTERFACE_NAME,INHERITED1,INHERITED2)
#define UNSUPPORTED_INTERFACE(INTERFACE_NAME,l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8)
#define LOTUS_INTERFACE(INTERFACE_NAME) DECLARE_ASWAN_INTERFACE1(INTERFACE_NAME)
#include <asintf.i>
#undef INTERFACE
#undef INTERFACE2
#undef INTERFACE3
#undef UNSUPPORTED_INTERFACE
#undef LOTUS_INTERFACE
#include <aspop.h>
#endif 
