/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswCtlBase
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    This class exists primarily to hold onto a pointer to the Aswan
 *    control object (CAswControl). All Aswan classes that need to hold
 *    onto a pointer to CAswControl should inherit from this class.
 *
 *    The overhead for this class is 4 bytes per Aswan class instance.
 *    Alternatives were considered and this was deemed a reasonable design
 *    choice given the granularity of Aswan objects.  
 *
 *    To keep overhead down, this class has no virtual functions,
 *    and hence no vtbl pointer, in normal operation.  When debugs
 *    are enabled, virtual functions are created to support 
 *    certain debugging functions.  
 *
 *****************************************************************************/

#ifndef ASCTLBAS_H
#define ASCTLBAS_H
#include <aspack.h>
#if defined(USE_CAP)
   #define CTLBASE_OUTOFLINE 1
#else
   #define CTLBASE_OUTOFLINE 0
#endif
#define USE_CONTROLHOLDER 0
#if USE_CONTROLHOLDER
#define CTLHOLDER_OUTOFLINE 0
class CAswCtlHolder
{
public:
#if CTLHOLDER_OUTOFLINE
   ASWPUBMETHOD CAswCtlHolder( PASWCONTROL pControl );
   PASWCONTROL ASWPUBMETHOD Get( void ) const;
#else
   inline CAswCtlHolder( PASWCONTROL pControl )
      : m_pControl( pControl )
   {
   }
   inline PASWCONTROL Get( void ) const
   {
      return ( m_pControl );
   }   
#endif
private:
   PASWCONTROL const m_pControl;
};
#endif
class ASWPUBCLASS CAswCtlBase : public CAswDebugBase
{
public:
#if CTLBASE_OUTOFLINE
   ASWPUBMETHOD CAswCtlBase( PASWCONTROL pControl, BOOL fAddToList = TRUE );
#else
   inline CAswCtlBase( PASWCONTROL pControl, BOOL fAddToList = TRUE )
      : CAswDebugBase( pControl, fAddToList )
#if USE_CONTROLHOLDER
      , m_ControlHolder( pControl ) 
#else
      , m_pControl( pControl )
#endif
   {
   }
#endif   
#if CTLBASE_OUTOFLINE
   BOOL ASWPUBMETHOD HasControl(void) const;
   PASWCONTROL ASWPUBMETHOD GetControl(void) const;
#else
   inline BOOL HasControl( void ) const
   {
#if USE_CONTROLHOLDER
      return ( ISRPTROK( m_ControlHolder.Get() ) );
#else
      return ( ISRPTROK( m_pControl ) );
#endif
   }
   inline PASWCONTROL GetControl( void ) const
   {
#if USE_CONTROLHOLDER
      ASWGLOBALASSERT_RPTROK( m_ControlHolder.Get() );   
      return m_ControlHolder.Get();
#else
      ASWGLOBALASSERT_RPTROK( m_pControl );   
      return m_pControl;
#endif
   }
#endif
private:
   CAswCtlBase(const CAswCtlBase& SourceObject);
   void operator= (const CAswCtlBase& SourceObject);
#if USE_CONTROLHOLDER
   CAswCtlHolder const m_ControlHolder;
#else
   PASWCONTROL const m_pControl;
#endif
};
#include <aspop.h>
#endif   
