/* NOSHIP */
/*****************************************************************************
 *
 * Source Module: asdebug.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Purpose:       Declarations for debugging aids in the Aswan Toolkit.
 *                SEE the entry on the Aswan Debug Trace Facility in the
 *                Aswan database or reference materials.
 *
 * Usage:         use -DASWANDEBUG and -DASWANASSERT on compile
 *                or #define them to achieve the following:
 *
 *                ASWANDEBUG - enables ALL of the debugging macros coded
 *                within the Aswan toolkit (even sets ASWANASSERT) and writes
 *                out the debug info to the specified channel logs.
 *
 *                ASWANASSERT - enables all of the ASWASSERT flavor of macros,
 *                causing a dialog box to be invoked on failure of the tested
 *                expression.  If ASWANDEBUG is on, the assertion is also
 *                written out to the appropriate channel log.  ASWANASSERT
 *                only builds are useful for client beta testing.
 *
 *****************************************************************************/

#ifndef ASDEBUG_H
#define ASDEBUG_H
#ifndef _INC_STDARG
#include <stdarg.h>     
#endif      
#include <aspack.h>
#define ASWSHOWTF(flag) ((flag)?(LPSTR)"TRUE":(LPSTR)"FALSE")
const UINT MAX_ASWCLASSNAME = 31;
const UINT CB_ASWCLASSNAME = MAX_ASWCLASSNAME + 1;
#define NAMETOOBIGSTRING ">nametoobig!<"
#if defined(ASWANASSERT)
void TerminateCurrentApplication( void );
extern void ASWPUBFUNC __DoGlobalAssert( LPCSTR lpszMessage );
extern void ASWPUBFUNC __DoGlobalAssertTitle( LPCSTR lpszMessage,
                                              LPCSTR lpszTitle );
extern BOOL ASWPUBFUNC AswanGlobalAssertControl( BOOL bDisplay );
#endif 
const DWORD c_dwCAswDebugBaseSignature = 0xcadbcadb; 
class ASWPUBCLASS CAswDebugBase : public CAswMemoryBase
{
#ifdef ASWANASSERT
private:
DWORD    m_dwSignature;
public:
inline BOOL HasDebugBaseSignature( void ) const
{
   return ( c_dwCAswDebugBaseSignature == m_dwSignature );
}
inline BOOL IsDebugBase( void ) const
{
   if ( NULL != this )
      if ( ISRPTROKS( this ) )
            return ( HasDebugBaseSignature() );
   return ( FALSE );
}
#endif
public:
#ifdef ASWANASSERT
   ASWPUBMETHOD CAswDebugBase( PASWCONTROL pControl, 
                               BOOL        fAddToShutdownList = TRUE );
   ASWPUBMETHOD ~CAswDebugBase( void );
   inline void Setup( void )
   {
      m_AddlChannel = CH_UNUSED;       
      DebugSetSetup( TRUE );           
      DebugSetNeedsShutdown( FALSE );  
   }                                   
   inline void Shutdown( void )
   {
      DebugSetSetup( FALSE );          
      DebugSetNeedsShutdown( FALSE );  
   }
   inline BOOL HasControl( void ) const
   {
      return ( ISRPTROK( m_pControl ) );
   }
   inline PASWCONTROL GetControl( void ) const
   {
      ASWGLOBALASSERT_RPTROK( m_pControl );   
      if ( ISRPTROK( m_pControl ) )
         return m_pControl;
      else
         return NULL;
   }
   void SetListedControl( PASWCONTROL pControl );
   virtual BOOL ASWPUBMETHOD DebugCrossCheck( void );
   virtual BOOL ASWPUBMETHOD DebugNeedsShutdown( void );
   virtual void ASWPUBMETHOD DebugSetNeedsShutdown( BOOL fDoesItorDoesntIt );
   virtual void ASWPUBMETHOD DebugSetSetup( BOOL fSetup );
   virtual BOOL ASWPUBMETHOD DebugIsSetup( void ) const;
   virtual const ULONG ASWPUBMETHOD GetInstanceID( void ) const;
   virtual void ASWPUBMETHOD SetInstanceID( void );
   inline void SetInstanceTime( void );
   inline DWORD GetInstanceTime( void );
   void ASWPUBMETHOD SetClassName( LPCSTR pID );
   virtual LPCSTR ASWPUBMETHOD GetAswanClassName( void ) const;
   inline LPCSTR GetClassID( void ) const
   {
      return ( m_szClassName );
   }
   inline ASWCHANNEL AddChannel( ASWCHANNEL NewChannel )
   {
      ASWCHANNEL OldChannel = m_AddlChannel;    
      m_AddlChannel = NewChannel;               
      return ( OldChannel );                    
   }
   LPCSTR ASWPUBMETHOD GetAswanBuildString( void ) const;
#if defined(MAKEDLL) || defined(ASWDLL)
   void LoseControl( PASWCONTROL pDisappearingControl );
private:
   CAswListEl  m_leNeedsLoseControl;
public:
#endif   
protected:
   ASWCHANNEL m_AddlChannel;     
private:
   friend class CAswControl;          
   friend class CAswDebugList;        
   CAswListEl  m_leNeedsShutdown;
   BOOL        m_fNeedsShutdown;
   BOOL        m_fSetup;
   PASWCONTROL m_pControl;  
   CAswDebugBase( const CAswDebugBase & SourceObject );
   void operator= ( const CAswDebugBase & SourceObject );
#define DEBUGBASE_BLDSTRINGSZ 80
   static char NEAR sm_szBuildNameString[ DEBUGBASE_BLDSTRINGSZ ];
   static ULONG NEAR sm_ulUniqueID;
   ULONG       m_ulInstance;     
#if defined(ASWANDEBUG)
   DWORD       m_dwInstanceTime; 
#endif
#define ASW_DEFAULT_CLASSNAME "unnamed"
   char        m_szClassName[ CB_ASWCLASSNAME ];
#else    
   inline CAswDebugBase( PASWCONTROL pControl,
                         BOOL        fAddToShutdownList = TRUE );
   inline void Setup( void )
   {
   }
   inline void Shutdown( void )
   {
   }
#define DebugCrossCheck()
#define DebugNeedsShutdown()
#define DebugIsSetup()
#define DebugSetNeedsShutdown(x)
#define DebugSetSetup(x)
#endif   
#if defined(ASWANDEBUG)
public:
   void ASWPUBMETHOD InitDebugBuffer( void ) const;
   LPCSTR ASWPUBMETHOD ConstructClassId( PCASWDEBUGBASE pAswClass ) const;
   LPCSTR ASWPUBMETHOD DumpGUID( REFCLSID  rclsid ) const;
   LPCSTR ASWPUBMETHOD GetInstanceIDString( void ) const;
   LPSTR ASWPUBMETHOD GetDebugIDStringBuffer( void ) const;
#endif      
public:
   inline BOOL IsShutdownOK( void ) const
   {
      return TRUE;
   }
   inline PASWSHUTDOWN GetShutdownObject( void ) const
   {
      return NULL;
   }
   inline void DoShutdownProcessing( BOOL  )
   {
   }
};
#ifndef ASWANASSERT
#pragma warning(disable:4705)
inline CAswDebugBase::CAswDebugBase( PASWCONTROL ,
                                     BOOL         )
{
}
#pragma warning(default:4705)
#endif
DWORD ASWPUBFUNC GetAswanVersion( void );
#ifdef ASWANASSERT     
MAKELISTHEADCLASS(CAswDebugBase);
class ASWPUBCLASS CAswDebugBaseList : public CAswDebugBaseListHead,
                                             public CAswDebugBase
{
public:
   inline CAswDebugBaseList( PASWCONTROL pControl,
                             CAswListEl CAswDebugBase::*le )
      : CAswDebugBase( pControl, FALSE ),
        CAswDebugBaseListHead( le )
   {
      SetClassName( "BaseList" );
   }
   inline ~CAswDebugBaseList()
   {
   }                        
   virtual BOOL ASWPUBMETHOD CrossCheck();
};
LPCSTR ASWPUBMETHOD FAR PASCAL GetIIDValueString( REFIID riid );
LPCSTR ASWPUBMETHOD FAR PASCAL GetIIDString( REFIID riid );
BOOL CopyWidthPrecision( LPSTR  * ppszDest,
                         LPCSTR * ppszSrc );
void CopyFormatString( LPSTR  pszDest,
                       LPCSTR pszSrc );
BOOL CopyFlags( LPSTR  * ppszDest,
                LPCSTR * ppszSrc );
#endif
#include <aspop.h>
#endif   
