/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswObjectInstance
 *        CAswInstanceDataImpl
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    This class is provides the OLE 2 functionality necessary for a server
 *    object.  It includes the ability for the server object to do either
 *    InPlace Editing or Launch to Edit.  The application will create an
 *    instance of this class for each object requested by the user, typically
 *    in the application specific part of CreateInstance processing.
 *
 *****************************************************************************/

#ifndef ASOBJINS_H
#define ASOBJINS_H
#include <aspack.h>
class CAswInstanceDataImpl : public CAswDataObjectImpl2
{
public:
   CAswInstanceDataImpl(PASWCONTROL          pCtl,
                        PASWCOMBASE          pCOMBase,
                        PASWOBJECTINSTANCE   pObjInstance)
      : CAswDataObjectImpl2(pCtl,pCOMBase),
        m_pObjInstance(pObjInstance)
      {  };
   virtual CAswResult SetData(LPFORMATETC       pFormatEtc,
                              PASWSTGMEDUNION   pAswStgMedium);
   virtual CAswResult GetData(LPFORMATETC     pFormatEtc,
                              PASWSTGMEDUNION pAswStgMedium);
private:
   virtual PASWSOURCEFORMATSET FindFormatSet(void);
   PASWOBJECTINSTANCE const  m_pObjInstance;
};
class ASWPUBCLASS CAswObjectInstance : public CAswCOMBase
{
   friend class CAswOleObjectImpl;
   friend class CAswInstanceDataImpl;
   friend class CAswOleInPlaceObjectImpl;
   friend class CAswOleInPlaceActiveObjectImpl;
   friend class CAswServMenu;
   friend class CAswServSpace;
   friend class CAswIOleItemContainerImpl;
public:
   enum OBJECTINSTANCETYPE
   {
      oitNone = 0x00,                  
      oitImplementedFile = 0x01,       
      oitImplementedStorage = 0x02     
   };
   enum DOCTYPE
   {
      dtUnknown = 0,    
      dtFromFile,       
      dtEmbedded,       
      dtFromStorage,    
      dtNew             
   };
   ASWPUBMETHOD CAswObjectInstance(PASWCONTROL  pControl,
                                   LPUNKNOWN    pUnknownOuter = NULL);
   ASWPUBMETHOD ~CAswObjectInstance(void);
   CAswResult ASWPUBMETHOD Setup(PASWOBJECTCLASS      pObjectClass,
                                 UINT                 oit,
                                 PASWSOURCEFORMATSET  pSourceFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   CAswResult ASWPUBMETHOD SetVerbTable(PASWVERBTABLE pVerbTable,
                                        WORD          cVerbEntries);
   virtual CAswResult ASWPUBMETHOD
   QueryVerbTable(PASWVERBTABLE ASWPTR * ppVerbTable, LPWORD pcVerbEntries);
   inline void
   GetVerbTable(PASWVERBTABLE ASWPTR * ppVerbTable, LPWORD pcVerbEntries) const
   {
      *ppVerbTable   = m_pVerbTable;
      *pcVerbEntries = m_cVerbEntries;
   }
   virtual WORD ASWPUBMETHOD CheckVerb(LONG     iVerb,
                                       HWND     hwndParent,
                                       LPCRECT  prcPosRect);
   virtual CAswResult OnClose(DWORD dwSaveOption) PURE;
   virtual CAswResult Hide(LONG iVerb) PURE;
   virtual CAswResult SetHostNames(LPASWCOLESTR pszContainerApp,
                                   LPASWCOLESTR pszContainerObj) PURE;
   virtual CAswResult Launch(LONG iVerb) PURE;
   CAswResult ASWPUBMETHOD CloseObjectInstance(DWORD dwSaveOption = OLECLOSE_SAVEIFDIRTY);
   inline DOCTYPE GetDocumentType(void) const
   {
      return m_dtDocType;
   }
   CAswResult ASWPUBMETHOD 
   SetInPlaceOptions(PASWSERVMENU   pServMenu       = NULL,
                     PASWSERVSPACE  pServFrameSpace = NULL,
                     PASWSERVSPACE  pServDocSpace   = NULL,
                     PASWSERVACCEL  pServAccel      = NULL,
                     PASWSERVUNDO   pServUndo       = NULL,
                     LPASWCOLESTR   pszObjName      = NULL);
   CAswResult ASWPUBMETHOD ChangeInPlaceAdornments(void);
   virtual HWND ASWPUBMETHOD GetInPlaceServerWindow(HWND    hwndParent,
                                                    RECT    rcClip,
                                                    LPRECT  prcPos);
   virtual void ASWPUBMETHOD SetInPlaceServerFocus(HWND hwndServer);
   virtual BOOL ASWPUBMETHOD HideInPlaceWindow(void);
   virtual BOOL ASWPUBMETHOD ShowInPlaceWindow(void);
   virtual void ASWPUBMETHOD OnInPlaceActivate(void);
   virtual void ASWPUBMETHOD OnInPlaceDeactivate(void);
   virtual void ASWPUBMETHOD AdjustObjectRect(LPRECT prcObject);
   inline BOOL IsInsideOut(void)
   {
      return (   (CAswObjectInstance::dtEmbedded == GetDocumentType())
              && (m_fInsideOut));
   }
   CAswResult ASWPUBMETHOD LeftMouseButtonDown(void);
   CAswResult ASWPUBMETHOD SetStatusText(LPASWCOLESTR pszText);
   CAswResult ASWPUBMETHOD EnableContainersModelessWindows(BOOL fEnable);
   virtual CAswResult ASWPUBMETHOD EnableModeless(BOOL fEnable);
   CAswResult ASWPUBMETHOD InPlaceDeactivate(void);
   CAswResult ASWPUBMETHOD UIDeactivate(void);
   virtual CAswResult SetExtent(DWORD dwDrawAspect, LPSIZEL lpsizel) PURE;
   virtual CAswResult GetExtent(DWORD dwDrawAspect, LPSIZEL lpsizel) PURE;
   virtual void ASWPUBMETHOD SetIsDirty(BOOL fDirty = TRUE);
   virtual BOOL ASWPUBMETHOD GetIsDirty(void) const;
   virtual CAswResult ASWPUBMETHOD LoadFile(LPASWCOLESTR pszFileName, DWORD grfMode);
   virtual CAswResult ASWPUBMETHOD SaveFile(LPASWCOLESTR pszFileName, BOOL fRemember);
   CAswResult ASWPUBMETHOD FileRenamed(LPASWCOLESTR pszFileName);
   virtual CAswResult ASWPUBMETHOD SaveFileCompleted(LPASWCOLESTR pszFileName);
   virtual CAswResult ASWPUBMETHOD GetCurFile(LPASWOLESTR pszFileName, UINT cchMax);
   virtual CAswResult ASWPUBMETHOD InitNewStorage(PASWSTORAGE pAswStg);
   virtual CAswResult ASWPUBMETHOD LoadStorage(PASWSTORAGE pAswStg);
   virtual CAswResult ASWPUBMETHOD SaveStorage(PASWSTORAGE  pAswStgSave,
                                               BOOL         fSameAsLoad);
   virtual CAswResult ASWPUBMETHOD SaveStorageCompleted(PASWSTORAGE pAswStgNew);
   virtual CAswResult ASWPUBMETHOD HandsOffStorage(void);
   virtual CAswResult ASWPUBMETHOD GetNativeClipFormat(LPCLIPFORMAT pFormat);
   inline void SetContainerClass(PASWCONTAINER pContainer)
   {
      ASWMETHOD("SetContainerClass", (CH_CLASS_OBJINST));
      ASWENTRYF("pContainer=%s", (ASWSTR,ASWCLASSID(pContainer)));
      m_pContainer = pContainer;
      ASWEXIT();
   }
   inline PASWCONTAINER GetContainerClass(void)
   {
      ASWMETHOD("GetContainerClass", (CH_CLASS_OBJINST));
      ASWENTRY();
      ASWEXITF(m_pContainer, "%p", (ASWSTR, m_pContainer));
   }
   virtual CAswResult ASWPUBMETHOD IsUpToDate(void);
   virtual CAswResult ASWPUBMETHOD Update(void);
   inline void SetFormatSet(PASWSOURCEFORMATSET pAswSourceFormatSet)
   {
      m_pSourceFormatSet = pAswSourceFormatSet;
   }
   inline PASWSOURCEFORMATSET GetFormatSet(void)
   {
      return m_pSourceFormatSet;
   }
   inline void ClearFormatSet(void)
   {
      m_pSourceFormatSet = NULL;
   }
   virtual PASWSOURCEFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   virtual CAswResult ASWPUBMETHOD SetData(LPFORMATETC      lpFormatEtc,
                                           PASWSTGMEDUNION  pAswStgMedium);
   virtual CAswResult ASWPUBMETHOD GetData(LPFORMATETC     lpFormatEtc,
                                           PASWSTGMEDUNION pAswStgMedium) PURE;
   CAswResult ASWPUBMETHOD GetEmbedSourceData(PASWSTORAGE pAswStorage);
   CAswResult ASWPUBMETHOD GetLinkSourceData(PASWSTREAM pAswStream);
   inline CAswResult AdviseOnSave(void)
      { return SendAdvise(CAswObjectInstance::ocSaved); };
   inline CAswResult AdviseOnClose(void)
      { return SendAdvise(CAswObjectInstance::ocClosed); };
   inline CAswResult AdviseOnRename(LPMONIKER pmk)
      { return SendAdvise(CAswObjectInstance::ocRenamed,pmk); };
   inline CAswResult AdviseOnDataChange(DWORD advf = 0)
      { return SendAdvise(CAswObjectInstance::ocDataChanged,NULL,advf); };
   inline CAswResult AdviseOnViewChange(DWORD advf = 0)
      { return SendAdvise(CAswObjectInstance::ocViewChanged,NULL,advf); };
   inline CAswResult AdviseOnShowWindow(void)
      { return SendAdvise(CAswObjectInstance::ocShowWindow); };
   inline CAswResult AdviseOnHideWindow(void)
      { return SendAdvise(CAswObjectInstance::ocHideWindow); };
   inline CAswResult SaveObject(void)
      { return SendAdvise(CAswObjectInstance::ocSaveObject); };
   inline CAswResult ShowObject(void)
      { return SendAdvise(CAswObjectInstance::ocShowObject); };
   virtual CAswResult ASWPUBMETHOD SetColorScheme(LPLOGPALETTE pLogPal);
   virtual CAswResult ASWPUBMETHOD GetMiscStatus(DWORD   dwAspect,
                                                 LPDWORD pdwStatus);
   virtual CAswResult ASWPUBMETHOD GetUserClassID(LPCLSID pClsid);
   virtual CAswResult ASWPUBMETHOD GetUserType(DWORD                 dwFormOfType,
                                               LPASWOLESTR ASWPTR*   pszUserType);
   virtual PASWPSEUDOOBJECT ASWPUBMETHOD QueryPseudoObject(LPASWCOLESTR pszItem);
   virtual HRESULT ASWPUBOLEMETHOD STDMETHODCALLTYPE
   QueryInterface(REFIID riid, LPVOID ASWPTR* ppvObj);
   virtual PASWSHUTDOWN ASWPUBMETHOD GetShutdownObject(void);
   virtual PASWBASE ASWPUBMETHOD GetParentObject(void);
   virtual BOOL ASWPUBMETHOD IsShutdownOK(void);
   inline BOOL IsVisible(void) const
   {
      ASWMETHOD("IsVisible", (CH_CLASS_OBJINST));
      ASWENTRY();
      ASWEXITB(m_fVisible);
   }
   inline BOOL GetIsClosing(void) const
   {
      ASWMETHOD("GetIsClosing", (CH_CLASS_OBJINST));
      ASWENTRY();
      ASWEXITB(m_fObjectClosing);
   }
   inline BOOL IsIPActive(void) const
   {
      ASWMETHOD("IsIPActive", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITB(m_fIPActive);
   }
   inline void SetFromOnPosRectChange(BOOL fFrom)
   {
      ASWMETHOD("SetFromOnPosRectChange", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fFrom)));
      m_fFromOnPosChg = fFrom;
      ASWEXITX();
   }
   inline BOOL GetFromOnPosRectChange(void)
   {
      ASWMETHOD("GetFromOnPosRectChange", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITB(m_fFromOnPosChg);
   }
   inline BOOL IsIPVisible(void) const
   {
      ASWMETHOD("IsIPVisible", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITB(m_fIPVisible);
   }
   inline BOOL IsUIActive(void) const
   {
      ASWMETHOD("IsUIActive", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITB(m_fUIActive);
   }
   inline PCASWREFHOLDER GetClientSiteRH(void) const
   {
      return &m_rhOleClientSite;
   }
   inline PCASWREFHOLDER GetOleAdviseHolderRH(void) const
   {
      return &m_rhOleAdviseHolder;
   }
   inline LPOLECLIENTSITE GetClientSite(void) const
   {
      ASWMETHOD("GetClientSite", (CH_CLASS_OBJINST));
      PCASWREFHOLDER prhClientSite = GetClientSiteRH();
      #ifdef ASWANASSERT
      if (prhClientSite->IsSet())
      {
         ASWASSERT_RPTROKLEN(prhClientSite->Get(), sizeof(IOleClientSite));
         prhClientSite->CheckIID( IID_IOleClientSite, TRUE );
      }
      #endif
      ASWEXITX();
      return((LPOLECLIENTSITE)prhClientSite->Get());
   }
   inline PASWREFHOLDER GetIPSiteRH(void)
   {
      return &m_rhIPSite;
   }
   inline LPOLEINPLACESITE GetIPSite(void)
   {
      ASWMETHOD("GetIPSite", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      PCASWREFHOLDER prhIPSite = GetIPSiteRH();
      #ifdef ASWANASSERT
      if (prhIPSite->IsSet())
      {
         ASWASSERT_RPTROKLEN(prhIPSite->Get(), sizeof(IOleInPlaceSite));
         prhIPSite->CheckIID( IID_IOleInPlaceSite, TRUE );
      }
      #endif
      ASWEXITX();
      return( (LPOLEINPLACESITE)prhIPSite->Get() );
   }
   inline PASWREFHOLDER GetIPFrameRH(void)
   {
      return &m_rhIPFrame;
   }
   inline LPOLEINPLACEFRAME GetIPFrame(void)
   {
      ASWMETHOD("GetIPFrame", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      PCASWREFHOLDER prhIPFrame = GetIPFrameRH();
      #ifdef ASWANASSERT
      if (prhIPFrame->IsSet())
      {
         ASWASSERT_RPTROKLEN(prhIPFrame->Get(), sizeof(IOleInPlaceFrame));
         prhIPFrame->CheckIID( IID_IOleInPlaceFrame, TRUE );
      }
      #endif
      ASWEXITX();
      return( (LPOLEINPLACEFRAME)prhIPFrame->Get() );
   }
   inline PASWREFHOLDER GetIPUIWindowRH(void)
   {
      return &m_rhIPUIWindow;
   }
   inline LPOLEINPLACEUIWINDOW GetIPUIWindow(void)
   {
      ASWMETHOD("GetIPUIWindow", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      PCASWREFHOLDER prhIPUIWindow = GetIPUIWindowRH();
      #ifdef ASWANASSERT
      if (NULL != prhIPUIWindow->Get())
         ASWASSERT_RPTROKLEN(prhIPUIWindow->Get(), sizeof(IOleInPlaceUIWindow));
      prhIPUIWindow->CheckIID( IID_IOleInPlaceUIWindow, TRUE );
      #endif
      ASWEXITX();
      return( (LPOLEINPLACEUIWINDOW)prhIPUIWindow->Get() );
   }
   inline PASWOBJECTCLASS GetObjectClass(void)
   {
      return m_pObjectClass;
   }
   CAswResult SetClientSite(LPOLECLIENTSITE pClientSite);
   CAswResult SetOleAdvise(LPADVISESINK pAdvSink,LPDWORD pdwConnection);
   CAswResult ASWPUBMETHOD SetMoniker(DWORD dwWhichMoniker, LPMONIKER pmk);
   CAswResult ASWPUBMETHOD GetMoniker(DWORD               dwAssign,
                                      DWORD               dwWhichMoniker,
                                      LPMONIKER ASWPTR*   ppmk);
   inline void SetDocumentType(DOCTYPE dt)
   {
      m_dtDocType = dt;
   }
   void GetAdjustedPositionRect(LPRECT prcNewHatch, LPRECT prcPos);
   inline void AddPseudoToList(PASWPSEUDOOBJECT pObject)
   {
      m_lhPseudoObjects.Insert(pObject);
   }
   inline void RemovePseudoObjectFromList(PASWPSEUDOOBJECT pObject)
   {
      m_lhPseudoObjects.Remove(pObject);
   }
   PASWPSEUDOOBJECT GetPseudoObject(LPASWCOLESTR pszItem);
   CAswListEl              m_leObjectInstance;
private:
   enum OBJECTCODE
   {
      ocSaved       = 0,   
      ocClosed      = 1,   
      ocRenamed     = 2,   
      ocSaveObject  = 3,   
      ocDataChanged = 4,   
      ocViewChanged = 5,   
      ocShowWindow  = 6,   
      ocHideWindow  = 7,   
      ocShowObject  = 8    
   };
   inline PASWSOURCEFORMATSET FindFormatSet(void)
   {
      return (m_pSourceFormatSet) ? m_pSourceFormatSet : QueryFormatSet();
   }
   CAswResult ASWPUBMETHOD SendAdvise(const UINT   uCode,
                                      LPMONIKER    pmk = NULL,
                                      DWORD        advf = 0);
   void NotifyOfRename(LPMONIKER pmk);
   void DoCloseInstance(void);
   inline void SetRunning(void)
   {
      ASWMETHOD("SetRunning", (CH_CLASS_OBJINST));
      ASWENTRY();
      m_fRunning = TRUE;
      ASWEXITX();
   }
   inline void SetNotRunning(void)
   {
      ASWMETHOD("SetNotRunning", (CH_CLASS_OBJINST));
      m_fRunning = FALSE;
      m_IExternalConn.ClearCount();
      ASWEXITX();
   }
   inline BOOL IsRunning(void) const
   {
      ASWMETHOD("IsRunning", (CH_CLASS_OBJINST));
      ASWENTRY();
      ASWEXITB(m_fRunning);
   }
   inline void SetIsVisible(BOOL fVisible)
   {
      ASWMETHOD("SetIsVisible", (CH_CLASS_OBJINST));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fVisible)));
      m_fVisible = fVisible;
      ASWEXITX();
   }
   inline void SetIsClosing(BOOL fClosing)
   {
      ASWMETHOD("SetIsClosing", (CH_CLASS_OBJINST));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fClosing)));
      m_fObjectClosing = fClosing;
      ASWEXITX();
   }
   inline void SetIPActive(BOOL fIPActive)
   {
      ASWMETHOD("SetIPActive", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fIPActive)));
      m_fIPActive = fIPActive;
      ASWEXITX();
   }
   inline void SetIPVisible(BOOL fIPVisible)
   {
      ASWMETHOD("SetIPVisible", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fIPVisible)));
      m_fIPVisible = fIPVisible;
      ASWEXITX();
   }
   inline void SetUIActive(BOOL fUIActive)
   {
      ASWMETHOD("SetUIActive", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fUIActive)));
      m_fUIActive = fUIActive;
      ASWEXITX();
   }
   inline void SetObjectWindow(HWND hwndObject)
   {
      ASWMETHOD("SetObjectWindow", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRYF("%x", (ASWSTR,ASWHANDLE(hwndObject)));
      m_hwndObject = hwndObject;
      ASWEXITX();
   }
   inline HWND GetObjectWindow(void) const
   {
      ASWMETHOD("GetObjectWindow", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITF(m_hwndObject, "returns: %x", (ASWSTR,ASWHANDLE(m_hwndObject)));
   }
   inline HWND GetHatchWindow(void) const
   {
      ASWMETHOD("GetHatchWindow", (CH_CLASS_OBJINST, CH_SERVER_INPLACE));
      ASWENTRY();
      ASWEXITF(m_hwndHatch, "returns: %x", (ASWSTR,ASWHANDLE(m_hwndHatch)));
   }
   CAswResult CreateHatchWindow(HWND ASWPTR *phwndHatch);
   inline
   CAswResult FindVerbTable(PASWVERBTABLE ASWPTR*  ppVerbTable,
                            LPWORD                 pcVerbEntries)
   {
      CAswResult ar;
      GetVerbTable(ppVerbTable, pcVerbEntries);
      if (*ppVerbTable)
         ar.Set(S_OK, ASD_OK);
      else
         ar = QueryVerbTable(ppVerbTable, pcVerbEntries);
      return ar;
   }
   CAswResult InPlaceActivate(LONG lVerb);
   CAswResult InPlaceHide(void);
   CAswResult InPlaceResizeWindow(RECT rcClip, LPRECT prcPos);
   CAswResult AddFrameLevelUI(void);
   void GetIPObjectTitle(CAswOLEString& rAswTitle);
   inline PASWSERVMENU GetServMenu(void)
   {
      return m_pServMenu;
   }
   void SetServMenu(PASWSERVMENU pServMenu);
   inline PASWSERVSPACE GetServFrameSpace(void)
   {
      return m_pServFrameSpace;
   }
   void SetServFrameSpace(PASWSERVSPACE pFrameSpace);
   inline PASWSERVSPACE GetServDocSpace(void)
   {
      return m_pServDocSpace;
   }
   void SetServDocSpace(PASWSERVSPACE pDocSpace);
   PASWSERVACCEL GetServAccel(void)
   {
      return m_pServAccel;
   }
   void SetServAccel(PASWSERVACCEL pServAccel);
   PASWSERVUNDO GetServUndo(void)
   {
      return m_pServUndo;
   }
   void SetServUndo(PASWSERVUNDO pServUndo);
   LPASWCOLESTR GetInPlaceObjectName(void);
   ASWAN_DECLARATIONS(14);
   CAswOleObjectImpl                m_OleObject;      
   CAswInstanceDataImpl             m_DataObject;     
   CAswPersistStorageImpl           m_PersistStorage; 
   CAswPersistFileImpl              m_PersistFile;    
   CAswOleInPlaceObjectImpl         m_IPObject;       
   CAswOleInPlaceActiveObjectImpl   m_IPActiveObject; 
   CAswExternalConnectionImpl       m_IExternalConn;  
   CAswIOleItemContainerImpl        m_IOleItemCont;   
   CAswShutdown            m_shutdown;
   CAswRefHolder           m_rhOleClientSite;
   CAswRefHolder           m_rhOleAdviseHolder;
   CAswRefHolder           m_rhMoniker;
   CAswRefHolder           m_rhIPSite;
   CAswRefHolder           m_rhIPFrame;
   CAswRefHolder           m_rhIPUIWindow;
   PASWOBJECTCLASS         m_pObjectClass;
   PASWCONTAINER           m_pContainer;
   PASWSOURCEFORMATSET     m_pSourceFormatSet;
   PASWSERVMENU            m_pServMenu;
   PASWSERVSPACE           m_pServFrameSpace;
   PASWSERVSPACE           m_pServDocSpace;
   PASWSERVACCEL           m_pServAccel;
   PASWSERVUNDO            m_pServUndo;
   CAswOLEString           m_aostrObjName;   
   PASWVERBTABLE           m_pVerbTable;     
   WORD                    m_cVerbEntries;   
   CAswPseudoObjectListHead   m_lhPseudoObjects;
   OBJECTINSTANCETYPE      m_oit;            
   DWORD                   m_dwRegROT;       
   DOCTYPE                 m_dtDocType;      
   BOOL                    m_fSetup;         
   BOOL                    m_fRunning;       
   BOOL                    m_fDirty;         
   BOOL                    m_fObjectClosing; 
   BOOL                    m_fVisible;       
   BOOL                    m_fInsideOut;     
   BOOL                    m_fFromOnPosChg;  
   BOOL m_fIPActive;             
   BOOL m_fIPVisible;            
   BOOL m_fUIActive;             
   HWND m_hwndHatch;             
   HWND m_hwndObject;            
   RECT m_rcPos;                 
   RECT m_rcClip;                
   OLEINPLACEFRAMEINFO     m_IPFrameInfo;
};
MAKELISTHEADCLASS(CAswObjectInstance);
#include <aspop.h>
#endif      
