/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswPseudoObject
 *        CAswPseudoDataImpl
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    This class is used for all OLE pseudo objects.  A pseudo object
 *    is a portion of a document that can be linked to (like a range).
 *
 *****************************************************************************/

#ifndef ASPSEUDO_H
#define ASPSEUDO_H
#include <aspack.h>
class CAswPseudoDataImpl : public CAswDataObjectImpl2
{
public:
   CAswPseudoDataImpl(PASWCONTROL      pCtl,
                      PASWCOMBASE      pCOMBase,
                      PASWPSEUDOOBJECT pPseudoObj)
      : CAswDataObjectImpl2(pCtl, pCOMBase),
        m_pPseudoObj(pPseudoObj)
      {  };
   virtual CAswResult SetData(LPFORMATETC       pFormatEtc,
                              PASWSTGMEDUNION   pAswStgMedium);
   virtual CAswResult GetData(LPFORMATETC     pFormatEtc,
                              PASWSTGMEDUNION pAswStgMedium);
private:
   virtual PASWSOURCEFORMATSET FindFormatSet(void);
   PASWPSEUDOOBJECT const  m_pPseudoObj;
};
class ASWPUBCLASS CAswPseudoObject : public CAswCOMBase
{
   friend class CAswOleObjectImpl;
   friend class CAswPseudoDataImpl;
   ASWAN_DECLARATIONS(2);
public:
   ASWPUBMETHOD CAswPseudoObject(PASWCONTROL   pControl,
                                 LPUNKNOWN     pUnknownOuter = NULL);
   ASWPUBMETHOD ~CAswPseudoObject(void);
   CAswResult ASWPUBMETHOD Setup(PASWOBJECTINSTANCE   pObjInstance,
                                 LPASWCOLESTR         pObjName,
                                 PASWSOURCEFORMATSET  pSourceFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   inline void SetFormatSet(PASWSOURCEFORMATSET pAswSourceFormatSet)
   {
      m_pSourceFormatSet = pAswSourceFormatSet;
   }
   inline PASWSOURCEFORMATSET GetFormatSet(void)
   {
      return m_pSourceFormatSet;
   }
   inline void ClearFormatSet(void)
   {
      m_pSourceFormatSet = NULL;
   }
   virtual PASWSOURCEFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   virtual CAswResult ASWPUBMETHOD SetData(LPFORMATETC      pFormatEtc,
                                           PASWSTGMEDUNION  pAswStgMedium);
   virtual CAswResult ASWPUBMETHOD GetData(LPFORMATETC     pFormatEtc,
                                           PASWSTGMEDUNION pAswStgMedium) PURE;
   virtual void ASWPUBMETHOD SetSelection(void);
   CAswResult ASWPUBMETHOD GetLinkSourceData(PASWSTREAM pAswStream);
   inline CAswResult AdviseOnDataChange(DWORD advf = 0)
      { return SendAdvise(CAswPseudoObject::ocDataChanged, NULL, advf); };
   inline CAswResult AdviseOnViewChange(DWORD advf = 0)
      { return SendAdvise(CAswPseudoObject::ocViewChanged, NULL, advf); };
   inline CAswResult AdviseOnSave(void)
      { return SendAdvise(CAswPseudoObject::ocSaved); };
   inline CAswResult AdviseOnClose(void)
      { return SendAdvise(CAswPseudoObject::ocClosed); };
   inline CAswResult AdviseOnRename(LPMONIKER pmk)
      { return SendAdvise(CAswPseudoObject::ocRenamed, pmk); };
   inline LPASWCOLESTR GetPseudoObjName(void)
   {
      return m_ostrName;
   }
   inline PASWOBJECTINSTANCE GetObjectInstance(void) const
   {
      return m_pObjInstance;
   }
   virtual CAswResult GetExtent(LPSIZEL lpsizel) PURE;
   inline CAswResult ClosePseudoObject(void)
   {
      HRESULT hr = m_OleObject.Close(OLECLOSE_NOSAVE);
      return CAswResult(hr, SUCCEEDED(hr) ? ASD_OK : ASD_UNCHANGED);
   }
   CAswResult GetMoniker(DWORD               dwAssign,
                         DWORD               dwWhichMoniker,
                         LPMONIKER ASWPTR*   ppmk);
   virtual BOOL ASWPUBMETHOD IsShutdownOK(void);
   CAswListEl     m_lePseudoObject;
private:
   inline void SetIsClosing(BOOL fClosing)
   {
      ASWMETHOD("SetIsClosing", (CH_CLASS_OBJINST));
      ASWENTRYF("%s",(ASWSTR,ASWSHOWTF(fClosing)));
      m_fObjectClosing = fClosing;
      ASWEXITX();
   }
   inline BOOL GetIsClosing(void) const
   {
      ASWMETHOD("GetIsClosing", (CH_CLASS_OBJINST));
      ASWENTRY();
      ASWEXITB(m_fObjectClosing);
   }
   inline PASWSOURCEFORMATSET FindFormatSet(void)
   {
      return (m_pSourceFormatSet) ? m_pSourceFormatSet : QueryFormatSet();
   }
   CAswResult SetOleAdvise(LPADVISESINK pAdvSink, LPDWORD pdwConnection);
   inline PCASWREFHOLDER GetOleAdviseHolderRH(void) const
   {
      return &m_rhOleAdviseHolder;
   }
   CAswResult ASWPUBMETHOD SendAdvise(const UINT   uCode,
                                      LPMONIKER    pmk = NULL,
                                      DWORD        advf = 0);
   void DoClosePseudoObject(void);
   enum PSEUDOADVISECODE
   {
      ocSaved       = 0,   
      ocClosed      = 1,   
      ocRenamed     = 2,   
      ocDataChanged = 3,   
      ocViewChanged = 4    
   };
   CAswOleObjectImpl       m_OleObject;   
   CAswPseudoDataImpl      m_DataObject;  
   CAswRefHolder           m_rhOleAdviseHolder;
   PASWOBJECTINSTANCE      m_pObjInstance;
   PASWSOURCEFORMATSET     m_pSourceFormatSet;
   CAswOLEString           m_ostrName;
   BOOL                    m_fObjectClosing;
};
MAKELISTHEADCLASS(CAswPseudoObject);
#include <aspop.h>
#endif
