/* NOSHIP */
/*****************************************************************************
 *
 * asregkey.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *
 *    Contains the definition of classes: CAswBoolRegValue.
 *                                        CAswReadOnlyStringValue.
 *
 *    CAswBoolRegValue represents a BOOL value that is found in the Windows
 *    Registration Database, also known as the Registry.
 *
 *    CAswReadOnlyStringValue represents a string value found in the Windows
 *    Registration Database, also known as the Registry.
 *
 *    The constructor reads the RegDB (creating the value if it is not there).
 *
 *    Inline accessors read the current state.
 *
 *    A method is provided to refresh the value from the registry...
 *
 *    The keys this creates are all found in HKEY_LOCAL_MACHINE under
 *    SOFTWARE\Lotus\Aswan, also known as ASWAN_REGSUBKEYNAME
 *
 *****************************************************************************/

#ifndef ASREGKEY_H
#define ASREGKEY_H
#include <aspack.h>
#ifdef _WIN32
#define ASWAN_REGKEY                 HKEY_LOCAL_MACHINE
#else    
#define ASWAN_REGKEY                 HKEY_CLASSES_ROOT
#endif   
#define ASWAN_REGSUBKEYNAME          "SOFTWARE\\Lotus\\Aswan"
#define ASWGAD_REGSUBKEYVALUENAME    "AswanGlobalAssert"
#define ASWGAD_REGSUBKEYVALUE        ASWAN_REGSUBKEYNAME "\\" ASWGAD_REGSUBKEYVALUENAME
const BOOL ASWGAD_OFF = FALSE;
const BOOL ASWGAD_ON  = TRUE;
const BOOL ASWGAD_DEFAULT = ASWGAD_ON;
#define ASWASSERT_REGSUBKEYVALUENAME "DisplayAssert"
const BOOL ASWASSERT_DEFAULT = TRUE;
#define ASWASSERT_REGSUBKEYVALUE     ASWAN_REGSUBKEYNAME "\\" ASWASSERT_REGSUBKEYVALUENAME
#define ASWDEBUG_REGSUBKEYVALUENAME "FormatDebug"
const BOOL ASWDEBUG_DEFAULT = TRUE;
#define ASWDEBUG_REGSUBKEYVALUE     ASWAN_REGSUBKEYNAME "\\" ASWDEBUG_REGSUBKEYVALUENAME
#define ASWSHOWTICKCOUNT_REGSUBKEYVALUENAME "ShowTickCount"
const BOOL ASWSHOWTICKCOUNT_DEFAULT = FALSE;
#define ASWSHOWTICKCOUNT_REGSUBKEYVALUE ASWAN_REGSUBKEYNAME "\\" ASWSHOWTICKCOUNT_REGSUBKEYVALUENAME
#define ASWSHOWTASKID_REGSUBKEYVALUENAME "ShowTaskID"
const BOOL ASWSHOWTASKID_DEFAULT = FALSE;
#define ASWSHOWTASKID_REGSUBKEYVALUE ASWAN_REGSUBKEYNAME "\\" ASWSHOWTASKID_REGSUBKEYVALUENAME
#define ASWAN_REGKEYTRUE  "Y"
#define ASWAN_REGKEYFALSE "N"
class ASWPUBCLASS CAswBoolRegValue : public CAswMemoryBase
{
public:
   ASWPUBMETHOD CAswBoolRegValue( LPCSTR  lpszKeyName,
                                  BOOL    bDefault = TRUE );
   ASWPUBMETHOD ~CAswBoolRegValue();
   inline void Set( BOOL bValue )
   {
      m_bValue = bValue;
   }
   inline BOOL Get( void ) const
   {
      return ( m_bValue );
   }
   inline BOOL True( void ) const
   {
      return ( Get() );
   }
   inline BOOL False( void ) const
   {
      return ( ! True() );
   }
   inline BOOL Refresh( void )
   {
      Read();
      return ( Get() );
   }
   void ASWPUBMETHOD Update( void ) const;
private:
   void Read( void );
   BOOL            m_bValue;
   CAswTCHARString m_Key;
};
class ASWPUBCLASS CAswReadOnlyStringValue : public CAswMemoryBase
{
public:
   ASWPUBMETHOD CAswReadOnlyStringValue( LPCSTR lpszKeyName,
                                         LPCSTR lpszDefault = NULL );
   ASWPUBMETHOD ~CAswReadOnlyStringValue();
   inline void Get( CAswANSIString ASWPTR * preturnString ) const
   {
      if ( preturnString )
         preturnString->Copy( m_szValue.GetANSIPtr() );
   }
private:
   void Read( void );
   inline void Set( LPCSTR lpszValue )
   {
      if ( lpszValue )
         m_szValue.Copy( lpszValue );
      else
         m_szValue.Copy( NULL );
   }
   void Update( void ) const;
   CAswANSIString m_szValue;
   CAswTCHARString m_Key;
};
#include <aspop.h>
#endif   
