/* NOSHIP */
/*****************************************************************************
 *
 * assrvmen.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * LIST OF CLASSES IN THIS FILE
 *      CAswServMenu
 *
 * Description:
 *    Support for CAswServMenu, the in-place server menu class.
 *
 *****************************************************************************/

#ifndef ASSRVMEN_H
#define ASSRVMEN_H
#include <aspack.h>
class ASWPUBCLASS CAswServMenu : public CAswCtlBase
{
   friend CAswObjectInstance;    
public:
   ASWPUBMETHOD CAswServMenu(PASWCONTROL pControl);
   ASWPUBMETHOD ~CAswServMenu(void);
   CAswResult ASWPUBMETHOD Setup(void);
   inline CAswResult Setup(             HWND             hwndMenuHandler, 
                                        HMENU            hmenuServer,
                                        int              nCount,
                                        PASWSHAREDMENU   pSharedMenuArray)
   {
      CAswResult ar = Setup();
      SetMenuInfo(hwndMenuHandler, hmenuServer, nCount, pSharedMenuArray);
      return(ar);
   }
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   inline BOOL IsSetup(void)
   {
      return(m_fSrvMenuSetup);
   }
   void ASWPUBMETHOD SetMenuInfo(HWND           hwndMenuHandler, 
                                 HMENU          hmenuServer,
                                 int            nCount, 
                                 PASWSHAREDMENU pSharedMenuArray);
   inline void ClearMenuInfo(void)
   {
      SetMenuInfo(NULL, NULL, 0, NULL);
   }
   inline void GetMenuInfo(HWND ASWPTR *           phwndMenuHandler,
                           HMENU ASWPTR *          phmenuServer,
                           LPINT                   pnCount,
                           PASWSHAREDMENU ASWPTR * ppSharedMenuArray)
   {
      *phwndMenuHandler  = m_ServerMenuInfo.hwndServerHandler;
      *phmenuServer      = m_ServerMenuInfo.hMenu;
      *pnCount           = m_ServerMenuInfo.nCount;
      *ppSharedMenuArray = m_ServerMenuInfo.pMenuArray;
   }
   virtual CAswResult ASWPUBMETHOD QueryMenuInfo(
      HWND ASWPTR *           hwndMenuHandler,
      HMENU ASWPTR *          phmenuServer,
      LPINT                   pnCount,
      PASWSHAREDMENU ASWPTR * ppSharedMenuArray);
   CAswResult ASWPUBMETHOD RebuildSharedMenu(void);
   CAswResult SetSharedMenu(void);
private:
   struct tagSharedMenu
   {
      inline tagSharedMenu(void);
      HMENU                hMenu;               
      HOLEMENU             hOleMenuDescriptor;  
      HWND                 hwndSharedHandler;   
      OLEMENUGROUPWIDTHS   rgMenuGroupWidths;   
      int                  nServerHandles;      
      HMENU ASWPTR*        pServerHandles;      
   }
      m_SharedMenu[2];
   int   m_nActiveSharedMenu;
   struct tagServerMenuInfo
   {
      inline tagServerMenuInfo(void)
      {
         hMenu = NULL;
         hwndServerHandler = NULL;
         pMenuArray = NULL;
         nCount = 0;
      }
      HMENU          hMenu;               
      HWND           hwndServerHandler;   
      PASWSHAREDMENU pMenuArray;          
      int            nCount;              
   }
      m_ServerMenuInfo;
   PASWOBJECTINSTANCE   m_pObjInstance;
   BOOL                 m_fSrvMenuSetup;
   inline PASWOBJECTINSTANCE GetObjectInstance(void)
   {
      return(m_pObjInstance);
   }
   inline void SetObjectInstance(PASWOBJECTINSTANCE pObjInstance)
   {
      m_pObjInstance = pObjInstance;
   }
   inline void ClearMenuGroupWidths(int n)
   {
      tagSharedMenu ASWPTR*   pSharedMenu = m_SharedMenu + n;
      for (int i = 0;
           i < ARRAYELEMENTS(pSharedMenu->rgMenuGroupWidths.width);
           i++)
      {
         pSharedMenu->rgMenuGroupWidths.width[i] = 0L;
      }
   }
   inline CAswResult FindMenuInfo(HWND ASWPTR *             phwndMenuHandler,
                                  HMENU ASWPTR *            phmenuServer,
                                  LPINT                     pnCount,
                                  PASWSHAREDMENU ASWPTR *   ppSharedMenuArray)
   {
      CAswResult ar(S_OK, ASD_OK);
      GetMenuInfo(phwndMenuHandler, phmenuServer, pnCount, ppSharedMenuArray);
      if (NULL == *phmenuServer)
      {
         ar = QueryMenuInfo(phwndMenuHandler,
                            phmenuServer,
                            pnCount,
                            ppSharedMenuArray);
      }
      return(ar);
   }
   CAswResult CreateSharedMenu(void);
   CAswResult DestroySharedMenu(void);
   #ifdef ASWANASSERT
   inline void CheckSharedMenu(void)
   {
      ASWMETHOD("CheckSharedMenu", (CH_CLASS_SERVMENU, CH_SERVER_MENU));
      ASWENTRY();
      tagSharedMenu ASWPTR*   pSharedMenu = m_SharedMenu + m_nActiveSharedMenu;
      int cItems = GetMenuItemCount(pSharedMenu->hMenu);
      ASWASSERT(-1 != cItems);
      LONG lTotalWidth = 0;
      for (int i = 0; 
           i < ARRAYELEMENTS(pSharedMenu->rgMenuGroupWidths.width);
           i++)
      {
         lTotalWidth += pSharedMenu->rgMenuGroupWidths.width[i];
      }
      ASWASSERTF(cItems == lTotalWidth,
         "cItems=%d,lTotalWidth=%ld", (ASWSTR, cItems, lTotalWidth));
      ASWEXIT();
   }
   #endif 
   #ifdef ASWANDEBUG
   void ShowMenus(HMENU hmenu, LPSTR szDesc=NULL);
   #endif 
};
#ifdef ASWANASSERT
#define CHECKSHAREDMENU() CheckSharedMenu()
#else
#define CHECKSHAREDMENU()
#endif
#pragma warning(disable:4705)
inline CAswServMenu::tagSharedMenu::tagSharedMenu(void)
{
   hMenu = NULL;
   hOleMenuDescriptor = NULL;
   hwndSharedHandler = NULL;
   pServerHandles = NULL;
   nServerHandles = 0;
   for (int i = 0; i < ARRAYELEMENTS(rgMenuGroupWidths.width); i++)
      rgMenuGroupWidths.width[i] = 0L;
}
#pragma warning(default:4705)
#include <aspop.h>
#endif      
