/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswTask
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:   Define all the task-related classes and APIs:
 *
 *                CAswTask          - a task
 *                CAswTaskWithPath  - a task that has a pathname
 *                CAswTaskWithLock  - a task that adds locking semantics
 *                CAswTaskInfo      - information held on a per-task basis
 *                CAswPerTaskInfo   - a holder for all per-task info
 *
 *****************************************************************************/

#ifndef ASTASK_H
#define ASTASK_H
#include <aspack.h>
#define OS_WIN16         16
#define OS_WINNT         35
#define OS_WIN95         95
#define OS_OS2            2
class ASWPUBCLASS CAswWinVersion
{
public:
   inline CAswWinVersion( void )
#if !defined(_WIN32)
      : m_dwVersion( OS_WIN16 ) {  }   
#else
#if defined(LTSOS2)
      : m_dwVersion( OS_OS2 )   {  }   
#else
      : m_dwVersion( OS_WIN95 )   
   {
      OSVERSIONINFO VersionInfo;
      VersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
      if ( ::GetVersionEx( &VersionInfo ) )
      {
         switch ( VersionInfo.dwPlatformId )
         {
         case VER_PLATFORM_WIN32s:
            m_dwVersion = OS_WIN16;
            break;
         case VER_PLATFORM_WIN32_NT:
            m_dwVersion = OS_WINNT;
            break;
         default:
            break;
         }
      }
   }
#endif 
#endif 
#if defined(_WIN32)
   inline BOOL IsWin95( void ) const
   {
      return ( OS_WIN95 == m_dwVersion );
   }
   inline BOOL IsWinNT( void ) const
   {
      return ( OS_WINNT == m_dwVersion );
   }
   inline BOOL IsOS2( void ) const
   {
      return ( OS_OS2 == m_dwVersion );
   }
   inline BOOL IsWin16( void ) const
   {
      return ( OS_WIN16 == m_dwVersion );
   }
#endif
#if defined(ASWANASSERT)
   inline LPCSTR GetWinString( void ) const
   {
      switch ( m_dwVersion )
      {
      case OS_WIN95: return("Win95");
      case OS_WINNT: return("WinNT");
      case OS_WIN16: return("Win16");
      case OS_OS2:   return("OS/2");
      default:       return("Unknown OS");
      }
   }
#endif 
private:
   DWORD    m_dwVersion;
};
extern CAswWinVersion NEAR s_WindowsVersion;
class ASWPUBCLASS CAswTask : public CAswMemoryBase
{
public:
   inline CAswTask( void )
   {
      Shutdown();
   }
   inline BOOL operator== ( const CAswTask & refTask ) const
   {
#ifdef WIN16
      return ( refTask.GetHTASK() == GetHTASK() );
#else
      return (    (refTask.GetThreadID()  == GetThreadID() )
               && (refTask.GetProcessID() == GetProcessID()) );
#endif 
   }
   inline void Setup( void )
   {
#ifdef WIN16
      m_hTask = ::GetCurrentTask();
#else
      m_dwProcessID = ::GetCurrentProcessId(); \
      m_dwThreadID  = ::GetCurrentThreadId();
#endif   
   }
   inline BOOL IsValid( void ) const
   {
#ifdef WIN16
      return ( NULL != m_hTask );
#else
      return ( (NULL != m_dwProcessID) && (NULL != m_dwThreadID) );
#endif 
   }
   inline void Shutdown( void )
   {
#ifdef WIN16
      m_hTask = NULL;
#else
      m_dwProcessID = NULL;
      m_dwThreadID  = NULL;
#endif   
   }
#if defined(ASWANDEBUG)
   void DisplayState( LPSTR lpszStateInfo );
#endif   
private:
#ifdef WIN16
   inline const HTASK GetHTASK( void ) const
   {
      return m_hTask;
   }
#else
   inline const DWORD GetProcessID( void ) const
   {
      return m_dwProcessID;
   }
   inline const DWORD GetThreadID( void ) const
   {
      return m_dwThreadID;
   }
#endif   
#ifdef WIN16
   HTASK    m_hTask;
#else
   DWORD    m_dwProcessID;
   DWORD    m_dwThreadID;
#endif   
};
class ASWPUBCLASS CAswTaskWithPath : public CAswTask
{
public:
   inline CAswTaskWithPath( void )
   {
      Shutdown();
   }
   void Setup( void );
#if defined(ASWANDEBUG)
   LPCSTR GetTaskPath( void ) const
   {
      return ( m_szPath );
   }
   enum eShowPath
   {
      SHOWPATH = 1,
      NOSHOWPATH = 2
   };
   void DisplayState( LPSTR lpszStateInfo, eShowPath ShowPath = SHOWPATH );
#endif   
private:
#if defined(ASWANDEBUG)
   char     m_szPath[ _MAX_PATH ];
#endif   
};
#if defined(ASWANDEBUG)
   const UINT CASWTASKWITHLOCK_DISPLAYSTATESIZE = 32;
   const UINT CASWINFO_DISPLAYSTATESIZE = 80;
   #if defined(WIN16)
      const UINT CASWTASK_DISPLAYSTATESIZENOPATH = 16;
   #else
      const UINT CASWTASK_DISPLAYSTATESIZENOPATH = 48;
   #endif   
   const UINT CASWTASK_DISPLAYSTATESIZE =   CASWINFO_DISPLAYSTATESIZE\
                                          + CASWTASKWITHLOCK_DISPLAYSTATESIZE\
                                          + CASWTASK_DISPLAYSTATESIZENOPATH\
                                          + _MAX_PATH;
#endif   
#if defined(ASWANASSERT)
#if defined(WIN16) && !(defined(MAKEDLL) && defined(COMPASW))
   const UINT ASWMAXTASKS = 1;
#else 
   const UINT ASWMAXTASKS = 32;
#endif   
const UINT ASW_BAD_TASK_INDEX = ~((UINT)0);
class CAswTaskWithLock : public CAswTaskWithPath
{
public:
   inline CAswTaskWithLock( void )
   {
      m_dwLockCount = 0;
   }
   inline ~CAswTaskWithLock( void )
   {
      Shutdown();
   }
   void Setup( void );
   void Shutdown( void );
   inline void Lock( void )
   {
      m_dwLockCount++;
   }
   inline void UnLock( void )
   {
      if ( 0 != m_dwLockCount )
         m_dwLockCount--;
   }
   inline BOOL IsLocked( void ) const
   {
      return ( 0 != m_dwLockCount );
   }
#if defined(ASWANDEBUG)
   void DisplayState( LPSTR lpszDataInfo );
#endif   
private:
   DWORD             m_dwLockCount;
};
class CAswTaskInfo : public CAswTaskWithLock
{
public:
   inline CAswTaskInfo( void )
      : CAswTaskWithLock()
      , m_dwBalanceCounter( 0 )
      , m_dwMinimumBalanceForFreeChecking( 0 )
   {
   }
   inline void Setup( void )
   {
      CAswTaskWithLock::Setup();
      SetMinimumBalanceForFreeChecking( GetBalanceCounter() );
   }
   inline void Shutdown( void )
   {
      CAswTaskWithLock::Shutdown();
      SetBalanceCounter( 0 );
      SetMinimumBalanceForFreeChecking( 0 );
#if defined(ASWANDEBUG)
      ASWSIMPLEASSERT( NULL == GetStringListHead(), 
                       "CAswTaskWithLock::~CAswTaskWithLock - NULL != GetStringListHead()" );
#endif   
   }
   inline void SetBalanceCounter( DWORD dwBalanceCounter )
   {
      m_dwBalanceCounter = dwBalanceCounter;
   }
   inline DWORD GetBalanceCounter( void ) const
   {
      return m_dwBalanceCounter;
   }
   inline void SetMinimumBalanceForFreeChecking( DWORD dwMinimumBalanceForFreeChecking )
   {
      m_dwMinimumBalanceForFreeChecking = dwMinimumBalanceForFreeChecking;
   }
   inline DWORD GetMinimumBalanceForFreeChecking( void ) const
   {
      return m_dwMinimumBalanceForFreeChecking;
   }
#if defined(ASWANDEBUG)
   void DisplayState( HWND hWndListBox );
private:
   PASWSTRINGBASE m_pAswStringListHead;
public:
   void AddStringToList( PASWSTRINGBASE pStringBase );
   void RemoveStringFromList( PASWSTRINGBASE pStringBase );
   void AddAllStrings( HWND hWndLBStrings );
   inline void SetStringListHead( PASWSTRINGBASE pHead )
   {
      m_pAswStringListHead = pHead;
   }
   inline PASWSTRINGBASE GetStringListHead( void ) const
   {
      return m_pAswStringListHead;
   }
   void UpdateNumStrings( HWND hWndDlg );
#endif   
private:
   DWORD m_dwBalanceCounter;
   DWORD m_dwMinimumBalanceForFreeChecking;
};
FORWARD_CLASS_REF( CAswTaskInfo, TASKINFO );
FORWARD_CLASS_REF( CAswPerTaskInfo, PERTASKINFO );
#if defined(ASWANDEBUG)
   #define DO_CTOR_CHECK 0
   #if DO_CTOR_CHECK
      const LPCSTR lpszCTOR_CHECK = "CAswPerTaskInfo: CTOR_CHECK() failed, indicating a static initialization ordering error.";
      const LPCSTR lpszCTOR_CHECKWindowClass = "STATIC";
      const LPCSTR lpszCTOR_CHECKWindowName = "CAswTaskInfo_CTOR";
      #define CTOR_CHECK()\
      ASWSIMPLEASSERT(NULL != ::FindWindow(lpszCTOR_CHECKWindowClass,lpszCTOR_CHECKWindowName),lpszCTOR_CHECK)
      #define CTOR_CHECK_DESTROY() ::DestroyWindow(::FindWindow(lpszCTOR_CHECKWindowClass,lpszCTOR_CHECKWindowName))
   #else
      #define CTOR_CHECK()
      #define CTOR_CHECK_DESTROY()
   #endif   
#else    
   #define CTOR_CHECK()
   #define CTOR_CHECK_DESTROY()
#endif   
class CAswPerTaskInfo : public CAswMemoryBase
{
public:
#if defined(ASWANDEBUG)
   enum eShowDialog
   {
      SHOW_MODELESS = 1,
      SHOW_MODAL    = 2,
      HIDE_DIALOG   = 3
   };
   enum eForceShowStrings
   {
      NO_FORCE_SHOW_STRINGS = 1,
      FORCE_SHOW_STRINGS = 2,
   };
#endif   
   CAswPerTaskInfo( void );
   ~CAswPerTaskInfo( void );
   void AddCurrentTask( void );
   void DropCurrentTask( void );
   DWORD ReadCurrentBalanceCounter( void );
   void IncrementCurrentBalanceCounter( void );
   void DecrementCurrentBalanceCounter( void );
   DWORD ReadMinimumBalanceForFreeChecking( void );
#if defined(ASWANDEBUG)
   void DisplayState( eShowDialog         ShowDialog   = SHOW_MODELESS,
                      eForceShowStrings   ForceStrings = NO_FORCE_SHOW_STRINGS );
   void AddStringToList( PASWSTRINGBASE pStringBase );
   void RemoveStringFromList( PASWSTRINGBASE pStringBase );
   void AddAllStrings( HWND hWndLBStrings );
   inline void SetDialogHandle( HWND hWnd )
   {
      CTOR_CHECK();
      m_hWndDialog = hWnd;
   }
   inline HWND GetDialogHandle( void )
   {
      CTOR_CHECK();
      return ( m_hWndDialog );
   }
   inline void SetStringsMode( eForceShowStrings ForceStrings )
   {
      CTOR_CHECK();
      m_StringsMode = ForceStrings;
   }
   inline eForceShowStrings GetStringsMode( void )
   {
      CTOR_CHECK();
      return ( m_StringsMode );
   }
   inline void SetShowMode( eShowDialog ShowDialog )
   {
      CTOR_CHECK();
      m_ShowDialog = ShowDialog;
   }
   inline eShowDialog GetShowMode( void )
   {
      CTOR_CHECK();
      return ( m_ShowDialog );
   }
   inline BOOL IsModeless( void )
   {
      CTOR_CHECK();
      return ( SHOW_MODELESS == GetShowMode() );
   }
   inline BOOL IsModal( void )
   {
      CTOR_CHECK();
      return ( SHOW_MODAL == GetShowMode() );
   }
   inline BOOL IsHideDialog( void )
   {
      CTOR_CHECK();
      return ( HIDE_DIALOG == GetShowMode() );
   }
   inline void BlockRecursion( void )
   {
      CTOR_CHECK();
      m_bRecursionIsAllowed = FALSE;
   }
   inline void AllowRecursion( void )
   {
      CTOR_CHECK();
      m_bRecursionIsAllowed = TRUE;
   }
   inline BOOL IsRecursionAllowed( void )
   {
      CTOR_CHECK();
      return ( m_bRecursionIsAllowed );
   }
   void UpdateDisplay( void );
   static BOOL StringListAvailable( void );
   PASWTASKINFO GetTaskInfoPtr( DWORD dwIndex )
   {
      if ( dwIndex < m_uiNumTasks )
      {
         return ( &m_TaskInfo[ dwIndex ] );
      }
      else
      {
         ASWSIMPLEASSERT( FALSE,
                          "CAswPerTaskInfo::GetTaskInfoPtr: task index out of range" );
         return ( NULL );
      }
   }
   int LoadString( UINT uiID, LPSTR lpsz, int iSize )
   {
      int iRet = 0;
      if ( lpsz )
         *lpsz = NULL;
      if ( INSTANCEHANDLE_OK ( m_hInstResDLL ) )
         iRet = ::LoadString( m_hInstResDLL, uiID, lpsz, iSize );
      return ( iRet );
   }
   void ShowHelp( HWND hwndDlg );
#endif   
private:
#if defined(ASWANDEBUG)
   void Unwind( void );
#endif   
   PASWTASKINFO GetCurrentTaskPointer( void );
   inline void SetCache( PASWTASKINFO pTaskInfo )
   {
      CTOR_CHECK();
      m_pCurrentTask = pTaskInfo;
   }
   UINT                    m_uiNumTasks;
   PASWTASKINFO            m_pCurrentTask;
   CAswTaskInfo            m_TaskInfo[ ASWMAXTASKS ];
#if defined(ASWANDEBUG)
   HWND                    m_hWndDialog;
   HINSTANCE               m_hInstResDLL;
   eForceShowStrings       m_StringsMode;
   eShowDialog             m_ShowDialog;
   BOOL                    m_bRecursionIsAllowed;
   static CAswBoolRegValue NEAR cm_MaintainStringList;
#endif   
};
#endif   
#include <aspop.h>
#endif
