/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswSourceFormatSet
 *        CAswClipboardSource
 *        CAswDragSource
 *        CAswClipData
 *        CAswDragData
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *
 *    CAswSourceFormatSet
 *       The CAswSourceFormatSet is a class which holds an array for a set
 *       of data transfer (clipboard) formats that the application is 
 *       willing to source its data in.  Most applications will have a
 *       defined number of static sets that are known up front. An instance
 *       of CAswSourceFormatSet will be allocated for each of these sets. 
 *
 *       PUBLIC API:
 *          INITIALIZATION
 *             CAswSourceFormatSet
 *             ~CAswSourceFormatSet
 *             Setup
 *             Shutdown
 *
 *           FORMAT MANAGEMENT
 *             GetCount
 *             GetFormatArray
 *
 *    CAswDragSource
 *       CAswDragSource class is responsible for handling OLE 2 drag source.
 *       An instance of this class must be allocated for each drag operation
 *       or application instance.  It is important to note that CAswDragSource
 *       inherits publicly from CAswSourceObject.
 *
 *       PUBLIC API:
 *          OBJECT CREATION AND DELETION
 *             CAswDragSource
 *             ~CAswDragSource
 *
 *          DRAG/DROP METHODS
 *             DoDragDrop
 *             GiveDragFeedback     *** Callout ***
 *             QueryDragContinue    *** Callout ***
 *
 *          SOURCE DATA METHODS
 *             SetFormatSet
 *             QueryFormatSet       *** Overrideable ***
 *             GetData              *** Callout ***
 *
 *    CAswClipboardSource
 *       CAswClipboardSource class is responsible for handling OLE 2
 *       clipboard source support.  An instance of this class must be
 *       allocated for each application instance.  It is important to note
 *       that CAswClipboardSource inherits publicly from CAswSourceObject.
 *
 *       PUBLIC API:
 *          CONSTRUCTION/DESTRUCTION METHODS
 *             CAswClipboardSource
 *             ~CAswClipboardSource
 *
 *          CLIPBOARD SOURCE METHODS
 *             PutObjectOnClipboard
 *             RemoveObjectFromClipboard
 *             IsObjectOnClipboard
 *             FlushClipboard
 *
 *          SOURCE DATA METHODS
 *             SetFormatSet
 *             QueryFormatSet       *** Overrideable ***
 *             GetData              *** Callout ***
 *
 *****************************************************************************/

#ifndef ASWSRC_H
#define ASWSRC_H
#include <aspack.h>
class ASWPUBCLASS CAswSourceFormatSet : public CAswDebugBase
{
public:
   inline CAswSourceFormatSet(PASWCONTROL pCtl)
      : CAswDebugBase(pCtl,TRUE),      
        m_pFormatEtc(NULL),            
        m_nFormatEtc(0)
   {
      ASWSETCLASSNAME("CAswSourceFormatSet");
      ASWMETHOD("CAswSourceFormatSet", (CH_CLASS_CASWSOURCEFORMATSET));
   	ASWCONSTRUCTORENTRY();
      ASWCONSTRUCTOREXIT();
   }
   inline ~CAswSourceFormatSet(void)
   {
      ASWMETHOD("~CAswSourceFormatSet", (CH_CLASS_CASWSOURCEFORMATSET));
      ASWDESTRUCTORENTRY();
   	Shutdown(TRUE);
      ASWDESTRUCTOREXIT();
   }
   CAswResult ASWPUBMETHOD Setup(LPFORMATETC pFormatEtc, UINT nEntries);
   CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   inline UINT GetFormatCount(void)
   {
      return m_nFormatEtc;
   }
   inline LPFORMATETC GetFormatArray(void)
   {
      return m_pFormatEtc;
   }
private:
   LPFORMATETC m_pFormatEtc;     
   UINT        m_nFormatEtc;     
};
class CAswClipData : public CAswDataObjectImpl2
{
public:
   CAswClipData(PASWCONTROL pCtl,PASWCOMBASE pCOMBase,PASWCLIPBOARDSOURCE pSrc)
      : CAswDataObjectImpl2(pCtl,pCOMBase),
        m_pSrc(pSrc) 
      {  };
   virtual CAswResult GetData(LPFORMATETC     pFormatEtc, 
                              PASWSTGMEDUNION pAswStgMedium);
private:
   virtual PASWSOURCEFORMATSET FindFormatSet(void);
   PASWCLIPBOARDSOURCE const  m_pSrc;
};
class CAswDragData : public CAswDataObjectImpl2
{
public:
   CAswDragData(PASWCONTROL pCtl,PASWCOMBASE pCOMBase,PASWDRAGSOURCE pSrc)
      : CAswDataObjectImpl2(pCtl,pCOMBase),
        m_pSrc(pSrc) 
      {  };
   virtual CAswResult GetData(LPFORMATETC     pFormatEtc, 
                              PASWSTGMEDUNION pAswStgMedium);
private:
   virtual PASWSOURCEFORMATSET FindFormatSet(void);
   PASWDRAGSOURCE const  m_pSrc;
};
class ASWPUBCLASS CAswClipboardSource : public CAswCOMBase
{
   friend CAswClipData;
   ASWAN_DECLARATIONS(1);
public:
   ASWPUBMETHOD CAswClipboardSource(PASWCONTROL pControl,
                                    LPUNKNOWN   pUnkOuter = NULL);
   ASWPUBMETHOD ~CAswClipboardSource(void);
   CAswResult ASWPUBMETHOD Setup(PASWSOURCEFORMATSET pAswSourceFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   CAswResult ASWPUBMETHOD PutObjectOnClipboard(void);
   CAswResult ASWPUBMETHOD RemoveObjectFromClipboard(void);
   BOOL ASWPUBMETHOD FlushClipboard(void);
   BOOL ASWPUBMETHOD IsObjectOnClipboard(void);
   inline void SetFormatSet(PASWSOURCEFORMATSET pAswSourceFormatSet)
   {
      m_pFormatSet = pAswSourceFormatSet;
   }
   inline PASWSOURCEFORMATSET GetFormatSet(void)
   {
      return m_pFormatSet;
   }
   inline void ClearFormatSet(void)
   {
      m_pFormatSet = NULL;
   }
   virtual PASWSOURCEFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   virtual CAswResult ASWPUBMETHOD GetData(LPFORMATETC     pFormatEtc, 
                                           PASWSTGMEDUNION pAswStgMedium) PURE;
private:
   inline PASWSOURCEFORMATSET FindFormatSet(void)
   {
      return (m_pFormatSet) ? m_pFormatSet : QueryFormatSet();
   }
   CAswClipData         m_IDataObject;
   PASWSOURCEFORMATSET  m_pFormatSet;
   BOOL                 m_fSetup;
};
class ASWPUBCLASS CAswDragSource : public CAswCOMBase
{
   friend CAswDragData;
   friend CAswIDropSourceImpl;
   ASWAN_DECLARATIONS(2);
public:
   ASWPUBMETHOD CAswDragSource(PASWCONTROL pControl,LPUNKNOWN pUnkOuter = NULL);
   ASWPUBMETHOD ~CAswDragSource(void);
   CAswResult ASWPUBMETHOD Setup(PASWSOURCEFORMATSET pAswSourceFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   CAswResult ASWPUBMETHOD DoDragDrop(DWORD dwEffect,LPDWORD pdwEffect);
   virtual CAswResult ASWPUBMETHOD GiveDragFeedback(DWORD dwEffect);
   virtual CAswResult ASWPUBMETHOD QueryDragContinue(BOOL   fEscapePressed,
                                                     DWORD  grfKeyState);
   inline BOOL IsInternalAction(void)
   {
      return m_fInternalDrop;
   }
   inline void SetFormatSet(PASWSOURCEFORMATSET pAswSourceFormatSet)
   {
      m_pFormatSet = pAswSourceFormatSet;
   }
   inline PASWSOURCEFORMATSET GetFormatSet(void)
   {
      return m_pFormatSet;
   }
   inline void ClearFormatSet(void)
   {
      m_pFormatSet = NULL;
   }
   virtual PASWSOURCEFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   virtual CAswResult ASWPUBMETHOD GetData(LPFORMATETC     pFormatEtc, 
                                           PASWSTGMEDUNION pAswStgMedium) PURE;
   inline void SetInternalDrop(BOOL fInternal = TRUE)
   {
      m_fInternalDrop = fInternal;
   }
private:
   inline PASWSOURCEFORMATSET FindFormatSet(void)
   {
      return (m_pFormatSet) ? m_pFormatSet : QueryFormatSet();
   }
   CAswDragData         m_IDataObject;
   CAswIDropSourceImpl  m_IDropSource;
   PASWSOURCEFORMATSET  m_pFormatSet;
   BOOL                 m_fSetup;
   BOOL                 m_fInternalDrop;
};
#include <aspop.h>
#endif
