/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswTargetFormatSet
 *        CAswDropTarget
 *        CAswPasteTarget
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *
 *    CAswTargetFormatSet
 *
 *       The CAswTargetFormatSet is a class which holds an array for a set of
 *       available data transfer (clipboard) formats defined by the target
 *       application.  The application must maintain the list of formats for
 *       each set in PRIORITY order.  Most applications will have a defined
 *       number of static sets that are known up front. An instance of
 *       CAswTargetFormatSet will be allocated for each of these sets.
 *
 *       The structures used by the class are (more information can be
 *       found in the OLE 2.01 UI Reference and the OLE 2
 *       Programmer's Reference)
 *
 *    CAswDropTarget
 *
 *       CAswDropTarget class is responsible for handling OLE 2 drop target.
 *       An instance of this class must be allocated for each target window.
 *
 *    CAswPasteTarget
 *
 *       CAswPasteTarget class is used by any application wishing to process
 *       Edit+Paste, Edit+Paste Link, or Edit+Paste Special for OLE 2 objects.
 *       One instance of this class should be allocated per paste target
 *       area (typically associated with a frame).
 *
 *****************************************************************************/

#ifndef ASWTARG_H
#define ASWTARG_H
#include <aspack.h>
class ASWPUBCLASS CAswTargetFormatSet : public CAswDebugBase
{
public:
   inline CAswTargetFormatSet(PASWCONTROL pCtl)
      : CAswDebugBase(pCtl,TRUE),      
        m_pFormats(NULL),              
        m_nFormats(0),
        m_pLinkFormats(NULL),
        m_nLinkFormats(0),
        m_pBuf(NULL)
   {
      ASWSETCLASSNAME("CAswTargetFormatSet");
      ASWMETHOD("CAswTargetFormatSet", (CH_CLASS_CASWTARGETFORMATSET));
      ASWCONSTRUCTORENTRY();
      ASWCONSTRUCTOREXIT();
   }
   inline ~CAswTargetFormatSet(void)
   {
      ASWMETHOD("~CAswTargetFormatSet", (CH_CLASS_CASWTARGETFORMATSET));
      ASWDESTRUCTORENTRY();
      Shutdown(TRUE);
      ASWDESTRUCTOREXIT();
   }
   CAswResult ASWPUBMETHOD Setup(LPOLEUIPASTEENTRY pFormats,
                                 UINT              nFormats,
                                 LPUINT            pLinkFormats = NULL,
                                 UINT              nLinkFormats = 0);
   CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   inline UINT GetFormatCount(void) const
   {
      return m_nFormats;
   }
   inline LPOLEUIPASTEENTRY GetFormatArray(void) const
   {
      return m_pFormats;
   }
   inline UINT GetLinkCount(void) const
   {
      return m_nLinkFormats;
   }
   inline LPUINT GetLinkArray(void) const
   {
      return m_pLinkFormats;
   }
private:
   LPOLEUIPASTEENTRY m_pFormats;       
   UINT              m_nFormats;       
   LPUINT            m_pLinkFormats;   
   UINT              m_nLinkFormats;   
   LPVOID            m_pBuf;
};
class ASWPUBCLASS CAswDropTarget : public CAswCOMBase
{
   friend CAswIDropTargetImpl;
   ASWAN_DECLARATIONS(1);
public:
   enum SCROLLDIRECTION
   {
      SCROLL_NONE          = 0x00,
      SCROLL_UP            = 0x01,
      SCROLL_RIGHT         = 0x02,
      SCROLL_DOWN          = 0x04,
      SCROLL_LEFT          = 0x08,
      SCROLL_UPANDRIGHT    = (SCROLL_UP   | SCROLL_RIGHT),
      SCROLL_DOWNANDRIGHT  = (SCROLL_DOWN | SCROLL_RIGHT),
      SCROLL_DOWNANDLEFT   = (SCROLL_DOWN | SCROLL_LEFT),
      SCROLL_UPANDLEFT     = (SCROLL_UP   | SCROLL_LEFT)
   };
   enum FEEDBACK
   {
      FEEDBACK_START       = 0x01,     
      FEEDBACK_INPROCESS   = 0x02,     
      FEEDBACK_END         = 0x03      
   };
   ASWPUBMETHOD CAswDropTarget(PASWCONTROL pControl,LPUNKNOWN pUnkOuter);
   ASWPUBMETHOD ~CAswDropTarget(void);
   CAswResult ASWPUBMETHOD
   Setup(HWND                hwndDrop,
         PASWDRAGSOURCE      pDragSource = NULL,
         PASWTARGETFORMATSET pAswTargetFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   virtual PASWTARGETFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   inline void SetFormatSet(PASWTARGETFORMATSET pAswTargetFormatSet)
   {
      m_pfsTarget = pAswTargetFormatSet;
   }
   inline PASWTARGETFORMATSET GetFormatSet(void) const
   {
      return m_pfsTarget;
   }
   inline void ClearFormatSet(void)
   {
      m_pfsTarget = NULL;
   }
   virtual PASWDRAGSOURCE ASWPUBMETHOD QueryDragSource(void);
   inline void SetDragSource(PASWDRAGSOURCE pDragSource)
   {
      m_pDragSource = pDragSource;
   }
   inline PASWDRAGSOURCE GetDragSource(void) const
   {
      return m_pDragSource;
   }
   inline void ClearDragSource(void)
   {
      m_pDragSource = NULL;
   }
   virtual BOOL ASWPUBMETHOD ObjectDropped(POINTL        pt,
                                           LPFORMATETC   pBestFormatetc,
                                           DWORD         dwEffect,
                                           BOOL          fRightMouseDrop) PURE;
   virtual void ASWPUBMETHOD DragEnter(POINTL pt);
   virtual void ASWPUBMETHOD DragOver(POINTL pt);
   virtual void ASWPUBMETHOD DragLeave(void);
   virtual void ASWPUBMETHOD RenderTargetFeedback(POINTL  pt,
                                                  DWORD   dwEffect,
                                                  WORD    wState);
   virtual void ASWPUBMETHOD DetermineEffect(DWORD     grfKeyState,
                                             BOOL      fDragScroll,
                                             LPDWORD   pdwEffect);
   BOOL ASWPUBMETHOD IsInternalAction(void);
   BOOL ASWPUBMETHOD IsAppSource(void);
   void ASWPUBMETHOD SetScrollHotZone(LPCRECT prcInset,
                                      BOOL    fUseOleDefault = FALSE);
   void ASWPUBMETHOD SetScrollTime(DWORD  dwMilliSeconds = DD_DEFSCROLLINTERVAL,
                                   BOOL   fUseOleDefault = FALSE);
   virtual void ASWPUBMETHOD Scroll(WORD wScrollDirection);
   virtual LPFORMATETC ASWPUBMETHOD
   GetHighestFormatMatch(PASWTARGETFORMATSET pAswTargetFormatSet = NULL,
                         LPUINT              pIndex = NULL);
   virtual BOOL ASWPUBMETHOD DoDragScroll(POINTL pt);
   inline void UpdateTargetFeedbackBackground(void)
   {
      if (m_pTargetFeedback)
         m_pTargetFeedback->RedoBackground();
   }
   inline BOOL QueryObjectFormat(LPFORMATETC pFormatEtc)
   {
      return m_DataObjectHolder.QueryObjectFormat(pFormatEtc);
   }
   inline CAswResult GetSourceData(LPFORMATETC     pFmtEtc,
                                   PASWSTGMEDUNION pAswStgMedium,
                                   PASWSTORAGE     pAswStorage = NULL,
                                   PASWSTREAM      pAswStream = NULL)
   {
      return m_DataObjectHolder.GetData(pFmtEtc,pAswStgMedium,
                                        pAswStorage,pAswStream);
   }
   inline PASWOBJECTDESCRIPTOR GetSourceObjectDescriptor(void)
   {
      return m_DataObjectHolder.GetObjectDescriptor();
   }
   inline PCASWREFHOLDER GetDataObjectRH(void)
   {
      return m_DataObjectHolder.GetDataObjectRH();
   }
   inline HWND GetDropWindow(void) const
   {
      return m_hwnd;
   }
   inline void SetTargetFeedback(PASWTARGETFEEDBACK pTargetFeedback)
   {
      m_pTargetFeedback = pTargetFeedback;
   }
   inline PASWTARGETFEEDBACK GetTargetFeedback(void) const
   {
      return m_pTargetFeedback;
   }
private:
   inline PASWTARGETFORMATSET FindFormatSet(void)
   {
      return (m_pfsTarget) ? m_pfsTarget : QueryFormatSet();
   }
   inline PASWDRAGSOURCE FindDragSource(void)
   {
      return (m_pDragSource) ? m_pDragSource : QueryDragSource();
   }
   inline void ReleaseDataObject(void)
   {
      m_DataObjectHolder.Release();
   }
   CAswIDropTargetImpl  m_IDropTarget;
   CAswDataObjectHolder m_DataObjectHolder;
   PASWTARGETFORMATSET  m_pfsTarget;
   HWND                 m_hwnd;
   PASWDRAGSOURCE       m_pDragSource;
   BOOL                 m_fCanDropLink;
   DWORD                m_dwScrollTime;      
   RECT                 m_rcInset;
   DWORD                m_dwTicksEnterScroll;
   WORD                 m_wLastScrollDirection;
   PASWTARGETFEEDBACK   m_pTargetFeedback;
};
class ASWPUBCLASS CAswPasteTarget : public CAswCtlBase
{
public:
   enum PASTEMENURESULTS
   {
      PASTE_NONE     = 0x00,  
      PASTE_EDIT     = 0x01,  
      PASTE_SPECIAL  = 0x02,  
      PASTE_LINK     = 0x04   
   };
   ASWPUBMETHOD CAswPasteTarget(PASWCONTROL pControl);
   ASWPUBMETHOD ~CAswPasteTarget(void);
   CAswResult ASWPUBMETHOD Setup(PASWTARGETFORMATSET pTargFormatSet = NULL);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   virtual PASWTARGETFORMATSET ASWPUBMETHOD QueryFormatSet(void);
   inline void SetFormatSet(PASWTARGETFORMATSET pAswTargetFormatSet)
   {
      m_pfsTarget = pAswTargetFormatSet;
   }
   inline PASWTARGETFORMATSET GetFormatSet(void)
   {
      return m_pfsTarget;
   }
   inline void ClearFormatSet(void)
   {
      m_pfsTarget = NULL;
   }
   virtual BOOL ASWPUBMETHOD ObjectPasted(LPFORMATETC pBestFormatetc,
                                          BOOL        fLink,
                                          HGLOBAL     hMetaPict) PURE;
   CAswResult ASWPUBMETHOD
   ProcessPaste(BOOL fLink = FALSE, LPFORMATETC pFormatEtc = NULL);
   CAswResult ASWPUBMETHOD ProcessPasteSpecial(HWND         hWndOwner,
                                               DWORD        dwFlags = PSF_SHOWHELP,
                                               LPASWCOLESTR pszCaption = NULL);
   void ASWPUBMETHOD PasteInitMenuPopup(LPWORD pPasteMenuResults);
   inline CAswResult GetSourceData(LPFORMATETC     pFmtEtc,
                                   PASWSTGMEDUNION pAswStgMedium,
                                   PASWSTORAGE     pAswStorage = NULL,
                                   PASWSTREAM      pAswStream = NULL)
   {
      return m_DataObjectHolder.GetData(pFmtEtc,pAswStgMedium,
                                        pAswStorage,pAswStream);
   }
   inline PASWOBJECTDESCRIPTOR GetSourceObjectDescriptor(void)
   {
      return m_DataObjectHolder.GetObjectDescriptor();
   }
   inline PCASWREFHOLDER GetDataObjectRH(void)
   {
      return m_DataObjectHolder.GetDataObjectRH();
   }
private:
   inline PASWTARGETFORMATSET FindFormatSet(void)
   {
      return (m_pfsTarget) ? m_pfsTarget : QueryFormatSet();
   }
   inline void ReleaseDataObject(void)
   {
      m_DataObjectHolder.Release();
   }
   CAswDataObjectHolder m_DataObjectHolder;
   PASWTARGETFORMATSET  m_pfsTarget;
   CLIPFORMAT           m_cfLinkSource;
};
#include <aspop.h>
#endif
