/* NOSHIP */
/*******************************************************************************
* FILE NAME: icmpfram.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IComponentFrameWindow                                                    *
*     IAswFrameWindow                                                          *
*     IAswFrameSpace                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _ICMPFRAM_
#define _ICMPFRAM_

// base class
#include <iframe.hpp>

#include <istattxt.hpp>
#include <itbar.hpp>
#include <itbarbut.hpp>
#include <iaccel.hpp>

// forward declarations
class CAswFrameWindow;
class CAswControl;
class IToolBar;

// classes defined in this module
class IComponentFrameWindow;
class IAswFrameWindow;
class IAswFrameSpace;

/*******************************************************************************
*                                                                              *
*   Class IComponentFrameWindow                                                *
*                                                                              *
*   Main frame window for the component                                        *
*                                                                              *
*******************************************************************************/

class IComponentFrameWindow : public IFrameWindow
{

public:
// Constuction/Destruction
                                IComponentFrameWindow();
    CAswResult                  setup( CAswControl* pAswControl );
                                ~IComponentFrameWindow();


// Public Methods
public:
    void                        addDefaultMenuBar( Boolean bForceStandalone = false );
    void                        changedNegotiatedSpace();
    void                        delayAdjustment();
    void                        adjustToolBar();
    void                        showToolBar( Boolean );

    IMenuBar&                   menuBar() { return *fpMenuBar; }
    IToolBar&                   toolBar() { return *fpToolBar; }
    IAccelerator&               accelerator() { return *fpAccel; }
    IStaticText&                dummy() { return fDummy; }

    // casting operator to make this class look like it was derived from CAswFrameWindow
    operator CAswFrameWindow* () const  { return (CAswFrameWindow*)fpAswFrameWindow; }


// Private members
private:
    IMenuBar*                   fpMenuBar;
    IToolBar*                   fpToolBar;
    IAccelerator*               fpAccel;

    IAswFrameWindow*            fpAswFrameWindow;
    CAswContMenu*               fpMenuBase;
    IAswFrameSpace*             fpFrameSpace;
    PASWINPLACEPRESSITE         fpActivePresSite;

    IStaticText                 fDummy;
    ASWSHAREDMENU               fSharedMenu[3];

    Boolean                     fbDeactivateFlag;
    Boolean                     fbDelayedAdjustment;
    
friend class IAswFrameWindow;
friend class IAswFrameSpace;
}; // class IComponentFrameWindow


/*******************************************************************************
*                                                                              *
*   Class IComponentToolBar                                                    *
*                                                                              *
*******************************************************************************/

class IComponentToolBar : public IToolBar {
public:
                            IComponentToolBar( IFrameWindow& frame );
    virtual                 ~IComponentToolBar();
    virtual IToolBarFrameWindow* createFloatingFrame ( IFrameWindow* owner );

private:
    // these buttons appear on all toolbars
    IToolBarButton          fButtonCut;
    IToolBarButton          fButtonCopy;
    IToolBarButton          fButtonPaste;
    IToolBarButton          fButtonAbout;

    // these 3 are only required when running stand-alone (not embedded)
    IToolBarButton*         fpButtonNew;
    IToolBarButton*         fpButtonOpen;
    IToolBarButton*         fpButtonSave;

};


/*******************************************************************************
*                                                                              *
*   Class IAswFrameWindow                                                      *
*                                                                              *
*   Implementation of CAswFrameWindow                                          *
*                                                                              *
*******************************************************************************/

class IAswFrameWindow : public CAswFrameWindow
{

public:
                                IAswFrameWindow( IComponentFrameWindow& );

protected:
// Base class overrides
    virtual HRESULT ASWPUBMETHOD OnInPlaceActivatePassThru( PASWINPLACEPRESSITE pIPPresSite,
                                                            PASWREFHOLDER       pRefHoldOleInPlaceObj);

    virtual void    ASWPUBMETHOD OnInPlaceDeactivatePassThru( PASWINPLACEPRESSITE pIPPresSite);

private:
    // back pointer to actual frame window
    IComponentFrameWindow*      fpComponentFrameWindow;

}; // class IAswFrameWindow


/*******************************************************************************
*                                                                              *
*   Class IAswFrameSpace                                                       *
*                                                                              *
*   Implementation of CAswSpaceBase - toolbar space negotiation                *
*                                                                              *
*******************************************************************************/

class IAswFrameSpace : public CAswSpaceBase
{

public:
// Constuction/Destruction
                                IAswFrameSpace( IComponentFrameWindow& );
    virtual                     ~IAswFrameSpace();

protected:
// Base class overrides

   // Returns a RECT structure in which the object can put toolbars while
   // active InPlace.
   virtual CAswResult ASWPUBMETHOD GetBorder(LPRECT prcBorder);

   // Checks if tools can be installed around the  window frame while the
   // object is active InPlace.
   virtual CAswResult ASWPUBMETHOD RequestBorderSpace( LPCBORDERWIDTHS lpBorderwidths);

   // Allocates space for the border requested in the call to the
   // RequestBorderSpace method.
   virtual CAswResult ASWPUBMETHOD SetBorderSpace( LPCBORDERWIDTHS lpBorderwidths );

private:
    // back pointer to actual frame window
    IComponentFrameWindow*      fpComponentFrameWindow;

}; // class IAswFrameSpace


#endif // _ICMPFRAM_
