/*******************************************************************************
* FILE NAME: immabuf.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immabuf.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_OS2MM
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#include <immabuf.hpp>
#include <istring.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMABUF_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PM
// Initialize static objects
#pragma data_seg(ICLStaticConst)
const unsigned long
  IMMAudioBuffer::audio  = MMIO_MEDIATYPE_AUDIO;
#pragma data_seg()
#endif

/*------------------------------------------------------------------------------
| IMMAudioBufferData                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
class IMMAudioBufferData : public IBase
{
public:
  IMMAudioBufferData()
     { }
};


/*------------------------------------------------------------------------------
| IMMAudioBuffer::IMMAudioBuffer                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::IMMAudioBuffer( void *audioData,
                                unsigned long len)
              : fBuffer(new IString(audioData,len)),
                fLength(len)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::IMMAudioBuffer(*audioData, length)") ;
#ifdef IC_PM
  fHeader = new MMAUDIOHEADER;
  memset(fHeader, 0, sizeof(MMAUDIOHEADER));
  PMMAUDIOHEADER(fHeader)->ulHeaderLength = sizeof(MMAUDIOHEADER);
#endif
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::IMMAudioBuffer                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::IMMAudioBuffer(unsigned long len)
              : fBuffer(new IString(0,len)),
                fLength(len)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::IMMAudioBuffer(length)") ;
#ifdef IC_PM
  fHeader = new MMAUDIOHEADER;
  memset(fHeader, 0, sizeof(MMAUDIOHEADER));
  PMMAUDIOHEADER(fHeader)->ulHeaderLength = sizeof(MMAUDIOHEADER);
#endif
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::IMMAudioBuffer                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::IMMAudioBuffer(const IMMAudioBuffer &audioBuffer)
              : fBuffer(audioBuffer.fBuffer),
                fLength(audioBuffer.fLength)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::IMMAudioBuffer(audioBuffer, length)") ;
#ifdef IC_PM
  fHeader = new MMAUDIOHEADER;
  memcpy(fHeader, audioBuffer.fHeader, sizeof(MMAUDIOHEADER));
#endif
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::operator =                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::operator = (const IMMAudioBuffer &audioBuffer)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::operator=") ;
#ifdef IC_PM
  memcpy(fHeader, audioBuffer.fHeader, sizeof(MMAUDIOHEADER));
  if (fBuffer)
     delete fBuffer;
  fBuffer = new IString(audioBuffer.fBuffer,audioBuffer.fLength);
  fLength = audioBuffer.fLength;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::~IMMAudioBuffer                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::~IMMAudioBuffer()
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::~IMMAudioBuffer") ;
  if (fHeader)
     delete fHeader;
  if (fBuffer)
     delete fBuffer;
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IMMAudioBuffer::headerData                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioBuffer::headerData() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::headerData") ;
  return IString(fHeader,sizeof(MMAUDIOHEADER));
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::data                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioBuffer::data() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::data") ;
  return IString(*fBuffer);
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setData                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer& IMMAudioBuffer::setData( const void* data,
                                         unsigned long dataLength)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setData") ;
  if (fBuffer)
     delete fBuffer;
  fBuffer = new IString(data,dataLength);
  fLength = dataLength;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::length                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::length() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::length") ;
  return fLength;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::bytesPerSecond                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::bytesPerSecond() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::bytesPerSecond") ;
  return PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.ulAvgBytesPerSec;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::bitsPerSample                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::bitsPerSample() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::bitsPerSample") ;
  return PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usBitsPerSample;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::blockAlignment                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::blockAlignment() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::blockAlignment") ;
  return PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usBlockAlign;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::channels                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::channels() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::channels") ;
  return PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usChannels;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::contentType                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::ContentType IMMAudioBuffer::contentType() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::contentsType") ;
  return IMMAudioBuffer::ContentType(PMMAUDIOHEADER(fHeader)->ulContentType);
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::format                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::Format IMMAudioBuffer::format() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::format") ;
  return IMMAudioBuffer::Format(PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usFormatTag);
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::mediaType                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::mediaType() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::mediaType") ;
  return PMMAUDIOHEADER(fHeader)->ulMediaType;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::samplesPerSecond                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioBuffer::samplesPerSecond() const
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::samplesPerSecond") ;
  return PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.ulSamplesPerSec;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setBytesPerSecond                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setBytesPerSecond(unsigned long avgBytes)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setBytesPerSecond") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.ulAvgBytesPerSec = avgBytes;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setBitsPerSample                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setBitsPerSample(unsigned long bitsPer)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setBitsPerSample") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usBitsPerSample = USHORT(bitsPer);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setBlockAlignment                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setBlockAlignment(unsigned long blockAlign)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setBlockAlignment") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usBlockAlign =
                                                        USHORT(blockAlign);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setChannels                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setChannels(unsigned long channels)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setChannels") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usChannels = (unsigned short)channels;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setContentType                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setContentType(IMMAudioBuffer::ContentType contentType)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setContentType") ;
  PMMAUDIOHEADER(fHeader)->ulContentType = ULONG(contentType);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setFormat                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setFormat(IMMAudioBuffer::Format tag)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setFormat") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.usFormatTag = USHORT(tag);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setMediaType                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setMediaType(unsigned long mediaType)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setMediaType") ;
  PMMAUDIOHEADER(fHeader)->ulMediaType = mediaType;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioBuffer::setSamplesPerSecond                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer &IMMAudioBuffer::setSamplesPerSecond(unsigned long samps)
{
  IMODTRACE_DEVELOP("IMMAudioBuffer::setSamplesPerSecond") ;
  PMMAUDIOHEADER(fHeader)->mmXWAVHeader.WAVEHeader.ulSamplesPerSec = samps;
  return *this;
}
#endif
