/*******************************************************************************
* FILE NAME: immcdda.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immcdda.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <immdefs.h>
#ifdef IC_PM
   #include <cdaudos2.h>
#endif
   }

#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMTTIME_
   #include <immttime.hpp>
#endif

#include <immcdda.hpp>

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#ifndef _ITHREAD_
   #include <ithread.hpp>
#endif

#ifndef _ITIMER_
   #include <itimer.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif
#ifndef _IMMCDHDR_
  #include <immcdhdr.hpp>
#endif

#ifndef _IPLATFRM_
  #include <iplatfrm.hpp>
#endif

#include <immowin.hpp>

#ifdef IC_WIN
// Win32 does not have an MCI_GETTOC command. We'll fake it ...
typedef struct _MCI_TOC_REC {
    BYTE      TrackNum;      // Returned Track Number
    ULONG     ulStartAddr;   // Starting address of the track in MMTIME format
    ULONG     ulEndAddr;     // Ending address of the track in MMTIME format
    BYTE      Control;       // Track Control information
    USHORT    usCountry;     // Country
    ULONG     ulOwner;       // Owner
    ULONG     ulSerialNum;   // Serial Number
} MCI_TOC_REC;
typedef MCI_TOC_REC   *PTOCREC;
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMCDDA_CPP_
  #include <ipagetun.h>
#endif


// Initialize static objects
#pragma data_seg(ICLStaticConst)
const INotificationId IMMAudioCD::positionTimerId   = "IMMAudioCD::positionTimerId";
const INotificationId IMMAudioCD::trackStartedId    = "IMMAudioCD::trackStartedId";

const IString IMMAudioCD::discTitleKey("IMMAudioCDDiscTitle");
const IString IMMAudioCD::discProgramKey("IMMAudioCDDiscProgram");
#pragma data_seg()

class TOCRep : public IBase
{
private:

  TOCRep();
  TOCRep(void *newToc, unsigned long num);
  ~TOCRep();

int
  count;
unsigned long
  numTracks;
void
  *toc;

friend class IMMAudioCDContents;
friend class IMMAudioCDContentsData;
};

class IMMAudioCDData : public IBase
{
public:
  IMMAudioCDData(IMMAudioCDContents *toc2,
                 IProfile *prof2,
                 Boolean cdPlaying2,
                 Boolean conPlay2,
                 Boolean loadAndGo2,
                 IMMAudioCDContents::Cursor cursor2,
                 IMMAudioCD::ScanStatus scanStatus2,
                 ITimer* pScanTimer2,
                 ITimer* pUpdateTimer2 );           // 7810

  ~IMMAudioCDData();           // 7810

IMMAudioCDContents
  *toc;
IMMAudioCD::ScanStatus
  scanStatus;
ITimer
  *pScanTimer,
  *pUpdateTimer;
Boolean
  needToDeleteProfile,
  conPlay,
  loadAndGo,
  cdPlaying;
IProfile
  *prof;
IMMAudioCDContents::Cursor
  cursor;
IMMTrackMinSecFrameTime
  scanTime;
IMMAudioCDHandler
  *deviceHandler;
};

IMMAudioCDData::IMMAudioCDData(IMMAudioCDContents *toc2,
                 IProfile *prof2,
                 Boolean cdPlaying2,
                 Boolean conPlay2,
                 Boolean loadAndGo2,
                 IMMAudioCDContents::Cursor cursor2,
                 IMMAudioCD::ScanStatus scanStatus2,
                 ITimer* pScanTimer2,
                 ITimer* pUpdateTimer2 )           // 7810
      : toc(toc2),
        prof( prof2),
        cdPlaying(cdPlaying2),
        conPlay(conPlay2),
        loadAndGo(loadAndGo2),
        cursor(cursor2),
        scanTime(*toc2,1),
        scanStatus(scanStatus2),
        pScanTimer(pScanTimer2),
        pUpdateTimer(pUpdateTimer2),
        deviceHandler(0),
        needToDeleteProfile(false)
{ 
}

IMMAudioCDData::~IMMAudioCDData()           // 7810
{
  if (toc)
     delete toc;
  if (pScanTimer)
     delete pScanTimer;
  if (pUpdateTimer)
     delete pUpdateTimer;
  if (needToDeleteProfile)
     delete prof;
};

class IMMCursorData : public IBase
{
public:
  IMMCursorData(unsigned long ind, Boolean val,
                IMMAudioCDContents* Toc)
         : index(ind),
           valid(val),
           table(Toc),
           sameValidation(0)
     { };
  ~IMMCursorData()
     { };

IMMAudioCDContents
  *table;
unsigned long
  index,
  sameValidation;
Boolean
  valid;
};

class IMMAudioCDContentsData : public IBase
{
public:
  IMMAudioCDContentsData(Boolean val,IString discID,unsigned long *list,
                         TOCRep *reps,unsigned long num);          // 7810

  ~IMMAudioCDContentsData();            // 7810

Boolean
  valid;
IString
  discId;
TOCRep
  *rep;
unsigned long
  *trackList;
unsigned long
  trackNum,
  needValidation;
};

IMMAudioCDContentsData::IMMAudioCDContentsData(Boolean val,IString discID,
                    unsigned long *list, TOCRep *reps,unsigned long num)          // 7810
    : rep(reps),
      discId(discID),
      trackList(list),
      valid(val),
      trackNum(num),
      needValidation(0)
{ 
}

IMMAudioCDContentsData::~IMMAudioCDContentsData()            // 7810
{
   if (trackList)
      delete trackList;
   rep->count--;
   if (rep->count <= 0)
      delete rep;
}

/*------------------------------------------------------------------------------
| TOCRep::TOCRep                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::TOCRep()
      : count(0),
        numTracks(0),
        toc(0)
{
  IMODTRACE_DEVELOP("TOCRep::TOCRep") ;
}

/*------------------------------------------------------------------------------
| TOCRep::TOCRep                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::TOCRep(void *newToc, unsigned long num)
      : count(0),
        numTracks(num),
        toc(newToc)
{
  IMODTRACE_DEVELOP("TOCRep::TOCRep(void*, unsigned long)") ;
}

/*------------------------------------------------------------------------------
| TOCRep::~TOCRep                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::~TOCRep()
{
  IMODTRACE_DEVELOP("TOCRep::~TOCRep()") ;
  if (toc)
     delete toc;
}

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor(const IMMAudioCDContents &Toc)
                          : fCursorData(new IMMCursorData(0,true,0))
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::Cursor(IMMAudioCDContents&)") ;
  fCursorData->table = (IMMAudioCDContents*)(&Toc);
  if ((!Toc.numberOfEntries()) ||
      (!Toc.isValid()        )  )
      invalidate();
  else
      setToFirst();
}

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor(const IMMAudioCDContents::Cursor &cursor)
   : fCursorData(new IMMCursorData(cursor.fCursorData->index,
                                   cursor.fCursorData->valid,
                                   cursor.fCursorData->table))
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::Cursor(IMMAudioCDContents::Cursor&)") ;
}

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor()
                          : fCursorData(new IMMCursorData(0,false, 0))
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::Cursor()") ;
}

/*------------------------------------------------------------------------------
| Cursor::operator =                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator = (const IMMAudioCDContents::Cursor &newCursor)
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::operator=") ;
   if (this != &newCursor)  // check for assignment to self
   {
      fCursorData->valid = newCursor.fCursorData->valid;
      fCursorData->index = newCursor.fCursorData->index;
      fCursorData->table = newCursor.fCursorData->table;
      fCursorData->sameValidation = newCursor.fCursorData->sameValidation;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::~Cursor                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::~Cursor()
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::~Cursor") ;
  if (fCursorData)
     delete fCursorData;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents()
  : fContentsData(new IMMAudioCDContentsData(false,IString(),0,new TOCRep,0))
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::IMMAudioCDContents") ;
   fContentsData->rep->count = 1;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents(const IMMAudioCDContents &newToc)
   : fContentsData(new IMMAudioCDContentsData(newToc.fContentsData->valid,
                                              newToc.fContentsData->discId,
                                              0,
                                              newToc.fContentsData->rep,
                                              newToc.numberOfEntries()))

{
  IMODTRACE_DEVELOP("IMMAudioCDContents::IMMAudioCDContents(IMMAudioCDContents&)") ;
  fContentsData->rep->count++;
  unsigned long tr = newToc.numberOfEntries();
  if (tr)
  {
     fContentsData->trackList = new unsigned long[tr];
     memcpy(fContentsData->trackList, newToc.fContentsData->trackList, sizeof(unsigned long) * tr);
  }
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::operator =                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents &IMMAudioCDContents::operator = (const IMMAudioCDContents &newToc)
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::operator=") ;
   if (this != &newToc)  // check for assignment to self
   {
      fContentsData->valid = newToc.fContentsData->valid;
      fContentsData->discId = newToc.fContentsData->discId;
      fContentsData->trackNum = newToc.fContentsData->trackNum;

      fContentsData->rep->count--;
      if (fContentsData->rep->count <= 0)
         delete fContentsData->rep;

      newToc.fContentsData->rep->count++;
      fContentsData->rep = newToc.fContentsData->rep;
      if (newToc.fContentsData->trackList)
      {
         delete fContentsData->trackList;
         unsigned long tr = newToc.numberOfEntries();
         if (tr)
         {
            fContentsData->trackList = new unsigned long[tr];
            memcpy(fContentsData->trackList, newToc.fContentsData->trackList, sizeof(unsigned long) * tr);
         }
      }
     fContentsData->needValidation++;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents(void *newToc,
                                       const IString &id,
                                       unsigned long tracks)
   : fContentsData(new IMMAudioCDContentsData(true,id,0,new TOCRep(newToc,tracks),tracks))
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::IMMAudioCDContents(void*, IString, unsigned long)") ;
   fContentsData->rep->count = 1;
   if (tracks)
   {
      fContentsData->trackList = new unsigned long[tracks];
      unsigned long *pul = fContentsData->trackList;
      for (unsigned long i = 1; i <= tracks; i++)
         *pul++ = i;
   } /* endif */
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::~IMMAudioCDContents                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::~IMMAudioCDContents()
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::~IMMAudioCDContents") ;
  if (fContentsData)
     delete fContentsData;
}


/*------------------------------------------------------------------------------
| IMMAudioCDContents::controlInformation                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::controlInformation(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::controlInformation") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +  //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->Control;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::country                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::country(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::country") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->usCountry;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::timeAt                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::timeAt(unsigned long track,Boolean begin) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::timeAt") ;
  IASSERTSTATE(isValid());
  if (begin)
     return IMMTime((PTOCREC(fContentsData->rep->toc) + track - 1)->ulStartAddr);
  else
     return IMMTime((PTOCREC(fContentsData->rep->toc) + track - 1)->ulEndAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::endOfTrack                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::endOfTrack(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::endOfTrack") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return IMMTime((PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
                  fContentsData->trackList[cursor.asIndex()-1])->ulEndAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::owner                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::owner(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::owner") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->ulOwner;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::serialNumber                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::serialNumber(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::serialNumber") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->ulSerialNum;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::startOfTrack                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::startOfTrack(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::startOfTrack") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return IMMTime((PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
                 fContentsData->trackList[cursor.asIndex()-1])->ulStartAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::track                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::track(const IMMAudioCDContents::Cursor &cursor) const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::track") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  if (fContentsData->trackList)
     return fContentsData->trackList[cursor.asIndex()-1];
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::replaceEntryAt                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::replaceEntryAt(unsigned long newTrack,
                                                       const Cursor &cursor)
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::replaceEntryAt") ;
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->trackList[cursor.asIndex()-1] = newTrack;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::addEntryAsFirst                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::addEntryAsFirst(unsigned long newTrack)
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::addEntryAsFirst") ;
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr+1];
     newTrackList[0] = newTrack;
     newTrackList++;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * tr);
     newTrackList--;
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum++;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::addEntryAsNext                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::addEntryAsNext(unsigned long newTrack,
                                                       const Cursor &cursor)
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::addEntryAsNext") ;
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr+1];
     unsigned long index = cursor.asIndex() ;  //Since the cursor is 0-based
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * index);
     newTrackList[index] = newTrack;
     newTrackList+=index + 1;
     fContentsData->trackList+=index;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * (tr-index));
     newTrackList-=(index + 1);
     fContentsData->trackList-=index;
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum++;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::removeEntryAt                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::removeEntryAt( const Cursor &cursor)
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::removeEntryAt") ;
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr-1];
     unsigned long index = cursor.asIndex()-1;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * index);
     newTrackList+=index;
     fContentsData->trackList+=index + 1;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * (tr-index-1));
     newTrackList-=index;
     fContentsData->trackList-=(index + 1);
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum--;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::numberOfTracks                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::numberOfTracks() const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::numberOfTracks") ;
  return fContentsData->rep->numTracks;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::numberOfEntries                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::numberOfEntries() const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::numberOfEntries") ;
  return fContentsData->trackNum;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::IMMAudioCD                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::IMMAudioCD( IProfile* profile,
                        Boolean openNow,
                        unsigned long instance,
                        Boolean openShareable)
          :  IMMRemovableMedia("CDAUDIO", openNow, instance, openShareable),
             fAudioCDData(new IMMAudioCDData(new IMMAudioCDContents(),
                ((profile) ? profile : new IProfile(IProfile::userProfile())),
                false, false, false, IMMAudioCDContents::Cursor(),
                notScanning, 0, 0 )),
             fDelay(!openNow)
{
  IMODTRACE_DEVELOP("IMMAudioCD::IMMAudioCD(IProfile*, Boolean, unsigned long, Boolean)") ;
  if (!profile)
     fAudioCDData->needToDeleteProfile = true;
  if (openNow)
  {
     if (!(mode()==IMMDevice::notReady))
     {
     if (isMediaPresent())
     {
        setTimeFormat(IMMTime::trackMinSecFrame, wait);
        setTableOfContents(getTableFromDisc());
     }
     } /* endif */
  } /* endif */
  fAudioCDData->pScanTimer = new ITimer();
  fAudioCDData->pUpdateTimer = new ITimer();
  fAudioCDData->cursor.setToFirst();
  fAudioCDData->deviceHandler = new IMMAudioCDHandler();
  fAudioCDData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::IMMAudioCD                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::IMMAudioCD(unsigned long newDeviceId,
                        const IString &newAlias)
          : IMMRemovableMedia(newDeviceId, newAlias),
            fAudioCDData(new IMMAudioCDData(new IMMAudioCDContents(),
                               new IProfile(IProfile::userProfile()),
                               false, false, false, IMMAudioCDContents::Cursor(),
                               notScanning, 0, 0 )),
            fDelay(true)
{
  IMODTRACE_DEVELOP("IMMAudioCD::IMMAudioCD(unsigned long, IString&)") ;
  fAudioCDData->needToDeleteProfile = true;
  fAudioCDData->pScanTimer = new ITimer();
  fAudioCDData->pUpdateTimer = new ITimer();
  fAudioCDData->cursor.setToFirst();
  fAudioCDData->deviceHandler = new IMMAudioCDHandler();
  fAudioCDData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::~IMMAudioCD                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::~IMMAudioCD()
{
  IMODTRACE_DEVELOP("IMMAudioCD::~IMMAudioCD") ;

  // If playing, stop (CD will remain playing otherwise).
  // This may be a neat function, but the option to remain playing
  // or to stop on object delete should be left to the app. Perhaps
  // at som later point we could add a setStopOnDestroy() function.
  //
  // if (IMMDevice::playing == mode())
  //    stop();

  if (fAudioCDData)
  {
     if (fAudioCDData->deviceHandler)
     {
        fAudioCDData->deviceHandler->stopHandlingEventsFor(this);
        delete fAudioCDData->deviceHandler;
     }
     delete fAudioCDData;
  }
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setProfile                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setProfile (IProfile &profile)
{
  IMODTRACE_DEVELOP("IMMAudioCD::setProfile") ;
  if (fAudioCDData->needToDeleteProfile)
  {
     delete fAudioCDData->prof;
     fAudioCDData->needToDeleteProfile = false;
  }
  fAudioCDData->prof = &profile;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::discId                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::discId(CallType call) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::discId") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  MCI_INFO_PARMS parms;
#ifdef PM
  MCI_CD_ID      cdid;
#endif
#ifdef IC_WIN
  const int IMM_UNIQUEIDLENGTH = 16 ;
  char           cdid[IMM_UNIQUEIDLENGTH];
#endif
  unsigned long ulMCIFlags = 0 ;

  memset(&parms, 0, sizeof(parms));
  memset(&cdid, 0, sizeof(cdid));
#ifdef IC_PM
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = PSZ(&cdid);
  parms.ulRetSize    = sizeof(cdid);
  ulMCIFlags         = MCI_CD_INFO_ID | call ;
#endif
#ifdef IC_WIN
  parms.dwCallback  = deviceWindow().handle().asUnsigned() ;
  parms.lpstrReturn = cdid ;
  parms.dwRetSize   = IMM_UNIQUEIDLENGTH ;
  ulMCIFlags        = MCI_INFO_MEDIA_IDENTITY | call ;
#endif

  ((IMMAudioCD*)this)->sendCommand(MCI_INFO,
              ulMCIFlags,
              &parms,
              0);
#ifdef IC_PM
  // TTD why is this code here ?
  if (lastError()== MCIERR_INVALID_BUFFER)
     ((IMMAudioCD*)this)->setLastError(0);
#endif
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

#ifdef IC_PM
  return IString(parms.pszReturn);
#endif
#ifdef IC_WIN
  return IString(parms.lpstrReturn);
#endif

} // discId

/*------------------------------------------------------------------------------
| IMMAudioCD::contents                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents IMMAudioCD::contents() const
{
  IMODTRACE_DEVELOP("IMMAudioCD::contents") ;
  IMMAudioCDContents result;
  if (fAudioCDData->toc)
     result = *(fAudioCDData->toc);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::currentContentsEntry                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCD::currentContentsEntry() const
{
  IMODTRACE_DEVELOP("IMMAudioCD::currentContentsEntry") ;

  if ((fAudioCDData->cursor).isValid())
     return (fAudioCDData->cursor).asIndex() ;
  else
     return 0 ;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::numberOfTracks                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCD::numberOfTracks(CallType call) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::numberOfTracks") ;
  IASSERTSTATE(isMediaPresent()) ;

  unsigned long rc = itemStatus(MCI_STATUS_NUMBER_OF_TRACKS,0, call); // same call for OS/2 and Win32
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::currentTrack                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCD::currentTrack() const
{
  IMODTRACE_DEVELOP("IMMAudioCD::currentTrack") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent()) ;

  return (fAudioCDData->toc)->track(fAudioCDData->cursor) ;

}

/*------------------------------------------------------------------------------
| IMMAudioCD::uniqueDiscIdentifier                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::uniqueDiscIdentifier() const
{
  IMODTRACE_DEVELOP("IMMAudioCD::uniqueDiscIdentifier") ;
  IASSERTSTATE(isMediaPresent()) ;

  IString result;

// TTD should throw exception if no disc, otherwise would return garbage value
// if (isMediaPresent())
   result = IString("IMMAudioCD") + discId(wait);
   return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::upc                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::upc(CallType call) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::upc") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent()) ;      // TTD shouldn't we check for this ?


  MCI_INFO_PARMS parms;
  char    temp[8];
  unsigned long ulMCIFlags = 0 ;
  IString result;


  memset(&parms, 0, sizeof(parms));
  memset(temp, 0, sizeof(temp));

#ifdef IC_PM
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = temp;
  parms.ulRetSize    = sizeof(temp);
  ulMCIFlags = MCI_CD_INFO_UPC | call ;
#endif
#ifdef IC_WIN
  parms.dwCallback   = deviceWindow().handle().asUnsigned() ;
  parms.lpstrReturn  = temp;
  parms.dwRetSize    = sizeof(temp);
  ulMCIFlags         = MCI_INFO_MEDIA_UPC | call ;
#endif

  ((IMMAudioCD*)this)->sendCommand(MCI_INFO,
                                   ulMCIFlags,
                                   &parms, 0);
#ifdef IC_PM
  if (lastError()== MCIERR_INVALID_BUFFER)   // TTD need to find the Win32 equivalent
     ((IMMAudioCD*)this)->setLastError(0);
#endif
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

  return IString(temp) ;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isOpenStringValid                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isOpenStringValid(const IString &deviceName) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::isOpenStringValid") ;
  IString tester(deviceName);
  return (tester.upperCase().indexOf("CDAUDIO") == 1 ? true : false);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::stopT                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::stopT()
{
  IMODTRACE_DEVELOP("IMMAudioCD::stopT") ;
  fAudioCDData->pUpdateTimer->stop();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::playing                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::playing()
{
  IMODTRACE_DEVELOP("IMMAudioCD::playing") ;
  return fAudioCDData->cdPlaying;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::getTableFromDisc                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents IMMAudioCD::getTableFromDisc()
{
  IMODTRACE_DEVELOP("IMMAudioCD::getTableFromDisc") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

#ifdef IC_PM
  MCI_TOC_PARMS         parms;
#endif
#ifdef IC_WIN
  MCI_STATUS_PARMS      parms;
#endif
  IMMAudioCDContents    Toc;
  PTOCREC               ptocrec = 0;
  unsigned long         numTracks = numberOfTracks(wait);

  ptocrec = new (MCI_TOC_REC[numTracks]);

#ifdef IC_PM
  parms.pBuf      = ptocrec;
  parms.ulBufSize = sizeof(MCI_TOC_REC) * numTracks;
  sendCommand(MCI_GETTOC, MCI_WAIT, &parms);
  if (lastError() == MCIERR_INVALID_BUFFER)
  {
     delete ptocrec;
     parms.pBuf = ptocrec = PTOCREC(new (char[parms.ulBufSize]));
     memset(ptocrec, 0, parms.ulBufSize) ;
     sendCommand(MCI_GETTOC,
                 MCI_WAIT,
                 &parms);
  }
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
#endif // IC_PM

#ifdef IC_WIN
   // Win32 does not have an MCI_GETTOC command so we'll fake it.
   // loop through the number of tracks and for each track, get
   // start position and length and from there, calculate the start
   // and end address in the TOC_REC
   // NOTE: Win32 MCI returns the time in milliseconds but, since
   //       IMMTime on Win is defined to be milliseconds, we do not
   //       have to do any conversion

   IMMTime::Format oldTimeFormat = timeFormat(IMMDevice::wait) ;
   if (oldTimeFormat != IMMTime::mmTime)
      setTimeFormat(IMMTime::mmTime, IMMDevice::wait) ;

   // for each track, get the starting position and length,
   // calculate end position and stuff everything into the toc
   unsigned short i ;
   for (i=1; i<=numTracks; i++) {
      memset(&(ptocrec[i-1]), 0, sizeof(MCI_TOC_REC) ) ;
      parms.dwTrack = i ;
      parms.dwItem = MCI_STATUS_POSITION ;
      sendCommand(MCI_STATUS,
                  MCI_STATUS_ITEM | MCI_TRACK | MCI_WAIT,
                  &parms) ;
      if (lastError())
         ITHROWMMERROR(lastError(),"sendCommand");

      ptocrec[i-1].TrackNum = i ;
      ptocrec[i-1].ulStartAddr = parms.dwReturn ;

      parms.dwTrack = i ;
      parms.dwItem = MCI_STATUS_LENGTH ;
      sendCommand(MCI_STATUS,
                  MCI_STATUS_ITEM | MCI_TRACK | MCI_WAIT,
                  &parms) ;
      if (lastError())
         ITHROWMMERROR(lastError(),"sendCommand");

      ptocrec[i-1].ulEndAddr = ptocrec[i-1].ulStartAddr + parms.dwReturn ;

   } // for

   // restore the original time format if we mucked with it beforehand ...
   if (oldTimeFormat != IMMTime::mmTime)
      setTimeFormat(oldTimeFormat, IMMDevice::wait) ;
#endif // IC_WIN

     Toc = IMMAudioCDContents(ptocrec, uniqueDiscIdentifier(), numTracks);
  return Toc;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setTableOfContents                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setTableOfContents(const IMMAudioCDContents &newToc)
{
   IMODTRACE_DEVELOP("IMMAudioCD::setTableOfContents") ;
   if (fAudioCDData->toc)
      delete fAudioCDData->toc;
   IString discId = uniqueDiscIdentifier();
   if (fAudioCDData->prof->containsKeyName(discProgramKey, discId))
   {
      IString temp = fAudioCDData->prof->elementWithKey(discProgramKey, discId);
      fAudioCDData->toc = new IMMAudioCDContents(newToc);
      if (fAudioCDData->toc->fContentsData->trackList)
         delete fAudioCDData->toc->fContentsData->trackList;
      unsigned long * list = (unsigned long *)(unsigned char *)temp;
      unsigned long tr = list[0];
      list++;
      fAudioCDData->toc->fContentsData->trackNum = tr;
      fAudioCDData->toc->fContentsData->trackList = new unsigned long[tr];
      memcpy(fAudioCDData->toc->fContentsData->trackList, list, sizeof(unsigned long) * tr);
   }
   else
      fAudioCDData->toc = new IMMAudioCDContents(newToc);
   IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
   fAudioCDData->cursor = temp;
   IMMTime::Format format = timeFormat(wait);
   setTimeFormat(IMMTime::mmTime, wait);
   seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
   setTimeFormat(format, wait);
   IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
   notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::discTitle                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::discTitle() const
{
  IMODTRACE_DEVELOP("IMMAudioCD::discTitle") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  IString result;
  IString discId(uniqueDiscIdentifier());
  if (fAudioCDData->prof->containsKeyName(discTitleKey, discId))
     result = fAudioCDData->prof->elementWithKey(discTitleKey, discId);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToEntry                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::goToEntry(unsigned long index)
{
  IMODTRACE_DEVELOP("IMMAudioCD::goToEntry(unsigned long)") ;
  fAudioCDData->cursor.setToIndex(index);
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  goToCursor();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToEntry                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::goToEntry(IMMAudioCDContents::Cursor reqCursor)
{
  IMODTRACE_DEVELOP("IMMAudioCD::goToEntry(IMMAudioCDContents::Cursor)") ;
  IASSERTPARM(reqCursor.isValid());
  fAudioCDData->cursor.setToIndex(reqCursor.asIndex());
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  goToCursor();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::pause                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::pause( CallType call)
{
  IMODTRACE_DEVELOP("IMMAudioCD::pause") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  IMMPlayableDevice::pause(call);
  if (!lastError() && call == wait && IMMDevice::paused == mode())
     putInStoppedState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::play                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::play(const IMMTime &from,
                             const IMMTime &to,
                             Boolean resumeIfPaused,
                             CallType call)
{
  IMODTRACE_DEVELOP("IMMAudioCD::play") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  // if wait put in playing state before play call.
  if (call == wait)
     putInPlayingState();

  if (from.isValid() || to.isValid())
     Inherited::play(from, to, resumeIfPaused, call);
  else
  {
     IMMTime::Format format = timeFormat(wait);
#ifdef IC_PM
     setTimeFormat(IMMTime::mmTime, wait);
     IMMTime curPos(position());
#endif
#ifdef IC_WIN
     // TTD
     // There is a bug in MCI_STATUS, POSITION when using milliseconds.
     // It returns a position < actual position which throws us off.
     // for now, I'm faking it by getting the position in TMSF then
     // translating it
     if (format != IMMTime::trackMinSecFrame)
        setTimeFormat(IMMTime::trackMinSecFrame, wait) ;
     IMMTrackMinSecFrameTime curPosTMSF(*(fAudioCDData->toc), position()) ;
     IMMTime delta(( (curPosTMSF.minutes()* 60) + curPosTMSF.seconds() ) * 1000) ;
     IMMTime curPos(fAudioCDData->toc->timeAt(curPosTMSF.track(), true)) ;
     curPos           = curPos + delta ;
     setTimeFormat(IMMTime::mmTime, wait) ;
#endif

     // The NT CD device driver does not return the correct start time
     // for a track.
     // I'm working around that by just starting at the current position

     IMMTime startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
     IMMTime endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     if (curPos >= startTrack && curPos <= endTrack)
        startTrack = curPos;
     else if (curPos < startTrack)
     {
        if (! (IPlatform::isWinNT() | IPlatform::isNTNewShell()) )
           fAudioCDData->cursor.setToFirst();
        startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
        endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     }
     else
     {
        if (! (IPlatform::isWinNT() | IPlatform::isNTNewShell()) )
           fAudioCDData->cursor.setToLast();
        startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
        endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     }

     ITRACE_DEVELOP(IString("starttrack = ") + startTrack.asString());
     Inherited::play(startTrack, endTrack, call);
     setTimeFormat(format, wait);
  }

  // if called for play to complete, put in stopped state
  // else if play started ok, put in playing state
  if (call == wait && IMMDevice::paused == mode())
     putInStoppedState();
  else if (!lastError())
          putInPlayingState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::resume                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::resume(Boolean resume, CallType call)
{
  IMODTRACE_DEVELOP("IMMAudioCD::resume") ;
  if (!(!resume && IMMDevice::paused == mode()))
#ifdef IC_PM
     IMMPlayableDevice::resume(call);
#endif
#ifdef IC_WIN
     //Win32 MCI CD does not support MCI_RESUME, so we simulate.                                                  |
     play(IMMTime(),IMMTime(),false,call);
#endif
  if (call == wait && IMMDevice::playing == mode())
     putInPlayingState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setDiscTitle                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::setDiscTitle(const IString &title)
{
  IMODTRACE_DEVELOP("IMMAudioCD::setDiscTitle") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  fAudioCDData->prof->addOrReplaceElementWithKey(discTitleKey, title,
                                   uniqueDiscIdentifier());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setProgram                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setProgram(const IMMAudioCDContents &prog)
{
  IMODTRACE_DEVELOP("IMMAudioCD::setProgram") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (IMMDevice::playing == mode())
     stop();
  // Need to make sure the user has the right program for the right
  // disc
  IASSERTPARM(uniqueDiscIdentifier() == prog.discId());

  unsigned long num=prog.numberOfEntries();
  IString temp(&num,sizeof(unsigned long),prog.fContentsData->trackList,
               prog.numberOfEntries()*sizeof(unsigned long));
  fAudioCDData->prof->addOrReplaceElementWithKey(discProgramKey, temp,
                                 prog.discId());
  setTableOfContents(prog);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setTrackTitle                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::setTrackTitle(const IString &trackTitle,
                                                    unsigned long track)
{
  IMODTRACE_DEVELOP("IMMAudioCD::setTrackTitle") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  fAudioCDData->prof->addOrReplaceElementWithKey(IString(track), trackTitle,
                                   uniqueDiscIdentifier());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::stop                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::stop(CallType call)
{
  IMODTRACE_DEVELOP("IMMAudioCD::stop") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  putInStoppedState();
  IMMPlayableDevice::stop(call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackBackward                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::trackBackward(unsigned long number)
{
  IMODTRACE_DEVELOP("IMMAudioCD::trackBackward") ;
  if (number)
     moveOneTrack(false, number);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackForward                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::trackForward(unsigned long number)
{
  IMODTRACE_DEVELOP("IMMAudioCD::trackForward") ;
  if (number)
     moveOneTrack(true, number);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackTitle                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::trackTitle(unsigned long track) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::trackTitle") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IString result;

  IString discId(uniqueDiscIdentifier());
  IString trackNum(track);
  if (fAudioCDData->prof->containsKeyName(trackNum, discId))
     result = fAudioCDData->prof->elementWithKey(trackNum, discId);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::processInternal                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::processInternal(const IMMNotifyEvent &evt)
{
  IMODTRACE_DEVELOP("IMMAudioCD::processInternal") ;
  switch (evt.command())
  {
    // just finished an openOnThread or loadOnThread.
    case IMMNotifyEvent::open:
         if (isOpen())
         {
            setTimeFormat(IMMTime::trackMinSecFrame);
            if (isMediaPresent())
               setTableOfContents(getTableFromDisc());
         } /* endif */
         break;
    // if play is finished see where we are and go on to the next
    // track in the program
    case IMMNotifyEvent::play:
         switch (evt.successCode())
         {
           case IMMNotifyEvent::successful:
                {
                  putInStoppedState();
                  Boolean doPlay = false;
                  if ((fAudioCDData->cursor.asIndex() != fAudioCDData->toc->numberOfEntries()) ||
                      (fAudioCDData->conPlay))
                  {
                     // we are not at the end of the CD - just increment to the next track
                     ++(fAudioCDData->cursor);
                     if (fAudioCDData->cursor.isValid())
                     {
                           doPlay = true;
                     }
                     else {
                        // we are at the end of the CD. Go back to the beginning
                        fAudioCDData->cursor = IMMAudioCDContents::Cursor(*(fAudioCDData->toc));
                        if (fAudioCDData->conPlay)
                           doPlay = true;
                     }
                  }
                  IMMTime::Format format = timeFormat(wait);
                  setTimeFormat(IMMTime::mmTime, wait);
                  seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
                  setTimeFormat(format, wait);
                  IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
                  if (doPlay)
                  {
                     setTimeFormat(IMMTime::mmTime, wait);
                     play(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), fAudioCDData->toc->endOfTrack(fAudioCDData->cursor));
                     setTimeFormat(format, wait);
                  }
                  notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
                }
                break;

            // could have done if/else but may need these cases
            // later; leave for now
            case IMMNotifyEvent::preempted:
            case IMMNotifyEvent::aborted:
            case IMMNotifyEvent::error:
            default:
               if (IMMDevice::playing != mode())
                  putInStoppedState();
            }
         break;

      case IMMNotifyEvent::stop:
      case IMMNotifyEvent::pause:
         if (IMMNotifyEvent::successful == evt.successCode())
            putInStoppedState();
         break;

      case IMMNotifyEvent::resume:
         if (IMMNotifyEvent::successful == evt.successCode() &&
             IMMDevice::playing == mode())
            putInPlayingState();
         break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::handleScan                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::handleScan(unsigned long timerId)
{
  IMODTRACE_DEVELOP("IMMAudioCD::handleScan") ;
  Boolean trackChanged = false;

  if (fAudioCDData->scanStatus == scanningForward)
  {
     fAudioCDData->scanTime += IMMTime(3000);
     if (!fAudioCDData->scanTime.isValid())
     {
        fAudioCDData->cursor.setToNext();
        trackChanged = true;
        //if (!fAudioCDData->toc->overValidTrack(fAudioCDData->cursor))
        if (!fAudioCDData->cursor.isValid())
        {
           IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
           fAudioCDData->cursor = temp;
        }
        fAudioCDData->scanTime = IMMTrackMinSecFrameTime(*(fAudioCDData->toc), fAudioCDData->toc->track(fAudioCDData->cursor), 0, 0, 0);
     }
  }
  else if (fAudioCDData->scanStatus == scanningBackward)
       {
          fAudioCDData->scanTime -= IMMTime(3000);
          if (!fAudioCDData->scanTime.isValid())
          {
             fAudioCDData->cursor.setToPrevious();
             trackChanged = true;
             if (!fAudioCDData->cursor.isValid())
             {
                IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
                fAudioCDData->cursor = temp;
                fAudioCDData->cursor.setToLast();
             }
             IMMMinSecFrameTime msf(fAudioCDData->toc->endOfTrack(fAudioCDData->cursor) -
                     fAudioCDData->toc->startOfTrack(fAudioCDData->cursor));
             fAudioCDData->scanTime = IMMTrackMinSecFrameTime(*(fAudioCDData->toc),
                     fAudioCDData->toc->track(fAudioCDData->cursor),
                     msf.minutes(), msf.seconds(), msf.frames());
          }
       }
  if (trackChanged)
     notifyObservers(INotificationEvent(trackStartedId ,*this,true,&fAudioCDData->scanTime));

  notifyObservers(INotificationEvent(positionTimerId,*this,true,&fAudioCDData->scanTime));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToCursor                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::goToCursor()
{
  IMODTRACE_DEVELOP("IMMAudioCD::goToCursor()") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IASSERTSTATE(fAudioCDData->toc);
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  IMMDevice::Mode currentMode = mode(wait);
  if (IMMDevice::playing == currentMode)
     stop(wait);

  IMMTime::Format format = timeFormat(wait);
  setTimeFormat(IMMTime::mmTime, wait);
  seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
  setTimeFormat(format, wait);
  IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
  notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
  if (IMMDevice::playing == currentMode)
  {
     setTimeFormat(IMMTime::mmTime, wait);
     play(IMMTime(), //fAudioCDData->toc->startOfTrack(fAudioCDData->cursor),
     fAudioCDData->toc->endOfTrack(fAudioCDData->cursor));
     setTimeFormat(format, wait);
  }
  return true;
}
/*------------------------------------------------------------------------------
| IMMAudioCD::moveOneTrack                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::moveOneTrack(Boolean forward, unsigned long number)
{
  IMODTRACE_DEVELOP("IMMAudioCD::moveOneTrack") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IASSERTSTATE(fAudioCDData->toc);
  unsigned long loop=number;

  while (loop)
  {
     if (forward)
        ++(fAudioCDData->cursor);
     else
        --(fAudioCDData->cursor);
     if (!fAudioCDData->cursor.isValid())
     {
        IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
        fAudioCDData->cursor = temp;
        if (!forward)
           fAudioCDData->cursor.setToLast();
     }
     loop--;
  } /* endwhile */
  return goToCursor();
}

/*------------------------------------------------------------------------------
| IMMAudioCD::putInPlayingState                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::putInPlayingState()
{
  IMODTRACE_DEVELOP("IMMAudioCD::putInPlayingState") ;
  fAudioCDData->cdPlaying = true;
  if (! fAudioCDData->pUpdateTimer->isStarted())
     fAudioCDData->pUpdateTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleUpdate));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::putInStoppedState                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::putInStoppedState()
{
  IMODTRACE_DEVELOP("IMMAudioCD::putInStoppedState") ;
  fAudioCDData->cdPlaying = false;
  fAudioCDData->pUpdateTimer->stop();
}

/*------------------------------------------------------------------------------
| IMMAudioCD::handleUpdate                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::handleUpdate(unsigned long timerid)
{
  IMODTRACE_DEVELOP("IMMAudioCD::handleUpdate") ;
   IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
   notifyObservers(INotificationEvent(positionTimerId,*this,true,&posTime));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::startScanningForward                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::startScanningForward()
{
  IMODTRACE_DEVELOP("IMMAudioCD::startScanningForward") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  // if not stopped, stop CD
  // set scanTime to the current position
  // set scanStatus variable, start scantimer
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  if (mode() != IMMDevice::stopped)
     stop();

  IMMTime::Format temp = timeFormat(wait);
  setTimeFormat(IMMTime::trackMinSecFrame, wait);
  IMMTrackMinSecFrameTime mmTime(*(fAudioCDData->toc), position(wait));
  fAudioCDData->scanTime =  mmTime;
  setTimeFormat(temp, wait);

  fAudioCDData->scanStatus = scanningForward;
  if (! fAudioCDData->pScanTimer->isStarted())
     fAudioCDData->pScanTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleScan),100);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::startScanningBackward                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::startScanningBackward()
{
  IMODTRACE_DEVELOP("IMMAudioCD::startScanningBackward") ;
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  // if not stopped, stop CD
  // set scanTime to the current position
  // set scanStatus variable, start scantimer
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  if (mode() != IMMDevice::stopped)
     stop();
  IMMTime::Format temp = timeFormat(wait);
  setTimeFormat(IMMTime::trackMinSecFrame, wait);
  IMMTrackMinSecFrameTime mmTime(*(fAudioCDData->toc), position(wait));
  fAudioCDData->scanTime =  mmTime;
  setTimeFormat(temp, wait);
  fAudioCDData->scanStatus = scanningBackward;
  if (! fAudioCDData->pScanTimer->isStarted())
     fAudioCDData->pScanTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleScan),100);
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::isValid                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDContents::Cursor::isValid() const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::isValid") ;
  if ((!fCursorData->valid) || (!fCursorData->table))
     return false;
  if ((fCursorData->index <  fCursorData->table->numberOfEntries()) &&
      (fCursorData->sameValidation == fCursorData->table->fContentsData->needValidation))
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::isValid                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDContents::isValid() const
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::isValid") ;
  if (fContentsData->trackList && fContentsData->valid)
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| Cursor::invalidate                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::invalidate()
{
  IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::invalidate") ;
   fCursorData->valid = false;
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToFirst                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToFirst()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::setToFirst") ;
   fCursorData->index = 0;
   if (fCursorData->table)
   {
      fCursorData->valid = true;
      fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
   } /* endif */
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToIndex                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToIndex(unsigned long toIndex)
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::setToIndex") ;
  if (fCursorData->table)
  {
     if ((toIndex - 1 < fCursorData->table->numberOfEntries()) && toIndex)
     {
        fCursorData->index = toIndex - 1;
        fCursorData->valid = true;
        fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
     }
     else
        invalidate();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToLast                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToLast()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::setToLast") ;
  if (fCursorData->table)
  {
     fCursorData->index = fCursorData->table->numberOfEntries()-1;
     fCursorData->valid = true;
     fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToNext                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToNext()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::setToNext") ;
  if (fCursorData->table)
  {
     if (!(isValid()))
        invalidate();
     else
     {
        if (fCursorData->index+1 >= fCursorData->table->numberOfEntries())
           setToFirst();
        else
        {
           fCursorData->index++;
           fCursorData->valid = true;
           fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
        }
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToPrevious                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToPrevious()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::setToPrevious") ;
  if (fCursorData->table)
  {
     if (!(isValid()))
        invalidate();
     else
     {
        if (fCursorData->index == 0)
           setToLast();
        else
        {
           fCursorData->index--;
           fCursorData->valid = true;
           fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
        }
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::operator ++                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator ++()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::operator++") ;
  return setToNext();
}

/*------------------------------------------------------------------------------
| Cursor::operator --                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator --()
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::operator--") ;
  return setToPrevious();
}


/*------------------------------------------------------------------------------
| Cursor::asIndex                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::Cursor::asIndex() const
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::Cursor::asIndex") ;
  return fCursorData->index + 1;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::discId                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCDContents::discId() const
{
   IMODTRACE_DEVELOP("IMMAudioCDContents::discId") ;
  return IString(fContentsData->discId);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isPlayEnabled                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isAutoPlayEnabled() const
{
   IMODTRACE_DEVELOP("IMMAudioCD::isAutoPlayEnabled") ;
  return fAudioCDData->loadAndGo;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isContinuousPlayEnabled                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isContinuousPlayEnabled() const
{
   IMODTRACE_DEVELOP("IMMAudioCD::isContinuousPlayEnabled") ;
  return fAudioCDData->conPlay;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::enableContinuousPlay                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::enableContinuousPlay(Boolean continuous)
{
   IMODTRACE_DEVELOP("IMMAudioCD::enableContinuousPlay") ;
  fAudioCDData->conPlay = continuous;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::disableContinuousPlay                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::disableContinuousPlay()
{
   IMODTRACE_DEVELOP("IMMAudioCD::disableContinuousPlay") ;
  fAudioCDData->conPlay = false;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::disableAutoPlay                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::disableAutoPlay()
{
   IMODTRACE_DEVELOP("IMMAudioCD::disableAutoPlay") ;
  fAudioCDData->loadAndGo = false;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::enableAutoPlay                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::enableAutoPlay(Boolean autoPl)
{
   IMODTRACE_DEVELOP("IMMAudioCD::enableAutoPlay") ;
  fAudioCDData->loadAndGo = autoPl;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::profile                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IProfile IMMAudioCD::profile() const
{
   IMODTRACE_DEVELOP("IMMAudioCD::profile") ;
  return *(fAudioCDData->prof);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::changeTable                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::changeTable ()
{
  IMODTRACE_DEVELOP("IMMAudioCD::changeTable") ;
  if (fAudioCDData->toc)
     delete fAudioCDData->toc;
  fAudioCDData->toc = 0;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IMMAudioCD::volume                                                           |
| Override of IMMDevice::volume. Necessary for Win32 since volume (and sound   |
| shaping in general) is not consolidated by the MCI.                          |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCD::volume(AudioChannel channel ,CallType call) const
{
  IMODTRACE_DEVELOP("IMMAudioCD::volume");

  unsigned long       ulDetails[2];
  Boolean             uniform = false;  // Assume a specific channel is requested

  if (channel == all)
     uniform = true;       // 1 value returned

  // cast away the const to call mixerControlValues (which is not const)
  IMMAudioCD * const nonConstThis = (IMMAudioCD * const) this;

  if (!nonConstThis->mixerControlValues(
              MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY, // class specific
              MIXERCONTROL_CONTROLTYPE_VOLUME,
              ulDetails, uniform))
     ITHROWMMERROR(lastError(),"mixerControlValues");

  if (channel == right)
     return ( ulDetails[1] );  // Right channel is in second position
  else
     return ( ulDetails[0] );  // Left channel or uniform value in first position
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setVolume                                                         |
| Override of IMMDevice::setVolume. Necessary for Win32 since volume (and sound|
| shaping in general) is not consolidated by the MCI.                          |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setVolume(unsigned long volume,
                                IMMDevice::AudioChannel channel,
                                const IMMMillisecondTime &over,
                                CallType call)
{
  IMODTRACE_DEVELOP("IMMAudioCD::setVolume");

  unsigned long       ulDetails[2];
  Boolean             uniform = false;  // Assume a specific channel is requested

  if (channel == all)
  {
     uniform = true;       // 1 value returned
  }
  else
  {
     // Get current values of both channels, since one won't change
     if (!mixerControlValues(
                 MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY, // class specific
                 MIXERCONTROL_CONTROLTYPE_VOLUME,
                 ulDetails, uniform))
        ITHROWMMERROR(lastError(),"mixerDetails");
  }

  if (channel == right)
     ulDetails[1] = volume;  // Right channel is in second position
  else
     ulDetails[0] = volume;  // Left channel or uniform value in first position

  if (!setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY, // class specific
              MIXERCONTROL_CONTROLTYPE_VOLUME,
              ulDetails, uniform))
     ITHROWMMERROR(lastError(),"mixerDetails");

  return *this;
}
#endif
