/*******************************************************************************
* FILE NAME: immdvhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immdvhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <immdefs.h>
  }

#include <immdvhdr.hpp>
#include <icconst.h>
#include <immdev.hpp>
#include <inotifev.hpp>
#include <immevt.hpp>
#include <immowin.hpp>
#include <itrace.hpp>
#include <immexcpt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMDVHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMDeviceHandler::dispatchHandlerEvent                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::dispatchHandlerEvent(IEvent &evt)
{
  IMODTRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent");

  Boolean result = false;

  // Win MCI does not give us the command name in the event, which
  // kills everything since all the logic depends on knowing what
  // command the event message is for.
  // I'm going to work around that by keeping a list of unacknowledged
  // commands in IMMDevice and re-post a message with the command info.
  if (evt.eventId() == MM_MCINOTIFY &&
      evt.parameter2().highNumber() == 0)
  {
     IMMNotifyEvent notifyEvt(evt) ;
     IMMMessageWindow* pWin = (IMMMessageWindow*) evt.window() ;
     unsigned short usCommand = (notifyEvt.device())->firstUnacknowledgedCommand() ;
     IEventParameter2 param2(evt.parameter2().lowNumber() ,
                             usCommand) ;
     pWin->postEvent(IC_UM_MM_MCI_CMDNOTIFY,
                    evt.parameter1(),
                    param2) ;
     return true ;
  }

  // Do a switch here to see what type of notification is
  // taking place and construct the appropriate event. Then call the
  // virtual function to which the message corresponds.

  switch (evt.eventId())
  {
#ifdef IC_WIN
     case IC_UM_MM_MCI_CMDNOTIFY:
#endif
     case MM_MCINOTIFY:
         {
         ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, received MM_MCINOTIFY");
         IMMNotifyEvent notifyEvent(evt);
         IMMDevice* device = notifyEvent.device();
         ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, notifyEvent.command() = "
                         + IString(notifyEvent.command()));
         ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, notifyEvent.successCode() = "
                         + IString(notifyEvent.successCode()));

         if (notifyEvent.command() == IMMNotifyEvent::open)
         {
            ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, processing IMMNotifyEvent::open");

            if (notifyEvent.successCode() == IMMNotifyEvent::successful)
            {
               device->setOpenStatus(IMMDevice::openSuccessful);
#ifdef IC_WIN
               // emulate the MMPM/2 pass device architecture
               device->setPassDeviceRequested(true);
#endif
            }
            else if (notifyEvent.errorId())
                 {
                    device->setLastError(notifyEvent.errorId());
                    device->setOpenStatus(IMMDevice::openFailed);
                 }
            else
               device->setOpenStatus(IMMDevice::openFailed);
         }
         break;
         }
    case MM_MCIPASSDEVICE:
         {
         ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, received MM_MCIPASSDEVICE");
         IMMPassDeviceEvent passEvent(evt);
         IMMDevice* device = passEvent.device();
         result = passDevice(passEvent);
         break;
         }

#ifdef IC_WIN
    case IC_UM_MM_MCI_POSITIONCHANGE:
#endif
#ifdef IC_PM
    case MM_MCIPOSITIONCHANGE:
#endif
         {
            ITRACE_DEVELOP("IMMDeviceHandler::dispatchHandlerEvent, received MM_MCIPOSITIONCHANGE") ;
         IMMPositionChangeEvent posEvent(evt);
         IMMDevice* device = posEvent.device();
         result = positionChange(posEvent);
         break;
         }

#ifdef IC_PM
    case MM_MCICUEPOINT:
         {
         IMMCuePointEvent cueEvent(evt);
         IMMDevice* device = cueEvent.device();
         result = cuePoint(cueEvent);
         break;
         }

      case MM_MCIEVENT:
         {
         IMMDeviceEvent devEvent(evt);
         IMMDevice* device = devEvent.device();
         result = deviceEvent(devEvent);
         break;
         }
#endif
  }
  return result;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::IMMDeviceHandler                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler::IMMDeviceHandler()
                : IHandler()
{ IMODTRACE_DEVELOP("IMMDeviceHandler::ctor"); }


/*------------------------------------------------------------------------------
| IMMDeviceHandler::~IMMDeviceHandler                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler::~IMMDeviceHandler()
{ IMODTRACE_DEVELOP("IMMDeviceHandler::dtor"); }


/*------------------------------------------------------------------------------
| IMMDeviceHandler::passDevice                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::passDevice(const IMMPassDeviceEvent &evt)
{

  IMODTRACE_DEVELOP("IMMDeviceHandler::passDevice");
  evt.device()->setPassDeviceRequested(true);
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::positionChange                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::positionChange(const IMMPositionChangeEvent &evt)
{
  IMODTRACE_DEVELOP("IMMDeviceHandler::positionChange");
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::cuePoint                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::cuePoint(const IMMCuePointEvent &evt)
{
  IMODTRACE_DEVELOP("IMMDeviceHandler::cuePoint");
  return false;
}


/*------------------------------------------------------------------------------
| IMMDeviceHandler::deviceEvent                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::deviceEvent(const IMMDeviceEvent &evt)
{
  IMODTRACE_DEVELOP("IMMDeviceHandler::deviceEvent");
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::handleEventsFor                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::handleEventsFor       ( IMMDevice* device )
{
   IMODTRACE_DEVELOP("IMMDeviceHandler::handleEventsFor(device)");
   IHandler::handleEventsFor(&(device->deviceWindow()));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::handleEventsFor                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::handleEventsFor       ( IWindow* window )
{
   IMODTRACE_DEVELOP("IMMDeviceHandler::handleEventsFor(window)");
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::stopHandleEventsFor                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::stopHandlingEventsFor ( IMMDevice* device )
{
   IMODTRACE_DEVELOP("IMMDeviceHandler::stopHandlingEventsFor(device)");
   IHandler::stopHandlingEventsFor(&(device->deviceWindow()));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::stopHandleEventsFor                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::stopHandlingEventsFor ( IWindow* window )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}
