/*******************************************************************************
* FILE NAME: immevt.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immevt.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#include <itrace.hpp>
#include <immevt.hpp>
#include <imultsta.hpp>
#include <immtime.hpp>
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMEVT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMMNotifyEvent::IMMNotifyEvent                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::IMMNotifyEvent(const IEvent &evt)
              : IEvent(evt)
{ IMODTRACE_DEVELOP("IMMNotifyEvent::ctor"); }

/*------------------------------------------------------------------------------
| IMMNotifyEvent::successCode                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::SuccessCode IMMNotifyEvent::successCode() const
{
  IMODTRACE_DEVELOP("IMMNotifyEvent::successCode");

  IMMNotifyEvent::SuccessCode rc;

  // Following are codes as they are returned from the subsystem, before
  // macro mapping.
  ITRACE_DEVELOP("IMMNotifyEvent::successCode, low mp1 from MCI = "
                  + IString(parameter1().lowNumber()));
  ITRACE_DEVELOP("IMMNotifyEvent::successCode, high mp1 from MCI = "
                  + IString(parameter1().highNumber()));

  // Mapping macro - convert code to normalized internal form
  IMMSUCCESSCODE(rc,parameter1().lowNumber());

  ITRACE_DEVELOP("IMMNotifyEvent::successCode, value returning = "
                  + IString(rc));

  return rc;
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::userParameter                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMNotifyEvent::userParameter() const
{
  IMODTRACE_DEVELOP("IMMNotifyEvent::userParameter");
  return parameter1().highNumber();
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::command                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::Command IMMNotifyEvent::command() const
{
  IMODTRACE_DEVELOP("IMMNotifyEvent::command");

  ITRACE_DEVELOP("IMMNotifyEvent::command, low mp2 = "
                  + IString(parameter2().lowNumber()));
  ITRACE_DEVELOP("IMMNotifyEvent::command, high mp2 = "
                  + IString(parameter2().highNumber()));

  IMMNotifyEvent::Command mciCommand ;
  IMMCOMMAND(mciCommand, parameter2().highNumber()) ;
  return mciCommand ;
}


/*------------------------------------------------------------------------------
| IMMNotifyEvent::~IMMNotifyEvent                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::~IMMNotifyEvent()
{ IMODTRACE_DEVELOP("IMMNotifyEvent::dtor"); }

/*------------------------------------------------------------------------------
| IMMNotifyEvent::device                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMNotifyEvent::device() const
{
  IMODTRACE_DEVELOP("IMMNotifyEvent::device");

  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::errorId                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMNotifyEvent::errorId() const
{
   IMODTRACE_DEVELOP("IMMNotifyEvent::errorId");
   return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::errorText                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMNotifyEvent::errorText(unsigned long errorCode)
{
  IMODTRACE_DEVELOP("IMMNotifyEvent::errorText");

  IString errorString;
  IString achResult(0,513);
  if (!IMultimediaStatics::getErrorString(errorCode, achResult,
                                          (unsigned short)(achResult.length())))
     errorString = achResult;
  return errorString;
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::~IMMPassDeviceEvent                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPassDeviceEvent::~IMMPassDeviceEvent()
{ IMODTRACE_DEVELOP("IMMPassDeviceEvent::dtor"); }

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::isGainingUse                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPassDeviceEvent::isGainingUse() const
{
  IMODTRACE_DEVELOP("IMMPassDeviceEvent::isGainingUse");
  return ((MCI_GAINING_USE == parameter2().lowNumber()) ? true : false);
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::device                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMPassDeviceEvent::device() const
{
  IMODTRACE_DEVELOP("IMMPassDeviceEvent::device");
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::IMMPassDeviceEvent                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPassDeviceEvent::IMMPassDeviceEvent(const IEvent &evt)
                  : IEvent(evt)
{ IMODTRACE_DEVELOP("IMMPassDeviceEvent::ctor"); }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::IMMPositionChangeEvent                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPositionChangeEvent::IMMPositionChangeEvent(const IEvent &evt)
                      : IEvent(evt)
{ IMODTRACE_DEVELOP("IMMPositionChangeEvent::ctor"); }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::userParameter                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMPositionChangeEvent::userParameter() const
{
  IMODTRACE_DEVELOP("IMMPositionChangeEvent::userParameter");
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::~IMMPositionChangeEvent                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPositionChangeEvent::~IMMPositionChangeEvent()
{ IMODTRACE_DEVELOP("IMMPositionChangeEvent::dtor"); }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::device                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMPositionChangeEvent::device() const
{
  IMODTRACE_DEVELOP("IMMPositionChangeEvent::device");

  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::position                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMPositionChangeEvent::position() const
{
  IMODTRACE_DEVELOP("IMMPositionChangeEvent::position");
  return IMMTime((unsigned long)parameter2());
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::~IMMCuePointEvent                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMCuePointEvent::~IMMCuePointEvent()
{ IMODTRACE_DEVELOP("IMMCuePointEvent::dtor"); }

/*------------------------------------------------------------------------------
| IMMCuePointEvent::IMMCuePointEvent                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMCuePointEvent::IMMCuePointEvent(const IEvent &evt)
                : IEvent(evt)
{ IMODTRACE_DEVELOP("IMMCuePointEvent::ctor"); }

/*------------------------------------------------------------------------------
| IMMCuePointEvent::userParameter                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMCuePointEvent::userParameter() const
{
  IMODTRACE_DEVELOP("IMMCuePointEvent::userParameter");
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::device                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMCuePointEvent::device() const
{
  IMODTRACE_DEVELOP("IMMCuePointEvent::device");

  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::position                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMCuePointEvent::position() const
{
  IMODTRACE_DEVELOP("IMMCuePointEvent::position");
  return IMMTime((unsigned long)parameter2());
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::IMMDeviceEvent                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceEvent::IMMDeviceEvent(const IEvent &evt)
              : IEvent(evt)
{ IMODTRACE_DEVELOP("IMMDeviceEvent::ctor"); }

/*------------------------------------------------------------------------------
| IMMDeviceEvent::eventCode                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMDeviceEvent::eventCode() const
{
{ IMODTRACE_DEVELOP("IMMDeviceEvent::eventCode"); }
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::eventData                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void *IMMDeviceEvent::eventData() const
{
  IMODTRACE_DEVELOP("IMMDeviceEvent::eventData");
  return ((char *)parameter2());
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::~IMMDeviceEvent                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceEvent::~IMMDeviceEvent()
{ IMODTRACE_DEVELOP("IMMDeviceEvent::dtor"); }

/*------------------------------------------------------------------------------
| IMMDeviceEvent::device                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMDeviceEvent::device() const
{
{ IMODTRACE_DEVELOP("IMMDeviceEvent::device"); }

  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}
