/*******************************************************************************
* FILE NAME: immowin.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immowin.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #include <immdefs.h>
  }

#include <immdev.hpp>
#include <iobjwin.hpp>
#include <immowin.hpp>
#include <immexcpt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMOWIN_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMMessageWindow::IMMMessageWindow                                           |
------------------------------------------------------------------------------*/
IMMMessageWindow ::IMMMessageWindow(IMMDevice& device)
                         : IObjectWindow(),
                           relatedDevice(&device)

{
  IMODTRACE_DEVELOP("IMMMessageWindow::IMMMessageWindow") ;
  setParent(objectWindow());
}

/*------------------------------------------------------------------------------
| IMMMessageWindow::~IMMMessageWindow                                          |
------------------------------------------------------------------------------*/
IMMMessageWindow ::~IMMMessageWindow()
{
  IMODTRACE_DEVELOP("IMMMessageWindow::~IMMMessageWindow") ;
}

/*------------------------------------------------------------------------------
| IMMMessageWindow::device                                                     |
------------------------------------------------------------------------------*/
IMMDevice& IMMMessageWindow::device() const
{
  IMODTRACE_DEVELOP("IMMMessageWindow::device") ;
  return *relatedDevice;
}

/*------------------------------------------------------------------------------
| IMMMessageWindow::notificationHandler        **aj**                          |
|                                                                              |
| Returns a pointer to the notification handler                                |
------------------------------------------------------------------------------*/
IWindowNotifyHandler* IMMMessageWindow::notificationHandler    ( ) const
{
  IMODTRACE_DEVELOP("IMMMessageWindow::notificationHandler") ;
  return Inherited::notificationHandler();
}

/*------------------------------------------------------------------------------
| IMMMessageWindow::setNotificationHandler           **aj**                    |
|                                                                              |
| Sets the notification handler to the passed value.                           |
------------------------------------------------------------------------------*/
IMMMessageWindow& IMMMessageWindow::setNotificationHandler (
                                       IWindowNotifyHandler* notifyHandler )
{
  IMODTRACE_DEVELOP("IMMMessageWindow::setNotificationHandler") ;
  IASSERTSTATE(notifyHandler!=0);
  Inherited::setNotificationHandler(notifyHandler);
  return *this;
}

