#ifndef _IMMWAVE_
   #define _IMMWAVE_
/*******************************************************************************
* FILE NAME:  immwav.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IMMWaveAudio                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <immaud.hpp>
#include <istring.hpp>
#include <immabuf.hpp>

#ifdef IC_WIN
class IMMDeviceSettings;
#endif

#pragma pack(4)

#ifdef IC_PMWIN

class IMMWaveAudio : public IMMConfigurableAudio {
typedef IMMConfigurableAudio
  Inherited;
public:
/*----------------------------- Constructors ---------------------------------*/
  IMMWaveAudio  ( Boolean openNow = true,
                  unsigned long instance = 0,
                  Boolean openShareable = true );

virtual
  ~IMMWaveAudio ();

/*-------------------------------- Buffers -----------------------------------*/
#ifdef IC_PM
virtual unsigned long
  cutCopyBufferSize   (const IMMTime& from = IMMTime(),
                       const IMMTime& to   = IMMTime()) const;
virtual IMMWaveAudio
  &copyFromBuffer     (const IMMAudioBuffer& audioBuffer,
                       CallType call = wait),
  &copyToBuffer       (IMMAudioBuffer& audioBuffer,
                       const IMMTime& from = IMMTime(),
                       const IMMTime& to   = IMMTime(),
                       CallType call = wait) const,
  &cutToBuffer        (IMMAudioBuffer& audioBuffer,
                       const IMMTime& from = IMMTime(),
                       const IMMTime& to   = IMMTime(),
                       CallType call = wait),
  &pasteFromBuffer    (const IMMAudioBuffer& audioBuffer,
                       const IMMTime& from = IMMTime(),
                       const IMMTime& to   = IMMTime(),
                       CallType call = wait),
  &pasteToBuffer      (IMMAudioBuffer& audioBuffer,
                       CallType call = wait) const;

virtual Boolean
  supportsWaveFormat  (unsigned long bitsPerSample,
                       unsigned long samplesPerSecond,
                       unsigned long channels,
                       IMMAudioBuffer::Format format,
                       CallType call = wait) const;
#endif

protected:
/*----------------------------- Constructors ---------------------------------*/
  IMMWaveAudio ( unsigned long deviceIdentifier,
                 const IString& newAlias = IString() );

/*-------------------------------- Implementation ----------------------------*/
virtual Boolean
  isOpenStringValid (const IString& deviceName) const;

#ifdef IC_WIN
virtual IMMWaveAudio
  &saveDeviceSettings     (),
  &restoreDeviceSettings  (Boolean newRecordMode = false);
#endif

private:
/*--------------------------------- Private ----------------------------------*/

/*--------------------------------- Hidden Functions--------------------------*/
  IMMWaveAudio          ( const IMMWaveAudio& device );
IMMWaveAudio
 &operator=             ( const IMMWaveAudio& device );

#ifdef IC_WIN
IMMDeviceSettings
  *fDeviceSettings ;
#endif

}; //IMMWaveAudio

#endif /* IC_PMWIN  */

#pragma pack()

#endif /* _IMMWAVE_ */
