#ifndef _IBIDISET_
#define _IBIDISET_
/*******************************************************************************
* FILE NAME: ibidiset.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IBidiSettings                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ivbase.hpp>
#include <ihandle.hpp>

class IWindow;
class IGraphicContext;
class IBidiSettingsData;

#pragma pack(4)

#ifdef IC_PMWIN

class IBidiSettings : public IVBase {
typedef IVBase
  Inherited;

public:
/*------------------------------- Constructors -------------------------------*/
  IBidiSettings ( const IWindow&         window );
  IBidiSettings ( const IWindowHandle    windowHandle );
#ifndef IC_WIN_FLAGNOP
  IBidiSettings ( const IGraphicContext& graphicContext );
  IBidiSettings ( const IPresSpaceHandle presSpace );
#endif
  IBidiSettings ( const IBidiSettings&   settings );
IBidiSettings
 &operator=     ( const IBidiSettings&   settings );

virtual
 ~IBidiSettings ( );

/*-------------------------- Bidirectional Support ---------------------------*/
static Boolean
  isBidiSupported ( );

virtual IBidiSettings
 &apply ( const IWindow&         window,
          Boolean                childInherit = true,
          Boolean                refresh      = true ),
 &apply ( const IWindowHandle    windowHandle,
          Boolean                childInherit = true,
          Boolean                refresh      = true );
#ifndef IC_WIN_FLAGNOP
virtual IBidiSettings
 &apply ( const IGraphicContext& graphicContext ),
 &apply ( const IPresSpaceHandle presSpace );
#endif

/*---------------------------- Application Support ---------------------------*/
static void
  setApplicationDefaults ( const IBidiSettings& applicationSettings );
static IBidiSettings
  applicationDefaults    ( );

/*-------------------------------- Attributes --------------------------------*/
enum BidiTextType {
  visual,
  implicit
  };

enum BidiTextOrientation {
  textLeftToRight,
  textRightToLeft,
  textContextual
  };

enum BidiLayout {
  layoutLeftToRight,
  layoutRightToLeft
  };

enum BidiNumeralType {
  arabic,
  asStored,
  national,
  contextual
  };

enum BidiTextShape {
  displayShaped,
  saveShaped,
  nominalShape,
  initialShape,
  middleShape,
  finalShape,
  isolatedShape
  };

#ifndef IC_WIN_FLAGNOP
virtual IBidiSettings
 &setTextType                   ( BidiTextType        textType );
#endif
virtual IBidiSettings
 &setTextOrientation            ( BidiTextOrientation textOrientation ),
 &setWindowLayout               ( BidiLayout          windowLayout );
#ifndef IC_WIN_FLAGNOP
virtual IBidiSettings
 &setNumeralDisplay             ( BidiNumeralType     numeralDisplay ),
 &enableSymmetricSwapping       ( Boolean             enable = true ),
 &disableSymmetricSwapping      ( ),
 &enableWordByWordReordering    ( Boolean             enable = true ),
 &disableWordByWordReordering   ( ),
 &setTextShape                  ( BidiTextShape       textShape );
#endif

BidiTextType
  textType                      ( ) const;
BidiTextOrientation
  textOrientation               ( ) const;
BidiLayout
  windowLayout                  ( ) const;
BidiNumeralType
  numeralDisplay                ( ) const;
Boolean
  isSymmetricSwappingEnabled    ( ) const;
Boolean
  isWordByWordReorderingEnabled ( ) const;
BidiTextShape
  textShape                     ( ) const;

/*-------------------------------- Comparisons -------------------------------*/
Boolean
  operator== ( const IBidiSettings& settings ) const;

protected:
/*------------------------------- Constructors -------------------------------*/
  IBidiSettings ( );

private:
/*--------------------------------- Private ----------------------------------*/
#ifndef IC_PM
static IBidiSettings
  fgApplicationDefaults;
#endif

void
  initialize         ( const IPresSpaceHandle& presSpace ),
  initialize         ( const IWindowHandle&    window );
void
  applyToWindow      ( IWindowHandle&          windowHandle,
                       Boolean                 childInherit,
                       Boolean                 refresh ) const,
  applyToFrameWindow ( IWindow&                frameWindow,
                       Boolean                 childInherit,
                       Boolean                 refresh ) const;
IBidiSettingsData
 *fBidiSettingsData;
}; // IBidiSettings

#endif // IC_PMWIN

#pragma pack()

#endif /* _IBIDISET_ */
