/*******************************************************************************
* FILE NAME: icnrctlb.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrnhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSTDCNR
extern "C" {
   #include <iwindefs.h>
}

#include <iwindow.hpp>
#include <icnrctl.hpp>
#include <icnrnhdr.hpp>
#include <ievent.hpp>
#include <ictlevt.hpp>
#include <iwcname.hpp>
#include <icnrevt.hpp>
#include <inotifev.hpp>
#ifdef IC_WIN
#include <iclcnrw.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTLB_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IContainerControlNotifyStatics
{
public:

  ~IContainerControlNotifyStatics ()
  {
    delete containerControlNotifyHandler;
    containerControlNotifyHandler = 0;
  }

static IContainerControlNotifyHandler
  &notificationHandler ( );

static IContainerControlNotifyHandler
  *containerControlNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IContainerControlNotifyStatics containerControlStatics;
INotificationId const IContainerControl::addId
                                      = "IContainerControl::add";
INotificationId const IContainerControl::removeId
                                      = "IContainerControl::remove";
INotificationId const IContainerControl::enterId
                                      = "IContainerControl::enter";
INotificationId const IContainerControl::selectId
                                      = "IContainerControl::select";
INotificationId const IContainerControl::titleId
                                      = "IContainerControl::title";
INotificationId const IContainerControl::titleVisibleId
                                      = "IContainerControl::titleVisible";
INotificationId const IContainerControl::detailsViewTitlesId
                                      = "IContainerControl::detailsViewTitles";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IContainerControlNotifyHandler*
  IContainerControlNotifyStatics::containerControlNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IContainerControlNotifyStatics::notificationHandler                          |
|                                                                              |
| Static function to create and return a notification handler for              |
| IContainerControl                                                            |
------------------------------------------------------------------------------*/
IContainerControlNotifyHandler&
                 IContainerControlNotifyStatics::notificationHandler ()
{
  if (!(IContainerControlNotifyStatics::containerControlNotifyHandler))
  {
    IContainerControlNotifyStatics::containerControlNotifyHandler =
                                   new IContainerControlNotifyHandler();
  }
  return(*IContainerControlNotifyStatics::containerControlNotifyHandler);
}

/*------------------------------------------------------------------------------
| IContainerControl::enableNotification                                        |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IContainerControl, does not send notifications to observer objects until it  |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IContainerControlNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::IContainerControlNotifyHandler               |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerControlNotifyHandler::IContainerControlNotifyHandler( )
  : IContainerControlNotifyHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::~IContainerControlNotifyHandler              |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerControlNotifyHandler::~IContainerControlNotifyHandler( )
{ }

/*------------------------------------------------------------------------------
| IContainerControlNotifyHandler::dispatchHandlerEvent                         |
|                                                                              |
| This notifies IContainerControl Windows of the following events:             |
|                                                                              |
| title(Id)                                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean IContainerControlNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  switch (anEvent.eventId())
  {
    case CM_SETCNRINFO:

      if (anEvent.parameter2() == CMA_CNRTITLE)
      {
        if (thisWindow)
        {
          /************************************************************/
          /* Post private message to delay notification until after   */
          /* the selection state has been modified                    */
          /************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_CNR_TITLE ));
        }
      }
      break;

    /****************************************************************/
    /* This is the delayed notification message.  Check to see which*/
    /* notification should be sent to the observers based on info   */
    /* stored in parameter 2 of the private message.  Parameter 1   */
    /* contains the information passed in the original event that   */
    /* needs to be sent with the notification                       */
    /****************************************************************/
    case IC_UM_DELAYNOTIFY:

      if (thisWindow && (anEvent.parameter2() == IC_NOTDEL_CNR_TITLE))
      {
        CNRINFO *pcnrInfo = (CNRINFO*)(anEvent.parameter1().asUnsignedLong());
        thisWindow->notifyObservers(INotificationEvent(
                                    IContainerControl::titleId,
                                    *thisWindow, true,
                                    (void*)pcnrInfo->pszCnrTitle));
      }
      break;

    case WM_CONTROL:
    {                              // Allocate class name string.
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
        if (className == WC_CONTAINER || className == WC_NATIVECONTAINER)
        {
          if (ulMsg == CN_ENTER)
          {
            ICnrEnterEvent enterEvt(ctlevt);
            thisWindow->notifyObservers(INotificationEvent(
                                        IContainerControl::enterId,
                                        *thisWindow, true,
                                        (void*)&enterEvt));
          }
          else if (ulMsg == CN_EMPHASIS)
          {
            ICnrEmphasisEvent emphasisEvt(ctlevt);
            if (emphasisEvt.changed(IContainerObject::selected))
            {
              thisWindow->notifyObservers(INotificationEvent(
                                          IContainerControl::selectId,
                                          *thisWindow, true,
                                          (void*)&emphasisEvt));
            }
          }
        }
      }
      break;
    }

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

#ifdef IC_MOTIF

IBase::Boolean IContainerControlNotifyHandler::dispatchHandlerEvent(IEvent& anEvent)
{
   // IC_NOTYET
   // This is a stub for Motif needed for vft.
   return false;
}

#endif // IC_MOTIF
