/*******************************************************************************
* FILE NAME: icnrdiev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrdiev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <stdio.h>
  #define INCL_WINSTDCNR
  #define INCL_WINWINDOWMGR
  #define INCL_WINMENUS
  }

#include <icnrrec.hpp>  // Must be first for OS flags

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#include <icnrdiev.hpp>
#include <irect.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRDIEV_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::~ICnrDrawItemEvent                                        |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent  :: ~ICnrDrawItemEvent( ) {}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::~ICnrDrawBackgroundEvent                            |
------------------------------------------------------------------------------*/
ICnrDrawBackgroundEvent :: ~ICnrDrawBackgroundEvent( ) {}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::ICnrDrawItemEvent                                         |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent ::  ICnrDrawItemEvent(const IEvent& evt)
    : IDrawItemEvent(evt) {;}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::ICnrDrawBackgroundEvent                             |
------------------------------------------------------------------------------*/
ICnrDrawBackgroundEvent :: ICnrDrawBackgroundEvent(const IEvent& evt)
    : IEvent(evt) {;}


/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::container                                                 |
|                                                                              |
| Returns the container being drawn.                                           |
------------------------------------------------------------------------------*/
IContainerControl* ICnrDrawItemEvent :: container  ( ) const
{
   HWND hwndCtl =  IWindow::handleWithParent(parameter1(), handle());
   IContainerControl* pcnrctl =
                    (IContainerControl*)IWindow::windowWithHandle(
                                                     IWindowHandle(hwndCtl));
   return pcnrctl;
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::object                                                    |
|                                                                              |
| Returns the object in the container being drawn.                             |
| If a column title is being drawn in detail view, 0 is returned.              |
------------------------------------------------------------------------------*/
IContainerObject* ICnrDrawItemEvent :: object  ( ) const
{
#ifdef IC_WIN
   LPCNRDRAWITEMINFO pditeminfo = (LPCNRDRAWITEMINFO)((LPDRAWITEMSTRUCT)
                                            (void*)parameter2())->itemData;
#else  // !IC_WIN
   PCNRDRAWITEMINFO pditeminfo = (PCNRDRAWITEMINFO)((POWNERITEM)
                                                   (void*)parameter2())->hItem;
#endif // !IC_WIN

  IMiniCnrRecord* pcnrrec =
    ( pditeminfo == NULL ? NULL : (IMiniCnrRecord*)(pditeminfo->pRecord));

   return (IObjFromRec(pcnrrec));
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::column                                                    |
|                                                                              |
| If an object being drawn is in the details view, the column is returned.     |
| If the column title is being drawn in details view, the column is returned.  |
| If the view is other than a details view, 0 is returned.                     |
------------------------------------------------------------------------------*/
IContainerColumn* ICnrDrawItemEvent :: column ( ) const
{
  IContainerColumn* pcnrcol=0;

#ifdef IC_WIN
   LPCNRDRAWITEMINFO pditeminfo = (LPCNRDRAWITEMINFO)((LPDRAWITEMSTRUCT)
                                            (void*)parameter2())->itemData;
#else  // !IC_WIN
  PCNRDRAWITEMINFO pditeminfo =
                   (PCNRDRAWITEMINFO)((POWNERITEM)(void*)parameter2())->hItem;
#endif // !IC_WIN

  if ( pditeminfo && pditeminfo->pFieldInfo )
    pcnrcol = (IContainerColumn*)(pditeminfo->pFieldInfo->pUserData);

  return (pcnrcol);
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemType                                                  |
|                                                                              |
| Returns the type of the item being drawn.                                    |
------------------------------------------------------------------------------*/
ICnrDrawItemEvent::ItemType ICnrDrawItemEvent :: itemType () const
{
#ifdef IC_WIN
   LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
   long itemId = pitem->itemID;
#else  // !IC_WIN
   POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
   long itemId = pitem->idItem;
#endif // !IC_WIN

   ItemType type;

   switch(itemId)
   {
     case CMA_TEXT :
     {
        type = text;
        break;
     }
     case CMA_ICON :
     {
        type = icon;
        break;
     }
     case CMA_TREEICON :
     {
        type = treeIcon;
        break;
     }
     case CMA_CNRTITLE :
     {
        type = title;
        break;
     }

     default :
     {
        type = detailsView;
        break;
     }
   }
   return type;
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemPresSpaceHandle                                       |
|                                                                              |
| Returns the items's presentation space handle.                               |
------------------------------------------------------------------------------*/
IPresSpaceHandle ICnrDrawItemEvent::itemPresSpaceHandle() const
{
  if ( (void*)parameter2() )
  {
#ifdef IC_WIN
    LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
    return IPresSpaceHandle(pitem->hDC);
#else  // !IC_WIN
    POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
    return IPresSpaceHandle(pitem->hps);
#endif // !IC_WIN
  }
  return 0;
}

/*------------------------------------------------------------------------------
| ICnrDrawItemEvent::itemRect                                                  |
|                                                                              |
| Returns the rectangle of the item.                                           |
------------------------------------------------------------------------------*/
IRectangle ICnrDrawItemEvent:: itemRect() const
{
  if ( (void*)parameter2() )
  {
#ifdef IC_WIN
    LPDRAWITEMSTRUCT pitem = (LPDRAWITEMSTRUCT)(void*)parameter2();
    return IRectangle(pitem->rcItem);
#else  // !IC_WIN
    POWNERITEM pitem = (POWNERITEM)(void*)parameter2();
    return IRectangle(pitem->rclItem);
#endif // !IC_WIN
  }
  return IRectangle();
}


/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::container                                           |
|                                                                              |
| Returns the container being drawn.                                           |
------------------------------------------------------------------------------*/
IContainerControl* ICnrDrawBackgroundEvent :: container  ( ) const
{
   IContainerControl* pcnrctl =
        (IContainerControl*)IWindow::windowWithHandle(IWindowHandle(handle()));
   return pcnrctl;
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemPresSpaceHandle                                 |
|                                                                              |
| Returns the item's presentation space handle.                                |
------------------------------------------------------------------------------*/
IPresSpaceHandle ICnrDrawBackgroundEvent::itemPresSpaceHandle() const
{
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return IPresSpaceHandle(pback->hps);
  }
  return 0;
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemId                                              |
|                                                                              |
| Returns the item's id.                                                       |
------------------------------------------------------------------------------*/
unsigned long ICnrDrawBackgroundEvent::itemId() const
{
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return (unsigned long) (pback->idWindow);
  }
  return 0;
}

/*------------------------------------------------------------------------------
| ICnrDrawBackgroundEvent::itemRect                                            |
|                                                                              |
| Returns the rectangle of the item.                                           |
------------------------------------------------------------------------------*/
IRectangle ICnrDrawBackgroundEvent:: itemRect() const
{
  if ( (void*)parameter1() )
  {
    POWNERBACKGROUND pback = (POWNERBACKGROUND)(void*)parameter1();
    return IRectangle(pback->rclBackground);
  }
  return IRectangle();
}
