/*******************************************************************************
* FILE NAME: icritsec.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     ICritSec                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #define INCL_DOSPROCESS
  #include <iwindefs.h>
}

#include <icritsec.hpp>

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICRITSEC_CPP_
  #include <ipagetun.h>
#endif

// Define static members.
unsigned long
  ICritSec :: fgCount = 0;
#ifdef IC_WIN

#pragma data_seg(ICLNonConst)

/*------------------------------------------------------------------------------
| Static objects to ensure default handlers get deleted at shutdown.           |
------------------------------------------------------------------------------*/
static struct ICritSec_Static {
static CRITICAL_SECTION
 *pCriticalSection;

  ~ICritSec_Static ( );                      // 7810
} statics;

ICritSec_Static::~ICritSec_Static ( )        // 7810
{
  IMODTRACE_DEVELOP( "ICritSec_Static dtor" );
//  Deleting this causes race conditions at process shutdown.  We
//  let the critical section be cleaned up by the system.
//    DeleteCriticalSection(pCriticalSection);
//    delete pCriticalSection;
//    pCriticalSection = 0;
}

CRITICAL_SECTION
 *ICritSec_Static::pCriticalSection = 0;

#pragma data_seg()
#endif


/*------------------------------------------------------------------------------
| ICritSec::ICritSec                                                           |
|                                                                              |
| Issue DosEnterCritSec and increment count.                                   |
------------------------------------------------------------------------------*/
ICritSec :: ICritSec ( )
                    : fCritSecData( 0 )
{
#ifdef IC_WIN
  if ( !statics.pCriticalSection )
  {
     statics.pCriticalSection= new CRITICAL_SECTION;
     InitializeCriticalSection(statics.pCriticalSection);
  }
  EnterCriticalSection(statics.pCriticalSection);
#endif
#ifdef IC_PM
  DosEnterCritSec();
#endif
  ICritSec::fgCount++;
}

/*------------------------------------------------------------------------------
| ICritSec::~ICritSec                                                          |
|                                                                              |
| Issue DosExitCritSec and decrement count.                                    |
------------------------------------------------------------------------------*/
ICritSec :: ~ICritSec ( )
{
#ifdef IC_WIN
  LeaveCriticalSection(statics.pCriticalSection);
#endif
#ifdef IC_PM
  DosExitCritSec();
#endif
  ICritSec::fgCount--;
}

/*------------------------------------------------------------------------------
| ICritSec::count                                                              |
|                                                                              |
| Retrieve the critical section inuse count                                    |
------------------------------------------------------------------------------*/
unsigned long ICritSec::count( )
{
  return fgCount;
}

