/*******************************************************************************
* FILE NAME: icslider.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icslider.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #define INCL_WINMESSAGEMGR
   #define INCL_WINWINDOWMGR
   #define INCL_WINDIALOGS
   #define INCL_WINBUTTONS
   #define INCL_OS2MM
   #define INCL_SW
   #define INCL_WINCIRCULARSLIDER
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#include <ireslib.hpp>
#include <iwcname.hpp>
#include <istring.hpp>
#include <ievent.hpp>
#include <ihandler.hpp>
#include <icolor.hpp>
#include <ibrshmgr.hpp>    // 25227

#ifdef IC_PM
#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif
#endif

#include <ipoint.hpp>
#include <immexcpt.hpp>

#ifdef IC_WIN
   #include <icctlsta.hpp>         // statics to load dll
   #include <iclcsldw.h>
#endif
#include <icslider.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICSLIDER_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const ICircularSlider::Style
  ICircularSlider::buttons           ( 0, ICSS_BUTTONS ),
  ICircularSlider::label             ( 0, ICSS_LABEL   ),
  ICircularSlider::displayValue      ( 0, ICSS_VALUE   ),
  ICircularSlider::jumpToPointer     = CSS_POINTSELECT,
  ICircularSlider::full360           = CSS_360,
  ICircularSlider::midpoint          = CSS_MIDPOINT,
  ICircularSlider::proportionalTicks = CSS_PROPORTIONALTICKS,
  ICircularSlider::noTicks           = CSS_NOTICKS,
  ICircularSlider::circularArm       = CSS_CIRCULARVALUE,
  ICircularSlider::classDefaultStyle ( WS_VISIBLE,
                                       ICSS_BUTTONS | ICSS_LABEL | ICSS_VALUE);
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ICircularSlider::Style ICircularSlider::currentDefaultStyle ( WS_VISIBLE,
                                    ICSS_BUTTONS | ICSS_LABEL | ICSS_VALUE);
#ifdef IC_PM
Boolean ICircularSlider::fRegistered = false;
#endif
#pragma data_seg()

#ifdef IC_PM
class ICircularSliderData
{
public:

IBitmapHandle rightUp;
IBitmapHandle rightDown;
IBitmapHandle leftUp;
IBitmapHandle leftDown;
};
#endif
#ifdef IC_WIN
class ICircularSliderData : public IBase {
/*------------------------------------------------------------------------------
| The ICircularSliderData class encapsulates private data and functions            |
| used by the IEntryField class.  An object of this class is created in the    |
| IEntryField constructors.  This is the os2/windows version.                  |
------------------------------------------------------------------------------*/
public:
  ICircularSliderData ( );

virtual
 ~ICircularSliderData ( );

HBRUSH
  bkgrndbrush;
IBitmapHandle
  rightUp;
IBitmapHandle
  rightDown;
IBitmapHandle
  leftUp;
IBitmapHandle
  leftDown;

}; // ICircularSliderData

/*------------------------------------------------------------------------------
| ICircularSliderData::ICircularSliderData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSliderData::ICircularSliderData ( )
  : bkgrndbrush( 0 )
{ }

/*------------------------------------------------------------------------------
| ICircularSliderData::~ICircularSliderData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSliderData::~ICircularSliderData ( )
{
  if ( bkgrndbrush != 0 )
     IBrushManager::freeBrush( bkgrndbrush );    // 25227
}

class ICSliderHandler : public IHandler {
typedef IHandler
  Inherited;

public:
/*------------------------------- Constructors -------------------------------*/
  ICSliderHandler ( );
virtual
 ~ICSliderHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

}; // class ICSliderHandler

/*------------------------------------------------------------------------------
| ICSliderHandler::ICSliderHandler                                                 |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
ICSliderHandler::ICSliderHandler()
{ }

/*------------------------------------------------------------------------------
| ICSliderHandler::~ICSliderHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ICSliderHandler::~ICSliderHandler()
{ }

/*------------------------------------------------------------------------------
| ICSliderHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Intercepts enter or escape key event and sends it to the owner.              |
| If there is no owner the event is sent to the parent.                        |
| Needed to emulate single line control but still get alignment                |
| characteristics.                                                             |
| Also achieves dialog behavior in canvases and frames without                 |
| use of IsDialogMessage in the event loop.                                    |
|                                                                              |
| Intercepts control color events                                              |
|                                                                              |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean ICSliderHandler::dispatchHandlerEvent( IEvent& event )
{
  IMODTRACE_ALL("ICSliderHandler::dispatchHandlerEvent");
  Boolean
    result = false;

  switch( event.eventId() )
  {
    case WM_CTLCOLORCSLIDER:
    {
      ICircularSlider* ctlwin = (ICircularSlider*)
           IWindow::windowWithHandle(IWindowHandle(event.parameter2()) );
      IColor clrFgnd = ctlwin->foregroundColor();
      IColor clrBgnd = ctlwin->backgroundColor();

      // Set foreground and background text colors in device context
      SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
      SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

      // If a previous background brush set, delete it
      if ( ctlwin->fCircularSliderData->bkgrndbrush != 0 )  // 25227
         IBrushManager::freeBrush( ctlwin->fCircularSliderData->bkgrndbrush );
      // Create a new background brush using the color retrieved
      ctlwin->fCircularSliderData->bkgrndbrush =
         IBrushManager::useBrush( clrBgnd.asRGBLong() );

      // Return HBRUSH for painting controls background
      event.setResult( ctlwin->fCircularSliderData->bkgrndbrush );
      result = true;
      break;
    }

    default:
      break;
  }
  return result;
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler
{
  operator ICSliderHandler* ();
ICSliderHandler
 *operator -> ()
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
ICSliderHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator ICSliderHandler* ( )
  {
  if ( !ptr )
    ptr = new ICSliderHandler;
  return ptr;
  }
#endif // IC_WIN


/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(unsigned long Id,
                                 IWindow* parent,
                                 IWindow* owner,
                                 const IRectangle& initial,
                                 const Style& style)
               : fData(0),
                 fCircularSliderData( new ICircularSliderData )
{
   IMODTRACE_DEVELOP("ICircularSlider::ctor1");
   IASSERTPARM(parent!=0);     //parent must be valid

   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PM
   if (!fRegistered)
   {
      fRegistered = true;
      IMultimediaStatics::registerCircularSlider();
   }
#endif

#ifdef IC_WIN
   IControlStatics::loadControlDLL();
#endif

   IWindowHandle handle(create(Id,
                               (PSZ)"",
                               convertToGUIStyle( style ),
                               WC_CIRCULARSLIDER,
                               parent->handle(),
                               owner->handle(),
                               initial,
                               PVOID(0),
                               PVOID(0)));

#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
   IWindow::startHandlingEventsFor(handle);
}

/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(unsigned long Id,
                                 IWindow *parentDialog)
               : fData(0),
                 fCircularSliderData( new ICircularSliderData )
{
  IMODTRACE_DEVELOP("ICircularSlider::ctor2");
  setAutoDestroyWindow(false);
#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(Id, parentDialog);
}

/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(const IWindowHandle& handle)
               : fData(0),
                 fCircularSliderData( new ICircularSliderData )
{
  IMODTRACE_DEVELOP("ICircularSlider::ctor1");
  setAutoDestroyWindow(false);
#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(handle);
}

/*------------------------------------------------------------------------------
| ICircularSlider::~ICircularSlider                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::~ICircularSlider()
{
  IMODTRACE_DEVELOP("ICircularSlider::dtor");
#ifdef IC_WIN
  removeHandler(defaultHandler);
#endif
  if (fData)
     delete fData;
  if (fCircularSliderData)
     delete fCircularSliderData;
}

/*------------------------------------------------------------------------------
| ICircularSlider::defaultStyle                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::Style ICircularSlider::defaultStyle()
{
   IMODTRACE_DEVELOP("ICircularSlider::defaultStyle");
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDefaultStyle                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void ICircularSlider::setDefaultStyle(const Style& style)
{
   IMODTRACE_DEVELOP("ICircularSlider::setDefaultStyle");
   currentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| ICircularSlider::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ICircularSlider :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   Boolean bExtOnly) const
{
  IMODTRACE_DEVELOP("ICircularSlider::convertToGUIStyle");

  // Obtain the style from the class (ITextControl) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // Let the clean CSS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & ICSS_MASK;

    // add notext style if labels are not desired
    if (!(extendedStyle() & label.asExtendedUnsignedLong()))
       ulStyle |= CSS_NOTEXT;

    // add nobuttons style if buttons are not desired
    if (!(extendedStyle() & buttons.asExtendedUnsignedLong()))
       ulStyle |= CSS_NOBUTTON;

    // add nonumber style if displaying the value is not desired
    if (!(extendedStyle() & displayValue.asExtendedUnsignedLong()))
       ulStyle |= CSS_NONUMBER;
  }

#ifdef IC_WIN
  ulStyle |= WS_CHILD;
#endif

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| ICircularSlider::armRange                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRange ICircularSlider::armRange() const
{
  short low  = 0;
  short high = 0;

  IMODTRACE_DEVELOP("ICircularSlider::armRange");

  IEventResult evt(handle().sendEvent(CSM_QUERYRANGE, &low, &high));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYRANGE");
  return IRange(low, high);
}

/*------------------------------------------------------------------------------
| ICircularSlider::value                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
long ICircularSlider::value() const
{
  short value = 0;

  IMODTRACE_DEVELOP("ICircularSlider::value");

  IEventResult evt(handle().sendEvent(CSM_QUERYVALUE, &value));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYVALUE");
  return value;
}

/*------------------------------------------------------------------------------
| ICircularSlider::radius                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::radius() const
{
  unsigned short rad = 0;

  IMODTRACE_DEVELOP("ICircularSlider::radius");

  IEventResult evt(handle().sendEvent(CSM_QUERYRADIUS, &rad));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYRADIUS");
  return rad;
}

/*------------------------------------------------------------------------------
| ICircularSlider::rotationIncrement                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::rotationIncrement() const
{
  unsigned short buttonInc = 0;
  unsigned short tickInc   = 0;

  IMODTRACE_DEVELOP("ICircularSlider::rotationIncrement");

  IEventResult evt(handle().sendEvent(CSM_QUERYINCREMENT,
                                      &buttonInc, &tickInc));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYINCREMENT");
  return buttonInc;
}

/*------------------------------------------------------------------------------
| ICircularSlider::tickSpacing                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::tickSpacing() const
{
  unsigned short buttonInc = 0;
  unsigned short tickInc   = 0;

  IMODTRACE_DEVELOP("ICircularSlider::tickSpacing");

  IEventResult evt(handle().sendEvent(CSM_QUERYINCREMENT,
                                      &buttonInc, &tickInc));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYINCREMENT");
  return tickInc;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setArmRange                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setArmRange(const IRange& rng)
{
  IMODTRACE_DEVELOP("ICircularSlider::setArmRange");

  IEventResult evt(handle().sendEvent(CSM_SETRANGE,
                   (unsigned long)rng.lowerBound(),
                   (unsigned long)rng.upperBound()));

  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETRANGE");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setValue                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setValue(long value)
{
  IMODTRACE_DEVELOP("ICircularSlider::setValue");

  IEventResult evt(handle().sendEvent(CSM_SETVALUE,(int)value,(unsigned long)0));

  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETVALUE");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setRotationIncrement                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setRotationIncrement(unsigned long scroll)
{
  IMODTRACE_DEVELOP("ICircularSlider::setRotationIncrement");

  IEventResult evt(handle().sendEvent(CSM_SETINCREMENT,
                   (unsigned long)scroll,
                   (unsigned long)tickSpacing()));

  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETINCREMENT");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setTickSpacing                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setTickSpacing(unsigned long tick)
{
  IMODTRACE_DEVELOP("ICircularSlider::setTickSpacing");

  IEventResult evt(handle().sendEvent(CSM_SETINCREMENT,
                   (unsigned long)rotationIncrement(),
                   (unsigned long)tick));

  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETINCREMENT");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDecrementBitmaps                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setDecrementBitmaps(const IResourceId& leftUp,
                                                      const IResourceId& leftDown)
{
  IMODTRACE_DEVELOP("ICircularSlider::setDecrementRange");

  return setDecrementBitmaps(leftUp.resourceLibrary().loadBitmap(leftUp.id()),
                             leftDown.resourceLibrary().loadBitmap(leftDown.id()) );
}

/*------------------------------------------------------------------------------
| ICircularSlider::setIncrementBitmaps                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setIncrementBitmaps(const IResourceId& rightUp,
                                                      const IResourceId& rightDown)
{
  IMODTRACE_DEVELOP("ICircularSlider::setIncrementBitmaps");

  return setIncrementBitmaps(rightUp.resourceLibrary().loadBitmap(rightUp.id()),
                             rightDown.resourceLibrary().loadBitmap(rightDown.id()) );
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDecrementBitmaps                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setDecrementBitmaps(const IBitmapHandle& leftUp,
                                                      const IBitmapHandle& leftDown)
{
  IMODTRACE_DEVELOP("ICircularSlider::setDecrementBitmaps");

  if (!fCircularSliderData)
    fCircularSliderData = new ICircularSliderData;

  // Save away the bitmap handles so if they are reference
  // counted, they will not be deleted.
  fCircularSliderData->leftUp   = leftUp;
  fCircularSliderData->leftDown = leftDown;

  CSBITMAPDATA csBitmapData;

  csBitmapData.hbmLeftUp    = fCircularSliderData->leftUp;
  csBitmapData.hbmLeftDown  = fCircularSliderData->leftDown;
  csBitmapData.hbmRightUp   = fCircularSliderData->rightUp;
  csBitmapData.hbmRightDown = fCircularSliderData->rightDown;

  IEventResult evt(handle().sendEvent(CSM_SETBITMAPDATA,
                   PVOID(&csBitmapData),(unsigned long)0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETBITMAPNDATA");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setIncrementBitmaps                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setIncrementBitmaps(const IBitmapHandle& rightUp,
                                                      const IBitmapHandle& rightDown)
{
  IMODTRACE_DEVELOP("ICircularSlider::setIncrementBitmaps2");

  if (!fCircularSliderData)
    fCircularSliderData = new ICircularSliderData;

  // Save away the bitmap handles so if they are reference
  // counted, they will not be deleted.
  fCircularSliderData->rightUp   = rightUp;
  fCircularSliderData->rightDown = rightDown;

  CSBITMAPDATA csBitmapData;

  csBitmapData.hbmLeftUp    = fCircularSliderData->leftUp;
  csBitmapData.hbmLeftDown  = fCircularSliderData->leftDown;
  csBitmapData.hbmRightUp   = fCircularSliderData->rightUp;
  csBitmapData.hbmRightDown = fCircularSliderData->rightDown;

  IEventResult evt(handle().sendEvent(CSM_SETBITMAPDATA,
                   PVOID(&csBitmapData),(unsigned long)0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETBITMAPNDATA");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::calcMinimumSize                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISize ICircularSlider::calcMinimumSize() const
{
  IMODTRACE_DEVELOP("ICircularSlider::calcMinimumSize");

  ISize i(100,70);
  return i;
}
