/*******************************************************************************
* FILE NAME: icustbhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINMESSAGEMGR
  #define INCL_WINBUTTONS
  #define INCL_WININPUT
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <icustbut.hpp>
#include <icustbhd.hpp>
#include <icustbev.hpp>
#include <icolor.hpp>
#include <icoordsy.hpp>
#include <igrafctx.hpp>
#include <igline.hpp>
#include <igrect.hpp>
#include <igstring.hpp>
#include <ifont.hpp>
#include <istring.hpp>
#include <igbundle.hpp>
#include <igregion.hpp>
#include <igpyline.hpp>
#include <itrace.hpp>
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBHD_CPP_
  #include <ipagetun.h>
#endif


#define DRAW_STANDARD_BUTTON  1  // undefine to get Win95 toolbar button appearance
#ifndef IC_WIN
#define LINE_ENDS_PAINTED     1  // define this if IGLine/IGPolyLine paint end point
#endif

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::ICustomButtonDrawHandler                           |
------------------------------------------------------------------------------*/
ICustomButtonDrawHandler :: ICustomButtonDrawHandler ()
{
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::~ICustomButtonDrawHandler                          |
------------------------------------------------------------------------------*/
ICustomButtonDrawHandler :: ~ICustomButtonDrawHandler ()
{
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::dispatchHandlerEvent                               |
------------------------------------------------------------------------------*/
Boolean ICustomButtonDrawHandler :: dispatchHandlerEvent ( IEvent &event )
{
  Boolean rc = false;

  switch ( event.eventId() )
  {
    // Check for paint message
#ifdef IC_PM
    case WM_CONTROL:
         if ( event.parameter1().number2() == BN_PAINT )
         {
           ICustomButtonDrawEvent evt(event);
           if ( evt.drawDown() )
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, true);
           else
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, false);
           if ( rc = drawButton(evt) )
           {
             event.setResult(evt.result());
           }
         }
         break;
#endif
#ifdef IC_WIN
    case WM_DRAWITEM:
         {
           ICustomButtonDrawEvent evt(event);
           if ( evt.drawDown() )
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, true);
           else
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, false);
           if ( rc = drawButton(evt) )
           {
             event.setResult(evt.result());
           }
         }
         break;
#endif

  }
  return rc;
}

/*------------------------------------------------------------------------------
| IBase::Boolean ICustomButtonDrawHandler::drawButton                          |
------------------------------------------------------------------------------*/
IBase::Boolean ICustomButtonDrawHandler::drawButton ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawButton");
  drawBorder ( event );
  drawBackground ( event );
  drawForeground ( event );
  if ( !event.isButtonEnabled() )
  {
    drawDisabledEmphasis ( event );
  }
  return true;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawBorder                                         |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawBorder ( ICustomButtonDrawEvent& event,
                                              ISize margin )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawBorder");
  // Get needed colors and sizes

  IColor  parentBgnd     (event.customButton()->parent()->backgroundColor());
  IColor  borderColor    (IColor::defaultButton);
  IColor  highlightDark  (IColor::buttonDark);
  IColor  highlightBright(IColor::buttonLight);

  unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);

  IRectangle rect = event.drawingArea();
  ITRACE_ALL(IString("rect=")  + rect.asString() +
             IString(" parentBgnd=")      + IString(parentBgnd.asRGBLong()).d2x() +
             IString(" borderColor=")     + IString(borderColor.asRGBLong()).d2x() +
             IString(" highlightDark=")   + IString(highlightDark.asRGBLong()).d2x() +
             IString(" highlightBright=") + IString(highlightBright.asRGBLong()).d2x() );


  // Get the current pen colors

  IColor oldPenColor (event.graphicContext().penColor());
  IColor oldFillColor(event.graphicContext().fillColor());

  // Clear the outermost border area to the parent background color
  // (This includes the margin and notched border area)

  unsigned long limit;
  unsigned long loop;
  event.graphicContext().setPenColor(parentBgnd)
                        .setDrawOperation(IGraphicBundle::frame);
  limit = cxBorder + margin.width();
  if (limit < (cyBorder + margin.height()) )
    limit = cyBorder + margin.height();
  for ( loop = 0; loop < limit; loop++ )
  {
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX() + loop,
                                  rect.minY() + loop,
                                  rect.maxX() - loop - 1,
                                  rect.maxY() - loop - 1 )));
     ITRACE_ALL( IString("clearing border rect=") +
                 IRectangle ( rect.minX() + loop,
                              rect.minY() + loop,
                              rect.maxX() - loop - 1,
                              rect.maxY() - loop - 1 ).asString() );
  }
  rect.shrinkBy ( margin );
  ITRACE_ALL( IString("margin removed rect=") + rect.asString() );


  // ------------------------------------------------------------
  // The buttons drawing code is different mainly due to
  // coordinate differences.
  // ------------------------------------------------------------
#ifdef IC_ORIGINLOWERLEFT
  event.graphicContext().setPenColor(borderColor)
                        .setFillColor(borderColor)
                        .setDrawOperation(IGraphicBundle::fillAndFrame);
  // Draw a notched border for the button
  //top
  event.graphicContext().draw(
    IGRectangle ( IRectangle ( rect.minX() + cxBorder,
                               rect.maxY() - cyBorder,
                               rect.maxX() - cxBorder - 1,
                               rect.maxY() - 1 )));
  //bottom
  event.graphicContext().draw(
    IGRectangle ( IRectangle ( rect.minX() + cxBorder,
                               rect.minY(),
                               rect.maxX() - cxBorder - 1,
                               rect.minY() + cyBorder - 1)));
  //left
  event.graphicContext().draw(
    IGRectangle ( IRectangle ( rect.minX(),
                               rect.minY() + cyBorder,
                               rect.minX() + cxBorder - 1,
                               rect.maxY() - cyBorder - 1 )));
  //right
  event.graphicContext().draw(
    IGRectangle ( IRectangle ( rect.maxX() - cxBorder,
                               rect.minY() + cyBorder,
                               rect.maxX() - 1,
                               rect.maxY() - cyBorder - 1)));

  rect.shrinkBy ( IPair ( cxBorder, cyBorder ));
  ITRACE_ALL( IString("border removed rect=") + rect.asString() );

  // Draw the 3-D borders

  if ( event.drawUp() )
  {
     // Draw bright highlight on left and top

     event.graphicContext().setPenColor(highlightBright);
     event.graphicContext().setFillColor(highlightBright);
     //top
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.maxY() - cyBorder,
                                  rect.maxX() - cxBorder - 1,
                                  rect.maxY() - 1 )));
     //left
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.minY() + cyBorder,
                                  rect.minX() + cxBorder - 1,
                                  rect.maxY() - cyBorder - 1 )));

     // Draw dark highlight on right and bottom (double wide in PM)

     event.graphicContext().setPenColor(highlightDark);
     event.graphicContext().setFillColor(highlightDark);
     // bottom
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.minY(),
                                  rect.maxX() - 1,
                                  rect.minY() + cyBorder - 1)));
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX() + cxBorder,
                                  rect.minY() + cyBorder,
                                  rect.maxX() - 1,
                                  rect.minY() + cyBorder + cyBorder - 1)));
     // right
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.maxX() - cxBorder,
                                  rect.minY() + cyBorder + cyBorder,
                                  rect.maxX() - 1,
                                  rect.maxY() - 1)));
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.maxX() - cxBorder - cxBorder,
                                  rect.minY() + cyBorder + cyBorder,
                                  rect.maxX() - cxBorder - 1,
                                  rect.maxY() - cyBorder - 1)));

     rect = IRectangle ( rect.minX() + cxBorder,
                         rect.minY() + cyBorder + cyBorder,
                         rect.maxX() - cxBorder - cxBorder,
                         rect.maxY() - cyBorder );

  }
  else
  {
     // Draw dark highlight on left and top

     event.graphicContext().setPenColor(highlightDark);
     event.graphicContext().setFillColor(highlightDark);
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.maxY() - cyBorder,
                                  rect.maxX() -  1,
                                  rect.maxY() - 1 )));
     event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.minY(),
                                  rect.minX() + cxBorder - 1,
                                  rect.maxY() - cyBorder - 1 )));

     rect = IRectangle ( rect.minX() + cxBorder,
                         rect.minY(),
                         rect.maxX(),
                         rect.maxY() - cyBorder );
  }

#endif // IC_ORIGINLOWERLEFT

#ifdef IC_ORIGINUPPERLEFT

#ifdef DRAW_STANDARD_BUTTON
  // adjust the rectangle by one since we are drawing using lines.
  // This is to insure that we draw bottom/right sides inside the region,
  // and the bottom/right dimensions of the rectangle are NOT part of
  // the region (definition of IRectangle)
  rect = IRectangle( rect.minX(), rect.minY(),
                     rect.maxX()-1, rect.maxY()-1 );

  event.graphicContext().setPenWidth( cxBorder );

  // Draw the notched border
  event.graphicContext().setPenColor(borderColor);
  event.graphicContext()
     .draw( IGLine( rect.minXMinY()+IPoint(cxBorder,0),
#ifndef LINE_ENDS_PAINTED
                    IPoint(1,0) +
#endif
                    rect.maxXMinY()-IPoint(cxBorder,0)) )         //top
     .draw( IGLine( rect.minXMinY()+IPoint(0,cyBorder),
#ifndef LINE_ENDS_PAINTED
                    IPoint(0,1) +
#endif
                    rect.minXMaxY()-IPoint(0,cyBorder)) )         //left
     .draw( IGLine( rect.minXMaxY()+IPoint(cxBorder,0),
#ifndef LINE_ENDS_PAINTED
                    IPoint(1,0) +
#endif
                    rect.maxXMaxY()-IPoint(cxBorder,0)) )         //bottom
     .draw( IGLine( rect.maxXMinY()+IPoint(0,cyBorder),
#ifndef LINE_ENDS_PAINTED
                    IPoint(0,1) +
#endif
                    rect.maxXMaxY()-IPoint(0,cyBorder)) );        //right

  // adjust rect for next inner line set
  rect.shrinkBy ( IPair ( cxBorder, cyBorder ));

  // draw top and left
  if ( event.drawUp() )
     event.graphicContext().setPenColor(highlightBright);
  else
     event.graphicContext().setPenColor(highlightDark);

  IPointArray anArray(3);
  anArray[0] = rect.minXMaxY();      // lower left
  anArray[1] = rect.minXMinY();      // upper left
#ifdef LINE_ENDS_PAINTED
  anArray[2] = rect.maxXMinY();      // upper right
#else
  anArray[2] = rect.maxXMinY() + IPoint(1,0);      // upper right
#endif
  event.graphicContext().draw( IGPolyline( anArray ) );

  // Draw the 3-D borders
  if ( event.drawUp() )
  {
     // Draw dark highlight on bottom and right
     event.graphicContext().setPenColor(highlightDark);
     anArray[1] = rect.maxXMaxY();      // lower right
#ifndef LINE_ENDS_PAINTED
     anArray[2] = rect.maxXMinY() - IPoint(0,1);      // upper right
#endif
     event.graphicContext().draw( IGPolyline( anArray ) );

     // draw the double wide dark highlight
     anArray[0] = anArray[0] + IPoint(cxBorder,-cyBorder);  // lower left
     anArray[1] = anArray[1] + IPoint(-cxBorder,-cyBorder); // lower right
     anArray[2] = anArray[2] + IPoint(-cxBorder,+cyBorder); // upper right
     event.graphicContext().draw( IGPolyline( anArray ) );

     // adjust rect leaving only "client area" of button.  Be sure to put back
     // the ISize(1,1) we took out before drawing.
     rect = IRectangle ( rect.minX() + cxBorder,
                         rect.minY() + cyBorder,
                         rect.maxX() - cxBorder*2 + 1,
                         rect.maxY() - cyBorder*2 + 1 );
  }
  else
  {
     // draw down.  Done drawing.  Just need to adjust the rect.
     rect = IRectangle ( rect.minX() + cxBorder,
                         rect.minY() + cyBorder,
                         rect.maxX() + 1,          // restoring ISize(1,1)
                         rect.maxY() + 1 );
  }

#else
  // This code draws Win95 native appearance toolbar buttons
  IColor  highlightMiddle(IColor(IColor::buttonMiddle));

  // draw down simply flips colors used
  if (!event.drawUp())
     {
     borderColor = highlightBright;
     highlightDark = highlightMiddle;
     highlightBright = IColor(IColor::defaultButton);
     highlightMiddle = IColor(IColor::buttonDark);
     }

  // draw top and left
  event.graphicContext().setPenColor(highlightBright);
  IPointArray anArray(3);
  anArray[0] = rect.minXMaxY();      // lower left
  anArray[1] = rect.minXMinY();      // upper left
  anArray[2] = rect.maxXMinY();      // upper right
  event.graphicContext().draw( IGPolyline( anArray ) );

  // draw bottom and right
  event.graphicContext().setPenColor(borderColor);
  anArray[1] = rect.maxXMaxY();      // lower right
  event.graphicContext().draw( IGPolyline( anArray ) );

  // adjust rect for next inner line set
  rect = IRectangle ( rect.minX() + cxBorder,
                      rect.minY() + cyBorder,
                      rect.maxX() - cxBorder,
                      rect.maxY() - cyBorder );

  // draw top and left
  event.graphicContext().setPenColor(highlightMiddle);
  anArray[0] = rect.minXMaxY();      // lower left
  anArray[1] = rect.minXMinY();      // upper left
  anArray[2] = rect.maxXMinY();      // upper right
  event.graphicContext().draw( IGPolyline( anArray ) );

  // draw bottom and right
  event.graphicContext().setPenColor(highlightDark);
  anArray[1] = rect.maxXMaxY();      // lower right
  event.graphicContext().draw( IGPolyline( anArray ) );

  // adjust rect leaving only "client area" of button.  Be sure to put back
  // the ISize(1,1) we took out before drawing.
  rect = IRectangle ( rect.minX() + cxBorder,
                      rect.minY() + cyBorder,
                      rect.maxX() - cxBorder + 1,
                      rect.maxY() - cyBorder + 1 );

#endif   // DRAW_STANDARD_BUTTON
#endif  //

  // Reset colors back to the original value

  event.graphicContext().setFillColor(oldFillColor);
  event.graphicContext().setPenColor(oldPenColor);

  // Save the new rectangle as the drawing area

  ITRACE_ALL( IString("New drawing area rect=") + rect.asString() );
  event.setDrawingArea ( rect );

}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawBackground                                     |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawBackground ( ICustomButtonDrawEvent& event )
{
  IRectangle rect = event.drawingArea();

  // Get the current pen color

  IColor oldPenColor = event.graphicContext().penColor();
  IColor oldFillColor = event.graphicContext().fillColor();

  // Get the background color

  IColor bgndColor(0,0,0);
  if ( event.drawLatched() )
    bgndColor = event.customButton()->latchedBackgroundColor();
  else
    bgndColor = event.customButton()->backgroundColor();

  // Fill the background

  event.graphicContext().setPenColor(bgndColor)
                        .setFillColor(bgndColor)
                        .setDrawOperation(IGraphicBundle::fillAndFrame);
  event.graphicContext().draw(
    IGRectangle ( IRectangle ( rect.minX(),
                               rect.minY(),
                               rect.maxX() - 1,
                               rect.maxY() - 1)));
  ITRACE_ALL( IString(" drawBackground rect=") +
              IRectangle ( rect.minX(),
                           rect.minY(),
                           rect.maxX() - 1,
                           rect.maxY() - 1).asString());

  // Halftone latched backgound (if needed)

  if ( event.drawLatched() &&
       event.customButton()->isLatchedBackgroundColorHalftone() )
  {
    unsigned long           oldPattern = event.graphicContext().fillPattern();
    IGraphicBundle::PenType oldType    = event.graphicContext().penType();
    IColor                  oldBgndClr = event.graphicContext().backgroundColor();

    event.graphicContext().setFillPattern(IGraphicBundle::halftone);
    event.graphicContext().setPenType(IGraphicBundle::alternate);
    event.graphicContext().setPenColor(bgndColor);
    event.graphicContext().setFillColor(IColor(IColor::white));
    event.graphicContext().setBackgroundColor(bgndColor);

    event.graphicContext().draw(
      IGRectangle ( IRectangle ( rect.minX(),
                                 rect.minY(),
                                 rect.maxX() - 1,
                                 rect.maxY() - 1)));

    event.graphicContext().setPenType(oldType);
    event.graphicContext().setFillPattern(oldPattern);
    event.graphicContext().setBackgroundColor(oldBgndClr);
  }

  // Reset colors back to the original value

  event.graphicContext().setFillColor(oldFillColor);
  event.graphicContext().setPenColor(oldPenColor);

}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawForeground                                     |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawForeground ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("ICustomButtonDrawHandler::drawForeground");
  IRectangle rect(event.drawingArea());
  ITRACE_ALL(IString("rect=") + rect.asString() );

  // Get the old clip region and create a new clip region based on
  // the drawing area rectangle (so we do not draw outside of the
  // foreground area)

#ifdef IC_WIN
  // On win32s graphicContext::clipRegion() is not supported.
  // This query is done as a precaution, and skipping it is acceptable

  IGRegion startRegion((IRegionHandle)0);
  if (!IPlatform::isWin32S() )
    startRegion = event.graphicContext().clipRegion();
#else
  IGRegion startRegion(event.graphicContext().clipRegion());
#endif

  IGRegion clipRegion(event.graphicContext(),rect);

  event.graphicContext().setClipRegion(clipRegion);

  // Get the current pen color

  IColor oldPenColor(event.graphicContext().penColor());

  // Get the current font

#ifdef IC_WIN
  IFont  fnt(event.controlWindow());
#else
  IFont  fnt(event.graphicContext().handle());
#endif

  // Get the current border size

  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);

  // Start drawing text on the top of the button

  IPoint textPos;
#ifdef IC_ORIGINLOWERLEFT
  textPos.setY ( rect.maxY() - cyBorder - fnt.maxAscender() );
#else
  textPos.setY ( rect.minY() + cyBorder );
#endif
  ITRACE_ALL( IString("textPos=") + textPos.asString() );

  // If button is not "up", shift foregroung drawing down 1 pel

  if ( !event.drawUp() )
    {
#ifdef IC_ORIGINLOWERLEFT
    textPos.setY ( textPos.y() - 1 );
#else
    textPos.setY ( textPos.y() + 1 );
#endif
    }

  // Use the foreground color to draw text

  if ( event.drawLatched() )
    event.graphicContext().setPenColor(event.customButton()->latchedForegroundColor());
  else
    event.graphicContext().setPenColor(event.customButton()->foregroundColor());

  // Loop through text to draw each line

  unsigned index = 0;
  unsigned start = 0;

  IString txt(event.customButton()->text());
  do
  {
    // Get the text for the current line
    IString subtxt;
    start = index + 1;
    index = txt.indexOf('\n',start);
    if (index != 0)
      subtxt = txt.subString(start,index-start).strip();
    else
      subtxt = txt.subString(start).strip();

    // Get the text width
    unsigned long txtWidth;
#ifdef IC_PMWIN
    if ( IBidiSettings::isBidiSupported() )
    {
      IBidiSettings settings(*(event.customButton()));
      txtWidth = fnt.textWidth(subtxt,settings);
    }
    else
#endif
    {
      txtWidth = event.customButton()->displaySize(subtxt).width();
    }

    // Center text (or clip if text is too big)
    if ( rect.width() >= txtWidth )
    {
      textPos.setX ( rect.minX() + (( rect.width() - txtWidth ) / 2 ));
    }
    else
    {
      textPos.setX ( rect.left() );
#ifdef IC_PMWIN
      if ( IBidiSettings::isBidiSupported() )
      {
        IBidiSettings settings(*(event.customButton()));
        if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
        {
          textPos.setX ( rect.right() - txtWidth );
        }
      }
#endif
    }

    // Draw text
    ITRACE_ALL( IString("drawGString pos=") + textPos.asString() +
                IString(" string=") + subtxt );
    IGString gs(subtxt,textPos,fnt);
    event.graphicContext().draw(gs);

    // Move position for next line of text (overlap one pel)
#ifdef IC_ORIGINLOWERLEFT
    textPos.setY ( textPos.y() - fnt.maxCharHeight() + 1 );
#else
    textPos.setY ( textPos.y() + fnt.maxCharHeight() - 1 );
#endif
  }
  while (index != 0);

  // Reset colors back to the original value

  event.graphicContext().setPenColor(oldPenColor);

  // Reset clip region back to original value (if it existed)

  event.graphicContext().clearClipRegion();

#ifdef IC_WIN
  if ((!IPlatform::isWin32S())  &&  ((IRegionHandle)startRegion))
#else
  if ((IRegionHandle)startRegion)
#endif
    event.graphicContext().setClipRegion(startRegion);
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawHandler::drawDisabledEmphasis                               |
------------------------------------------------------------------------------*/
void ICustomButtonDrawHandler :: drawDisabledEmphasis
                                 ( ICustomButtonDrawEvent& event )
{
    IRectangle rect(event.drawingArea());

    unsigned long           oldPattern  (event.graphicContext().fillPattern());
    IGraphicBundle::PenType oldType     (event.graphicContext().penType());
    IColor                  oldPenColor (event.graphicContext().penColor());
    IColor                  oldFillColor(event.graphicContext().fillColor());

    event.graphicContext().setFillPattern(IGraphicBundle::halftone)
                          .setPenType(IGraphicBundle::alternate)
                          .setFillColor(IColor(IColor::white))
#ifdef IC_WIN
                          .setPenColor(IColor(IColor::white))
                          .setBackgroundColor(IColor(IColor::black))
                          .setMixMode( IGraphicBundle::or )
#else
                          .setPenColor(event.customButton()->backgroundColor())
#endif
                          .setDrawOperation(IGraphicBundle::fillAndFrame);

    event.graphicContext().draw(
       IGRectangle ( IRectangle ( rect.minX(),
                                  rect.minY(),
                                  rect.maxX() - 1,
                                  rect.maxY() - 1)));

    event.graphicContext().setFillColor(oldFillColor)
                          .setPenColor(oldPenColor)
                          .setPenType(oldType)
                          .setFillPattern(oldPattern);
}

