/* NOSHIP */
#ifndef _ICVHDR_
  #define _ICVHDR_
/*******************************************************************************
* FILE NAME: icvhdr.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ICanvasHandler                                                           *
*     ICanvasStatics                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandler.hpp>
#include <ikeyhdr.hpp>

#pragma pack(4)

class IWindow;
class ICanvas;
class IFrameWindow;
class IEvent;
class IKeyboardEvent;
class ICanvasHandlerData;
class ICVKeyboardHandlerData;

class ICVKeyboardHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* This class provides keyboard event processing for canvas objects.
* Its primary purpose is to provide tabbing and cursor movement between
* controls in a manner similar to dialogs.  It also handle SETFOCUS events
* for the canvas.
*   In OS/2, a default object of this class is attached to ICanvas objects.
*   In Windows, a default object of this class is attached to ICanvas
*      objects and to IFrameWindow objects.
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any parameters.                     |
------------------------------------------------------------------------------*/
  ICVKeyboardHandler        ( );
virtual
 ~ICVKeyboardHandler        ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a canvas window:                                                             |
|   handleEventsFor       - Attaches the handler to the specified canvas.      |
|   stopHandlingEventsFor - Detaches the handler from the specified canvas.    |
| In Windows, the handler object may also be attached to an IFrameWindow       |
| object.                                                                      |
------------------------------------------------------------------------------*/
virtual ICVKeyboardHandler
 &handleEventsFor       ( ICanvas* canvas ),
 &stopHandlingEventsFor ( ICanvas* canvas );

#ifdef IC_WIN
virtual ICVKeyboardHandler
 &handleEventsFor       ( IFrameWindow* canvas ),
 &stopHandlingEventsFor ( IFrameWindow* canvas );
#endif


protected:
/*-------------------------------- Overrides -----------------------------------
| This class overrides the following inherited functions:                      |
|   dispatchHandlerEvent - Router for (and provides processing for other)      |
|                          class-specific events.                              |
------------------------------------------------------------------------------*/
virtual Boolean
  dispatchHandlerEvent  ( IEvent& event );

/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   key          - Processes tabbing and cursor movement keys                  |
------------------------------------------------------------------------------*/
virtual Boolean
  key                  ( IKeyboardEvent& event );

/*------------------------------ Implementation --------------------------------
| These functions are supplied in order to provide a canvas window with        |
| support typically expected from a dialog window.                             |
|   gotFocus     - Gives input focus to the appropriate child window           |
|   handleCursorMovement - Handles tabbing between groups and canvases and     |
|                          cursor movement within a group.                     |
|   handleEnter          - Handles selection of a default pushbutton when the  |
|                          Enter key is pressed.                               |
------------------------------------------------------------------------------*/
Boolean
  gotFocus              ( IEvent& event ),
  handleCursorMovement  ( IKeyboardEvent& event ),
  handleEnter           ( IKeyboardEvent& event );

/*------------------------------ Button State ----------------------------------
|   buttonSetup          - Handles cursor selection of radio buttons and       |
|                          default push button hiliting with cursor movement.  |
------------------------------------------------------------------------------*/
enum State {
  enteringGroup, withinGroup, leavingGroup
};
static void
  buttonSetup           ( const IWindow* canvas,
                          const IWindowHandle& child,
                          State state,
                          Boolean fClickRadio = TRUE );  // DEFECT 8015

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );

ICVKeyboardHandlerData
 *fCVKeyboardHandlerData;
};    // class ICVKeyboardHandler

class ICanvasHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* This class provides event processing for canvas objects.  This includes:     *
*   - expected client area behavior                                            *
*     (e.g. ability to be resized, passing of command events to the owner      *
*   - calling the layout routine when the window is repainted                  *
*                                                                              *
* The management of tabbing and cursor movement between controls,              *
* auto-selecting of radio buttons, processing of mnemonics, etc. is handled    *
* by ICVKeyboardHandler.  For canvas controls default ICanvasHandler and       *
* ICVKeyboardHandler objects are attached to the canvas.  In Windows, the      *
* ICVKeyboardHandler object is also used on IFrameWindows but the              *
* ICanvasHandler object is not.                                                *
*                                                                              *
* The only way to construct instances of this class is by using the            *
* default constructor, which does not take any parameters.                     *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructors ---------------------------
| The only way to construct instances of this class is by using the            |
| default constructor, which does not take any parameters.                     |
------------------------------------------------------------------------------*/
  ICanvasHandler        ( );
virtual
 ~ICanvasHandler        ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a canvas window:                                                             |
|   handleEventsFor       - Attaches the handler to the specified canvas.      |
|   stopHandlingEventsFor - Detaches the handler from the specified canvas.    |
------------------------------------------------------------------------------*/
virtual ICanvasHandler
 &handleEventsFor       ( ICanvas* canvas ),
 &stopHandlingEventsFor ( ICanvas* canvas );

protected:
/*-------------------------------- Overrides -----------------------------------
| This class overrides the following inherited functions:                      |
|   dispatchHandlerEvent - Router for (and provides processing for other)      |
|                          class-specific events.                              |
------------------------------------------------------------------------------*/
virtual Boolean
  dispatchHandlerEvent  ( IEvent& event );

/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   layoutCanvas - Calls the virtual layout function.                          |
------------------------------------------------------------------------------*/
virtual Boolean
#ifdef IC_MOTIF
  gotFocus              ( IEvent& event ),
  key                   ( IKeyboardEvent& event ),
#endif
  layoutCanvas          ( IEvent& event );
#ifdef IC_MOTIF
Boolean
  handleCursorMovement  ( IKeyboardEvent& event ),
  handleEnter           ( IKeyboardEvent& event );
enum State {
  enteringGroup, withinGroup, leavingGroup
};
ICanvasHandler
 &buttonSetup           ( const ICanvas* canvas,
                          const IWindowHandle& child,
                          State state );
#endif

private:
/*--------------------------------- Private ----------------------------------*/
virtual IHandler
 &handleEventsFor       ( IWindow* window  ),
 &stopHandlingEventsFor ( IWindow* window  );

ICanvasHandlerData
 *fCanvasHandlerData;
}; // class ICanvasHandler


class ICanvasStatics : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* This class manages the lifetime of the static canvas handler.  An object of  *
* this class is constructed during static object initialization.  The handler  *
* object is constructed only if needed, and is deleted at termination of the   *
* application.                                                                 *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
 ~ICanvasStatics      ( );

/*-------------------------------- Accessors -----------------------------------
| This function is used to access the static canvas handler.  This allows one  |
| object to be constructed when needed, and shared by all canvases.            |
|   canvasHandler   - Returns the canvas handler.                              |
|   keyboardHandler   - Returns the canvas keyboard handler.                   |
------------------------------------------------------------------------------*/
static ICanvasHandler
 &canvasHandler         ( );
static ICVKeyboardHandler
 &keyboardHandler     ( );


/*---------------------------- Dialog navigation -------------------------------
| Dialog navigation functions support dialog like keyboard navigation in       |
| canvases (and frame windows in some environments)                            |
|   origDefaultButtonHandle - Returns the original default button known by     |
|                             the enclosing frame, if any.                     |
|   defaultPushButtonHandle - Return the first child window that is a          |
|                             default push button, if one exists.              |
|   matchForMnemonic        - Returns the first child window that uses the     |
|                             specified character as a mnemonic.               |
|   nextDialogItem          - Portable interface for enumerating child         |
|                             controls.  Usage like PM WinEnumDlgItem.         |
|   setDefaultPushButton    - Sets the specified pushbutton as the default     |
|                             button.                                          |
|                                                                              |
------------------------------------------------------------------------------*/
static IWindowHandle
  origDefaultButtonHandle( const IWindowHandle& canvas );

static IWindowHandle
  defaultPushButtonHandle( const IWindow* window );

static IWindowHandle
  matchForMnemonic       ( const IWindow* window,
                           unsigned short character );

#ifdef IC_WIN
static IWindowHandle
  nextDialogItem         ( const IWindowHandle& dialog,
                           const IWindowHandle& child,
                           unsigned long        flags );

static IEventResult
  setDefaultPushButton   ( const IWindowHandle&  pushbutton,
                           Boolean               setAsDefault );
#endif

private:
/*--------------------------------- Private ----------------------------------*/
static ICanvasHandler
 *pClCanvasHandler;
static ICVKeyboardHandler
 *fgcvKeyboardHandler;
}; // class ICanvasStatics

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#ifdef IC_PM
/**********************************************************************/
/* support for dialog enumeration and button setting                  */
/**********************************************************************/
#define IENUMDLGITEM(dlg,ctl,flag )   (WinEnumDlgItem(dlg, ctl, flag ))
#define ISETDEFAULTPUSHBUTTON(hwnd,flg) (hwnd.sendEvent( BM_SETDEFAULT, flg, 0))

#endif

#ifdef IC_WIN
/**********************************************************************/
/* support for dialog enumeration and button setting                  */
/**********************************************************************/
#define IENUMDLGITEM(dlg,ctl,flag)      \
         (ICanvasStatics::nextDialogItem(dlg, ctl, flag))
#define ISETDEFAULTPUSHBUTTON(hwnd,flg) \
         (ICanvasStatics::setDefaultPushButton(hwnd, flg))

// Define these flags for Win.  They are used in ICanvasStatics::nextDialogItem
#define EDI_FIRSTTABITEM           0
#define EDI_LASTTABITEM            1
#define EDI_NEXTTABITEM            2
#define EDI_PREVTABITEM            3
#define EDI_FIRSTGROUPITEM         4
#define EDI_LASTGROUPITEM          5
#define EDI_NEXTGROUPITEM          6
#define EDI_PREVGROUPITEM          7
#endif

#endif /* _ICVHDR_ */
