/*******************************************************************************
* FILE NAME: idmsrch.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMSourceHandler                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
  // define what subsections of PM headers to include
  #define INCL_WININPUT         // General window management
  #define INCL_WINSTDDRAG       // Standard drag stuff
  #define INCL_WINSTDCNR        // Container drag stuff
  #include <iwindefs.h>
}

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Include our header
#ifndef _IDMSRCH_
  #include <idmsrch.hpp>
#endif

#ifndef _IDMEVENT_
  #include <idmevent.hpp>
#endif

#ifndef _IDMSRCRN_
  #include <idmsrcrn.hpp>
#endif

#ifndef _IDMITEM_
  #include <idmitem.hpp>
#endif

#ifndef _IDMEFIT_
  #include <idmefit.hpp>
#endif

#ifndef _IDMMLEIT_
  #include <idmmleit.hpp>
#endif

#ifndef _IDMCNRIT_
  #include <idmcnrit.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _IDMCOMM_
  #include <idmcomm.hpp>
#endif

#ifndef _IDMPROV_
  #include <idmprov.hpp>
#endif

#ifndef _IENTRYFD_
  #include <ientryfd.hpp>
#endif

#ifndef _IMLE_
  #include <imle.hpp>
#endif

#ifndef _ICNRCTL_
  #include <icnrctl.hpp>
#endif


#ifndef _IMCEVT_
  #include <imcevt.hpp>
#endif

#ifndef _ICOORDSY_
  #include <icoordsy.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMSRCH_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
class IDMSourceHandlerData
{
  public:
    IDMSourceHandlerData()
    {;}
    virtual
      ~IDMSourceHandlerData()
    {;}

  IEventData
    m_parameter1,
    m_parameter2;
};
#endif

/*------------------------------------------------------------------------------
| IDMSourceHandler::IDMSourceHandler                                           |
|                                                                              |
| Constructor (for IWindow *)                                                  |
------------------------------------------------------------------------------*/
IDMSourceHandler :: IDMSourceHandler (IWindow *pWin)
  : IDMHandler()
#ifdef IC_WIN
  , fDMSourceHandlerData( new IDMSourceHandlerData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceHandler::ctor (win)");

  /********************************************************************/
  /* Add source renderer to source drag handler                       */
  /********************************************************************/
  addRenderer( IDMRenderer::defaultSourceRenderer() );

  /********************************************************************/
  /* Configure the window as a source drag handler                    */
  /********************************************************************/
  handleEventsFor(pWin);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::IDMSourceHandler                                           |
|                                                                              |
| Constructor (for entry field)                                                |
------------------------------------------------------------------------------*/
IDMSourceHandler :: IDMSourceHandler (IEntryField *pEF)
  : IDMHandler()
#ifdef IC_WIN
  , fDMSourceHandlerData( new IDMSourceHandlerData() )
#endif

{
  IMODTRACE_DEVELOP("IDMSourceHandler::ctor (ef)");

  /********************************************************************/
  /* Add source renderer to source drag handler                       */
  /********************************************************************/
  addRenderer( IDMRenderer::defaultSourceRenderer() );

  /********************************************************************/
  /* Designate drag item provider for the entry field                 */
  /********************************************************************/
  IDMHandler::setItemProvider(pEF);

  /********************************************************************/
  /* Configure the entry field as a source drag handler               */
  /********************************************************************/
  handleEventsFor(pEF);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::IDMSourceHandler                                           |
|                                                                              |
| Constructor (for MLE).                                                       |
------------------------------------------------------------------------------*/
IDMSourceHandler :: IDMSourceHandler (IMultiLineEdit *pMLE)
  : IDMHandler()
#ifdef IC_WIN
  , fDMSourceHandlerData( new IDMSourceHandlerData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceHandler::ctor (mle)");

  /********************************************************************/
  /* Add source renderer to source drag handler                       */
  /********************************************************************/
  addRenderer( IDMRenderer::defaultSourceRenderer() );

  /********************************************************************/
  /* Designate drag item provider for the MLE                         */
  /********************************************************************/
  IDMHandler::setItemProvider(pMLE);

  /********************************************************************/
  /* Configure the entry field as a source drag handler               */
  /********************************************************************/
  handleEventsFor(pMLE);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::IDMSourceHandler                                           |
|                                                                              |
| Constructor (for container)                                                  |
------------------------------------------------------------------------------*/
IDMSourceHandler :: IDMSourceHandler(IContainerControl *pCnr)
  : IDMHandler()
#ifdef IC_WIN
  , fDMSourceHandlerData( new IDMSourceHandlerData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceHandler::ctor (cnr)");

  /********************************************************************/
  /* Add source renderer to source drag handler                       */
  /********************************************************************/
  addRenderer( IDMRenderer::defaultSourceRenderer() );

  /********************************************************************/
  /* Designate drag item provider for the container                   */
  /********************************************************************/
  IDMHandler::setItemProvider(pCnr);

  /********************************************************************/
  /* Configure the container as a source drag handler                 */
  /********************************************************************/
  handleEventsFor(pCnr);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::IDMSourceHandler                                           |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMSourceHandler :: IDMSourceHandler ()
  : IDMHandler()
#ifdef IC_WIN
  , fDMSourceHandlerData( new IDMSourceHandlerData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceHandler::ctor ()");

  /********************************************************************/
  /* Add source renderer to source drag handler                       */
  /********************************************************************/
  addRenderer( IDMRenderer::defaultSourceRenderer() );
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::~IDMSourceHandler                                          |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMSourceHandler :: ~IDMSourceHandler()
{
#ifdef IC_WIN
  if (fDMSourceHandlerData )
    delete fDMSourceHandlerData;
#endif

  IMODTRACE_DEVELOP("IDMSourceHandler::dtor ()");
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Overload of Handler's dispatcher.                                            |
------------------------------------------------------------------------------*/
Boolean IDMSourceHandler :: dispatchHandlerEvent(IEvent& evt)
{
  Boolean bProcessed = false;

  switch(evt.eventId())
  {

    /******************************************************************/
    /* Process the begin drag message.  If container control, return  */
    /* false to allow container to process!!!                         */
    /******************************************************************/
    case WM_BEGINDRAG:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler WM_BEGINDRAG");

      IWindow *pCtrlWin = evt.window();

#ifdef IC_PM
      /****************************************************************/
      /* Container check                                              */
      /****************************************************************/
      if (isContainerControl(pCtrlWin))
        return(false);
#endif

      IDMSourceBeginEvent event = IDMSourceBeginEvent(evt);

#ifdef IC_PM
      bProcessed = sourceBegin(event, false);
#endif

#ifdef IC_WIN
      bProcessed = sourceBegin(event, isContainerControl(pCtrlWin));

      IEventData parm2 = evt.parameter1() ;
      evt.window()->postEvent( WM_LBUTTONUP,
         fDMSourceHandlerData->m_parameter1,
         parm2 );
#endif

      evt.setResult(event.result());
      break;
    }

    case DM_RENDERPREPARE:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler DM_RENDERPREPARE");

      IDMSourcePrepareEvent event = IDMSourcePrepareEvent(evt);
      bProcessed = sourcePrepare(event);
      evt.setResult(event.result());
      break;
    }

    case DM_RENDER:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler DM_RENDER");

      IDMSourceRenderEvent event = IDMSourceRenderEvent(evt);
      bProcessed = sourceRender(event);
      evt.setResult(event.result());
      break;
    }

    case DM_ENDCONVERSATION:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler DM_ENDCONVERSATION");

#ifdef IC_WIN
      IDMItem::Handle pSrcDIH;
      pSrcDIH = (IDMItem *)(evt.parameter1().asUnsignedLong());
      if (pSrcDIH)
        {
        // set the source operation based on the dwEffect passed in
        // event parameter 2
        pSrcDIH->sourceOperation()->setOperation(evt.parameter2().number2());
        }
#endif

      IDMSourceEndEvent event = IDMSourceEndEvent(evt);
      bProcessed = sourceEnd(event);
      evt.setResult(event.result());
      break;
    }

    case DM_DISCARDOBJECT:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler DM_DISCARDOBJECT");

      IDMSourceDiscardEvent event = IDMSourceDiscardEvent(evt);
      bProcessed = sourceDiscard(event);
      evt.setResult(event.result());

      ITRACE_DEVELOP("Who Discards = "+IString((int)event.whoDiscards()));
      break;
    }

    case DM_PRINTOBJECT:
    {
      IMODTRACE_DEVELOP("IDMSourceHandler DM_PRINTOBJECT");

      IDMSourcePrintEvent event = IDMSourcePrintEvent(evt);
      bProcessed = sourcePrint(event);
      evt.setResult(event.result());

      ITRACE_DEVELOP("Who Prints = "+IString((int)event.whoPrints()));
      break;
    }

#ifdef IC_PM
    case WM_CONTROL:
    {
      IControlEvent ctlevt(evt);
      IWindow* pwndControl = ctlevt.controlWindow();

      if (isContainerControl(pwndControl))
      {
        /************************************************************/
        /* Container if at this point                               */
        /************************************************************/
        if (evt.parameter1().number2() == CN_INITDRAG)
        {
          IMODTRACE_DEVELOP("IDMSourceHandler CN_INITDRAG");

          IDMSourceBeginEvent event = IDMSourceBeginEvent(evt);
          bProcessed = sourceBegin(event);
          evt.setResult(event.result());
        }
        break;
      }
    }
#endif

  } /* end switch */

  return (bProcessed);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourceBegin                                                |
|                                                                              |
| Process the begin drag event.                                                |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler::
           sourceBegin (IDMSourceBeginEvent& event, Boolean bCnr)
{
  /**********************************************************************/
  /* Create an instance of the IDMSourceOperation class                 */
  /**********************************************************************/
  IDMSourceOperation::Handle pSrcOpH = allocateOperation(event,
                                                   IDM::systemImages);

  /**********************************************************************/
  /* Obtain our provider from the source window which is set by the     */
  /* IDMSourceOperation ctor                                            */
  /**********************************************************************/
  IDMItemProvider *pProvider = pSrcOpH->sourceWindow()->itemProvider();

  /**********************************************************************/
  /* If the source window is a container set container specifics here   */
  /**********************************************************************/
  if (bCnr)
  {
    pSrcOpH->setContainerObject(event.object());
    pSrcOpH->setPointerOffset(event.offset());
  }

  /**********************************************************************/
  /* If the window provides us with some drag items, continue with the  */
  /* drag at hand.                                                      */
  /**********************************************************************/
  /* if pProvider is NULL, the user has not specified an item provider  */
  /* and has most likely enabled dm for an IWindow object, which has no */
  /* default itemProvider.                                              */
  /**********************************************************************/
  if (!pProvider || !(pProvider->provideSourceItems(pSrcOpH)))
  { 
    ITRACE_DEVELOP("IDMSourceHandler: source item provider failed");
    event.setResult(true);
    return (true);
  }

  /**********************************************************************/
  /* if we have no items, return.                                       */
  /**********************************************************************/
  if( pSrcOpH->numberOfItems() == 0 )
	  return( true );

  /**********************************************************************/
  /* Find the appropriate source renderers for the operation            */
  /**********************************************************************/
  if ( findRenderersFor(pSrcOpH) )
  {
    /********************************************************************/
    /* Commence the drag ...                                            */
    /********************************************************************/
    pSrcOpH->begin();
  }

  /**********************************************************************/
  /* Set the result and return ...                                      */
  /**********************************************************************/
  event.setResult(true);
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::allocateOperation                                          |
|                                                                              |
| Create an instance of the IDMSourceOperation class.                          |
------------------------------------------------------------------------------*/
IDMSourceOperation::Handle IDMSourceHandler ::
                           allocateOperation(IDMSourceBeginEvent &event,
                                             DragImageStyle diStyle) const
{
  IDMSourceOperation::Handle pSrcOpH = new IDMSourceOperation (event, diStyle);
#ifdef IC_WIN
  /**********************************************************************/
  /* The source operation object is static under windows.  Store the    */
  /* pointer for later retrieval.                                       */
  /**********************************************************************/
  IDMSourceOperation::setSourceOperation(pSrcOpH);
#endif
  return (pSrcOpH);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::findRenderersFor                                           |
|                                                                              |
| Called to find the appropriate source renderers for the operation.           |
------------------------------------------------------------------------------*/
Boolean IDMSourceHandler ::
        findRenderersFor(const IDMSourceOperation::Handle &pSrcOpH)
{
  IMODTRACE_DEVELOP( "IDMSourceHandler::findRenderersFor" );

  IString  rendererType;
  unsigned position;

  /**********************************************************************/
  /* Obtain number of items                                             */
  /**********************************************************************/
  unsigned long count = pSrcOpH->numberOfItems();

  /**********************************************************************/
  /* Start looping through all the items that are attached to this      */
  /* source operation.                                                  */
  /**********************************************************************/
  for (unsigned long i = 1; (i <= count); i++)
  {
    IDMItem::Handle pItemH;
    pItemH = pSrcOpH->item(i);

    if ((position = findRendererFor(pItemH)))
    {
      pItemH->setRenderer(position);
      resolveRMFConflict( pItemH, this->renderer( position ) );
    }
    else
      return(false);
  }
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::findRendererFor                                            |
|                                                                              |
| Called to find the appropriate source renderer for the item.  Returns the    |
| position of the renderer, 0 if none found.                                   |
------------------------------------------------------------------------------*/
unsigned IDMSourceHandler :: findRendererFor(const IDMItem::Handle &pItemH)
{
  IMODTRACE_DEVELOP( "IDMSourceHandler::findRendererFor" );
  IString allTypes,
          aType;
  Boolean bTypeFound;

  /**********************************************************************/
  /* Obtain number of source renderers                                  */
  /**********************************************************************/
  unsigned long ulRendererCnt = numberOfRenderers();

  /**********************************************************************/
  /* Start looping through all the IDMSourceRenderers that are          */
  /* attatched to this source handler.                                  */
  /**********************************************************************/
  for (unsigned long ulRenderer = 1;
       (ulRenderer <= ulRendererCnt);
        ulRenderer++)
  {
    /********************************************************************/
    /* Get the source renderer object                                   */
    /********************************************************************/
    IDMSourceRenderer *pSrcRndr = renderer(ulRenderer);

    /********************************************************************/
    /* Make a cursory glance at types to make sure there isn't a        */
    /* flagrant mismatch ...                                            */
    /********************************************************************/
    bTypeFound = false;
    allTypes = pSrcRndr->supportedTypes();
    if (allTypes.indexOf(IDM::any))
      bTypeFound = true;
    else
    {
      unsigned count = allTypes.occurrencesOf(',') + 1;
      unsigned startPos = 1;
      unsigned index;
      IString  aType;
      for (unsigned i=0; ((i < count) && (!bTypeFound)); i++)
      {
        index = allTypes.indexOf(',', startPos);
        aType = ((index)  ? allTypes.subString(startPos, index-1)  :
                            allTypes.subString(startPos));

        if (pItemH->hasType (aType))
        {
          bTypeFound = true;
          break;
        }
        startPos = index + 1;
      }
    }

    /********************************************************************/
    /* If type found, compare with supported RMFs                       */
    /********************************************************************/
    if (bTypeFound)
    {
      if ( IDMItem::matchingRMFs( pItemH->rmfs(),
                                  pSrcRndr->supportedRMFs(),
                                  true ).length() )
        {
        ITRACE_DEVELOP( "match on renderer " + IString( ulRenderer ) );
        return ulRenderer;
        }
    }
  } /* end for */

  ITRACE_DEVELOP( "no match found" );
  return(0);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourceRender                                               |
|                                                                              |
| Handles the rendering of a specific drag item.                               |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler :: sourceRender (IDMSourceRenderEvent& event)
{
  IMODTRACE_DEVELOP( "IDMSourceHandler::sourceRender" );

  Boolean  rc = false;
#ifdef IC_PM
  IDMItem::Handle item = event.dragItem();
#endif
#ifdef IC_WIN

  /*---------------------------------------------------------------
  | If an item handle was passed in the format lindex, use it.
  | Otherwise default to item 1.
  ---------------------------------------------------------------*/
  IDMSourceOperation::Handle pSrcOp;
  pSrcOp = IDMSourceOperation::sourceOperation();

  IDMItem::Handle item;
  item = pSrcOp->item(  1 );

  FORMATETC* pFormat = (FORMATETC*)event.parameter1().asUnsignedLong();
  if (pFormat->lindex != -1)
    item = (IDMItem*)(unsigned)pFormat->lindex;
  ITRACE_DEVELOP(IString("rendering item ") + IString(pFormat->lindex));


  ITRACE_DEVELOP(IString("Source op has ") +
        IString(pSrcOp->numberOfItems()) +
        " objects.");
#endif


  IDMSourceRenderer
   *pIDMSrcRenderer = this->renderer( item->renderer() );

  if ( pIDMSrcRenderer )
    {
    rc = true;
    pIDMSrcRenderer->sourceRender(event);
    }

  /**********************************************************************/
  /* Set the result and return ...                                      */
  /**********************************************************************/
  event.setResult (rc);
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourcePrepare                                              |
|                                                                              |
| Handles the preparation of a render event.                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler::sourcePrepare (IDMSourcePrepareEvent& event)
{
  Boolean  rc = false;
  IDMItem::Handle item = event.dragItem();

  IMODTRACE_DEVELOP( "IDMSourceHandler::sourcePrepare" );

  IDMSourceRenderer
   *pIDMSrcRenderer = this->renderer( item->renderer() );

  if ( pIDMSrcRenderer )
    {
    rc = true;
    pIDMSrcRenderer->sourcePrepare(event);
    }

  /**********************************************************************/
  /* Set the result and return ...                                      */
  /**********************************************************************/
  event.setResult (rc);
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::resolveRMFConflict                              (private)  |
|                                                                              |
| When a renderer has been chosen to render a particular drag item we need to  |
| make sure that a conflict does not exist between the rmfs for the item and   |
| the rmfs for the renderer.  What we are checking for is as follows:  If the  |
| rmf (rmFile, rfText), (rmLibrary, rfText) is common between the item and the |
| renderer, (rmLibrary, rfText) will be removed from the item's rmfs.          |
------------------------------------------------------------------------------*/
void IDMSourceHandler::resolveRMFConflict (IDMItem::Handle &item,
                                           IDMSourceRenderer* pIDMSrcRenderer)
{
  IString rmfsResolved;

  /**********************************************************************/
  /* Obtain number of source renderers                                  */
  /**********************************************************************/
  unsigned long ulRendererCnt = numberOfRenderers();

  /**********************************************************************/
  /* Start looping through all the IDMSourceRenderers that are          */
  /* attatched to this source handler.                                  */
  /**********************************************************************/
  for (unsigned long ulRenderer = 1;
       (ulRenderer <= ulRendererCnt);
        ulRenderer++)
  {
    /********************************************************************/
    /* Get the source renderer object                                   */
    /********************************************************************/
    IDMSourceRenderer *pSrcRndr = renderer(ulRenderer);

    IString rmfMatch = IDMItem::matchingRMFs(item->rmfs(),
                                             pSrcRndr->supportedRMFs());

    /********************************************************************/
    /* Resolve the conflict if it exists, as described in this          */
    /* function's header                                                */
    /********************************************************************/
    if (rmfMatch.length())
    {
      if ((rmfMatch.includes(IDMItem::rmfFrom(IDM::rmLibrary, IDM::rfText))) &&
          (rmfMatch.includes(IString("<") + IDM::rmFile + ",")))
      {
        IDMItem::deleteRMF(rmfMatch,
                           IDMItem::rmfFrom(IDM::rmLibrary, IDM::rfText));
      }
      if (rmfsResolved.length())
        rmfsResolved += "," + rmfMatch;
      else
        rmfsResolved = rmfMatch;
    }
  }

  /**********************************************************************/
  /* Set the rmfs for the item that do not have conflicts               */
  /**********************************************************************/
  if (rmfsResolved.length())
    item->setRMFs(rmfsResolved);

  pIDMSrcRenderer;    //prevent compiler warning
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourceEnd                                                  |
|                                                                              |
| Handles the ending of a conversation of a particular drag item.              |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler :: sourceEnd(IDMSourceEndEvent& event)
{
  IDMItem::Handle item;
  item = event.dragItem();

  IMODTRACE_DEVELOP( "IDMSourceHandler::sourceEnd" );

#ifdef IC_PMWIN
  IDMSourceRenderer
   *pIDMSrcRenderer = this->renderer( item->renderer() );
#endif


  if (pIDMSrcRenderer)
    pIDMSrcRenderer->sourceEnd(event);

#ifdef IC_WIN
  /**********************************************************************/
  /* The source operation object is static under windows.  Reset the    */
  /* pointer to force destruction of the object.                        */
  /**********************************************************************/
  // IDMSourceOperation::setSourceOperation( 0 );  happens every item!
#endif

  event.setResult (0);
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourceDiscard                                              |
|                                                                              |
| Called to indicate that an item or items have been dropped on a shredder     |
| object.  Source response determines who deletes the item(s) or cancels the   |
| operation entirely.  Default response gives the source window responsibility |
| for the operation.                                                           |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler :: sourceDiscard(IDMSourceDiscardEvent& event)
{
  Boolean  rc = false;

  IMODTRACE_DEVELOP( "IDMSourceHandler::sourceDiscard" );

  IDMSourceOperation::Handle
    srcOp = event.sourceOperation();

  // Get first item and discard via its renderer.  Note that we really
  // should dispatch renderers on a per-object basis, but that requires
  // to much change to the current renderer interface...
  IDMItem::Handle
    item = srcOp->item( 1 );

  IDMSourceRenderer
   *renderer = this->renderer( item->renderer() );

  if ( renderer )
    {
    rc = true;
    renderer -> sourceDiscard( event );
    }

  /**********************************************************************/
  /* Set the result and return ...                                      */
  /**********************************************************************/
  if (!rc)
    event.setWhoDiscards(IDM::abortDiscard);

  event.setResult(event.whoDiscards());
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::sourcePrint                                                |
|                                                                              |
| Called to indicate that an item or items have been dropped on a printer      |
| object.  Source response determines who prints the item(s) or cancels the    |
| operation entirely.  Default response gives the source window responsibility |
| for the operation.                                                           |
------------------------------------------------------------------------------*/
IBase::Boolean IDMSourceHandler :: sourcePrint(IDMSourcePrintEvent& event)
{
  Boolean  rc = false;

  IMODTRACE_DEVELOP( "IDMSourceHandler::sourcePrint" );

  IDMSourceOperation::Handle
    srcOp = event.sourceOperation();

  // Get first item and print via its renderer.  Note that we really
  // should dispatch renderers on a per-object basis, but that requires
  // to much change to the current renderer interface...
  IDMItem::Handle
    item = srcOp->item( 1 );


  IDMSourceRenderer
   *renderer = this->renderer( item->renderer() );

  if ( renderer )
    {
    rc = true;
    renderer -> sourcePrint( event );
    }

  /**********************************************************************/
  /* Set the result and return ...                                      */
  /**********************************************************************/
  if (!rc)
    event.setWhoPrints(IDM::abortPrint);

  event.setResult(event.whoPrints());
  return (true);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::numberOfRenderers                                          |
|                                                                              |
| Returns the number of renderers.                                             |
------------------------------------------------------------------------------*/
unsigned IDMSourceHandler :: numberOfRenderers ()
{
  return (Inherited::numberOfRenderers());
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::renderer                                                   |
|                                                                              |
| Returns the source renderer with the given position.                         |
------------------------------------------------------------------------------*/
IDMSourceRenderer* IDMSourceHandler :: renderer (unsigned position)
{
  return((IDMSourceRenderer *)Inherited::renderer(position));
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::replaceRenderer                                            |
|                                                                              |
| Replaces a given IDMSourceRenderer with another.                             |
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMSourceHandler :: replaceRenderer (unsigned position,
                                      const IDMSourceRenderer &replacement)
{
  Inherited::replaceRenderer(position, replacement);
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::addRenderer                                                |
|                                                                              |
| Adds another IDMSourceRenderer to this source operation.                     |
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMSourceHandler ::
                  addRenderer (const IDMSourceRenderer &newRenderer)
{
  Inherited::addRenderer(newRenderer);
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::removeRenderer                                             |
|                                                                              |
| Removes an IDMSourceRenderer from this source operation.                     |
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMSourceHandler ::
                  removeRenderer (const IDMSourceRenderer &rendererToRemove)
{
  Inherited::removeRenderer(rendererToRemove);
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::setRenderer                                                |
|                                                                              |
| Sets the renderer for this source operation.  Removes any existing renderers.|
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMSourceHandler ::
                  setRenderer (const IDMSourceRenderer &newRenderer)
{
  Inherited::setRenderer(newRenderer);
  return (*this);
}


/**************************************************************************/
/* Note:  The following functions are here to prevent compiler warnings   */
/*                                                                        */
/**************************************************************************/
/*------------------------------------------------------------------------------
| IDMSourceHandler::replaceRenderer                                            |
------------------------------------------------------------------------------*/
IDMHandler& IDMSourceHandler :: replaceRenderer (unsigned position,
                                                 const IDMRenderer &replacement)
{
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::addRenderer                                                |
------------------------------------------------------------------------------*/
IDMHandler& IDMSourceHandler :: addRenderer (const IDMRenderer &newRenderer)
{
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::removeRenderer                                             |
------------------------------------------------------------------------------*/
IDMHandler& IDMSourceHandler :: removeRenderer (unsigned position)
{
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::removeRenderer                                             |
------------------------------------------------------------------------------*/
IDMHandler& IDMSourceHandler ::
            removeRenderer (const IDMRenderer &rendererToRemove)
{
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceHandler::setRenderer                                                |
------------------------------------------------------------------------------*/
IDMHandler& IDMSourceHandler :: setRenderer (const IDMRenderer &newRenderer)
{
  return(*this);
}

