/*******************************************************************************
* FILE NAME: idmwin.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of direct manipulation functions     *
*   that need to be added to IWindow:                                          *
*     IWindow::itemProvider                                                    *
*     IWindow::setItemProvider                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ireslock.hpp>
#include <iwindow.hpp>
#include <idmwin.hpp>
#include <ikeyset.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMWIN_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
IDMProviderList*  IProviderStatics::pProviders = 0;
IPrivateResource* IProviderStatics::pLibKey = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IDMProviderList                                                              |
------------------------------------------------------------------------------*/
class IDMProviderList : public IKeySet< Assoc, const IWindow* >
{
public:
  IDMProviderList ( );
 ~IDMProviderList ( );
};

/*------------------------------------------------------------------------------
| IDMProviderList::IDMProviderList                                             |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMProviderList :: IDMProviderList ( )
  : IKeySet< Assoc, const IWindow* >( 10 )
{
}

/*------------------------------------------------------------------------------
| IDMProviderList::~IDMProviderList                                            |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMProviderList :: ~IDMProviderList ( )
{
}

/*------------------------------------------------------------------------------
| IProviderStatics::~IProviderStatics                                          |
------------------------------------------------------------------------------*/
IProviderStatics ::  ~IProviderStatics ( )
{
  if (pLibKey)
    delete pLibKey;

  if (pProviders)
    delete pProviders;
}

/*------------------------------------------------------------------------------
| IDMProviderList::providerList                                                |
------------------------------------------------------------------------------*/
IDMProviderList& IProviderStatics :: providerList( )
{
  if (!pProviders)
    pProviders = new IDMProviderList;

  return( *pProviders );
}

/*------------------------------------------------------------------------------
| IProviderStatics::libraryKey                                                 |
------------------------------------------------------------------------------*/
IPrivateResource& IProviderStatics :: libraryKey ( )
{
  if (!pLibKey)
    pLibKey = new IPrivateResource();

  return( *pLibKey );
}

/*------------------------------------------------------------------------------
| IWindow::itemProvider                                                        |
------------------------------------------------------------------------------*/
IDMItemProvider *IWindow :: itemProvider ( ) const
{
  IDMItemProvider
   *result = 0;

  IDMProviderList
   &providers = IProviderStatics::providerList();

  {
    IResourceLock providerLock( IProviderStatics::libraryKey() );

    IDMProviderList::Cursor
      cursor( providers );

    if ( providers.locateElementWithKey( this, cursor ) )
      result = providers.elementAt( cursor ).provider;
  }

  return( result );
}

/*------------------------------------------------------------------------------
| IWindow::setItemProvider                                                     |
------------------------------------------------------------------------------*/
IWindow &IWindow :: setItemProvider ( IDMItemProvider *provider )
{
  Assoc
    assoc = { provider, this };

  {
    IResourceLock providerLock( IProviderStatics::libraryKey() );
    IProviderStatics::providerList().addOrReplaceElementWithKey( assoc );
  }

  return( *this );
}
