/*******************************************************************************
* FILE NAME: ientryf1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ientrynh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSYS
#define INCL_WINENTRYFIELDS

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IENTRYFD_
  #include <ientryfd.hpp>
#endif

#ifndef _IENTRYNH_
  #include <ientrynh.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IENTRYF1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IEntryFieldNotifyStatics
{
public:

  ~IEntryFieldNotifyStatics ()
  {
    delete entryFieldNotifyHandler;
    entryFieldNotifyHandler = 0;
  }

static IEntryFieldNotifyHandler
  &notificationHandler ( );

static IEntryFieldNotifyHandler
  *entryFieldNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IEntryFieldNotifyStatics entryFieldStatics;
INotificationId const IEntryField::dataUpdateId = "IEntryField::dataUpdate";
INotificationId const IEntryField::limitId      = "IEntryField::limit";
INotificationId const IEntryField::characterTypeId = "IEntryField::characterType";
INotificationId const IEntryField::insertModeId = "IEntryField::insertMode";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IEntryFieldNotifyHandler*
  IEntryFieldNotifyStatics::entryFieldNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IEntryFieldNotifyStatics::notificationHandler                                |
|                                                                              |
| Static function to create and return a notification handler for IEntryField. |
------------------------------------------------------------------------------*/
IEntryFieldNotifyHandler& IEntryFieldNotifyStatics::notificationHandler ()
{
  if (!(IEntryFieldNotifyStatics::entryFieldNotifyHandler))
  {
    IEntryFieldNotifyStatics::entryFieldNotifyHandler =
                                   new IEntryFieldNotifyHandler();
  }
  return( *IEntryFieldNotifyStatics::entryFieldNotifyHandler );
}

/*------------------------------------------------------------------------------
| IEntryField::enableNotification                                              |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IEntryField, does not send notifications to observer objects until it is     |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IEntryFieldNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IEntryFieldNotifyHandler::IEntryFieldNotifyHandler                           |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IEntryFieldNotifyHandler::IEntryFieldNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IEntryFieldNotifyHandler::~IEntryFieldNotifyHandler                          |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IEntryFieldNotifyHandler::~IEntryFieldNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IEntryFieldNotifyHandler::dispatchHandlerEvent                               |
|                                                                              |
| This notifies IEntryField Windows of the following events:                   |
|                                                                              |
| setInsertMode(Id) - PM only                                                  |
| setReadOnly(Id)                                                              |
| setTextLimit(Id)                                                             |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN

Boolean IEntryFieldNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  switch (anEvent.eventId())
  {
#ifdef IC_PM
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();

      if ( pwndControl &&
           !(((IEntryField*)pwndControl)->queryInsMode))
      {
        unsigned long ulMsg  = ctlevt.parameter1().highNumber();
        IWindowHandle hwndControl;

        if (pwndControl)
        {
          hwndControl = pwndControl->handle();
          thisWindow = pwndControl;
        }
        else
        {
          hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                  ctlevt.handle());
          thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                                ctlevt.window());
        }

        if (hwndControl && thisWindow)
        {
          IWindowClassName className(hwndControl);
          if (className == WC_ENTRYFIELD)
          {
            if (ulMsg == EN_INSERTMODETOGGLE)
            {
              unsigned long insMode = IQUERYSYSVALUE(SV_INSERTMODE);
              thisWindow->notifyObservers(INotificationEvent(
                                          IEntryField::insertModeId,
                                          *thisWindow, true,
                                          (void *)insMode));
            }
          }
        }
      }
    break;
    } /* end WM_CONTROL */
#endif // IC_PM

    case EM_SETREADONLY:

      if (thisWindow)
      {
        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the readonly state has been modified                       */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_EF_DATAUPD ));
      }
      break;

    case EM_SETTEXTLIMIT:

      if (thisWindow)
      {
        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the limit has actually been set                            */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_EF_LIMIT ));
      }
      break;

    /******************************************************************/
    /* This is the delayed notification message.  Check to see which  */
    /* notification should be sent to the observers based on info     */
    /* stored in parameter 2 of the private message.  Parameter 1     */
    /* contains the information passed in the original event that     */
    /* needs to be sent with the notification                         */
    /******************************************************************/
    case IC_UM_DELAYNOTIFY:

      if (thisWindow)
      {
        if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_EF_DATAUPD )
        {
          thisWindow->notifyObservers(INotificationEvent(
                                      IEntryField::dataUpdateId,
                                      *thisWindow, true,
                   (void *)(!(anEvent.parameter1().asUnsignedLong()))));
        }
        else if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_EF_LIMIT )
        {
          thisWindow->notifyObservers(INotificationEvent(
                                      IEntryField::limitId,
                                      *thisWindow, true,
                                      (void*)anEvent.parameter1()));
        }
      }
      break;

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

