/*******************************************************************************
* FILE NAME: ientryfd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ientryfd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define IUSING_OLE
#ifdef WIN32_LEAN_AND_MEAN
  #undef WIN32_LEAN_AND_MEAN
#endif
#ifdef _OLE2_H_
  #undef _OLE2_H_
#endif

extern "C" {
  #define INCL_NLS                     //ES_ANY, ES_SBCS, ES_DBCS, ES_MIXED
  #define INCL_WINCLIPBOARD
  #define INCL_WINENTRYFIELDS
  #define INCL_WINERRORS               //PMERR_HEAP_MAX_SIZE_REACHED
  #define INCL_WININPUT                //WM_CHAR, etc.
  #define INCL_WINMESSAGEMGR           //WNDPARAM struct
  #define INCL_WINWINDOWMGR
  #define INCL_GPIPRIMITIVES
  #define INCL_WINSYS
  #define INCL_GPIBITMAPS
  #include <iwindefs.h>
}

#include <ientryfd.hpp>
#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  #include <iefpriv.hpp>
  #include <ikeyevt.hpp>
#endif
#include <ibidiset.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandle.hpp>
#include <inotifev.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <itrace.hpp>
#include <iwcname.hpp>
#include <iplatfrm.hpp>
#include <ibrshmgr.hpp>    // 25227

#ifdef IC_WIN
#ifndef _IMCEVT_
  #include <imcevt.hpp>
#endif
#ifndef _ICOORDSY_
  #include <icoordsy.hpp>
#endif
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IENTRYFD_CPP_
  #include <ipagetun.h>
#endif

//-----------------------------------------------------------------------
// DEFECT 4624 : required for PM and WIN
//-----------------------------------------------------------------------
#ifdef IC_PMWIN
#ifndef IES_AUTOTAB
   #define IES_AUTOTAB            0x00000008ul   //IEntryField::autoTab
#endif 
#endif // IC_PMWIN

/*------------------------------------------------------------------------------
| Public entry field styles.                                                   |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IEntryField::Style
  IEntryField::readOnly          = ES_READONLY,
  IEntryField::leftAlign         ( 0, IES_LEFTALIGN ),
  IEntryField::centerAlign       = ES_CENTER,
  IEntryField::rightAlign        = ES_RIGHT,
  IEntryField::autoScroll        = ES_AUTOSCROLL,
  IEntryField::margin            = ES_MARGIN,
  IEntryField::unreadable        = ES_UNREADABLE,
  IEntryField::anyData           ( 0, IES_ANYDATA ),
#ifdef IC_PMWIN   // DEFECT 4624 : use extendedStyle for PM and WIN
  IEntryField::autoTab           ( 0, IES_AUTOTAB ),
#else
  IEntryField::autoTab           = ES_AUTOTAB,
#endif
  IEntryField::command           = ES_COMMAND,
  IEntryField::sbcsData          = ES_SBCS,
  IEntryField::dbcsData          = ES_DBCS,
  IEntryField::mixedData         ( 0, IES_MIXEDDATA ),
  IEntryField::border3D          ( 0, IWS_BORDER3D ),
#ifdef IC_WIN
  IEntryField::upperCase         = ES_UPPERCASE,         //simulate under PM?
  IEntryField::lowerCase         = ES_LOWERCASE,         //simulate under PM?
#endif
  IEntryField::classDefaultStyle ( (WS_VISIBLE    |
                                    ES_MARGIN     |
                                    ES_AUTOSCROLL),
                                                    (IES_LEFTALIGN |
                                                     IES_ANYDATA | IWS_BORDER3D));
const long IEntryField::end = -2;
#pragma data_seg()

#ifdef IC_WIN
// Workaround for NT bug with not returning WS_BORDER style
#define IES_MARGINNTHACK           0x00000008ul
#endif

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IEntryField::Style
 IEntryField::currentDefaultStyle ( (WS_VISIBLE    |
                                     ES_MARGIN     |
                                     ES_AUTOSCROLL),
                                                     (IES_LEFTALIGN |
                                                      IES_ANYDATA | IWS_BORDER3D));
#pragma data_seg()

#ifdef IC_WIN
class IEntryFieldData : public IBase {
/*------------------------------------------------------------------------------
| The IEntryFieldData class encapsulates private data and functions            |
| used by the IEntryField class.  An object of this class is created in the    |
| IEntryField constructors.  This is the os2/windows version.                  |
------------------------------------------------------------------------------*/
public:
  IEntryFieldData ( );

virtual
 ~IEntryFieldData ( );

HBRUSH
  bkgrndbrush;

IBase::Boolean
  pointInSelectedText(const IEvent& evt);
unsigned long
  charLimit;
IBase::Boolean
  bCheckingRightDragStart;
IPoint
  pointRButtonDown;


}; // IEntryFieldData

/*------------------------------------------------------------------------------
| IEntryFieldData::IEntryFieldData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IEntryFieldData::IEntryFieldData ( )
  : bkgrndbrush( 0 )
  , charLimit(30000)
  , bCheckingRightDragStart(false)
  , pointRButtonDown()
{ }

/*------------------------------------------------------------------------------
| IEntryFieldData::~IEntryFieldData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IEntryFieldData::~IEntryFieldData ( )
{
  if ( bkgrndbrush != 0 )
     IBrushManager::freeBrush( bkgrndbrush );    // 25227
}

#endif // IC_WIN

#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler
{
  operator IEntryHandler* ();
IEntryHandler
 *operator -> ()
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IEntryHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator IEntryHandler* ( )
  {
  if ( !ptr )
    ptr = new IEntryHandler;
  return ptr;
  }
#endif //IC_PMWIN

/*------------------------------------------------------------------------------
| IEntryField::IEntryField                                                     |
|                                                                              |
| Default constructor (needed by combo box).                                   |
------------------------------------------------------------------------------*/
IEntryField::IEntryField()
#ifdef IC_WIN
  : fEntryFieldData( new IEntryFieldData ),
#else
  : fEntryFieldData( 0 ),
#endif
    eReadStatus(doesntMatter),
    queryInsMode(false)
{
}


/*------------------------------------------------------------------------------
| IEntryField::IEntryField                                                     |
|                                                                              |
| Constructor to create an entry field control on a standard window.           |
------------------------------------------------------------------------------*/
IEntryField::IEntryField ( unsigned long ulId,
                           IWindow* parent,
                           IWindow* pwndOwner,
                           const IRectangle& rectInit,
                           const Style& style )
#ifdef IC_WIN
  : fEntryFieldData( new IEntryFieldData ),
#else
  : fEntryFieldData( 0 ),
#endif
    eReadStatus(doesntMatter),
    queryInsMode(false)
{
  // assertions on input parms
  IASSERTPARM(parent!=0);

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  /********************************************************************/
  /* Need to check alignment styles for conflicting styles            */
  /********************************************************************/
  if (((style & IEntryField::rightAlign) && (style & IEntryField::centerAlign)) ||
      ((style & IEntryField::leftAlign) && (style & IEntryField::rightAlign)) ||
      ((style & IEntryField::leftAlign) && (style & IEntryField::centerAlign)))
  {
    ITHROWLIBRARYERROR( IC_INVALIDSTYLE,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
  }

  /****************************************************************/
  /* Borland has bug with embedded ? operator so below is needed. */
  /****************************************************************/
#ifdef __BORLANDC__
  IWindowHandle owner;
  if (pwndOwner == 0)
    owner = IWindowHandle(0);
  else
    owner = pwndOwner->handle();
#else
  IWindowHandle owner = (pwndOwner == 0) ? IWindowHandle(0)
                                         : pwndOwner->handle();
#endif

  Style
    createStyle = style;

  // Check if we need to supply bidi support to turn a left-aligned
  // entry field into a right-aligned one.
  if ( ! ( style & ( IEntryField::rightAlign |
                       IEntryField::centerAlign ) )  &&
       IBidiSettings::isBidiSupported() )
  {
     // The entry field inherits bidi attributes from its parent
     // window.
     IBidiSettings
       bidiValues( *parent );
     if ( bidiValues.windowLayout() == IBidiSettings::layoutRightToLeft )
     {
        // Change the left-aligned control into a right-to-left one.
        createStyle &= ~IEntryField::leftAlign;
        createStyle |= IEntryField::rightAlign;
        this->setExtendedStyle( this->extendedStyle() & ~IES_LEFTALIGN );
     }
  }

  IWindowHandle whEntryField =
         this -> create( ulId,
                         0,
                         convertToGUIStyle( createStyle ),
                         WC_ENTRYFIELD,
                         parent->handle(),
                         owner,
                         rectInit,
                         0,
                         0,
                         defaultOrdering(),
                         convertToGUIStyle( createStyle, true ) );

#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(whEntryField);
#ifdef IC_WIN
  // Set the limit to 32 (instead of 30000) to keep default the same as OS/2
  setLimit( 32 );
#endif
}


/*------------------------------------------------------------------------------
| IEntryField::IEntryField                                                     |
|                                                                              |
| Constructor to instantiate an entry field object for a dialog template.      |
------------------------------------------------------------------------------*/
IEntryField::IEntryField( unsigned long ulId,
                          IWindow*      parent )
#ifdef IC_WIN
  : fEntryFieldData( new IEntryFieldData ),
#else
  : fEntryFieldData( 0 ),
#endif
    eReadStatus(doesntMatter),
    queryInsMode(false)
{
  setAutoDestroyWindow(false);

#ifdef IC_PM
  // Work-around for window id breakage in Warp GA for the child
  // entry field of a spin button control (it should be the window
  // id of the parent spin button, but is 1).
  if (parent  &&
      parent->id() == ulId  &&
      IWindow::handleWithParent( ulId, parent->handle() ) == 0  &&
      IWindowClassName ( parent->handle() ) == WC_SPINBUTTON)
  {                // Can't find a child entry field of a spin button.
     ulId = 1;     // Replace the id with the one Warp GA uses.
  }
#endif  // IC_PM

#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(ulId, parent);

  setTextChangedFlag(false);   //Reset the changed status to FALSE,
                               //status is set to TRUE on creation.

  // Get the system style and look for extended styles.  Set if found.
  unsigned long sysStyle = style();

  // Extended style should always be zero
  unsigned long extStyle = extendedStyle();
#ifdef IC_WIN
  // Since we can't query the WS_BORDER style due to a NT bug, assume the
  // wrappered control always has a border (will be correct in most cases)
  extStyle |= IES_MARGINNTHACK;
#endif

  // Check for the 3D border style - will be WS_EX_CLIENTEDGE in NT and zero
  // otherwise
  if ((sysStyle & IWS_BORDER3D) || (sysStyle & ES_MARGIN))
     extStyle |= IWS_BORDER3D;

  // If neither center align nor right align is set, then left align must be
  if (!((sysStyle & ES_CENTER) || (sysStyle & ES_RIGHT)))
     extStyle |= IES_LEFTALIGN;

  // If SBCS and DBCS are set, then extended style MIXED is on
  if ((sysStyle & ES_SBCS) && (sysStyle & ES_DBCS))
     extStyle |= IES_MIXEDDATA;

  // If neither SBCS or DBCS is set, then set ANY extended style
  if (!((sysStyle & ES_SBCS) || (sysStyle & ES_DBCS)))
     extStyle |= IES_ANYDATA;

#ifdef IC_WIN
  // For Windows it is always a child and must be an MLE to get alignment,
  // and other required functions to work except password
  if (!(sysStyle & ES_UNREADABLE))
    sysStyle |= ES_MULTILINE;

  sysStyle |= WS_CHILD;
  // Set the style
  setStyle(sysStyle);
#endif
  // Set the extended style
  setExtendedStyle (extStyle);
}


/*------------------------------------------------------------------------------
| IEntryField::IEntryField                                                     |
|                                                                              |
| Constructor to instantiate from an existing entry field control.             |
------------------------------------------------------------------------------*/
IEntryField::IEntryField( const IWindowHandle& wh )
#ifdef IC_WIN
  : fEntryFieldData( new IEntryFieldData ),
#else
  : fEntryFieldData( 0 ),
#endif
    eReadStatus(doesntMatter),
    queryInsMode(false)
{
  setAutoDestroyWindow(false);
#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(wh);

  // Get the system style and look for extended styles.  Set if found.
  unsigned long sysStyle = style();

  // Extended style should always be zero
  unsigned long extStyle = extendedStyle();
#ifdef IC_WIN
  // Since we can't query the WS_BORDER style due to a NT bug, assume the
  // wrappered control always has a border (will be correct in most cases)
  extStyle |= IES_MARGINNTHACK;
#endif

  // Check for the 3D border style - will be WS_EX_CLIENTEDGE in NT and zero
  // otherwise
  if ((sysStyle & IWS_BORDER3D) || (sysStyle & ES_MARGIN))
     extStyle |= IWS_BORDER3D;

  // If neither center align nor right align is set, then left align must be
  if (!((sysStyle & ES_CENTER) || (sysStyle & ES_RIGHT)))
     extStyle |= IES_LEFTALIGN;

  // If SBCS and DBCS are set, then extended style MIXED is on
  if ((sysStyle & ES_SBCS) && (sysStyle & ES_DBCS))
     extStyle |= IES_MIXEDDATA;

  // If neither SBCS or DBCS is set, then set ANY extended style
  if (!((sysStyle & ES_SBCS) || (sysStyle & ES_DBCS)))
     extStyle |= IES_ANYDATA;

#ifdef IC_WIN
  // For Windows it is always a child and must be an MLE to get alignment,
  // and other required functions to work except password
  if (!(sysStyle & ES_UNREADABLE))
    sysStyle |= ES_MULTILINE;

  sysStyle |= WS_CHILD;
  // Set the style
  setStyle(sysStyle);
#endif
  // Set the extended style
  setExtendedStyle (extStyle);
}


/*------------------------------------------------------------------------------
| IEntryField::~IEntryField                                                    |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IEntryField::~IEntryField()
{
#ifdef IC_WIN
  removeHandler(defaultHandler);
  if ( fEntryFieldData != 0 )
    delete fEntryFieldData;
#endif
}

/*------------------------------------------------------------------------------
| IEntryField::defaultStyle                                                    |
|                                                                              |
| Return the default style for new entry field objects.                        |
------------------------------------------------------------------------------*/
IEntryField::Style IEntryField::defaultStyle()
{
  return( currentDefaultStyle );
}


/*------------------------------------------------------------------------------
| IEntryField::setDefaultStyle                                                 |
|                                                                              |
| Replace the default style for new entry field objects.                       |
------------------------------------------------------------------------------*/
void IEntryField::setDefaultStyle( const Style& style )
{
  currentDefaultStyle = style;
}


/*------------------------------------------------------------------------------
| IEntryField::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IEntryField::convertToGUIStyle( const IBitFlag& guiStyle,
                                              Boolean bExtOnly ) const
{
  // Obtain the style from the class (ITextControl) that we inherit from.
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range.
    // And extended GUI styles like client edge.
    ulStyle |= guiStyle.asExtendedUnsignedLong() & (IS_EXTMASK | ITC_EXTGUIMASK);
#ifdef IC_WIN
    // Hack around bug in Windows NT 3.51 which does not allow the WS_BORDER
    // style to be queried back after use on create - Use our own style for now
    unsigned long ulTmp = Inherited::convertToGUIStyle( guiStyle );

    if (ulTmp & margin.asUnsignedLong())
      ((IEntryField*)this)->setExtendedStyle( extendedStyle() | IES_MARGINNTHACK );
#endif
  }
  else
  {
    // Let the clean ES_ styles, flow thru.
    ulStyle |= guiStyle.asUnsignedLong() & IES_MASK;

#ifdef IC_MOTIFPM
    // ES_MIXED is the result of (ES_SBCS | ES_DBCS).  Therefore, we
    // define a unique bit for it in the extended styles.  Map the
    // extended style definition, back to the system definition,
    // prior to creating the control.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IES_EXTMASK;

    if (ulTmp & mixedData.asExtendedUnsignedLong())
    {
       ulStyle |= ES_MIXED;
    }
#endif
#ifdef IC_WIN
    // For Windows it is always a child and must be an MLE to get alignment,
    // and other required functions to work except password
    if (!(ulStyle & ES_UNREADABLE))
    {
      ulStyle |= ES_MULTILINE;
    }
    ulStyle |= WS_CHILD;
#endif
  }
  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IEntryField::setAlignment                                                    |
|                                                                              |
| Replaces the text alignment style of the entry field while preserving the    |
| marked area and cursor position.                                             |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setAlignment( Alignment alignType )
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  //Remove existing alignment
  ulStyle &= ~(centerAlign.asUnsignedLong() | rightAlign.asUnsignedLong());

  switch (alignType)
  {
    case left:
      // No additional processing required
    break;

    case right:
      ulStyle |= rightAlign.asUnsignedLong();
    break;

    case center:
      ulStyle |= centerAlign.asUnsignedLong();
    break;

    default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
    break;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force repaint
  }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::alignment                                                       |
|                                                                              |
| Returns the current alignment value for the entry field.                     |
------------------------------------------------------------------------------*/
IEntryField::Alignment  IEntryField::alignment() const
{
  unsigned long ulStyle = style();
  if (ulStyle & rightAlign.asUnsignedLong())
      return( right );

  if (ulStyle & centerAlign.asUnsignedLong())
      return( center );

  return( left );
}


/*------------------------------------------------------------------------------
| IEntryField::setCharType                                                     |
|                                                                              |
| Replaces the text data type style of the entry field while preserving the    |
| marked area and cursor position.                                             |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setCharType( CharType charType )
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;
    ulStyle &= ~(sbcsData.asUnsignedLong() | dbcsData.asUnsignedLong() |
                 ES_MIXED);

  switch (charType)
  {
     case sbcs:
       ulStyle |= sbcsData.asUnsignedLong();
     break;

     case dbcs:
       ulStyle |= dbcsData.asUnsignedLong();
     break;

     case any:
      // No additional processing required
     break;

     case mixed:
       ulStyle |= ES_MIXED;
     break;

     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
     break;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force refresh
    notifyObservers(INotificationEvent(IEntryField::characterTypeId,
                                       *this, true, (void *)charType));
  }
#endif
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::charType                                                        |
|                                                                              |
| Returns the text contents type.                                              |
------------------------------------------------------------------------------*/
IEntryField::CharType  IEntryField::charType() const
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  if ((ulStyle & sbcsData.asUnsignedLong()) &&
      (ulStyle & dbcsData.asUnsignedLong()))
     return( mixed );

  if (ulStyle & dbcsData.asUnsignedLong())
     return( dbcs );

  if (ulStyle & sbcsData.asUnsignedLong())
     return( sbcs );
#endif
  return( any );
}


/*------------------------------------------------------------------------------
| IEntryField::enableAutoScroll                                                |
|                                                                              |
| Sets the autoScroll style on the entry field control.                        |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableAutoScroll( Boolean enable )
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (enable)
  {
    ulStyle |= autoScroll.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~autoScroll.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force refresh
  }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::disableAutoScroll                                               |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableAutoScroll()
{
  enableAutoScroll(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::isAutoScroll                                                    |
------------------------------------------------------------------------------*/
Boolean IEntryField::isAutoScroll() const
{
  return( (style() & autoScroll.asUnsignedLong()) ? true : false );
}


/*------------------------------------------------------------------------------
| IEntryField::enableMargin                                                    |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableMargin ( Boolean enable )
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (enable)
  {
    ulStyle |= margin.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~margin.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force refresh
  }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::disableMargin                                                   |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableMargin()
{
  enableMargin(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::isMargin                                                        |
------------------------------------------------------------------------------*/
Boolean IEntryField::isMargin() const
{
  return(((style() & margin.asUnsignedLong())
#ifdef IC_WIN
           || (extendedStyle() & IES_MARGINNTHACK)
#endif
           ) ? true : false );
}


/*------------------------------------------------------------------------------
| IEntryField::enableCommand                                                   |
|                                                                              |
| Enables or disbles the command style on an entry field control.              |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableCommand( Boolean enable )
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  Boolean styleChanged = false;

  if (enable)
  {
    if (!(ulStyle & command.asUnsignedLong()))
    {
      ulStyle |= command.asUnsignedLong();
      styleChanged = true;
    }
  }
  else if (ulStyle & command.asUnsignedLong())
  {
    ulStyle &= ~command.asUnsignedLong();
    styleChanged = true;
  }

  if (styleChanged)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force refresh
  }
#endif
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::disableCommand                                                  |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableCommand()
{
  enableCommand(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::isCommand                                                       |
------------------------------------------------------------------------------*/
Boolean IEntryField::isCommand() const
{
  return( (style() & command.asUnsignedLong()) ? true : false );
}


/*------------------------------------------------------------------------------
| IEntryField::enableAutoTab                                                   |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableAutoTab( Boolean enable )
{
#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldStyle = ulExtStyle;

  if (enable)
    ulExtStyle |= autoTab.asExtendedUnsignedLong();
  else
    ulExtStyle &= ~autoTab.asExtendedUnsignedLong();

  if (ulExtStyle != ulOldStyle)
    setExtendedStyle(ulExtStyle);
#else
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (enable)
  {
    ulStyle |= autoTab.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~autoTab.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);                   //Change style
    refresh();                           //Force refresh
  }
#endif // IC_PMWIN
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::disableAutoTab                                                  |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableAutoTab()
{
  enableAutoTab(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::isAutoTab                                                       |
------------------------------------------------------------------------------*/
Boolean IEntryField::isAutoTab() const
{
#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
  return( (extendedStyle() & autoTab.asExtendedUnsignedLong()) ? true : false );
#else
  return( (style() & autoTab.asUnsignedLong()) ? true : false );
#endif
}


/*------------------------------------------------------------------------------
| IEntryField::cut                                                             |
|                                                                              |
| Removes the selected text from the entry field control and puts it in the    |
| clipboard.                                                                   |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::cut( unsigned long timestamp )
{
  IASSERTSTATE(hasSelectedText());

  IEventResult evt = handle().sendEvent(EM_CUT,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
#ifdef IC_PM
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_CUT");
#endif

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::copy                                                            |
|                                                                              |
| Copy marked text to the clipboard.                                           |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::copy( unsigned long timestamp )
{
  IASSERTSTATE(hasSelectedText());

  IEventResult evt = handle().sendEvent(EM_COPY,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
#ifdef IC_PM
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_COPY");
#endif

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::paste                                                           |
|                                                                              |
| Replaces the selected text with the text in the clipboard                    |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::paste()
{
  IASSERTSTATE(clipboardHasTextFormat());

  IEventResult evt = handle().sendEvent(EM_PASTE,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
#ifdef IC_PM
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_PASTE");
#endif

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::clear                                                           |
|                                                                              |
| Replaces the selected text in the entry field with blanks.                   |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::clear( unsigned long timestamp )
{
  IASSERTSTATE(hasSelectedText());
  // If disableDataUpdate() has been called, then do nothing (like other
  // system clipboard operations).
  if (isWriteable())
  {
     IRange rangMark = selectedRange();
     IString strText  = text();

     // Compensate for 0-based index from selected range
     // and 1-based index used by IString.
     strText.overlayWith(IString(0, (unsigned int)
                         (rangMark.upperBound() - rangMark.lowerBound()+1)),
                         (unsigned int)rangMark.lowerBound()+1);
     setText((char *)strText);
  }
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::discard                                                         |
|                                                                              |
| Deletes the selected text.  Compacts the marked area.                        |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::discard()
{
  IASSERTSTATE(hasSelectedText());

  IEventResult evt = handle().sendEvent(EM_CLEAR,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
#ifdef IC_PM
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_CLEAR");
#endif

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::removeAll                                                       |
|                                                                              |
| Deletes the entire contents of the Entryfield.                               |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::removeAll()
{
  /*******************************************************************/
  /* Check to insure that Entryfield is not already empty            */
  /*******************************************************************/
  if (!isEmpty())
  {
    /*****************************************************************/
    /* Turn update off, select all of the text, then call discard()  */
    /* to remove all of the text from the entryfield.                */
    /*****************************************************************/
    enableUpdate(false);
#ifdef IC_WIN
    // No delete all function in Windows so just set contents to NULL string
    setText((char*)IString());
#endif
#ifdef IC_PM
    selectRange();
    discard();
#endif
    enableUpdate(true);
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::isEmpty                                                         |
------------------------------------------------------------------------------*/
Boolean IEntryField::isEmpty() const
{
  if (text().length())
    return( false );
  else
    return( true );
}


/*------------------------------------------------------------------------------
| IEntryField::hasTextChanged                                                  |
|                                                                              |
| Queries whether the entry field has changed since the last query.            |
------------------------------------------------------------------------------*/
Boolean IEntryField::hasTextChanged( ) const
{
  //In PM, this message resets the change flag by default.
  //In Windows, this message does not reset the change flag by default.
  IEventResult evt = handle().sendEvent(EM_QUERYCHANGED,
                                        IEventParameter1(0),
                                        IEventParameter2(0));

  if (evt.asUnsignedLong())
  {
#ifdef IC_PM
    setTextChangedFlag( true );
#endif
    return( true );
  }
  else
    return( false );
}


/*------------------------------------------------------------------------------
| IEntryField::isWriteable                                                     |
------------------------------------------------------------------------------*/
Boolean IEntryField::isWriteable() const
{
#ifdef IC_WIN
  if ( style() & readOnly.asUnsignedLong() )
#else
  IEventResult evt = handle().sendEvent(EM_QUERYREADONLY,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  if (evt.asUnsignedLong())
#endif
    return( false );
  else
    return( true );
}


/*------------------------------------------------------------------------------
| IEntryField::hasSelectedText                                                 |
------------------------------------------------------------------------------*/
Boolean IEntryField::hasSelectedText() const
{
  if (selectedTextLength() == 0)
    return( false );
  else
    return( true );
}

/*------------------------------------------------------------------------------
| IEntryField::setTextChangedFlag                                              |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setTextChangedFlag( Boolean changed )
{
#ifdef IC_PM
  if (changed)
    ISETWINDOWTEXT(handle(), (PSZ)text());
  else
    IEventResult evt = handle().sendEvent(EM_QUERYCHANGED,
                                          IEventParameter1(0),
                                          IEventParameter2(0));
#endif

#ifdef IC_WIN
  handle().sendEvent(EM_SETMODIFY, IEventParameter1(changed),
                                   IEventParameter2(0));
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| IEntryField::resetTextChangedFlag                                            |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::resetTextChangedFlag( )
{
  setTextChangedFlag( false );
  return *this;
}

/*------------------------------------------------------------------------------
| IEntryField::disable                                                         |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disable()
{
  if (isWriteable())
  {
     eReadStatus = wasInputButReset;
     disableDataUpdate();
  }
  IWindow::disable();

  if (hasFocus())
  {
     /**********************************************************/
     /* Send tab message to self (this will be sent on to the  */
     /* owner to be processed).                                */
     /* NOTE: The only additional information sent with the    */
     /*       VK_TAB message is that it is a virtual key.      */
     /**********************************************************/
#ifdef IC_PM
     handle().sendEvent(WM_CHAR,
                        IEventParameter1((unsigned short)KC_VIRTUALKEY,0,0),
                        IEventParameter2((unsigned short)0,VK_TAB,0));
#endif

#ifdef IC_WIN
     handle().sendEvent(WM_KEYDOWN,
                        IEventParameter1(VK_TAB),
                        IEventParameter2(0));
#endif
  }

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::enable                                                          |
|                                                                              |
| Enables the entry field.  Remove the read-only (if turned on by disable()    |
| and reset by enableReadOnly()).                                              |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enable( Boolean enable )
{
  if (eReadStatus == wasInputButReset)
  {
     eReadStatus=doesntMatter;
     enableDataUpdate();
  }
  IENABLEWINDOW(handle(), enable);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::isInsertMode                                                    |
------------------------------------------------------------------------------*/
Boolean IEntryField::isInsertMode() const
{
#ifdef IC_PM
  ((IEntryField*)this)->queryInsMode = true;
  IEventResult evt = handle().sendEvent(EM_SETINSERTMODE,
                                        IEventParameter1(true),
                                        IEventParameter2(0));
  if (evt.asUnsignedLong())
  {
    ((IEntryField*)this)->queryInsMode = false;
    return( true );
  }

  handle().sendEvent(EM_SETINSERTMODE,
                     IEventParameter1(false),
                     IEventParameter2(0));

  ((IEntryField*)this)->queryInsMode = false;
  return( false );
#endif
#ifdef IC_WIN
  // Windows edit fields are always in insert mode
  return ( true );
#endif
}


/*------------------------------------------------------------------------------
| IEntryField::enableInsertMode                                                |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableInsertMode( Boolean insert )
{
#ifdef IC_PM
  handle().sendEvent(EM_SETINSERTMODE,
                     IEventParameter1(insert),
                     IEventParameter2(0));
#endif
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::disableInsertMode                                               |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableInsertMode()
{
  enableInsertMode(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::enableDataUpdate                                                |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::enableDataUpdate( Boolean update )
{
  if (update)
  {
    if ( !(this->isWriteable()) )
    {
      handle().sendEvent(EM_SETREADONLY,
                         IEventParameter1(false),
                         IEventParameter2(0));
    }
  }
  else if ( this->isWriteable() )
  {
    handle().sendEvent(EM_SETREADONLY,
                       IEventParameter1(true),
                       IEventParameter2(0));
  }
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::disableDataUpdate                                               |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::disableDataUpdate()
{
  enableDataUpdate(false);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::setLimit                                                        |
|                                                                              |
| Sets the maximum number of characters allowed in the entry field.            |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setLimit( unsigned long ulLimit )
{
  IASSERTSTATE(text().length() <= ulLimit);

  if (this->limit() != ulLimit)
  {
    /*************************************************************/
    /* Now set the new edit limit for the entry field.           */
    /*************************************************************/
    handle().sendEvent(EM_SETTEXTLIMIT,
                       IEventParameter1(ulLimit),
                       IEventParameter2(0));
#ifdef IC_WIN
    fEntryFieldData->charLimit = ulLimit;
#endif

    setLayoutDistorted(IWindow::minimumSizeChanged, 0);
  }

  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::setLimit                                                        |
|                                                                              |
| Sets the maximum number of characters allowed in the entry field given a     |
| string resource-id that defines the limit.                                   |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setLimit( const IResourceId& residTextLimit )
{
  IString strTextLimit =
     residTextLimit.resourceLibrary().loadString(residTextLimit);
  setLimit(strTextLimit.asInt());
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::limit                                                           |
|                                                                              |
| Returns the maximum number of characters allowed to entered into the entry   |
| field.                                                                       |
------------------------------------------------------------------------------*/
unsigned long IEntryField::limit() const
{
#ifdef IC_PM
  WNDPARAMS     wprmData;
  ENTRYFDATA    efdCtlData;
  unsigned long ulClTextLimit = 0;

  wprmData.fsStatus = WPM_CTLDATA;          //Get control data
  wprmData.pCtlData = &efdCtlData;          //Area for control data
  wprmData.cbCtlData = sizeof(ENTRYFDATA);  //Size of control data

  /*******************************************************************/
  /* Because of an error in PMMERGE.DLL on an OS2J machine, do       */
  /* NOT check the return value of this call.  On an OS2J machine    */
  /* the return code from PM is 1, but we are getting 0 back here    */
  /* due to the EBX register getting trashed in PMMERGE.  Hence, we  */
  /* throw an exception when we shouldn't.                           */
  /*******************************************************************/
  handle().sendEvent(WM_QUERYWINDOWPARAMS,
                     IEventParameter1(&wprmData),
                     IEventParameter2(0));

  ulClTextLimit = efdCtlData.cchEditLimit;  // Edit limit
                                   // Note: wprmData.fsStatus does not get
                                   //       cleared on success, as advertised!
  return( ulClTextLimit );
#endif

#ifdef IC_WIN
  return( fEntryFieldData->charLimit );  //Return local variable since
                                         //EM_GETTEXTLIMIT doesn't work
#endif
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IEntryField::setText                                                         |
|                                                                              |
| Changes the control text using the string table.                             |
------------------------------------------------------------------------------*/
IEntryField&  IEntryField::setText(const IResourceId& residText)
{
  IString str(residText.resourceLibrary().loadString(residText));
  setText((char*)str);
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::setText                                                         |
|                                                                              |
| Changes the control text given a string.                                     |
------------------------------------------------------------------------------*/
IEntryField&  IEntryField::setText(const char* pszText)
{
   IString newText(pszText);
   /**************************************************************/
   /* Windows won't check the string length against the limit, so*/
   /* we have to do it ourselves.                                */
   /* If the text is too long, truncate it like in OS/2.         */
   /**************************************************************/
   Inherited::setText( newText.subString( 1,
                       (unsigned int)min(newText.length(), limit())) );
   return *this;
}
#endif    // IC_WIN


/*------------------------------------------------------------------------------
| IEntryField::selectRange                                                     |
|                                                                              |
| Marks (selects) text in the entry field.  Index of characters to mark is     |
| assumed to be 0-based.                                                       |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::selectRange( const IRange& range,
                                       unsigned long timestamp )
{
  //Make upper the character following the last selected one
  IRange temprange = IRange(range.lowerBound(), range.upperBound() + 1);

  if (range.upperBound() == end)
    temprange.setUpperBound(text().length() + 1);

  setCursorInfo(temprange);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::selectedRange                                                   |
|                                                                              |
| Returns the range of the selected text.  Index of characters marked is       |
| assumed to be 0-based.                                                       |
------------------------------------------------------------------------------*/
IRange IEntryField::selectedRange() const
{
  IASSERTSTATE(hasSelectedText());
  IRange rangMarked = getCursorInfo();
  if (rangMarked.lowerBound() != rangMarked.upperBound())
     rangMarked.setUpperBound(rangMarked.upperBound() - 1);
                                //Make upper the last selected character
  return( rangMarked );
}


/*------------------------------------------------------------------------------
| IEntryField::selectedTextLength                                              |
|                                                                              |
| Returns the size of the user marked area in bytes.  Does not count '\0'.     |
------------------------------------------------------------------------------*/
unsigned long IEntryField::selectedTextLength() const
{
  IRange rangMarked = getCursorInfo();

  return (rangMarked.upperBound() - rangMarked.lowerBound());
}


/*------------------------------------------------------------------------------
| IEntryField::selectedText                                                    |
|                                                                              |
| Returns the selected text.                                                   |
------------------------------------------------------------------------------*/
IString IEntryField::selectedText() const
{
  long    lMinOffset;     //0-based offset of 1st char in selection
  long    lMaxOffset;     //0-based offset of 1st char AFTER selection

  IRange rangMarked = getCursorInfo();

  if (rangMarked.lowerBound() == rangMarked.upperBound())
  {                                    //No selected text
    IString str("\0");
    return( str );
  }
  else
  {
    lMinOffset = rangMarked.lowerBound();
    lMaxOffset = rangMarked.upperBound();

    IString str((char *)text() + lMinOffset, (unsigned int)
                (lMaxOffset - lMinOffset), '\0');
    return( str );
  }
}


/*------------------------------------------------------------------------------
| IEntryField::setLeftIndex                                                    |
|                                                                              |
| Set the first character displayed at the left edge of the entry field        |
| control.  The index is 0-based.                                              |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setLeftIndex( unsigned long ulIndex )
{
#ifdef IC_PM
  IEventResult evt = handle().sendEvent(EM_SETFIRSTCHAR,
                                        IEventParameter1(ulIndex),
                                        IEventParameter2(0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_SETFIRSTCHAR");
#endif
#ifdef IC_WIN
   // Since we cannot get the current index, scroll to start of text
   // and then scroll forward the requested amount
   handle().sendEvent( EM_LINESCROLL,
                       IEventParameter1(-65000),
                       IEventParameter2(0) );
   handle().sendEvent( EM_LINESCROLL,
                       IEventParameter1(ulIndex),
                       IEventParameter2(0) );
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::leftIndex                                                       |
|                                                                              |
| Get the offset position of the first character displayed at the left edge    |
| of the entry field.  The index is 0-based.                                   |
------------------------------------------------------------------------------*/
unsigned long IEntryField::leftIndex() const
{
  IEventResult evt = handle().sendEvent(EM_QUERYFIRSTCHAR,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  return( evt.asUnsignedLong() );
}


/*------------------------------------------------------------------------------
| IEntryField::setStyle                                                        |
|                                                                              |
| Replaces the style of the entry field control, while preserving the          |
| cursor position and selected text.                                           |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setStyle( unsigned long ulStyle )
{
  IRange range = getCursorInfo();
  IWindow::setStyle(ulStyle);
  setCursorInfo(range);
  return( *this );
}


/*------------------------------------------------------------------------------
| IEntryField::calcMinimumSize                                                 |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The height is      |
| based on the font height and the width is based on the average font width    |
| multiplied by the text limit.                                                |
------------------------------------------------------------------------------*/
ISize IEntryField::calcMinimumSize() const
{
  ISize sizChar = ((IEntryField *)this)->characterSize();
                                            //Cast to non-const object
  unsigned long ulWidth = sizChar.width();  //Average char width
  unsigned long ulHeight= sizChar.height();

  // Hack similar to the one in ITextControl::displaySize.  This will
  // keep entry fields with borders and static text controls, which use
  // the same font, about equal in height on a multi-cell or set canvas.
#ifdef IC_PM
  ulHeight += 2;
#endif
#ifdef IC_WIN
  if ( !(IPlatform::isWin95() || IPlatform::isNTNewShell()) )
  {
    // For some reason we need a fudge factor of 7 for window platforms
    // that are using program manager. If this fudge factor is not added then
    // the entryfield will not accept focus and you can not type in it.
    ulHeight += 7;
  }
#endif
  unsigned long ulClTextLimit = limit();

  /*****************************************************************/
  /* Use a step function to reflect the fact that you are more     */
  /* likely to exceed an average character width with an           */
  /* individual character, but a long string will likely be ok     */
  /* with this average value.                                      */
  /*****************************************************************/

  IFont font(this);
  if ( !font.isFixed() )
  {
     // DEFECT 8842 : earlier algorithm gave uneven length increases;
     // simplify to a smooth curve of lengths over text limit range.
     if (ulClTextLimit <= 2)
        ulWidth = (unsigned long)(ulWidth * 3);
     else if (ulClTextLimit <=16)
        ulWidth = (unsigned long)(ulWidth + 5);
     else if (ulClTextLimit <=24)
        ulWidth = (unsigned long)(ulWidth + 4);
     else if (ulClTextLimit <=32)
        ulWidth = (unsigned long)(ulWidth + 3);
     else if (ulClTextLimit <=40)
        ulWidth = (unsigned long)(ulWidth + 2);
     else if (ulClTextLimit <=48)
        ulWidth = (unsigned long)(ulWidth + 1);
     ulWidth = ulWidth * ulClTextLimit;
  }  // is proportional
  else
  {
     ulWidth = (unsigned long)(ulWidth * ulClTextLimit);
  }  // not proportional

  if ( ulWidth > ( sizChar.width() * 35 ))
  {
     ulWidth = sizChar.width() * 35;
  }

  if (isMargin())
  {
     ulWidth +=(3 * IQUERYSYSVALUE(SV_CXBORDER));
     ulHeight+=(3 * IQUERYSYSVALUE(SV_CYBORDER));
  } /* endif */

  ISize sizMin(ulWidth, ulHeight);

  if ( ( extendedStyle() & border3D.asExtendedUnsignedLong() ) &&
       ( IPlatform::isWin95() || IPlatform::isNTNewShell() ))
  {
     // 22733 : use same API as that used in textctl::displaySize()
     int cx = IQUERYSYSVALUE( SV_CXSIZEBORDER );
     int cy = IQUERYSYSVALUE( SV_CYSIZEBORDER );

     sizMin += ISize(cx,cy);
  }

  return( sizMin );
}


/*------------------------------------------------------------------------------
| IEntryField::cursorPosition                                                  |
|                                                                              |
| Returns the character position of where the cursor currently is.             |
------------------------------------------------------------------------------*/
unsigned long IEntryField::cursorPosition() const
{
  IEventResult evt = handle().sendEvent(EM_QUERYSEL,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  return( evt.number2() );
}


/*------------------------------------------------------------------------------
| IEntryField::setCursorPosition                                               |
|                                                                              |
| Position the cursor in front of the character specified.                     |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setCursorPosition( unsigned long lCursorChar )
{
   IEventResult evt;

#ifdef IC_PM
   evt = handle().sendEvent(EM_SETSEL,
                      IEventParameter1((unsigned short)lCursorChar,
                                       (unsigned short)lCursorChar),
                      IEventParameter2(0));

   if (!(evt.asUnsignedLong()))
      ITHROWGUIERROR("EM_SETSEL");
#endif
#ifdef IC_WIN
   evt = handle().sendEvent(EM_SETSEL,
                      IEventParameter1(lCursorChar),
                      IEventParameter2(lCursorChar));
#endif

   //----------------------------------------------------------------
   // DEFECT 25702 : if not in view, scroll to 1st char position on line.
   //----------------------------------------------------------------
#ifndef IC_WIN
   unsigned long ulIndex;
   ulIndex = ( (lCursorChar > 0 ? lCursorChar-1 : 0) );
   evt = handle().sendEvent(EM_SETFIRSTCHAR,
                            IEventParameter1(ulIndex),
                            IEventParameter2(0));
#else
   evt = handle().sendEvent(EM_SCROLLCARET,
                            IEventParameter1(0),
                            IEventParameter2(0));
#endif
   //----------------------------------------------------------------

   return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::getCursorInfo                                                   |
|                                                                              |
| Gets the bound of the marked or selected text, which also includes where     |
| the cursor is placed.  (The values in the returned range have not been       |
| adjusted to match the format used by selectedRange.)                         |
------------------------------------------------------------------------------*/
IRange IEntryField::getCursorInfo() const
{
  IEventResult evt = handle().sendEvent(EM_QUERYSEL,
                                        IEventParameter1(0),
                                        IEventParameter2(0));

  return( IRange(evt.number1(), evt.number2()) );
}


/*------------------------------------------------------------------------------
| IEntryField::setCursorInfo                                                   |
|                                                                              |
| Marks (selects) text in the entry field, and sets the position of the cursor.|
------------------------------------------------------------------------------*/
void IEntryField::setCursorInfo( const IRange& range )
/***************************************************************/
/* Marks (selects) texts in the entry field, and also sets     */
/* the position of the cursor.                                 */
/***************************************************************/
{
#ifdef IC_PM
  IEventResult evt = handle().sendEvent(EM_SETSEL,
                      IEventParameter1((unsigned short)range.lowerBound(),
                                       (unsigned short)range.upperBound()),
                      IEventParameter2(0));

  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("EM_SETSEL");
#endif
#ifdef IC_WIN
  IEventResult evt = handle().sendEvent(EM_SETSEL,
                      IEventParameter1((int)range.lowerBound()),
                      IEventParameter2((int)range.upperBound()));
#endif
}


/*------------------------------------------------------------------------------
| IEntryField::foregroundColor                                                 |
|                                                                              |
| Returns the foreground color of the IEntryField.                             |
------------------------------------------------------------------------------*/
IColor IEntryField::foregroundColor () const
{
  IGUIColor guiColor(IGUIColor::windowText);

  if (!isWriteable())
  {
    guiColor = IGUIColor(IGUIColor::outputText);
  }
  return (IWindow::color(PP_FOREGROUNDCOLOR, guiColor));
}

/*------------------------------------------------------------------------------
| IEntryField::backgroundColor                                                 |
|                                                                              |
| Returns the background color of the IEntryField.                             |
------------------------------------------------------------------------------*/
IColor IEntryField::backgroundColor () const
{
  return (IWindow::color(PP_BACKGROUNDCOLOR,
                         IGUIColor(IGUIColor::entryFieldBgnd)));
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IEntryField::isDragStarting                                                  |
|                                                                              |
| Windows does not generate drag messages, so therefore it is up to a          |
| a control to decide if the conditions exist to start a drag. A drag          |
| occurs if text is selected in the entry field and a button down occurs       |
| whose position is within the selected text rectangle.                        |
|                                                                              |
| Note: if you don't want the event to continue up the owner chain,            |
|       then set the event result to true.                                     |
------------------------------------------------------------------------------*/
Boolean IEntryField::isDragStarting( IEvent &event )
{
  IMODTRACE_DEVELOP("IEntryField::isDragStarting");
  Boolean brc = false;

  switch ( event.eventId() )
    {
    case WM_LBUTTONDOWN:
      {
      ITRACE_DEVELOP("Got a left button down, start to look for drag");

      // only allow drag operation if text selected
      if (hasSelectedText())
        {
        ITRACE_DEVELOP("Has selected text...");
        // If the mouse is in the selected text, initiate the drag
        if (fEntryFieldData->pointInSelectedText(event))
          {
          IMouseClickEvent mevt( event );
          ITRACE_DEVELOP("Mouse is in selected range");
          ITRACE_DEVELOP("We have a drag" );
          // We are going to start the drag from here instead of returning true, because
          // we need to control the lifetime of the text being selected.
          event.window()->sendEvent( WM_BEGINDRAG,
               IEventParameter1((unsigned short)mevt.mousePosition().x(),
                                (unsigned short)mevt.mousePosition().y()),
               IEventParameter2( 0 ));
          selectRange(IRange(-1,0));
          }
        }
      break;
      }
    case WM_RBUTTONDOWN:
      {
      ITRACE_DEVELOP("Got a right button down, start to look for drag");

      // only allow drag operation if text selected
      if (hasSelectedText())
        {
        // If the mouse is in the selected text, initiate the drag
        if (fEntryFieldData->pointInSelectedText(event))
          {
          ITRACE_DEVELOP("Mouse is in selected range");
          if ( ::GetCapture() == NULL )
            {
            IMouseClickEvent mevt( event );
            ::SetCapture( event.handle());
            fEntryFieldData->bCheckingRightDragStart = True;
            fEntryFieldData->pointRButtonDown = mevt.mousePosition();
            }
          }
        }
      break;
      }
    case WM_RBUTTONUP:
      {
      if ( fEntryFieldData->bCheckingRightDragStart )
        {
        ::ReleaseCapture();
        SetCapture( NULL );
        fEntryFieldData->bCheckingRightDragStart = False;
        fEntryFieldData->pointRButtonDown = IPoint(0,0);
        }
      break;
      }
    case WM_MOUSEMOVE:
      {
      if ( fEntryFieldData->bCheckingRightDragStart )
        {
        ITRACE_DEVELOP("Mouse move while checking drag start");
        IMouseClickEvent mevt( event );
        IPoint mPoint( fEntryFieldData->pointRButtonDown );

        if ( abs((int)(mevt.mousePosition().x() - mPoint.x())) >
                 DD_DEFDRAGMINDIST ||
             abs((int)(mevt.mousePosition().y() - mPoint.y())) >
                 DD_DEFDRAGMINDIST )
          {
          /*-----------------------------------------
          | We are going to start the drag from here instead
          | of returning true, because we need to control
          | the lifetime of the text being selected.
          -----------------------------------------*/
          ITRACE_DEVELOP("Starting a drag" );
          ::ReleaseCapture();
          fEntryFieldData->bCheckingRightDragStart = FALSE;
          event.window()->sendEvent( WM_BEGINDRAG,
             IEventParameter1((unsigned short)mevt.mousePosition().x(),
                              (unsigned short)mevt.mousePosition().y()),
             IEventParameter2( 0 ));
          selectRange(IRange(-1,0));
          }
        }
      break;
      }
    }
  return brc;
}
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IEntryFieldData::pointInSelectedText                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IEntryFieldData::pointInSelectedText(const IEvent& evt)
{
  IMODTRACE_DEVELOP("IEntryFieldData::pointInSelectedText");

  // Get the handle to the edit control
  IEntryField *pEditCntrl = (IEntryField *)evt.window();

  POINT pt;
  RECT rect, selRect;
  selRect.left = selRect.right = selRect.top = selRect.bottom = 0;

  IMouseClickEvent mevt( evt );

  // Get mouse position when button down
  IPoint pmPt = mevt.mousePosition();

  // convert if necessary.
  if ( ICoordinateSystem::applicationOrientation() ==
       ICoordinateSystem::originLowerLeft )
    {
    pmPt.setY(evt.window()->size().height() - mevt.mousePosition().y());
    }

  // get the range of the selected text
  IRange range = pEditCntrl->selectedRange();

  if ((IPlatform::isWin95()) || (IPlatform::isNTNewShell()))
    {
    ITRACE_DEVELOP("Platform is Win95 or NT newshell");
    // Retrieve the zero-based character index of the character nearest the
    // mouse point on the button down.
    // Note:  This message is currently supported only in Windows 95 and NT Newshell
    IEventResult charEvt = pEditCntrl->handle().sendEvent(EM_CHARFROMPOS, 0,
        IEventData((unsigned short)pmPt.x(), (unsigned short)pmPt.y()));

    long position = (long)charEvt.number1();

    ITRACE_DEVELOP("Character position of mouse in EF: " + IString(position));
    ITRACE_DEVELOP("Range: " + IString(range.lowerBound()) + "-" +
      IString(range.upperBound()));

    // Return true if the mouse position is within the selected range
    return (range.includes(position));
    }


  if (GetClientRect(pEditCntrl->handle(), &rect))
  {
    HDC hDC = GetDC(pEditCntrl->handle());
    IString text = pEditCntrl->text();

    IString selText = pEditCntrl->selectedText();

    SIZE sizeSel, sizeLead;
    sizeSel.cx = sizeSel.cy = sizeLead.cx = sizeLead.cy = 0;
    if (IPlatform::isWin32S())
    {
      ITRACE_DEVELOP("Win32s: Call GetTextExtentPoint not GetTextExtentPoint32");
      GetTextExtentPoint(hDC, selText, selText.length(), &sizeSel);
    }
    else
    {
      GetTextExtentPoint32(hDC, selText, selText.length(), &sizeSel);
    }
    IString leadingText("");
    if (range.lowerBound() > 0)
      leadingText = text.subString(0, (unsigned int)range.lowerBound());

    if (IPlatform::isWin32S())
    {
      ITRACE_DEVELOP("Win32s: Call GetTextExtentPoint not GetTextExtentPoint32");
      GetTextExtentPoint(hDC, leadingText, leadingText.length(), &sizeLead);
    }
    else
    {
      GetTextExtentPoint32(hDC, leadingText, leadingText.length(), &sizeLead);
    }

    // build the selection rectangle
    selRect = rect;
    // update the left and right positions to equate to the selection rectangle
    selRect.left = sizeLead.cx;
    selRect.right = sizeLead.cx + sizeSel.cx;

    ReleaseDC(pEditCntrl->handle(), hDC);
  }

  pt.x = pmPt.x();
  pt.y = pmPt.y();
  return (PtInRect(&selRect, pt));
}
#endif

#ifdef IC_PMWIN   // DEFECT 4624 : required for PM and WIN
/*------------------------------------------------------------------------------
| IEntryHandler::IEntryHandler                                                 |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IEntryHandler::IEntryHandler()
{ }

/*------------------------------------------------------------------------------
| IEntryHandler::~IEntryHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IEntryHandler::~IEntryHandler()
{ }

/*------------------------------------------------------------------------------
| IEntryHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Intercepts enter or escape key event and sends it to the owner.              |
| If there is no owner the event is sent to the parent.                        |
| Needed to emulate single line control but still get alignment                |
| characteristics.                                                             |
| Also achieves dialog behavior in canvases and frames without                 |
| use of IsDialogMessage in the event loop.                                    |
|                                                                              |
| Intercepts control color events                                              |
|                                                                              |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean IEntryHandler::dispatchHandlerEvent( IEvent& event )
{
  IMODTRACE_ALL("IEntryHandler::dispatchHandlerEvent");
  Boolean
    result = false;

  switch( event.eventId() )
  {
    case WM_CHAR:
    {
#ifdef IC_PM
      //--------------------------------------------------------------
      // DEFECT 4624 : Override auto-tabbing for PM
      //--------------------------------------------------------------
      IKeyboardEvent keyevt(event);
      IKeyboardEvent::VirtualKey vkKey;
      if (keyevt.isCharacter()  &&
          (keyevt.isVirtual() == false  ||
          ((vkKey = keyevt.virtualKey()) != IKeyboardEvent::tab  &&
         vkKey != IKeyboardEvent::backSpace  &&
         vkKey != IKeyboardEvent::esc)) )
      {
         IEntryField* ef = (IEntryField*)event.window();
         unsigned long efLimit = ef->limit() - 1;
         if ( ef->isAutoTab() && (ef->textLength() == efLimit) &&
              (ef->cursorPosition() == efLimit) )
         {
            ef->sendEvent(WM_CHAR,
                               IEventParameter1((unsigned short)KC_VIRTUALKEY,0,0),
                               IEventParameter2((unsigned short)0,VK_TAB,0));
         }
      }
      //--------------------------------------------------------------
      if ( (unsigned long)event.parameter1() == 0x000D )
        return true;
#endif   // IC_PM

#ifdef IC_WIN
      unsigned long ep1 = (unsigned long)event.parameter1();
      if ((ep1 == VK_RETURN ) || (ep1 == VK_ESCAPE ) || (ep1 == VK_TAB ))
      {
        IWindow* window = event.window();
        IWindow* owner  = window->owner();
        if (!owner)
          owner = window->parent();
  #ifdef IC_WIN
        // If owner is a combobox, then get its owner to send to
        // because on Windows, the combobox will just send it back.
        if ( IWindowClassName( owner->handle() ) == WC_COMBOBOX )
        {
          window = owner;
          owner  = window->owner();
          if (!owner)
            owner = window->parent();
        }
  #endif
        owner->sendEvent(event);
        return true;
      }
      // Lets emulate auto-tabbing for Windows
      else if ((ep1 != VK_BACK ) && (ep1 != VK_BREAK ))
      {
        // Check if autotab style is set and if when this key was
        // pressed we are one character short of the limit (meaning that
        // after this key is processed, we will be at our limit and
        // that the cursor is in the last position in the field
        IEntryField* ef = (IEntryField*)event.window();
        unsigned long efLimit = ef->limit() - 1;
        if ( ef->isAutoTab() && (ef->textLength() == efLimit) &&
             (ef->cursorPosition() == efLimit) )
        {
          ef->postEvent( WM_CHAR, VK_TAB );
        }
      }
#endif   // IC_WIN
      break;
    }

    // Adding code to trap accelerator/mnemonics and send directly to
    // owner (allowing it to naturally progress through dispatcher results
    // in an audible beep caused by the default window procedure because
    // it cannot find a match for a mnemonic that we process in
    // ICanvas or IFrameWindow), eating original event.
    case WM_SYSCHAR:
    {
      IWindow* window = event.window();
      IWindow* owner = window->owner();
      if (!owner)
        owner = window->parent();
      if (owner)
      {
        owner->sendEvent( event );
        result = true;
      }
      break;
    }

    case WM_CTLCOLOREDIT:
    case WM_CTLCOLORSTATIC:
    {
      IEntryField* ctlwin = (IEntryField*)
           IWindow::windowWithHandle(IWindowHandle(event.parameter2()) );
      IColor clrFgnd = ctlwin->foregroundColor();
      IColor clrBgnd = ctlwin->backgroundColor();

      // Set foreground and background text colors in device context
      SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
      SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

      // If a previous background brush set, delete it
      if ( ctlwin->fEntryFieldData->bkgrndbrush != 0 )   // 25227
           IBrushManager::freeBrush( ctlwin->fEntryFieldData->bkgrndbrush );

      // Create a new background brush using the color retrieved
      ctlwin->fEntryFieldData->bkgrndbrush =
           IBrushManager::useBrush( clrBgnd.asRGBLong() );

      // Return HBRUSH for painting controls background
      event.setResult( ctlwin->fEntryFieldData->bkgrndbrush );
      result = true;
      break;
    }

    default:
      break;
  }
  return result;
}
#endif   // IC_PMWIN

