/*******************************************************************************
* FILE NAME: ifiledlg.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifiledlg.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINSTDFILE
  #define INCL_WINWINDOWMGR
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #include <iwindefs.h>
  #include <string.h>
  #include <stdlib.h>
}

#include <ifiledlg.hpp>
#include <ifildprv.hpp>
#ifdef IC_PM
#include <ipmexsta.hpp>
#endif
#include <icoordsy.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <ihandler.hpp>
#include <iobjwin.hpp>
#include <ireslib.hpp>
#include <ithread.hpp>
#include <itrace.hpp>
#ifdef IC_WIN
#include <icolor.hpp>
#endif // IC_WIN
#include <ibrshmgr.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFILEDLG_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_WIN
#define IC_ENABLELISTBOX_SET 0x0008
#define IC_OPEN_SET        0x0010
#endif

#pragma data_seg(ICLStaticConst)
const IFileDialog::Style
  IFileDialog::helpButton        = FDS_HELPBUTTON,             // 0x00000008
  IFileDialog::multiSelection    = FDS_MULTIPLESEL,            // 0x00000400
  IFileDialog::filter            = FDS_FILTERUNION,            // 0x00000004
  IFileDialog::applyButton       = FDS_APPLYBUTTON,            // 0x00000010
  IFileDialog::includeEAS        = FDS_INCLUDE_EAS,            // 0x00000080
#ifdef IC_PM
  IFileDialog::modeless          = FDS_MODELESS,               // 0x00000040
#endif
#ifdef IC_WIN
  IFileDialog::modeless          ( 0, IFDS_MODELESS ),
#endif
  IFileDialog::preload           = FDS_PRELOAD_VOLINFO,        // 0x00000020
  IFileDialog::selectableListbox = FDS_ENABLEFILELB,           // 0x00000800
  IFileDialog::noStyle           = 0,
  IFileDialog::classDefaultStyle = 0;

const unsigned long
  IFileDialog::ok                = DID_OK,
  IFileDialog::cancel            = DID_CANCEL;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IFileDialog::Style IFileDialog::currentDefaultStyle = 0;
#pragma data_seg()

#ifdef IC_WIN
#define IC_OKBUTTON_SET    0x0001
#define IC_POSITION_SET    0x0004
#endif
//
// Implementation for nested Settings class
//

/*------------------------------------------------------------------------------
| IFileDialog::Settings::Settings                                              |
------------------------------------------------------------------------------*/
IFileDialog::Settings::Settings( ) :
  fSettingsData( new IFileDialogSettingsData ),
  titleStr(),
  OKButtonStr(),
  initialDriveStr(),
  initialFileTypeStr(),
  fileNameStr(),
  isOpenDlg(true),
  isPosSet(false)
{
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setTitle                       |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setTitle(const IResourceId& id)
{
  IString newText = id.resourceLibrary().loadString(id);
  if (newText.length())
    setTitle(newText);
  return *this;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setOKButtonText                |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setOKButtonText(const IResourceId& id)
{
  IString newText = id.resourceLibrary().loadString(id);
  if (newText.length())
    setOKButtonText(newText);
  return *this;
}

#ifndef IC_WIN
/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::addFileType                    |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::addFileType(const char* fileType)
{
   if (fSettingsData->fileTypes == 0)
     fSettingsData->fileTypes = new StringSeq;

   IString* pString = new IString(fileType);
   fSettingsData->fileTypes->add(pString);

   return *this;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::addDrive                       |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::addDrive(const char* drive)
{
   IMODTRACE_DEVELOP("IFileDialog::Settings::addDrive");
   if (fSettingsData->drives == 0)
     fSettingsData->drives = new StringSeq;

   IString* pString = new IString(drive);
   fSettingsData->drives->add(pString);

   return *this;
}
#endif


/*------------------------------------------------------------------------------
| IFileDialog::Settings::~Settings                                             |
------------------------------------------------------------------------------*/
IFileDialog::Settings::~Settings()
{
  IMODTRACE_DEVELOP("~IFileDialog::Settings");
  delete fSettingsData->fileTypes;
  delete fSettingsData->drives;
  delete fSettingsData;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setTitle                      |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setTitle (const char* newTitle)
{
  titleStr = newTitle;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setOKButtonText               |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setOKButtonText(const char* newText)
{
  OKButtonStr = newText;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setFileName                    |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setFileName(const char* fileName)
{
#ifdef IC_WIN
  IString temp( fileName );

  if ( temp.lastIndexOfAnyOf( ":\\" ) )
  {
    /******************************************************************/
    /* Get just path name out of the string (text up to last token)   */
    /******************************************************************/
    IString pathName = temp.subString( 1, temp.lastIndexOfAnyOf( ":\\" ));
    IString justFile = temp.subString( temp.lastIndexOfAnyOf( ":\\" )+1 );

    fSettingsData->fDirectory = pathName;
    fileNameStr = justFile;
    fSettingsData->isDirectorySet = true;
  }
  else
#endif
  fileNameStr = fileName;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setInitialDrive                |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setInitialDrive(const char* drive)
{
  initialDriveStr = drive;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setInitialFileType            |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setInitialFileType(const char* type)
{
  initialFileTypeStr = type;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings::setDialogTemplate                                     |
------------------------------------------------------------------------------*/
IFileDialog::Settings&
  IFileDialog::Settings::setDialogTemplate ( const IResourceId& resId )
{
  fSettingsData->isDlgTemplateSet = true;
  fSettingsData->dlgId = resId.id();
  fSettingsData->dlgHandle = resId.resourceLibrary().handle();
  return *this;
}

/*------------------------------------------------------------------------------
| StringSeq*  IFileDialog::Settings::fileTypes ( ) const                       |
------------------------------------------------------------------------------*/
StringSeq*  IFileDialog::Settings::fileTypes ( ) const
{
  return fSettingsData->fileTypes;
}

#ifndef IC_WIN
/*------------------------------------------------------------------------------
| StringSeq*  IFileDialog::Settings::drives( ) const                           |
------------------------------------------------------------------------------*/
StringSeq*  IFileDialog::Settings::drives( ) const
{
  return fSettingsData->drives;
}
#endif

/*------------------------------------------------------------------------------
| IBase::Boolean  IFileDialog::Settings::isDialogTemplateSet ( ) const         |
------------------------------------------------------------------------------*/
IBase::Boolean  IFileDialog::Settings::isDialogTemplateSet ( ) const
{
  return fSettingsData->isDlgTemplateSet;
}

/*------------------------------------------------------------------------------
| _IFileDlgProc                                                                |
|                                                                              |
| Override the WinFileDlg dialog procedure so that the handler scheme          |
| will work for Modal dialogs.                                                 |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
void* __stdcall _IFileDlgProc( void*
#else
void*   _System _IFileDlgProc( unsigned long
#endif
                                             hwnd,
                               unsigned long msg,
                               void* mp1, void* mp2 )
{
  void* result(FALSE);       // default return value
#ifdef IC_PM
   /*****************************************************************/
   /* Get the "this" pointer for window object from the window word */
   /* which was set by WM_CREATE in above.                          */
   /*****************************************************************/
   FILEDLG* pfildData = (FILEDLG*)WinQueryWindowPtr(hwnd, QWL_USER);
   IFileDialog* pfildlg = (IFileDialog*)pfildData->ulUser;

   if (pfildlg == (IFileDialog*)0)
      return IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);
#else
   IFileDialog* pfildlg = (IFileDialog*)IQUERYWINDOWULONG( hwnd, DWL_USER );
   if ((pfildlg == 0) && (msg != WM_INITDLG))     // No window object to pass
      return FALSE;
#endif

   switch(msg)
   {
#ifdef IC_WIN
      case WM_CTLCOLORDLG:
      case WM_CTLCOLORSTATIC:
      case WM_CTLCOLORBTN:
      {
        IColor clrBgnd = pfildlg->backgroundColor();
        IEvent evt(hwnd, msg, mp1, mp2);

        // Set background colors in device context
        SetBkColor( (HDC)evt.parameter1(), clrBgnd.asRGBLong() );

        // Return HBRUSH for painting controls background
        if ( pfildlg->fFileDialogData->fbkgrndbrush != 0 )  // 25227
            IBrushManager::freeBrush( pfildlg->fFileDialogData->fbkgrndbrush );
        pfildlg->fFileDialogData->fbkgrndbrush =
            IBrushManager::useBrush( clrBgnd.asRGBLong() );

        return( pfildlg->fFileDialogData->fbkgrndbrush );
      }
      break;
#endif //IC_WIN

      case WM_INITDLG:
      {
#ifdef IC_WIN
         /*************************************************************/
         /* Get the "this" pointer for font dialog object from lparam */
         /* (saved in custData) and typecast for usage.               */
         /*************************************************************/
         OPENFILENAME* cfs = (OPENFILENAME*)mp2;
         pfildlg = (IFileDialog*)cfs->lCustData;

         /*************************************************************/
         /* Since this is only passed on WM_INITDIALOG, save in the   */
         /* user window word so that we can retrieve it on other msgs */
         /*************************************************************/
         ISETWINDOWULONG( hwnd, DWL_USER, (LPARAM)pfildlg );

         /*************************************************************/
         /* Modify the preview text string, if one was set            */
         /*************************************************************/
         if (pfildlg->fFileDialogData->fSettingsChanged & IC_OKBUTTON_SET)
           ISETWINDOWTEXT( IWINDOWFROMID( hwnd, IDOK ),
                           pfildlg->fFileDialogData->fOKButton );

         /*************************************************************/
         /* If selectableListbox was requested on Save as, enable it. */
         /* Otherwise, disable it.                                    */
         /*************************************************************/

         if (!(pfildlg->fFileDialogData->fSettingsChanged & IC_OPEN_SET))
         {
            if (pfildlg->fFileDialogData->fSettingsChanged & IC_ENABLELISTBOX_SET)
               IENABLEWINDOW(IWINDOWFROMID( hwnd, lst1), true);
            else
               IENABLEWINDOW(IWINDOWFROMID( hwnd, lst1), false);
         }
         /*************************************************************/
         /* If the dialog position was set, move it to that spot      */
         /*************************************************************/
         if (pfildlg->fFileDialogData->fSettingsChanged & IC_POSITION_SET)
         {
           /***********************************************************/
           /* Get the position requested which is relative to the     */
           /*  owner window and convert it to screen coordinates      */
           /***********************************************************/
           RECT rectOwner;
           IPoint pos = pfildlg->fFileDialogData->fPosition;
           GetWindowRect( cfs->hwndOwner, &rectOwner );
           pos.setX( rectOwner.left + pos.x() );

           /***********************************************************/
           /* If coordinate system is different, recoordinate here    */
           /***********************************************************/
           if (ICoordinateSystem::applicationOrientation() ==
                                      ICoordinateSystem::originLowerLeft )
           {
             RECT rectDlg;
             GetWindowRect( hwnd, &rectDlg );
             pos.setY( rectOwner.bottom - pos.y() -
                       (rectDlg.bottom - rectDlg.top) );
           }
           else
             pos.setY( rectOwner.top + pos.y() );

           /***********************************************************/
           /* Once position is calculated in screen coord,move window */
           /***********************************************************/
           ISETWINDOWPOS( hwnd, NULL, pos.x(), pos.y(), 0, 0,
                          SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE );
        }
        if (cfs->hwndOwner)
           {
           // Insure that owner is behind this window
           ISETWINDOWPOS( cfs->hwndOwner, hwnd, 0,0,0,0,
                          SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE );
           }
        /*************************************************************/
        /*  Unless there is a user provided handler that processes   */
        /*  WM_INITDIALOG, we want to return true to Windows.  This  */
        /*  tells Windows to set focus to the dialog.                */
        /*************************************************************/
        result = (void*)1;
#endif
        pfildlg->addDefaultHandler();    // for IFrameWindow

        pfildlg->addToWindowSet(pfildlg, IWindowHandle(hwnd));

        if (pfildlg->fFileDialogData->handlerToAdd != 0)
           pfildlg->fFileDialogData->handlerToAdd->handleEventsFor(pfildlg);
     }
        break;

     case WM_COMMAND:
     {
       /***************************************************************/
       /* Setup id of apply pushbutton                                */
       /***************************************************************/
       unsigned short applyButton =
#ifdef IC_WIN
                                    psh3;
#else
                                    DID_APPLY_PB;
#endif
       /***************************************************************/
       /* Check if the apply button was hit for a modeless dialog     */
       /***************************************************************/
       if (pfildlg->isModeless() && (LOWORD((WPARAM)mp1) == applyButton)
           && (HIWORD((WPARAM)mp1) == BN_CLICKED))
       {
         /*************************************************************/
         /* then send message to self that file should be applied     */
         /*************************************************************/
         pfildlg->handle().sendEvent( IC_UM_DLG_APPLY_PRESSED,
#ifdef IC_WIN
                   IEventParameter1( pfildlg->pFileDlgData->lpstrFile ),
#else
                   IEventParameter1( pfildlg->pFileDlgData->szFullFile ),
#endif
                   IEventParameter2( 0 ) );
       }
#ifdef IC_WIN
       /***************************************************************/
       /* If OK button pressed, set that as return value              */
       /***************************************************************/
       if (( LOWORD((WPARAM)mp1) == DID_OK ) &&
           ( HIWORD((WPARAM)mp1) == BN_CLICKED ))
         pfildlg->fFileDialogData->fReturnVal = DID_OK;

       /***************************************************************/
       /* If Help button pressed,                                     */
       /***************************************************************/
       if (( LOWORD((WPARAM)mp1) == pshHelp ) &&
           ( HIWORD((WPARAM)mp1) == BN_CLICKED ))
       {

         /*************************************************************/
         /* then post help message for the file dialog window         */
         /*************************************************************/
         pfildlg->postEvent( IWindow::help,
                             IEventParameter1( 0 ),
                             IEventParameter2( 0 ) );
       }
#endif
     }
     case WM_SYSCOMMAND:            // Keep these from going to default
     case WM_CLOSE:                 // frame handler
#ifdef IC_WIN
         return FALSE;
#else
        return IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);
#endif

  } /* end switch */

  // call dispatch so that any handlers will get called
  IEvent newEvt(hwnd, msg, mp1, mp2);

  if (pfildlg->dispatch(newEvt))
     result = newEvt.result();
#ifdef IC_PM
   else
     result = IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);
#endif

  if(msg==WM_DESTROY)
  {
     pfildlg->removeFromWindowSet(pfildlg);
  }

  ISETWINDOWULONG(hwnd, DWL_MSGRESULT, (unsigned long)result);
  return result;
}

//
// Implementation for IFileDialog
//

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*  parent,
                           IWindow*  owner,
                           const Style& style )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  Settings tempSettings;
  initialize(parent, owner, style, 0, tempSettings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           const Settings& settings,
                           const Style&    style )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, 0, settings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           const Style&    style,
                           const Settings& settings )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, 0, settings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           IHandler*       handler,
                           const Style&    style,
                           const Settings& settings )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, handler, settings);
}


/*------------------------------------------------------------------------------
| IFileDialog::~IFileDialog                                                    |
------------------------------------------------------------------------------*/
IFileDialog::~IFileDialog( )
{
  IMODTRACE_DEVELOP("~IFileDialog");

#ifdef IC_PM
  int i;
  delete [] pFileDlgData->pszTitle;
  delete [] pFileDlgData->pszOKButton;
  delete [] pFileDlgData->pszIType;
  delete [] pFileDlgData->pszIDrive;

  if(pFileDlgData->papszITypeList)
  {
    for(i=0; *pFileDlgData->papszITypeList[i]; i++)
       delete [] *pFileDlgData->papszITypeList[i];
    delete [] pFileDlgData->papszITypeList;
  }

  if(pFileDlgData->papszIDriveList)
  {
    for(i=0; *pFileDlgData->papszIDriveList[i]; i++)
       delete [] *pFileDlgData->papszIDriveList[i];
    delete [] pFileDlgData->papszIDriveList;
  }

  if (pFileDlgData->papszFQFilename)
     IPMCtlStatics::freeFileDlgList((char**)(pFileDlgData->papszFQFilename));
#endif
#ifdef IC_WIN
  delete [] fFileDialogData->fTitle;
  delete [] pFileDlgData->lpstrFile;
#endif

  delete pFileDlgData;
  if (fFileDialogData)
    delete fFileDialogData;
}

/*------------------------------------------------------------------------------
| IFileDialog::isModeless                                                      |
------------------------------------------------------------------------------*/
Boolean IFileDialog::isModeless () const
{
#ifdef IC_PM
   return pFileDlgData->fl & FDS_MODELESS;
#endif
#ifdef IC_WIN
   return extendedStyle() & IFDS_MODELESS;
#endif
}

/*------------------------------------------------------------------------------
| IFileDialog::buttonPressedId                                                 |
------------------------------------------------------------------------------*/
unsigned long IFileDialog::buttonPressedId ( ) const
{
#ifdef IC_PM
   return pFileDlgData->lReturn;
#endif
#ifdef IC_WIN
   return fFileDialogData->fReturnVal;
#endif
}

/*------------------------------------------------------------------------------
| IFileDialog::pressedOK                                                       |
------------------------------------------------------------------------------*/
Boolean IFileDialog::pressedOK ( ) const
{
   return buttonPressedId() == DID_OK ;
}

/*------------------------------------------------------------------------------
| IFileDialog::selectedFileCount                                               |
------------------------------------------------------------------------------*/
unsigned IFileDialog::selectedFileCount() const
{
#ifdef IC_PM
   return pFileDlgData->ulFQFCount;
#endif
#ifdef IC_WIN
   /*******************************************************************/
   /* If multiple select, then need to count the number of filenames  */
   /* returned in lpstrFile buffer for Windows multifile select       */
   /*******************************************************************/
   if (pFileDlgData->Flags & FDS_MULTIPLESEL)
   {
     IString temp(pFileDlgData->lpstrFile);
     unsigned int count = temp.occurrencesOf( ' ' );
     return ( (count) ? count : 1 );
   }
   else
     return 1;
#endif
}


/*------------------------------------------------------------------------------
| IFileDialog::returnValue                                                     |
------------------------------------------------------------------------------*/
long IFileDialog::returnValue () const
{
#ifdef IC_PM
   return pFileDlgData->lSRC;
#endif
#ifdef IC_WIN
   return fFileDialogData->fSRC;
#endif
}

/*------------------------------------------------------------------------------
| IFileDialog::fileName                                                        |
------------------------------------------------------------------------------*/
IString IFileDialog::fileName (unsigned fileNumber) const
{
#ifdef IC_PM
   if (fileNumber == 0) {
     return IString(pFileDlgData->szFullFile);
   }

   if (fileNumber < 1 ||
       fileNumber > selectedFileCount() +1 ||
       !pFileDlgData->papszFQFilename )
        return IString();

  return IString((char*)(*pFileDlgData->papszFQFilename[fileNumber]));
#endif
#ifdef IC_WIN
  if ((fileNumber == 0) && (selectedFileCount() == 1))
    return IString(pFileDlgData->lpstrFile);
  else
  {
    /*****************************************************************/
    /* If an invalid fileNumber is passed, return a null string      */
    /*****************************************************************/
    if (fileNumber >= selectedFileCount() )
      return IString();

    /*****************************************************************/
    /* Create a temp string of the entire buffer returned            */
    /*****************************************************************/
    IString temp(pFileDlgData->lpstrFile);

    /*****************************************************************/
    /* Get just path name out of the string (text up to first space) */
    /*****************************************************************/
    IString pathName = temp.subString( 1, temp.indexOf( ' ' )-1 );

    /*****************************************************************/
    /* Loop through selected files to find the requested one         */
    /*****************************************************************/
    unsigned int nextIndex, i;
    unsigned int index = 0;
    for ( i=0; i <= fileNumber; i++ )
    {
      index = temp.indexOf( ' ', index+1 );
      nextIndex = temp.indexOf( ' ', index+1 );
    }

    /*****************************************************************/
    /* Finally, build the fully qualified name from the parts        */
    /*****************************************************************/
    IString currName;
    if (nextIndex)
      currName = temp.subString( index+1, (nextIndex-index-1) );
    else
      currName = temp.subString( index+1 );
    return pathName + temp.subString( 3,1 ) + currName;
  }
#endif
}

#ifndef IC_WIN
/*------------------------------------------------------------------------------
| IFileDialog::saveAsEAType                                                    |
------------------------------------------------------------------------------*/
IString IFileDialog::saveAsEAType ( ) const
{
   if ((pFileDlgData->sEAType != -1) && pFileDlgData->papszITypeList)
      return IString((char*)(*pFileDlgData->papszITypeList[pFileDlgData->sEAType]));

   return IString();
}
#endif

/*------------------------------------------------------------------------------
| IFileDialog::defaultStyle                                                    |
|                                                                              |
| Return the default style for new file dialog objects.                        |
------------------------------------------------------------------------------*/
IFileDialog::Style  IFileDialog::defaultStyle()
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IFileDialog::setDefaultStyle                                                 |
|                                                                              |
| Set the default style for new file dialog objects.                           |
------------------------------------------------------------------------------*/
void IFileDialog::setDefaultStyle( const IFileDialog::Style &style )
{
  currentDefaultStyle = style;
}
#endif

/*------------------------------------------------------------------------------
| IFileDialog::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IFileDialog::convertToGUIStyle(const IBitFlag& guiStyle,
                                             Boolean bExtOnly) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IFDS_EXTMASK;
  }
  else
  {
    // FDS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word.
    ulStyle |= guiStyle.asUnsignedLong() & IFDS_MASK;
  }

  return( ulStyle );
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IFileDialog::id                                                              |
|                                                                              |
| Get the object identity                                                      |
------------------------------------------------------------------------------*/
unsigned long IFileDialog::id() const
{
  return fFileDialogData->fDialogId;
}

/*------------------------------------------------------------------------------
| IFileDialog::setId                                                           |
|                                                                              |
| Set the object identifier                                                    |
------------------------------------------------------------------------------*/
IFileDialog& IFileDialog::setId( unsigned long ulId )
{
  fFileDialogData->fDialogId = ulId;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::backgroundColor                                                 |
|                                                                              |
| Returns the background color of the font dialog.                             |
------------------------------------------------------------------------------*/
IColor IFileDialog::backgroundColor () const
{
  return fFileDialogData->fclrbackground;
}

/*------------------------------------------------------------------------------
| IFileDialog::setBackgroundColor                                              |
|                                                                              |
| Stores the background color of the font dialog.                              |
------------------------------------------------------------------------------*/
IFileDialog& IFileDialog::setBackgroundColor (const IColor &color)
{
  fFileDialogData->fclrbackground = color;
  return *this;
}

#endif // IC_WIN

/*------------------------------------------------------------------------------
| IFileDialog::initialize                                                          |
|                                                                              |
| Called by constructor to actually start the dialog.                          |
| For a modal dialog does not return until user has dismissed dialog.          |
| Because of this, we have override the PM dialog procedure with our own       |
| so that the handler scheme will work.                                        |
------------------------------------------------------------------------------*/
void IFileDialog::initialize (IWindow*        parent,
                              IWindow*        owner,
                              const Style&    style,
                              IHandler*       handler,
                              const Settings& settings)
{
   IMODTRACE_DEVELOP("IFileDialog::initialize");
#ifdef IC_PM
   // It's possible that the first thing an application may do is
   // create a IFileDialog. Since we deferCreation of our parent IFrameWindow
   // we should be sure that PM has been initialized.
   IThread::current().initializeGUI();

   fFileDialogData->handlerToAdd = handler;

   pFileDlgData = new FILEDLG;
   memset(pFileDlgData, 0, sizeof(FILEDLG) );
   pFileDlgData->cbSize = sizeof(FILEDLG);

   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   // Convert the style bits into the GUI native style settings
   pFileDlgData->fl = convertToGUIStyle( style );

   if (settings.isOpenDialog()) {
      pFileDlgData->fl |= FDS_OPEN_DIALOG;
   } else {
      pFileDlgData->fl |= FDS_SAVEAS_DIALOG;
   } /* endif */

   if (settings.isPositionSet()) {
      pFileDlgData->x = (short)(settings.position().x());
      pFileDlgData->y = (short)(settings.position().y());
   } else {
      pFileDlgData->fl |= FDS_CENTER;
   } /* endif */

   if (settings.isDialogTemplateSet()) {
      pFileDlgData->fl |= FDS_CUSTOM;
      pFileDlgData->hMod = settings.fSettingsData->dlgHandle;
      pFileDlgData->usDlgId = (unsigned short)(settings.fSettingsData->dlgId);
   }

   pFileDlgData->ulUser = (unsigned long)this;

   if (settings.title().length()) {
      pFileDlgData->pszTitle = (PSZ)new char[settings.title().length()+1];
      strcpy(pFileDlgData->pszTitle, settings.title() );
   }

   if (settings.okButtonText().length()) {
      pFileDlgData->pszOKButton = (PSZ)new char[settings.okButtonText().length()+1];
      strcpy(pFileDlgData->pszOKButton, settings.okButtonText() );
   }

   if (settings.initialDrive().length()) {
      pFileDlgData->pszIDrive = (PSZ)new char[settings.initialDrive().length()+1];
      strcpy(pFileDlgData->pszIDrive, settings.initialDrive() );
   }

   if (settings.initialFileType().length()) {
      pFileDlgData->pszIType = (PSZ)new char[settings.initialFileType().length()+1];
      strcpy(pFileDlgData->pszIType, settings.initialFileType() );
   }

   strcpy(pFileDlgData->szFullFile, settings.fileName() );

   pFileDlgData->pfnDlgProc = (FNWP*)_IFileDlgProc;

   IString* curString;
   int i;

   // Settings maintains the list of Types and Drives as BB collections
   // of strings. These have to be filled into an array of char pointers
   // in the PM FILEDIALOG structure...
   StringSeq* curQ = settings.fileTypes();
   if (curQ) {
     i = 0;
     // allocate space for the table of char pointers
     pFileDlgData->papszITypeList = (PAPSZ) new char*[curQ->numberOfElements() + 1];
     StringSeq::Cursor current(*curQ);
     for (current.setToFirst(); current.isValid(); current.setToNext() ) {
        curString = curQ->elementAt(current);
        // allocate space for the current string
        *pFileDlgData->papszITypeList[i] = new char[curString->length() + 1];
        // copy current string into storage just allocated
        strcpy(*pFileDlgData->papszITypeList[i], *curString);
        i++;
     } /* end for */
     *pFileDlgData->papszITypeList[i] = 0;       // array terminator
   }

   curQ = settings.drives();
   if (curQ) {
     i = 0;
     // allocate space for the table of char pointers
     pFileDlgData->papszIDriveList = (PAPSZ) new char*[curQ->numberOfElements() + 1];
     StringSeq::Cursor current(*curQ);
     for (current.setToFirst(); current.isValid(); current.setToNext() ) {
        curString = curQ->elementAt(current);
        // allocate space for the current string
        *pFileDlgData->papszIDriveList[i] = new char[curString->length() + 1];
        // copy current string into storage just allocated
        strcpy(*pFileDlgData->papszIDriveList[i], *curString);
        i++;
     } /* end for */
     *pFileDlgData->papszIDriveList[i] = 0;
   }

   // Need to set a variable in IWindow that holds the default procedure
   // so that IFrameHandler will work.
   setDefaultProcedure(IPMCtlStatics::defFileDlgProcAddr());

   // Make call to WinFileDlg via IPMCtlStatics. Trying to avoid reference
   // to WinFileDlg until actually needed.
   if (!IPMCtlStatics::fileDlg(parent == 0 ? desktopWindow()->handle() : parent->handle(),
                   owner  == 0 ? IWindowHandle(0)        : owner->handle(),
                   pFileDlgData) )
   {
      ITHROWGUIERROR("WinFileDlg");
   }
#endif
#ifdef IC_WIN
   /*******************************************************************/
   /* Declare a variable for saving return code of API call and save  */
   /* handler passed into the dialog data (for use in subclass proc)  */
   /*******************************************************************/
   DWORD fileBufferLength = MAX_PATH;
   Boolean rc;
   fFileDialogData->handlerToAdd = handler;

   /*******************************************************************/
   /* Copy the settings data passed into a new settings data block    */
   /*******************************************************************/
   pFileDlgData = new OPENFILENAME;
   memset(pFileDlgData, 0, sizeof(OPENFILENAME) );
   pFileDlgData->lStructSize = sizeof(OPENFILENAME);

   /*******************************************************************/
   /* Save the extended style to make sure we have a copy of it stored*/
   /*******************************************************************/
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   /*******************************************************************/
   /* Convert the style bits into the GUI native style settings and   */
   /* remove read-only checkbox                                       */
   /*******************************************************************/
   pFileDlgData->Flags = convertToGUIStyle( style );
   pFileDlgData->Flags |= OFN_HIDEREADONLY | OFN_NOCHANGEDIR;

   /*******************************************************************/
   /* Check if a custom title was set                                 */
   /*******************************************************************/
   if (settings.title().length())
   {
     /*****************************************************************/
     /* If so, then copy the title over for updating                  */
     /*****************************************************************/
     fFileDialogData->fTitle = (LPSTR)new char[settings.title().length()+1];
     strcpy( fFileDialogData->fTitle, settings.titleStr );
     pFileDlgData->lpstrTitle = fFileDialogData->fTitle;
   }

   /*******************************************************************/
   /* Check if custom button text was set                             */
   /*******************************************************************/
   if (settings.okButtonText().length())
   {
     /*****************************************************************/
     /* If so, then copy the text over for updating and set flag to   */
     /* tell subclass procedure to do this on dialog initialization   */
     /*****************************************************************/
     fFileDialogData->fOKButton = settings.OKButtonStr;
     fFileDialogData->fSettingsChanged |= IC_OKBUTTON_SET;
   }

   /*******************************************************************/
   /* Check if a selectable list box is desired.                      */
   /*******************************************************************/
   if (style.asUnsignedLong() & selectableListbox.asUnsignedLong())
      fFileDialogData->fSettingsChanged |= IC_ENABLELISTBOX_SET;

   /*******************************************************************/
   /* Check if a custom position was set                              */
   /*******************************************************************/
   if (settings.isPositionSet())
   {
     /*****************************************************************/
     /* If so, then copy the position over for updating and set flag  */
     /* to tell subclass procedure to do this on dialog initialization*/
     /*****************************************************************/
     fFileDialogData->fPosition = settings.pos;
     fFileDialogData->fSettingsChanged |= IC_POSITION_SET;
   }

   /*******************************************************************/
   /* Check if a previous directory was set (via previous invocation) */
   /*******************************************************************/
   if (settings.fSettingsData->isDirectorySet)
   {
     /*****************************************************************/
     /* If so, then copy the drive over to initial directory string   */
     /*****************************************************************/
     fFileDialogData->fDrive =
         (LPSTR)new char[settings.fSettingsData->fDirectory.length()+1];
     strcpy( fFileDialogData->fDrive, settings.fSettingsData->fDirectory );
     pFileDlgData->lpstrInitialDir = fFileDialogData->fDrive;
   }

   /*******************************************************************/
   /* Otherwise check if an initial drive was specified               */
   /*******************************************************************/
   else if (settings.initialDrive().length())
   {
     /*****************************************************************/
     /* If so, then copy the drive over to initial directory string   */
     /*****************************************************************/
     fFileDialogData->fDrive =
                    (LPSTR)new char[settings.initialDrive().length()+1];
     strcpy( fFileDialogData->fDrive, settings.initialDriveStr );
     pFileDlgData->lpstrInitialDir = fFileDialogData->fDrive;
   }

   /*******************************************************************/
   /* Check if a custom dialog template was specified                 */
   /*******************************************************************/
   if (settings.isDialogTemplateSet())
   {
     /*****************************************************************/
     /* If so, copy the id and instance handle over for updating      */
     /*****************************************************************/
     pFileDlgData->Flags |= OFN_ENABLETEMPLATE;
     pFileDlgData->lpTemplateName = MAKEINTRESOURCE( settings.fSettingsData->dlgId );
     pFileDlgData->hInstance = settings.fSettingsData->dlgHandle;
   }

   /*******************************************************************/
   /* If multiple selection, make it a large buffer for many filenames*/
   /* NOTE: In testing, Windows does not seem to use any more than    */
   /*  2K of buffer, meaning only about 150-170 files can be selected */
   /*******************************************************************/
   if (style.asUnsignedLong() & multiSelection.asUnsignedLong())
     fileBufferLength = 4096;

   /*******************************************************************/
   /* Allocate a buffer for initializing AND receiving the path/file  */
   /* name.  Default the buffer to NULL and pass the buffer length    */
   /*******************************************************************/
   pFileDlgData->lpstrFile = (PSZ)new char[fileBufferLength];
   pFileDlgData->lpstrFile[0] = 0;
   pFileDlgData->nMaxFile = fileBufferLength;

   /*******************************************************************/
   /* Check if initial filename is to be specified and if so, copy it */
   /*******************************************************************/
   if (settings.fileName().length())
     strcpy( pFileDlgData->lpstrFile, settings.fileNameStr );

   /*******************************************************************/
   /* Set initial filter to All files in order to fill listbox on dlg */
   /* initialization.  Without this, the listbox will be blank        */
   /*******************************************************************/
   pFileDlgData->lpstrFilter = "All Files\0*.*\0\0";
   pFileDlgData->nFilterIndex = 1;

   /*******************************************************************/
   /* Set the owner of the dialog based on parameters passed          */
   /* If dialog is modeless, create an object window and make it owner*/
   /*******************************************************************/
   IObjectWindow* objWin = 0;
   if (extendedStyle() & IFDS_MODELESS)
   {
     objWin = new IObjectWindow();
     objWin->setAutoDestroyWindow(true);
     pFileDlgData->hwndOwner = objWin->handle();
   }
   /*******************************************************************/
   /*  otherwise use the owner handle passed (if any)                 */
   /*******************************************************************/
   else
     pFileDlgData->hwndOwner = owner == 0 ? IWindowHandle(0) :
                                                        owner->handle();

   /*******************************************************************/
   /* Save the this pointer in the application data field in template */
   /* to allow access back to IFileDialog object from within hook proc*/
   /*******************************************************************/
   pFileDlgData->lCustData = (unsigned long)this;

   /*******************************************************************/
   /* Set the hook proc to our dialog procedure since we need to get  */
   /* in to process interactions                                      */
   /*******************************************************************/
   pFileDlgData->lpfnHook = (LPOFNHOOKPROC)_IFileDlgProc;
   pFileDlgData->Flags |= OFN_ENABLEHOOK | OFN_PATHMUSTEXIST;

   /*******************************************************************/
   /* Set the window id for the dialog                                */
   /*******************************************************************/
   if (settings.isDialogTemplateSet())
     // use template id for dialog id
     fFileDialogData->fDialogId = settings.fSettingsData->dlgId;
   else
     if (pFileDlgData->Flags & FDS_MULTIPLESEL)
       // use system multiselect open file dialog id
       fFileDialogData->fDialogId = IC_FILE_DIALOG_MULTSELECT;
     else
       // use normal system open file dialog id
       fFileDialogData->fDialogId = IC_FILE_DIALOG;

   /*******************************************************************/
   /* Everything is now set up, so invoke the correct dialog          */
   /*******************************************************************/
   if (settings.isOpenDialog())
   {
     fFileDialogData->fSettingsChanged |= IC_OPEN_SET;
     rc = GetOpenFileName( pFileDlgData );
   }
   
   else
   {
     pFileDlgData->Flags |= OFN_OVERWRITEPROMPT;
     rc = GetSaveFileName( pFileDlgData );
   }

   /*******************************************************************/
   /* If complete, then perform any cleanup necessary                 */
   /*******************************************************************/
   if (objWin)
     delete objWin;

   /*******************************************************************/
   /* Check the return code from the dialog and set the return value  */
   /* accordingly                                                     */
   /*******************************************************************/
   if (rc)
   {
     fFileDialogData->fReturnVal = DID_OK;

     /*****************************************************************/
     /* If a selection was made, then update the settings initial path*/
     /* string with selection, for future invocations to start at the */
     /* same place that the previous one did                          */
     /*****************************************************************/
     settings.fSettingsData->fDirectory = pFileDlgData->lpstrFile;
     settings.fSettingsData->fDirectory.remove( pFileDlgData->nFileOffset );
     settings.fSettingsData->isDirectorySet = true;

     ((Settings&)settings).fileNameStr = pFileDlgData->lpstrFile;
     ((Settings&)settings).fileNameStr.remove( 1, pFileDlgData->nFileOffset );
   }
   else
   {
     fFileDialogData->fReturnVal = DID_CANCEL;

     /*****************************************************************/
     /* If NULL return, check for errors and if any, throw exception  */
     /*****************************************************************/
     fFileDialogData->fSRC = CommDlgExtendedError();
     if (fFileDialogData->fSRC)
       ITHROWGUIERROR("OpenFileName");
   }
#endif
}

