/* NOSHIP */
#ifndef _IFLYPRIV_
#define _IFLYPRIV_
/*******************************************************************************
* FILE NAME: iflypriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of iflyhhdr.hpp                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandle.hpp>
#include <itimer.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <iksssts.h>

//----------------------------------------------------------
// DEFECT 8032 : create our own IFlyText if not provided
//    (hidden creation as child of longText control)
//----------------------------------------------------------
#define DEFAULT_FLYTEXT_ID    999
//----------------------------------------------------------

class IFlyText;
class ITextControl;
struct tagNMHDR;
class IFlyOverData;

class IHandleStringElement {
public:
  IHandleStringElement( unsigned long      handle,
                        const IString&     flyText,
                        const IString&     longText );

  IHandleStringElement( unsigned long      handle,
                        const IResourceId& flyText,
                        const IResourceId& longText );

  ~IHandleStringElement( );

  // Note:  A union would appear to be storage representation of choice
  //        for this class since it will contain either strings or
  //        resource Ids.  It is my understanding though, that the compiler
  //        will generate 32 bytes of storage at a minumum for each
  //        structure which negates any savings by using a union.
  unsigned long handleKey;
  IString       flyStr;
  IString       longStr;
  unsigned long flyResId;
  unsigned long longResId;
  Boolean       fStrings;
#ifdef IC_WIN
  // Save the pointer to the subclassed window proc.
  unsigned long (* __stdcall subclassProc)(HWND hwnd, unsigned long ulMsg,
                                           WPARAM mp1, LPARAM mp2);
  IFlyOverData* flyData;
#endif
private:
  IHandleStringElement( const IHandleStringElement& );
  IHandleStringElement
    &operator =       ( const IHandleStringElement& );
};

inline unsigned long const& key (IHandleStringElement* const& k)
{
  return k->handleKey;
}

class IHandleStringSet : public IKeySortedSetOnSortedTabularSequence< IHandleStringElement*, unsigned long >
{
typedef IKeySortedSetOnSortedTabularSequence< IHandleStringElement*, unsigned long >
  Inherited;
public:
  IHandleStringSet();
  ~IHandleStringSet();
};

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IResourceLibrary;

class IFlyOverData : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* Objects of IFlyOverData contain the private data for the                     *
* IFlyOverHelpHandler class.                                                   *
*******************************************************************************/
friend class IFlyOverHelpHandler;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 ways to construct instances of this class:                       |
|   1. From a pointer to a fly text control, long text control and             |
|      an unsigned long indicating the timer delay.                            |
------------------------------------------------------------------------------*/
  IFlyOverData( IFlyText*            flyText,
                ITextControl*        longText,
                unsigned long        delay,
                unsigned long        initialDelay,
                IFlyOverHelpHandler* flyHelp );

  ~IFlyOverData();

#ifdef IC_WIN
void
  notifyEvent  ( tagNMHDR* notifyHeader );

IFlyOverData
  &setHelpText ( const IWindowHandle& handle,
                 IHandleStringElement& element );

#else

void
  timerpop     ( unsigned long timerId );
#endif

/*------------------------------ Data Members ----------------------------------
|  The following data members are used to different font values:               |
|   flytxt      - Pointer to fly text control.                                 |
|   longtxt     - Pointer to long text control.                                |
|   flyOffset - Offset into the string table for the short text control.       |
|   longOffset  - Offset into the string table for the long text control.      |
|   timer       - Timer object.                                                |
|   prevPos     - last position of the mouse from a WM_CONTROL pointer.        |
|   controlId   - Id of window that the mouse was over from the last control   |
|                 pointer message.                                             |
------------------------------------------------------------------------------*/
IFlyText
 *fFlytxt;
ITextControl
 *fLongtxt;
long
  fFlyOffset,
  fLongOffset;
#ifndef IC_WIN
ITimer
  fTimer;
IPoint
  fPrevPos;
#endif
IString
  fDefault;
Boolean
  fSetPrevToNull;

#ifndef IC_WIN
IWindowHandle
  fHandle,
  fPrevHandle;
#endif
IWindowHandle
  fControlHandle;

#ifndef IC_WIN
IWindow*
  fControlWindow;
#endif

IFlyOverHelpHandler*
  fFly;
IResourceLibrary
 *fResLib;
unsigned long
  fTimerDelay,
  fInitialTimerDelay;

//----------------------------------------------------------
// DEFECT 8032 : create our own IFlyText if not provided
//----------------------------------------------------------
Boolean
   bFlytxtCreated;
//----------------------------------------------------------

private:
IFlyOverData
  &operator=(const IFlyOverData&);
IFlyOverData(const IFlyOverData&);

#ifdef IC_WIN
IFlyOverData
  &setFlyText( const IString& helpText, char* szText, Boolean useWideChar );
IFlyOverData
  &setFlyTextForHandle( IWindowHandle controlHandle,
                        char* szText,
                        Boolean useWideChar );
#else  //!IC_WIN
IFlyOverData
  &positionTextWindow( ),
  &setFlyText( const IString& helpText );
#endif

IFlyOverData
  &setLongText( const IString& helpText );


}; // IFlyOverData;

#pragma pack()

#endif /* _IFLYPRIV_ */
