/* NOSHIP */
#ifndef _IFRAMPRV_
  #define _IFRAMPRV_
/*******************************************************************************
* FILE NAME: iframprv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IFrameWindowData                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandle.hpp>
#include <irect.hpp>
#include <istring.hpp>

class IFrameExtensions;
class IToolBarList;
class IMenuBar;
class IAccelerator;
class ITimer;

#pragma pack(4)

class IFrameWindowData : public IBase {
public:
#ifdef IC_PMWIN
  IFrameWindowData ( Boolean flags );
#endif
#ifdef IC_MOTIF
  IFrameWindowData ( unsigned long defState );
#endif
 ~IFrameWindowData ( );

#ifdef IC_MOTIF
  IMenuBar *menuBar();
#endif

IFrameExtensions
 *pExtensions;
unsigned
  flags;
unsigned long
  rc;

#ifdef IC_MOTIF
  IMenuBar *frameMenu;
  IAccelerator *frameAccel;
  IWindowHandle shell;
  IWindowHandle mainW;
  IWindowHandle motifClient;

  static unsigned long
    wmDeleteWindow,
    wmSaveYourself,
    wmTakeFocus;
#endif

IPointerHandle
  frameIcon;
IPointerHandle
  mousePtr;
IToolBarList
 *toolBarList;
void
 *fpHelpList;

#ifdef IC_WIN
IRectangle
  logicalClientRect;    // Client rectangle.
void
 *fpSubmenuList;

// Members to implement flashing
ITimer
 *ftimer;                                  // A timer
IWindowHandle
  ftimerHwnd;                              // window to flash
void
  timerExpired ( unsigned long timerId );  // flashes ftimerHwnd

// Members to implement setting title bar font
HFONT
  fhFont;               // font handle for title bar (== 0 if default)

// member for showModally
unsigned long
  fModalNesting;        // value of fgModalNesting when showModally done

// statics
static IRectangle
  fgNextRect;           // Next value for nextShellRect
static unsigned long
  fgClassID;            // Next ID word for Frame window class
static unsigned long
  fgTlsIndex;           // modal nesting counter tls index

// calcFrameRect performs conversion between a window rectangle and
// the window's client area rectangle.  The frameToClient argument controls
// which way the conversion is done.  If true, the input rectangle is
// the frame window rectangle, if false the rectangle is the client
// rectangle.  Similar in function to PM WinCalcFrameRect.
static Boolean calcFrameRect ( IWindowHandle  hwnd,
                               RECTL*         rect,
                               Boolean        frameToClient );

#endif

#ifdef IC_PM
Boolean
  disableBidiChildUpdate;
#endif

#ifdef IC_MOTIF
  static IWindow::SiblingOrder
    siblingCreateOrder;
#endif

static const unsigned
#ifdef IC_WIN
  needsClientSized,
  enableOwner,          // Enable owner on modal dismiss.
  beingSized,
  inTitlePaint,         // Painting title
#endif
#ifdef IC_PMWIN
  modalAutoDelete,      // Support auto-delete object for modal frames.
#endif
#ifdef IC_MOTIF
  verticalSB,
  horizontalSB,
#endif
  destroyOnClose,
  hasDefaultHandler,
  modal,
  needsUpdating,
  fromDlgTemplate,
  standardFrame,
  hasBeenClosed;

#ifdef IC_MOTIF
  static unsigned nextPositionx,
                  nextPositiony;

  IFrameWindowData &findDefaultButton( IFrameWindow *frame );
  IBase::Boolean findFirstButton( IWindowHandle whnd, IWindow *pFrame,
                                  Boolean anyButton );
  IBase::Boolean findFirstControl( IWindowHandle whnd, IWindow *pFrame);
private:
  IFrameWindowData::IFrameWindowData(const IFrameWindowData&);
  IFrameWindowData& operator= (const IFrameWindowData&);
#endif

//--------------------------------------------------------------
// DEFECT 24892 : child with focus when frame deactivated
//--------------------------------------------------------------
#ifdef IC_WIN
IWindowHandle
  fLastFocus;           // last focus window when deactivated
#endif
//--------------------------------------------------------------
}; // class IFrameWindowData

#pragma pack()

#endif // _IFRAMPRV_
