/*******************************************************************************
* FILE NAME: igarc.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igarc.hpp.                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igarc.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGARC_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGArc::IGArc                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc::IGArc( const IRectangle& rc, double startAngle, double sweepAngle )
  : IGArc::Inherited(), fRect(rc), fStart(startAngle), fSweep(sweepAngle),
    fAd(IGArc::counterclockwise)
{
}

/*------------------------------------------------------------------------------
| IGArc::IGArc                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc::IGArc( const IGArc& arc )
  : IGArc::Inherited(arc), fRect(arc.fRect), fStart(arc.fStart),
    fSweep(arc.fSweep), fAd(arc.fAd)
{
}

/*------------------------------------------------------------------------------
| IGArc::~IGArc                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc::~IGArc( )
{
}

/*------------------------------------------------------------------------------
| IGArc::drawOn                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc& IGArc::drawOn ( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::setRectangle                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc& IGArc::setEnclosingRect  ( const IRectangle& rectangle )
{
  this->fRect = rectangle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::enclosingRect                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IGArc::enclosingRect ( ) const
{
  return this->fRect;
}

/*------------------------------------------------------------------------------
| IGArc::setStartAngle                                                         |
|                                                                              |
|                                                                              |
-----------------------------------------------------------------------*/
IGArc& IGArc::setStartAngle ( double startAngle )
{
  fStart = startAngle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::setSweepAngle                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc& IGArc::setSweepAngle ( double sweepAngle )
{
  fSweep = sweepAngle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::startAngle                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double IGArc::startAngle ( ) const
{
  return this->fStart;
}

/*------------------------------------------------------------------------------
| IGArc::sweepAngle                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double IGArc::sweepAngle ( ) const
{
  return this->fSweep;
}

/*------------------------------------------------------------------------------
| IGArc::setDirection                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc& IGArc::setDirection( IGArc::Direction direction )
{
  fAd = direction;
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::arcDirection                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc::Direction IGArc::direction( ) const
{
  return fAd;
}

/*------------------------------------------------------------------------------
| IGArc::operator=                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGArc& IGArc::operator= ( const IGArc& arc )
{
  if (this == &arc)
    return *this;
  this->Inherited::operator=(arc);
  fStart   = arc.fStart;
  fSweep   = arc.fSweep;
  fRect    = arc.fRect;
  return *this;
}

/*------------------------------------------------------------------------------
| IGArc::operator=                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGArc::operator== ( const IGArc& arc ) const
{
  return (this->Inherited::operator==(arc) &&
          fRect == arc.fRect &&
          fStart == arc.fStart &&
          fSweep == arc.fSweep );
}

/*------------------------------------------------------------------------------
| IGArc::operator!=                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGArc::operator!= ( const IGArc& arc ) const
{
  return !( *this == arc );
}

/*------------------------------------------------------------------------------
| IG3PointArc::IG3PointArc                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc::IG3PointArc( const IG3PointArc& arc)
  : IG3PointArc::Inherited(), fStart(arc.fStart),
    fIntermediate(arc.fIntermediate), fEnd(arc.fEnd)
{
  if (arc.hasGraphicBundle())
    setGraphicBundle(arc.graphicBundle());
}

/*------------------------------------------------------------------------------
| IG3PointArc::IG3PointArc                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc::IG3PointArc( const IPoint& start,
                          const IPoint& intermediate,
                          const IPoint& end )
  : IG3PointArc::Inherited(), fStart(start), fIntermediate(intermediate),
    fEnd(end)
{
}

/*------------------------------------------------------------------------------
| IG3PointArc::~IG3PointArc                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc::~IG3PointArc()
{
}

/*------------------------------------------------------------------------------
| IG3PointArc::drawOn                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc& IG3PointArc::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IG3PointArc::startingPoint                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IG3PointArc::startingPoint() const
{
  return fStart;
}

/*------------------------------------------------------------------------------
| IG3PointArc::endingPoint                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IG3PointArc::endingPoint() const
{
  return fEnd;
}

/*------------------------------------------------------------------------------
| IG3PointArc::intermediate                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IG3PointArc::intermediatePoint() const
{
  return fIntermediate;
}

/*------------------------------------------------------------------------------
| IG3PointArc::setStartingPoint                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc& IG3PointArc::setStartingPoint(const IPoint& point)
{
  fStart = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IG3PointArc::setIntermediate                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc& IG3PointArc::setIntermediatePoint(const IPoint& point)
{
  fIntermediate = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IG3PointArc::setEndingPoint                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc& IG3PointArc::setEndingPoint(const IPoint& point)
{
  fEnd = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IG3PointArc::operator=                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IG3PointArc& IG3PointArc::operator= ( const IG3PointArc& arc )
{
  if (this == &arc)
    return *this;

  this->Inherited::operator=(arc);
  fStart        = arc.fStart;
  fIntermediate = arc.fIntermediate;
  fEnd          = arc.fEnd;
  return *this;
}

/*------------------------------------------------------------------------------
| IG3PointArc::operator=                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IG3PointArc::operator== ( const IG3PointArc& arc ) const
{
  return (this->Inherited::operator==(arc)   &&
          fStart == arc.fStart               &&
          fIntermediate == arc.fIntermediate &&
          fEnd == arc.fEnd );
}

/*------------------------------------------------------------------------------
| IG3PointArc::operator!=                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IG3PointArc::operator!= ( const IG3PointArc& arc ) const
{
  return !( *this == arc );
}
