/*******************************************************************************
* FILE NAME: igbundle.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igbundle.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
{
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#include <igbundle.hpp>
#include <igrafctx.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGBUNDLE_CPP_
  #include <ipagetun.h>
#endif

const unsigned long IGraphicBundle::dense1(PATSYM_DENSE1);
const unsigned long IGraphicBundle::dense2(PATSYM_DENSE2);
const unsigned long IGraphicBundle::dense3(PATSYM_DENSE3);
const unsigned long IGraphicBundle::dense4(PATSYM_DENSE4);
const unsigned long IGraphicBundle::dense5(PATSYM_DENSE5);
const unsigned long IGraphicBundle::dense6(PATSYM_DENSE6);
const unsigned long IGraphicBundle::dense7(PATSYM_DENSE7);
const unsigned long IGraphicBundle::dense8(PATSYM_DENSE8);
const unsigned long IGraphicBundle::vertical(PATSYM_VERT);
const unsigned long IGraphicBundle::horizontal(PATSYM_HORIZ);
const unsigned long IGraphicBundle::forewardDiag1(PATSYM_DIAG1);
const unsigned long IGraphicBundle::forewardDiag2(PATSYM_DIAG2);
const unsigned long IGraphicBundle::backwardDiag1(PATSYM_DIAG3);
const unsigned long IGraphicBundle::backwardDiag2(PATSYM_DIAG4);
const unsigned long IGraphicBundle::hollow(PATSYM_NOSHADE);
const unsigned long IGraphicBundle::filled(PATSYM_SOLID);
const unsigned long IGraphicBundle::halftone(PATSYM_HALFTONE);
const unsigned long IGraphicBundle::hatched(PATSYM_HATCH);
const unsigned long IGraphicBundle::hatchedDiag(PATSYM_DIAGHATCH);

/*------------------------------------------------------------------------------
| IGraphicBundle::IGraphicBundle                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle::IGraphicBundle( )
  : fPen    (IGraphicContext::defaultPenColor()),
    fFill   (IGraphicContext::defaultFillColor()),
    fBack   (IGraphicContext::defaultBackgroundColor()),
    fDrawOp (IGraphicContext::defaultDrawOperation()),
    fPmix   (IGraphicContext::defaultMixMode()),
    fBmix   (IGraphicContext::defaultBackgroundMixMode()),
    fPpat   (IGraphicContext::defaultPenPattern()),
    fFpat   (IGraphicContext::defaultFillPattern()),
    fPtype  (IGraphicContext::defaultPenType()),
    fPend   (IGraphicContext::defaultPenEndingStyle()),
    fPjoin  (IGraphicContext::defaultPenJoiningStyle()),
    fWidth  (IGraphicContext::defaultPenWidth()),
    fOrg    (IGraphicContext::defaultPatternOrigin()),
    fMask   (0)
{
}

/*------------------------------------------------------------------------------
| IGraphicBundle::IGraphicBundle                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle::IGraphicBundle( const IGraphicBundle& bundle )
  : fPen    (bundle.fPen),
    fFill   (bundle.fFill),
    fBack   (bundle.fBack),
    fDrawOp (bundle.fDrawOp),
    fPmix   (bundle.fPmix),
    fBmix   (bundle.fBmix),
    fPpat   (bundle.fPpat),
    fFpat   (bundle.fFpat),
    fPtype  (bundle.fPtype),
    fPend   (bundle.fPend),
    fPjoin  (bundle.fPjoin),
    fWidth  (bundle.fWidth),
    fOrg    (bundle.fOrg),
    fMask   (bundle.fMask)
{
}

/*------------------------------------------------------------------------------
| IGraphicBundle::IGraphicBundle                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle::IGraphicBundle( const IGraphicContext& graphicContext )
  : fPen    (graphicContext.penColor()),
    fFill   (graphicContext.fillColor()),
    fBack   (graphicContext.backgroundColor()),
    fDrawOp (graphicContext.drawOperation()),
    fPmix   (graphicContext.mixMode()),
    fBmix   (graphicContext.backgroundMixMode()),
    fPpat   (graphicContext.penPattern()),
    fFpat   (graphicContext.fillPattern()),
    fPtype  (graphicContext.penType()),
    fPend   (graphicContext.penEndingStyle()),
    fPjoin  (graphicContext.penJoiningStyle()),
    fWidth  (graphicContext.penWidth()),
    fOrg    (graphicContext.patternOrigin()),
    fMask   (IGraphicBundle::All)
{
}

/*------------------------------------------------------------------------------
| IGraphicBundle::IGraphicBundle                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle::~IGraphicBundle()
{
}

/*------------------------------------------------------------------------------
| IGraphicBundle::operator=                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle& IGraphicBundle::operator= ( const IGraphicBundle& graphicBundle )
{
  if (this == &graphicBundle)
    return *this;

  fPen    = graphicBundle.fPen;
  fFill   = graphicBundle.fFill;
  fBack   = graphicBundle.fBack;
  fDrawOp = graphicBundle.fDrawOp;
  fPmix   = graphicBundle.fPmix;
  fBmix   = graphicBundle.fBmix;
  fPpat   = graphicBundle.fPpat;
  fFpat   = graphicBundle.fFpat;
  fPtype  = graphicBundle.fPtype;
  fPend   = graphicBundle.fPend;
  fPjoin  = graphicBundle.fPjoin;
  fWidth  = graphicBundle.fWidth;
  fOrg    = graphicBundle.fOrg;
  fMask   = graphicBundle.fMask;
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphicBundle::operator==                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGraphicBundle::operator== ( const IGraphicBundle& graphicBundle )
{
  return ( fPen    == graphicBundle.fPen    &&
           fFill   == graphicBundle.fFill   &&
           fBack   == graphicBundle.fBack   &&
           fDrawOp == graphicBundle.fDrawOp &&
           fPmix   == graphicBundle.fPmix   &&
           fBmix   == graphicBundle.fBmix   &&
           fPpat   == graphicBundle.fPpat   &&
           fFpat   == graphicBundle.fFpat   &&
           fPtype  == graphicBundle.fPtype  &&
           fPend   == graphicBundle.fPend   &&
           fPjoin  == graphicBundle.fPjoin  &&
           fWidth  == graphicBundle.fWidth  &&
           fOrg    == graphicBundle.fOrg    &&
           fMask   == graphicBundle.fMask );
}

/*------------------------------------------------------------------------------
| IGraphicBundle::operator!=                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGraphicBundle::operator!= ( const IGraphicBundle& graphicBundle )
{
  return !(*this == graphicBundle);
}

/*------------------------------------------------------------------------------
| IGraphicBundle::applyTo                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
//const IGraphicBundle& IGraphicBundle::applyTo( IGraphicContext& graphicContext ) const
//{
//
//  if ( fMask & IGraphicBundle::pColor )
//    graphicContext.setPenColor( fPen );
//  if ( fMask & IGraphicBundle::fColor )
//    graphicContext.setFillColor( fFill );
//  if ( fMask & IGraphicBundle::bColor )
//    graphicContext.setBackgroundColor( fBack );
//  if ( fMask & IGraphicBundle::drawOp )
//    graphicContext.setDrawOperation( fDrawOp );
//  if ( fMask & IGraphicBundle::pMix )
//    graphicContext.setMixMode( fPmix );
//  if ( fMask & IGraphicBundle::bMix )
//    graphicContext.setBackgroundMixMode( fBmix );
//  if ( fMask & IGraphicBundle::pType )
//    graphicContext.setPenType( fPtype );
//  if ( fMask & IGraphicBundle::pEnd )
//    graphicContext.setPenEndingStyle( fPend );
//  if ( fMask & IGraphicBundle::pJoin )
//    graphicContext.setPenJoiningStyle( fPjoin );
//  if ( fMask & IGraphicBundle::pWidth )
//    graphicContext.setPenWidth( fWidth );
//  if ( fMask & IGraphicBundle::pPat )
//    graphicContext.setPenPattern( fPpat );
//  if ( fMask & IGraphicBundle::fPat )
//    graphicContext.setFillPattern( fFpat );
//  if ( fMask & IGraphicBundle::pOrg )
//    graphicContext.setPatternOrigin( fOrg );
//  return *this;
//}
