/* NOSHIP */
#ifndef _IGCDATA_
   #define _IGCDATA_
/*******************************************************************************
* FILE NAME: igcdata.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*                                                                              *
* Implementation Notes:                                                        *
*   Data class for IGraphicContext for Windows.                                *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#ifdef IC_WIN

#include <ibase.hpp>
#include <ihandle.hpp>

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)
#pragma info(none)

#define      DBL2FIXED(x)     ((FIXED)  ((x)*65536) )

typedef struct tagVECTOR2D
{
  double     x;
  double     y;
} VECTOR2D, *PVECTOR2D;

typedef struct tagPROJECTION
{
  VECTOR2D   ttProjection;
  VECTOR2D   ttPerpProjection;
  double     LenProjection;
  double     LenPerpProjection;
} PROJECTION, *PPROJECTION;

typedef struct _POINTD
{
  double x;
  double y;
} POINTD, *PPOINTD;

class IGCData : public IBase {
typedef IBase
  Inherited;
public:
enum Mode {
  drawing,
  hitTesting,
  accumBoundary
  };

  IGCData( Mode mode,
           IGraphicBundle::DrawOperation drawOperation,
           const IColor& penColor,
           const IColor& fillColor,
           const IColor& backgroundColor,
           IGraphicBundle::MixMode mixMode,
           IGraphicBundle::BackgroundMixMode backgroundMixMode,
           unsigned long penWidth,
           IGraphicBundle::PenType penType,
           IGraphicBundle::PenEndingStyle penEndingStyle,
           IGraphicBundle::PenJoiningStyle penJoiningStyle,
           unsigned long penPattern,
           unsigned long fillPattern );
  ~IGCData( );


IGCData
  &createPen          ( const IPresSpaceHandle& ps ),
  &createBrush        ( const IPresSpaceHandle& ps );
static IPoint
  radialPoint         ( Boolean textMapMode, const IRectangle& rect, float angle );
static void
  angleNormalize      ( double& degree );
static IPoint
  transformPoint      ( const ITransformMatrix& mat, const IPoint& point );
static void
  transformPoint      ( const ITransformMatrix& mat, double& x, double& y);
static void
  arcCenterPoint      (POINTD ptdPts[4]);
static void
  perpLineEnd         (POINTD ptdPoint[2], POINTD *pptdStart, POINTD *pptdEnd);
static void
  pointAtIntersect    (POINTD ptdPts[5]);
static double
  determinant         ( double e11, double e12, double e21, double e22 );
static PVECTOR2D
  subtractVectors     (PVECTOR2D v0, PVECTOR2D v1, PVECTOR2D v);
static double
  vectorSquared       (PVECTOR2D v0);
static double
  vectorMagnitude     (PVECTOR2D v0);
static double
  dotProduct          (PVECTOR2D v0, PVECTOR2D v1);
static void
  projectAndResolve   (PVECTOR2D v0, PVECTOR2D v1, PPROJECTION ppProj);
static double
  distFromPointToLine (LPPOINT pt0, LPPOINT pt1, LPPOINT ptTest);
static double
  distance            (PVECTOR2D v0, PVECTOR2D v1);
static Boolean
  hitTestLine         (POINT pt0, POINT pt1, POINT ptMouse, int nWidth);
static Boolean
  usingCustomFillPat  ( unsigned long fillPattern );
Boolean
  angleWithin         ( double angle1, double angle2, double angle );
static void
  rectTransformedBy   (IRectangle& rect, const ITransformMatrix& mat);
Boolean
  ellipseContains     (const IGraphicContext& gc, const IRectangle& rect, Boolean filled);
Boolean
  arcContains         (const IGraphicContext& gc, const IGArc& arc, const IRectangle& rect);
Boolean
  arc3PointContains   (const IGraphicContext& gc, const IRectangle& rect, const IPoint& startPt,
                       const IPoint& endPt);
Boolean
  pieContains         (const IGraphicContext& gc,
                       const IGPie&           pie,
                       const IRectangle&      rect,
                       Boolean                filled );
Boolean
  chordContains       (const IGraphicContext& gc, const IGChord& chord, const IRectangle& rect,
                       Boolean filled);
IRectangle
  boundingRectangle   (const IGraphicContext& gc, const IRectangle& rect, double startAngle,
                       double sweepAngle, Boolean isIGPie, Boolean closewise);
Boolean
  hitTestRegion       ( const IGraphicContext& gc, const IRegionHandle regionHandle, Boolean inLogicalSpace = false );
Boolean
  hitTestPointArray   ( const IGraphicContext& gc, POINT* lpPt, unsigned long numPoints );
Boolean
  hitTestPath         ( const IGraphicContext& gc );

// fTransformMatrix is used as the transform matrix of the graphic
// context when transform matrix is not natively supported on Windows95.
// fAdvancedMode is a flag to indicate whether in NT (advanced) mode
// or Windows95/Win32s mode.

void
  setWorldTransform   ( const ITransformMatrix&   tmx,
                        IGraphic::TransformMethod tmd );

static Boolean
  notRotated          ( const ITransformMatrix& matrix );

IGCData
 &addToBoundingRect   ( const IRectangle& rect );

Mode
  drawingMode;
IWindowHandle
  dcWindow;
IGraphicBundle::DrawOperation
  fDrawOperation;
IColor
  fPenColor,
  fFillColor,
  fBackgroundColor;
IGraphicBundle::MixMode
  fMixMode;
IGraphicBundle::BackgroundMixMode
  fBackgroundMixMode;
unsigned long
  fPenWidth;
IGraphicBundle::PenType
  fPenType;
IGraphicBundle::PenEndingStyle
  fPenEndingStyle;
IGraphicBundle::PenJoiningStyle
  fPenJoiningStyle;
unsigned long
  fPenPattern,
  fFillPattern;
Boolean
  deferCreation;
IPoint
  hitPoint;
ISize
  hitSize;
Boolean
  fAdvancedMode;
ITransformMatrix
  fTransformMatrix;
IRectangle
  fBoundingRect;
unsigned long
  recoordHeight;
Boolean
  fRecoordinate;
HGDIOBJ
  originalBrush;
HGDIOBJ
  originalPen;
HGDIOBJ
  originalBmp;

/*--------------- static IBitmapHandle's for pattern brushes -----------------*/
static IBitmapHandle
  fgBmpHalftone,
  fgBmpDense1,
  fgBmpDense2,
  fgBmpDense3,
  fgBmpDense4,
  fgBmpDense5,
  fgBmpDense6,
  fgBmpDense7,
  fgBmpDense8,
  fgBmpfDiag2,
  fgBmpbDiag2;
};

#pragma pack()

#endif  /* IC_WIN */
#endif  /* _IGCDATA_ */
