/*******************************************************************************
* FILE NAME: igline.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igline.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igline.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGLINE_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGLine::IGLine                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine::IGLine( const IGLine& line )
  : IGLine::Inherited(line), fStart(line.fStart), fEnd(line.fEnd)
{
}

/*------------------------------------------------------------------------------
| IGLine::IGLine                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine::IGLine( const IPoint& startPoint, const IPoint& endPoint )
  : IGLine::Inherited(), fStart(startPoint), fEnd(endPoint)
{
}

/*------------------------------------------------------------------------------
| IGLine::~IGLine                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine::~IGLine()
{
}

/*------------------------------------------------------------------------------
| IGLine::start                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IGLine::startingPoint() const
{
  return fStart;
}

/*------------------------------------------------------------------------------
| IGLine::end                                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IGLine::endingPoint() const
{
  return fEnd;
}

/*------------------------------------------------------------------------------
| IGLine::center                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IGLine::centerPoint() const
{
  return IPoint((fStart.x()+fEnd.x())/2, (fStart.y()+fEnd.y())/2);
}

/*------------------------------------------------------------------------------
| IGLine::setStart                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine& IGLine::setStartingPoint(const IPoint& point)
{
  fStart = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IGLine::setEnd                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine& IGLine::setEndingPoint(const IPoint& point)
{
  fEnd = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IGLine::slope                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double IGLine::slope() const
{
  return ((double)(fEnd.y()-fStart.y()))/((double)(fEnd.x()-fStart.x()));
}

/*------------------------------------------------------------------------------
| IGLine::drawOn                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine& IGLine::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGLine::operator=                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGLine& IGLine::operator= ( const IGLine& line )
{
  if (this == &line)
    return *this;

  this->Inherited::operator=( line );
  fStart = line.fStart;
  fEnd   = line.fEnd;
  return *this;
}

/*------------------------------------------------------------------------------
| IGLine::operator==                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGLine::operator== ( const IGLine& line ) const
{
  return (this->Inherited::operator==( line ) &&
          fStart == line.fStart &&
          fEnd == line.fEnd);
}

/*------------------------------------------------------------------------------
| IGLine::operator!=                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGLine::operator!= ( const IGLine& line ) const
{
  return !( *this == line );
}
