/*******************************************************************************
* FILE NAME: igpie.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igpie.hpp.                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igpie.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGPIE_CPP_
  #include <ipagetun.h>
#endif

#define      DBL2LONG(x)     ((long)  ((x)*65536) )
#define      LONG2DBL(x)     ((double) (((double)x)/(double)65536) )

/*------------------------------------------------------------------------------
| IGPie::IGPie                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie::IGPie( const IRectangle& rectangle,
              double startAngle, double sweepAngle )
  : IGPie::Inherited(), fRect(rectangle), fStart(DBL2LONG(startAngle)),
    fSweep(DBL2LONG(sweepAngle))
{}

/*------------------------------------------------------------------------------
| IGPie::IGPie                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie::IGPie( const IGPie& pie)
  : IGPie::Inherited(pie), fRect(pie.fRect), fStart(pie.fStart),
    fSweep(pie.fSweep)
{
}

/*------------------------------------------------------------------------------
| IGPie::IGPie                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie::~IGPie( )
{ }

/*------------------------------------------------------------------------------
| IGPie::enclosingRect                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IGPie::enclosingRect( ) const
{
  return fRect;
}

/*------------------------------------------------------------------------------
| IGPie::startAngle                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double IGPie::startAngle( ) const
{
  return LONG2DBL(fStart);
}

/*------------------------------------------------------------------------------
| IGPie::sweepAngle                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double IGPie::sweepAngle( ) const
{
  return LONG2DBL(fSweep);
}

/*------------------------------------------------------------------------------
| IGPie::setEnclosingRect                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie& IGPie::setEnclosingRect( const IRectangle& rectangle )
{
  fRect = rectangle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPie::setStartAngle                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie& IGPie::setStartAngle( double startAngle )
{
  fStart = DBL2LONG(startAngle);
  return *this;
}

/*------------------------------------------------------------------------------
| IGPie::setSweepAngle                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie& IGPie::setSweepAngle( double sweepAngle )
{
  fSweep = DBL2LONG(sweepAngle);
  return *this;
}

/*------------------------------------------------------------------------------
| IGPie::drawOn                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie& IGPie::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPie:operator=                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPie& IGPie::operator= (const IGPie& pie )
{
  if ( this == &pie )
    return *this;

  this->Inherited::operator=(pie);
  fRect    = pie.fRect;
  fStart   = pie.fStart;
  fSweep   = pie.fSweep;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPie:operator==                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGPie::operator== (const IGPie& pie ) const
{
  return ( fRect == pie.fRect && fStart == pie.fStart &&
           fSweep == pie.fSweep && this->Inherited::operator==(pie));
}

/*------------------------------------------------------------------------------
| IGPie:operator!=                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGPie::operator!= (const IGPie& pie ) const
{
  return !(*this == pie);
}

/*------------------------------------------------------------------------------
| IGChord::IGChord                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGChord::IGChord( const IRectangle& rect,
                  double startAngle, double sweepAngle )
  : IGChord::Inherited( rect, startAngle, sweepAngle )
{}

/*------------------------------------------------------------------------------
| IGChord::IGChord                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGChord::IGChord( const IGChord& chord )
  : IGChord::Inherited( chord )
{ }

/*------------------------------------------------------------------------------
| IGChord::IGChord                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGChord::~IGChord( )
{ }

/*------------------------------------------------------------------------------
| IGChord::draw                                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGChord& IGChord::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}
