/*******************************************************************************
* FILE NAME: igpyline.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igpyline.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igpyline.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGPYLINE_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGPolyline::IGPolyline                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline::IGPolyline( const IGPolyline& polyline )
  : IGPolyline::Inherited(polyline), fPtArray( polyline.fPtArray )
{
}

/*------------------------------------------------------------------------------
| IGPolyline::IGPolyline                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline::IGPolyline( const IPointArray& pointArray )
  : IGPolyline::Inherited(), fPtArray( pointArray )
{
}

/*------------------------------------------------------------------------------
| IGPolyline::~IGPolyline                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline::~IGPolyline()
{
}

/*------------------------------------------------------------------------------
| IGPolyline::drawOn                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::numberOfPoints                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGPolyline::numberOfPoints() const
{
  return fPtArray.size();
}

/*------------------------------------------------------------------------------
| IGPolyline::pointArray                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
const IPointArray& IGPolyline::pointArray() const
{
  return fPtArray;
}

/*------------------------------------------------------------------------------
| IGPolyline::pointArray                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
const IPoint& IGPolyline::point( unsigned long index ) const
{
  return fPtArray[index];
}

/*------------------------------------------------------------------------------
| IGPolyline::setPoints                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::setPoints( const IPointArray& pointArray )
{
  fPtArray = pointArray;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::setPoints                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::setPoint( unsigned long index, const IPoint& point )
{
  fPtArray[index] = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::insertPoint                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::insertPoint(  unsigned long index, const IPoint& point )
{
  fPtArray.insert( index, point );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::addPoint                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::addPoint( const IPoint& point )
{
  fPtArray.add( point );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::removePoint                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::removePoint( unsigned long index )
{
  fPtArray.remove( index );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::reversePoints                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::reversePoints( )
{
  fPtArray.reverse();
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::operator=                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolyline& IGPolyline::operator= ( const IGPolyline& polyline )
{
  if (this == &polyline)
    return *this;

  this->Inherited::operator=(polyline);
  fPtArray = polyline.fPtArray;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolyline::operator==                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGPolyline::operator== ( const IGPolyline& polyline ) const
{
  return ( fPtArray == polyline.fPtArray &&
           this->Inherited::operator==( polyline ));
}

/*------------------------------------------------------------------------------
| IGPolyline::operator!=                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGPolyline::operator!= ( const IGPolyline& polyline ) const
{
  return !( *this == polyline );
}

/*------------------------------------------------------------------------------
| IGPolygon::IGPolygon                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon::IGPolygon( const IGPolygon& polygon)
  : IGPolygon::Inherited( polygon ), fFm(polygon.fFm)
{
}

/*------------------------------------------------------------------------------
| IGPolygon::IGPolygon                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon::IGPolygon( const IPointArray& pointArray )
  : IGPolygon::Inherited(pointArray), fFm(IGPolygon::alternate)
{
}

/*------------------------------------------------------------------------------
| IGPolygon::~IGPolygon                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon::~IGPolygon()
{
}

/*------------------------------------------------------------------------------
| IGPolygon::drawOn                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon& IGPolygon::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolygon::fillMode                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon::FillMode IGPolygon::fillMode() const
{
  return fFm;
}

/*------------------------------------------------------------------------------
| IGPolygon::setFillMode                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon& IGPolygon::setFillMode( IGPolygon::FillMode fillmode )
{
  fFm = fillmode;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolygon::operator=                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGPolygon& IGPolygon::operator= ( const IGPolygon& polygon )
{
  if ( this == &polygon )
    return *this;

  this->Inherited::operator=(polygon);
  fFm      = polygon.fFm;
  return *this;
}

/*------------------------------------------------------------------------------
| IGPolygon::operator==                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IGPolygon::operator== ( const IGPolygon& polygon ) const
{
  return ( this->Inherited::operator==( polygon ) && fFm == polygon.fFm );
}

/*------------------------------------------------------------------------------
| IGPolygon::operator!=                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IGPolygon::operator!= ( const IGPolygon& polygon ) const
{
  return !(*this == polygon);
}

