/*******************************************************************************
* FILE NAME: igrect.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igrect.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igrect.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRECT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGRectangle::IGRectangle                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::IGRectangle( const IGRectangle& rect )
  : IGRectangle::Inherited(rect), fRect( rect.fRect ), fRounding(rect.fRounding)
{
}

/*------------------------------------------------------------------------------
| IGRectangle::IGRectangle                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::IGRectangle( const IRectangle& rect )
  : IGRectangle::Inherited(), fRect( rect ), fRounding(0,0)
{
}

/*------------------------------------------------------------------------------
| IGRectangle::~IGRectangle                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::~IGRectangle()
{
}

/*------------------------------------------------------------------------------
| IGRectangle::drawOn                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::setRounding                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::setRounding( const IPair& ellipseAxis )
{
  fRounding = ellipseAxis;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::rounding                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPair IGRectangle::rounding() const
{
  return fRounding;
}

/*------------------------------------------------------------------------------
| IGRectangle::enclosingRect                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IGRectangle::enclosingRect( ) const
{
  return fRect;
}

/*------------------------------------------------------------------------------
| IGRectangle::setRectangle                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::setEnclosingRect( const IRectangle& rectangle )
{
  fRect = rectangle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::operator=                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::operator= ( const IGRectangle& gRectangle )
{
  if ( this == &gRectangle )
    return *this;

  this->Inherited::operator=(gRectangle);
  fRect     = gRectangle.fRect;
  fRounding = gRectangle.fRounding;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::operator==                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGRectangle::operator== ( const IGRectangle& gRectangle ) const
{
  return (fRect == gRectangle.fRect && fRounding == gRectangle.fRounding &&
          this->Inherited::operator==(gRectangle));
}

/*------------------------------------------------------------------------------
| IGRectangle::operator!=                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGRectangle::operator!= ( const IGRectangle& gRectangle ) const
{
  return !(*this == gRectangle);
}
