/*******************************************************************************
* FILE NAME: ihelp0.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihelp.hpp.  (platform independent)                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first.
#pragma priority( -2147482102 )
#define INCL_WINHELP
#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #ifndef IC_PM
     #include <ipfx.h>
  #endif
}

#include <ihelp.hpp>
#include <ihelpprv.hpp>
#include <ihelpsta.hpp>
#ifdef IC_WIN
  #include <ihelptbl.hpp>
  #include <iframe.hpp>
#endif
#include <icconst.h>
#include <iexcept.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <iwindow.hpp>
#include <itrace.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IHELP0_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IHelpWindowData::IHelpWindowData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IHelpWindowData::IHelpWindowData ( )
  : helpHandle( 0 ),
    helpLibraries( "" ),
    newHTResLib(false),
    pHTResLib( 0 ),
    pMAResLib( 0 ),
#ifdef IC_WIN
    isUsingIPF( false )
#else
    isUsingIPF( true )
#endif
#ifdef IC_MOTIFWIN
   ,lastActiveHandle( 0 )
#endif
#ifdef IC_WIN
   ,winHelpHandler( 0 )
   ,pHlpTable ( 0 )
   ,usingHelpId ( 0 )
   ,ownerForIPF ( 0 )
#endif
{ }

/*------------------------------------------------------------------------------
| IHelpWindowData::~IHelpWindowData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IHelpWindowData::~IHelpWindowData ( )
{
  if ( newHTResLib && pHTResLib )
     delete pHTResLib;
  if ( pMAResLib )
     delete pMAResLib;
}


/*------------------------------------------------------------------------------
| IHelpWindow::Settings::Settings                                              |
|  Create a Settings object and initialize.                                    |
------------------------------------------------------------------------------*/
IHelpWindow::Settings::Settings ( )
  : title         (""),
    tutorial      (""),
    helpLibraries (""),
    HTResLib      (""),
    MAResLib      ("")
{
  usingHelpId   = 0;
  helpTableId   = 0;
  menuBarId     = 0;
  acceleratorId = 0;
  pHelpSettings = 0;
}


// IHelpWindow constructors and destructor in ihelp1.cpp.

/*------------------------------------------------------------------------------
| IHelpWindow::isIPFCompatible                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpWindow :: isIPFCompatible( ) const
{
  return fHelpWindowData->isUsingIPF;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setTitle                                                        |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setTitle ( const char* title )
{
  if (this->isIPFCompatible())
  {
    this->sendEvent( HM_SET_HELP_WINDOW_TITLE,
                     IEventParameter1( (char*)title ), 0 );
  }
  else
  {
    /* not supported by WinHelp */
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setTitle                                                        |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setTitle ( const IResourceId& id )
{
  IString temp = id.resourceLibrary().loadString( id );
  return this->setTitle( temp );
}

/*------------------------------------------------------------------------------
| IHelpWindow::setHelpTable                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setHelpTable ( const IResourceId& helpTable )
{
#ifdef IC_PM
  helpTable.resourceLibrary().loadHelpTable( this, helpTable.id() );
#endif
#ifdef IC_WIN
  if ( fHelpWindowData->pHlpTable )
    delete fHelpWindowData->pHlpTable;
  fHelpWindowData->pHlpTable =
      helpTable.id() ? new IHelpTable(helpTable) : 0;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::showPanelIds                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::showPanelIds ( Boolean visibleId )
{
  if (this->isIPFCompatible())
  {
    this->sendEvent( HM_SET_SHOW_PANEL_ID,
                     visibleId ? CMIC_SHOW_PANEL_ID : CMIC_HIDE_PANEL_ID,
                     0 );
  }
  else
  {
    /* not supported by WinHelp */
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::hidePanelIds                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::hidePanelIds ( )
{
  return this->showPanelIds( false );
}

// sizeTo, moveTo, moveSizeTo in ihelp1.cpp.

/*------------------------------------------------------------------------------
| IHelpWindow::handle                                                          |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::handle ( ) const
{
  if (this->isIPFCompatible())
  {
    return fHelpWindowData->helpHandle;
  }
  else
  {
    return IWindow::objectWindow()->handle();
  }
}

/*------------------------------------------------------------------------------
| IHelpWindow::addLibraries                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::addLibraries ( const char* helpLibraryNames )
{
  IFUNCTRACE_DEVELOP();

  // Since the OS/2 message actually replaces the previous libraries,
  // we must maintain a list and keep adding to it, separating them
  // with a blank space.  If they pass NULL then we clear out all previously
  // set libraries.  The old list of libraries is saved in case an error
  // occurs (in which case we want to reset them back to what they were).

  IString oldLibraries;
  if ( helpLibraryNames && (*helpLibraryNames != '\0') )
  {
     oldLibraries = fHelpWindowData->helpLibraries;
     if ( fHelpWindowData->helpLibraries != "" && this->isIPFCompatible())
     {
        fHelpWindowData->helpLibraries += " ";
        fHelpWindowData->helpLibraries += helpLibraryNames;
     }
     else
        fHelpWindowData->helpLibraries = helpLibraryNames;
  }
  else
     fHelpWindowData->helpLibraries = "";

  int rc=0;
  if (this->isIPFCompatible())
  {
//#ifdef IC_PM
    rc = this->sendEvent( HM_SET_HELP_LIBRARY_NAME,
                          IEventParameter1( (char*)fHelpWindowData->helpLibraries ),
                          IEventParameter2( 0 ));
//#endif
#ifdef IC_WIN_NOTYET
    rc = IHelpStatics::callIPF(fHelpWindowData->helpHandle,
                               HM_SET_HELP_LIBRARY_NAME,
                              (char*)fHelpWindowData->helpLibraries, 0 );
#endif
  }
// Doing the following native winhelp call causes help to be displayed.
// That is not always desirable when simply setting the help file name.
// The problem is that there will be no indication that the help file name is
// bad until a request is made that causes WinHelp to display.  jks.
//else
//{
//  rc = WinHelp((HWND)this->handle(),
//               fHelpWindowData->helpLibraries,
//               HELP_FORCEFILE,
//               0);
//}
  if (rc)
  {
    ITRACE_DEVELOP("HM_SET_HELP_LIBRARY_NAME or HELP_FORCEFILE failed");
     // Since an error occured, we need to reset the libraries and
     // throw an exception.
     if ( fHelpWindowData->helpLibraries != "" )
     {
        fHelpWindowData->helpLibraries = oldLibraries;

        if (this->isIPFCompatible())
        {
//#ifdef IC_PM
          this->sendEvent( HM_SET_HELP_LIBRARY_NAME,
                           IEventParameter1( (char*)fHelpWindowData->helpLibraries ),
                           IEventParameter2( 0 ));
//#endif
#ifdef IC_WIN_NOTYET
          IHelpStatics::callIPF(fHelpWindowData->helpHandle,
                                HM_SET_HELP_LIBRARY_NAME,
                                (char*)fHelpWindowData->helpLibraries, 0 );
#endif
        }

        ITHROWLIBRARYERROR( IC_HMERR_OPEN_LIB_FILE,
                            IBaseErrorInfo::invalidRequest,
                            IException::recoverable );
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::hide                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::hide ( )
{
  if (this->isIPFCompatible())
  {
    this->sendEvent( HM_DISMISS_WINDOW,
                     IEventParameter1( 0 ),
                     IEventParameter2( 0 ) );
  }
#ifdef IC_WIN
  else
  {
    HELPWININFO helpWinInfo;
    LPHELPWININFO lpHelpWinInfo=&helpWinInfo;

    helpWinInfo.wStructSize = sizeof(HELPWININFO);
    helpWinInfo.x           = 0;
    helpWinInfo.y           = 0;
    helpWinInfo.dx          = 0;
    helpWinInfo.dy          = 0;
    helpWinInfo.wMax        = SW_HIDE;
//  helpWinInfo.rgchMember  = 0;

    WinHelp((HWND)this->handle(),
            fHelpWindowData->helpLibraries,
            HELP_SETWINPOS,
            (DWORD)lpHelpWinInfo);
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( Boolean )
{
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( HelpType helpType )
{
#ifdef IC_PM
  switch ( helpType )
  {
     case index:
        this->sendEvent( HM_HELP_INDEX,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case contents:
        this->sendEvent( HM_HELP_CONTENTS,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case general:
        this->sendEvent( HM_GENERAL_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case keys:
        this->sendEvent( HM_KEYS_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case usingHelp:
        this->sendEvent( HM_DISPLAY_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
  } /* endswitch */
#endif // IC_PM

#ifdef IC_WIN
  if (fHelpWindowData->lastActiveHandle)
    fHelpWindowData->ownerForIPF = fHelpWindowData->lastActiveHandle;

  switch ( helpType )
  {
     case index:
        if (this->isIPFCompatible())
        {
          this->sendEvent( HM_HELP_INDEX,
                           IEventParameter1( 0 ),
                           IEventParameter2( 0 ) );
        }
        else
        {
          WinHelp((HWND)this->handle(),
                  fHelpWindowData->helpLibraries,
                  HELP_FINDER,
                  0);
        }
        break;
     case contents:
        if (this->isIPFCompatible())
        {
          this->sendEvent( HM_HELP_CONTENTS,
                           IEventParameter1( 0 ),
                           IEventParameter2( 0 ) );
        }
        else
        {
          WinHelp((HWND)this->handle(),
                  fHelpWindowData->helpLibraries,
                  HELP_CONTENTS,
                  0);
        }
        break;
     case general:
        {
          // search for active frame
          unsigned long helpID = IHelpTable::noHelpFound;
          IFrameWindow *activeFrame =
            ihelpwindowSearchChain(IQUERYFOCUS(0),
                                   fHelpWindowData->lastActiveHandle);
          if ( !activeFrame )
            return *this;

          // search help table for general help id
          // or get it directly from the frame
          if ( fHelpWindowData->pHlpTable )
          {
            unsigned long frameID = activeFrame->id();
            helpID = fHelpWindowData->pHlpTable->
                        extendedHelpForWindowID((unsigned short)frameID);
          }
          else
          {
            helpID = activeFrame->helpId();
          }

          //if general help is not found, send a message to the active frame
          if ( helpID == IHelpTable::noHelpFound || helpID == 0 )
          {
            activeFrame->sendEvent(HM_GENERAL_HELP_UNDEFINED,
                                   IEventParameter1( helpID ),
                                   IEventParameter2( 0 ) );
            return *this;
          }

          // request display of general help id
          unsigned long helpError = 0;
          if (this->isIPFCompatible())
          {
            helpError = this->sendEvent( HM_DISPLAY_HELP,
                                  IEventParameter1( helpID ),
                                  IEventParameter2( 0 ) );
          }
          else
          {
            helpError = WinHelp(this->handle(),
                         fHelpWindowData->helpLibraries,
                         HELP_CONTEXT,
                         helpID);
            if (helpError == 0)
              helpError = HMERR_PANEL_NOT_FOUND;
          }

          // if display of general help failed, send msg to active frame
          if (helpError != 0)
          {
            activeFrame->sendEvent(helpError,
                                   IEventParameter1( helpID ),
                                   IEventParameter2( 0 ) );
            return *this;
          }
        }
        break;
     case keys:
        {
          // search for active frame
          IFrameWindow *activeFrame =
            ihelpwindowSearchChain(IQUERYFOCUS(0),
                                   fHelpWindowData->lastActiveHandle);
          if ( !activeFrame )
            return *this;

          // send query keys help message to active frame and check event result
          // for the the id of the panel to display
          unsigned long keysHelpID =
            activeFrame->sendEvent(HM_QUERY_KEYS_HELP,
                                   IEventParameter1( 0 ),
                                   IEventParameter2( 0 ) );
          if ( !keysHelpID )
            return *this;

          // request display of keys help id
          unsigned long helpError = 0;
          if (this->isIPFCompatible())
          {
            helpError = this->sendEvent( HM_DISPLAY_HELP,
                                  IEventParameter1( keysHelpID ),
                                  IEventParameter2( 0 ) );
          }
          else
          {
            helpError = WinHelp(this->handle(),
                         fHelpWindowData->helpLibraries,
                         HELP_CONTEXT,
                         keysHelpID);
            if (helpError == 0)
              helpError = HMERR_PANEL_NOT_FOUND;
          }

          // if display of general help failed, send msg to active frame
          if (helpError != 0)
          {
            activeFrame->sendEvent(helpError,
                                   IEventParameter1( keysHelpID ),
                                   IEventParameter2( 0 ) );
            return *this;
          }
        }
        break;
     case usingHelp:
        // request display of help for help
        if (this->isIPFCompatible())
        {
          unsigned long usingHelpId = fHelpWindowData->usingHelpId;
          // Due to our inability to get usingHelp fixed at the source,
          // I have to hardcode in this id to make this work.
          if (usingHelpId == 0)
            usingHelpId = 64027;
          this->sendEvent( HM_DISPLAY_HELP,
                           IEventParameter1( usingHelpId ),
                           IEventParameter2( 0 ) );
        }
        else
        { // native windows help
          if (fHelpWindowData->usingHelpId)
          {
            WinHelp((HWND)this->handle(),
                    fHelpWindowData->helpLibraries,
                    HELP_CONTEXT,
                    fHelpWindowData->usingHelpId);
          }
          else
          {
            WinHelp((HWND)this->handle(),
                    fHelpWindowData->helpLibraries,
                    HELP_HELPONHELP,
                    0);
          }
        }
        break;
  } /* endswitch */
#endif // IC_WIN
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( const IResourceId& panelId )
{
#ifdef IC_WIN
  if (fHelpWindowData->lastActiveHandle)
    fHelpWindowData->ownerForIPF = fHelpWindowData->lastActiveHandle;
#endif
  if ( panelId.id() )
  {
    if (this->isIPFCompatible())
    {
      this->sendEvent( HM_DISPLAY_HELP,
                       IEventParameter1( panelId.id() ),
                       IEventParameter2( HM_RESOURCEID ) );
    }
#ifdef IC_WIN
    else
    {
      WinHelp((HWND)this->handle(),
              fHelpWindowData->helpLibraries,
              HELP_CONTEXT,
              panelId.id());
    }
#endif
  }
  else
  { // else - display system default help - "Using Help"
    if (this->isIPFCompatible())
    {
      this->sendEvent( HM_DISPLAY_HELP,
                       IEventParameter1( 0 ),
                       IEventParameter2( 0 ) );
    }
#ifdef IC_WIN
    else
    {
      WinHelp((HWND)this->handle(),
              fHelpWindowData->helpLibraries,
              HELP_HELPONHELP,
              0);
    }
#endif
  } /* endif */

  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( const char* panelName )
{
#ifdef IC_WIN
  if (fHelpWindowData->lastActiveHandle)
    fHelpWindowData->ownerForIPF = fHelpWindowData->lastActiveHandle;
#endif
  if ( panelName && this->isIPFCompatible())
  {
    this->sendEvent( HM_DISPLAY_HELP,
                     IEventParameter1( (char*)panelName ),
                     IEventParameter2( HM_PANELNAME ) );
  }
  else
  {  // else - display system default help
    if (this->isIPFCompatible())
    {
      this->sendEvent( HM_DISPLAY_HELP,
                       IEventParameter1( 0 ),
                       IEventParameter2( 0 ) );
    }
#ifdef IC_WIN
    else
    {
      WinHelp((HWND)this->handle(),
              fHelpWindowData->helpLibraries,
              HELP_PARTIALKEY,
              (DWORD)panelName);
    }
#endif
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setUsingHelp                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setUsingHelp ( unsigned long id )
{
  if (this->isIPFCompatible())
  {
    this->sendEvent( HM_REPLACE_USING_HELP,
                     IEventParameter1( id ),
                     IEventParameter2( 0 ) );
  }
#ifdef IC_WIN
  fHelpWindowData->usingHelpId = id;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::indexWindow                                                     |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::indexWindow ( ) const
{
  if (this->isIPFCompatible())
  {
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_INDEX ),
                                     IEventParameter2( 0 ) );
//#ifdef IC_PM
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::contentsWindow                                                  |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::contentsWindow ( ) const
{
  if (this->isIPFCompatible())
  {
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_TOC ),
                                     IEventParameter2( 0 ) );
//#ifdef IC_PM
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::searchListWindow                                                |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::searchListWindow ( ) const
{
  if (this->isIPFCompatible())
  {
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_SEARCH ),
                                     IEventParameter2( 0 ) );
//#ifdef IC_PM
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::viewedPagesWindow                                               |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::viewedPagesWindow ( ) const
{
  if (this->isIPFCompatible())
  {
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_VIEWPAGES ),
                                     IEventParameter2( 0 ) );
//#ifdef IC_PM
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::communicationWindow                                             |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::communicationWindow ( ) const
{
  if (this->isIPFCompatible())
  {
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_OBJCOM_WINDOW ),
                                     IEventParameter2( 0 ) );
//#ifdef IC_PM
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::coverPageWindow                                                 |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::coverPageWindow ( ) const
{
  if (this->isIPFCompatible())
  {
//#ifdef IC_PM
    unsigned long handle =
                    this->sendEvent( HM_QUERY,
                                     IEventParameter1( 0, HMQW_COVERPAGE ),
                                     IEventParameter2( 0 ) );
    return IWindowHandle( (HWND)handle );
//#endif
#ifdef IC_WIN_NOTYET
    long handle =
      IHelpStatics::callIPF(fHelpWindowData->helpHandle,
                            HM_SET_HELP_LIBRARY_NAME,
                           (char*)fHelpWindowData->helpLibraries, 0 );
    return IWindowHandle( (HWND)handle );
#endif
  }
  else
    return 0;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IHelpWindow::Settings& IHelpWindow::Settings::setTitle ( const char* title )
{
  this->title = IString( title );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setTitle ( const IResourceId& titleId )
{
  this->title = titleId.resourceLibrary().loadString( titleId );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setHelpResLibrary                                     |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setHelpResLibrary ( const char* helpTableResLibrary )
{
  this->HTResLib = IString( helpTableResLibrary );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setAccelResLibrary                                    |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setAccelResLibrary ( const char* menuAccelResLibrary )
{
  this->MAResLib = IString( menuAccelResLibrary );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTutorial                                           |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setTutorial ( const char* tutorial )
{
  this->tutorial = IString( tutorial );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setLibraries                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setLibraries ( const char* helpLibraryNames )
{
  this->helpLibraries = IString( helpLibraryNames );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setHelpTable                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setHelpTable ( unsigned long helpTable )
{
  this->helpTableId = helpTable;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setUsingHelp                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setUsingHelp ( unsigned long helpId )
{
  this->usingHelpId = helpId;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setMenuBar                                            |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setMenuBar ( unsigned long menubar )
{
  this->menuBarId = menubar;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setAccelerator                                        |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setAccelerator ( unsigned long accelerator )
{
  this->acceleratorId = accelerator;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::helpWindow                                                      |
------------------------------------------------------------------------------*/
IHelpWindow* IHelpWindow::helpWindow( const IWindow* window )
{
  if (window==0)
    return 0;

#ifdef IC_PM
  IWindowHandle handle = 0;
  if (IHelpWindow::currentDefaultStyle & IHelpWindow::ipfCompatible)
  {
    handle = IHelpStatics::queryHelpInstance( window->handle() );
  }
  if (handle == 0)
     return 0;
  else
     return (IHelpWindow*)( IWindow::windowWithHandle( handle ) );
#endif
#ifdef IC_WIN
  IHelpWindow* helpWin = 0;
  Boolean      bSearch = true;
  HWND         startWindow=window->handle();

  // Search for a frame that responds to the help window query
  while (bSearch)
  {
    IFrameWindow *frame = ihelpwindowSearchChain(startWindow, 0);
    if (frame)
    {
      helpWin = (IHelpWindow *) (void *)
        frame->sendEvent( IC_UM_QUERY_HELP_WINDOW,
                          IEventParameter1(0),
                          IEventParameter2(0) );
      // If a help window returned, then exit
      if ( helpWin )
        bSearch = false;
      // otherwise get the parent of this frame and start again
      else
      {
        startWindow = IPARENTOF( frame->handle() );
        if (!startWindow)
          startWindow = IOWNEROF( frame->handle() );
      }
    }
    // If no frame (desktop or invalid), then stop searching (returns 0)
    else
      bSearch = false;
  }
  return helpWin;
#endif
}

