/*******************************************************************************
* FILE NAME: ilistbo1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistbnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINLISTBOXES

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _ILISTBOX_
  #include <ilistbox.hpp>
#endif

#ifndef _ILISTBNH_
  #include <ilistbnh.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILISTBO1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IListBoxNotifyStatics
{
public:

~IListBoxNotifyStatics ()
{
  delete (listBoxNotifyHandler);
  listBoxNotifyHandler = 0;
}
static IListBoxNotifyHandler
 &notificationHandler ( );

static IListBoxNotifyHandler
 *listBoxNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IListBoxNotifyStatics listBoxStatics;
INotificationId const IListBox::addId    = "IListBox::add";
INotificationId const IListBox::removeId = "IListBox::remove";
INotificationId const IBaseListBox::selectId = "IBaseListBox::select";
INotificationId const IBaseListBox::enterId  = "IBaseListBox::enter";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IListBoxNotifyHandler* IListBoxNotifyStatics::listBoxNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IListBoxNotifyStatics::notificationHandler                                   |
|                                                                              |
| Static function to create and return a notification handler for IListBox.    |
------------------------------------------------------------------------------*/
IListBoxNotifyHandler& IListBoxNotifyStatics::notificationHandler ()
{
  if (!(IListBoxNotifyStatics::listBoxNotifyHandler))
  {
    IListBoxNotifyStatics::listBoxNotifyHandler =
                                   new IListBoxNotifyHandler();
  }
  return (*IListBoxNotifyStatics::listBoxNotifyHandler);
}

/*------------------------------------------------------------------------------
| IBaseListBox::enableNotification                                                 |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IBaseListBox, does not send notifications to observer objects until it is        |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IListBoxNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);

  return (*this);
}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::IListBoxNotifyHandler                                 |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IListBoxNotifyHandler::IListBoxNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::~IListBoxNotifyHandler                                |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IListBoxNotifyHandler::~IListBoxNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IListBoxNotifyHandler::dispatchHandlerEvent                                  |
|                                                                              |
| This notifies IBaseListBox Windows of the following events:                      |
|                                                                              |
| select(Id)                                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean IListBoxNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
        if (className == WC_LISTBOX)
        {
#ifdef IC_WIN
          Boolean isMultiple = ((IBaseListBox*)pwndControl)->isMultipleSelect(),
                  isExtended = ((IBaseListBox*)pwndControl)->isExtendedSelect();

          if (ulMsg == IC_UM_LN_SELECT ||
             (ulMsg == LN_SELECT &&
               (!isMultiple || (isMultiple && isExtended))))
#else
          if (ulMsg == LN_SELECT)
#endif
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseListBox::selectId,
                                        *thisWindow));
          }
          else if (ulMsg == LN_ENTER)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseListBox::enterId,
                                        *thisWindow));
          }
        }
      }
    break;
    } /* endcase */

#ifdef IC_WIN
    case WM_CHAR:
    {
      if ((unsigned long)anEvent.parameter1() == VK_RETURN)
      {
        IWindow* thisWindow = anEvent.window();
        if (thisWindow)
        {
          thisWindow->notifyObservers(INotificationEvent(
                                      IBaseListBox::enterId,
                                      *thisWindow));
        }
      }
      else if ((unsigned long)anEvent.parameter1() == VK_SPACE)
      {
         IBaseListBox *listBox = (IBaseListBox*)(anEvent.controlWindow());
         IWindow* thisWindow = anEvent.window();
         if (listBox->isMultipleSelect() && !listBox->isExtendedSelect())
         {
            // Since Windows sends LBN_SELCHANGE notifications on multiple
            // select listboxes whenever the user presses the arrow keys,
            // we need to handle the key/mouse events responsible for
            // selection.  This will ensure correct behavior of ISelectHandler
            // and selectId notifications.
            thisWindow->notifyObservers( INotificationEvent(
                                         IBaseListBox::selectId,
                                         *listBox ));
         }
      }
    break;
    }

    case WM_LBUTTONUP:
    {
      IBaseListBox *listBox = (IBaseListBox*)(anEvent.controlWindow());
      IWindow* thisWindow = anEvent.window();
      if (listBox->isMultipleSelect() && !listBox->isExtendedSelect())
      {
         thisWindow->notifyObservers( INotificationEvent(
                                      IBaseListBox::selectId,
                                      *listBox ));
      }
    break;
    }
#endif

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

