/*******************************************************************************
* FILE NAME: ilistbox.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistbox.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINLISTBOXES      // LIT_xxx, LM_xxx, LS_xxx
  #define INCL_WINSYS            // SV_xxx, PP_xxx
  #include <iwindefs.h>
}

#include <inotifev.hpp>
#include <ilistbox.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <ihandle.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <itrace.hpp>
#include <icolor.hpp>
#include <ifont.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILISTBOX_CPP_
  #include <ipagetun.h>
#endif

/***************************************************************/
/* Public list box styles.                                     */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
const IListBox::Style
  IListBox::classDefaultStyle ( LS_HORZSCROLL  |
                                LS_NOADJUSTPOS |
                                WS_VISIBLE,
                                WS_EX_CLIENTEDGE );
#pragma data_seg()

/***************************************************************/
/* Default style for new objects (initial value)               */
/***************************************************************/
#pragma data_seg(ICLNonConst)
  IListBox::Style IListBox::currentDefaultStyle ( LS_HORZSCROLL  |
                                                  LS_NOADJUSTPOS |
                                                  WS_VISIBLE,
                                                  WS_EX_CLIENTEDGE );
#pragma data_seg()


class IListBoxData {
/*******************************************************************************
* The IListBoxData class encapsulates private data and functions used by the   *
* IListBox class.  An object of this class is created in the IListBox          *
* constructors.                                                                *
*******************************************************************************/
public:
  IListBoxData ( );

int
  largestStringSize;
Boolean
  horizontalScrollStyle;

private:
  IListBoxData ( const IListBoxData& );
IListBoxData
 &operator=    ( const IListBoxData& );

}; // IListBoxData

/*------------------------------------------------------------------------------
| IListBoxData::IListBoxData                                                   |
------------------------------------------------------------------------------*/
IListBoxData::IListBoxData ( )
{
  largestStringSize = 0;
}

/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Construct an IListBox on an IWindow.                                         |
------------------------------------------------------------------------------*/
IListBox::IListBox ( unsigned long     Id,
                     IWindow*          pParent,
                     IWindow*          pOwner,
                     const IRectangle& rectInit,
                     const Style&      style )
  : IBaseListBox( Id,
                  pParent,
                  pOwner,
                  rectInit,
                  (const IBaseListBox::Style&) style )
{
  fListBoxData = new IListBoxData();
}

/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Construct an IListBox on a dialog.                                           |
------------------------------------------------------------------------------*/
IListBox::IListBox ( unsigned long id,
                     IWindow* parent )
  : IBaseListBox( id, parent )
{
  fListBoxData = new IListBoxData();
}

/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Wrapper an existing ListBox.                                                 |
------------------------------------------------------------------------------*/
IListBox::IListBox ( const IWindowHandle& handle )
  : IBaseListBox( handle )
{
  fListBoxData = new IListBoxData();
}

/*------------------------------------------------------------------------------
| IListBox::~IListBox                                                          |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
IListBox::~IListBox()
{
   delete fListBoxData;
}

/*------------------------------------------------------------------------------
| IListBox::Style IListBox::defaultStyle                                       |
------------------------------------------------------------------------------*/
IListBox::Style IListBox::defaultStyle()
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IListBox::setDefaultStyle                                                    |
------------------------------------------------------------------------------*/
void IListBox::setDefaultStyle(const Style& style)
{
  currentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IListBox::setFont                                                            |
------------------------------------------------------------------------------*/
IWindow& IListBox::setFont ( const IFont& font )
{
  IWindow::setFont ( font );
  if (isHorizontalScroll())
  {
     fListBoxData->largestStringSize = getLargestStringSize();
     handle().sendEvent( LB_SETHORIZONTALEXTENT,
                         fListBoxData->largestStringSize,
                         0 );
  }
}


#include <ilistbx4.hpp>
