/* NOSHIP */
#ifndef _IMENUPRV_
#define _IMENUPRV_
/*******************************************************************************
* FILE NAME: imenuprv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of IMenu and its derived classes.                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1996             *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iwindow.hpp>
#include <iframe.hpp>

class IMenuHandle;
class IResourceId;

#pragma pack(4)

/*--------------------------------------------------------------------*/
/* Menu lookup table layout (in support of old Windows menus)         */
/*--------------------------------------------------------------------*/
typedef struct _fmTable  {  /* fpmtbl */
  unsigned long    fId;               /* Menu item id to map       */
  HMENU            fHMenu;            /* Menu handle to map id to  */
  struct _fmTable* pfmtNext;          /* Ptr to next table entry   */
} FMTABLE;

/*--------------------------------------------------------------------*/
/* Help lookup table layout (in support of old Windows menus)         */
/*--------------------------------------------------------------------*/
typedef struct _mihTable  {  /* mihtbl */
  unsigned long     fId;                 /* Menu item id to map       */
  unsigned long     fHelpId;             /* Menu handle to map id to  */
  struct _mihTable* pmitNext;            /* Ptr to next table entry   */
} MIHELPTBL;

/*--------------------------------------------------------------------*/
/* Frame table, initialized for and updated in enumeration proc       */
/*--------------------------------------------------------------------*/
typedef struct _FRAMEINFO {   /* frmi */
  IMenuHandle   hmenu;           /* Menu handle to check table for    */
  IWindowHandle owner;           /* Owner window handle               */
  IFrameWindow* frame;           /* Frame window pointer              */
} FRAMEINFO, *PFRAMEINFO;

/*--------------------------------------------------------------------*/
/* Menu resource flags ids (used in new MENUEX format)                */
/*--------------------------------------------------------------------*/
#define MFR_POPUP 0x01
#define MFR_END   0x80

/*--------------------------------------------------------------------*/
/* Define length of menu text string to use to retrieve text from O/S */
/*--------------------------------------------------------------------*/
#define IC_MENUTEXTLENGTH 100


class IMenuPrivate : public IBase {
typedef IBase
  Inherited;
public:
 ~IMenuPrivate( );

#ifdef IC_WIN
static unsigned long
  findIdForPopup           ( const IMenuHandle& hBaseMenu,
                             const IMenuHandle& hMenuToFind );
static void
  addToLookUpTable         ( IFrameWindow*      owner,
                             const IMenuHandle& menuHandle,
                             unsigned long      itemId ),
  removeFromLookUpTable    ( IFrameWindow*      owner,
                             const IMenuHandle& menuHandle,
                             unsigned long      itemId ),
  deleteLookUpTable        ( IFrameWindow*      owner ),
  reverseMenuForBidi       ( unsigned long hMenu );
static void
 *getHMenuFromLookUpTable  ( IFrameWindow*      owner,
                             unsigned long      itemId );
static long
  getItemIdFromLookUpTable ( IFrameWindow*      owner,
                             const IMenuHandle& hMenu );
static IFrameWindow
 *locateFrame              ( IWindow*           window );

static Boolean
  buildLookUpTableFromResourceLoad( IFrameWindow*      owner,
                                    const IMenuHandle& menuHandle,
                                    const IResourceId& menuResId ),
  locateMenuItem                  ( IFrameWindow*      owner,
                                    const IMenuHandle& hMenu,
                                    unsigned long      itemId,
                                    long*              pItemPos,
                                    IMenuHandle*       pMenu );

static void*
  loadMenuForWin32s    ( void*          hinstance,
                         unsigned long  menuResId );

static int __stdcall
  EnumAllFrames        ( void* hwnd,
                         long  lpData );
#endif
static void
  setMenuItemHelpId    ( IFrameWindow*  owner,
                         unsigned long  menuItemId,
                         unsigned long  helpTopicId ),
  deleteHelpIdTable    ( IFrameWindow*  owner );
static unsigned long
  queryMenuItemHelpId  ( IFrameWindow*  owner,
                         unsigned long  menuItemId );

private:
  IMenuPrivate( );

}; // IMenuPrivate


class IMenuData : public IBase {
public:
  IMenuData ( );
 ~IMenuData ( );

Boolean
  fUpdateAllowed;

}; // IMenuData

#pragma pack()

#endif  /* _IMENUPRV_ */

