/*******************************************************************************
* FILE NAME: imndihdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imndihdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482112 )
extern "C"
{
  #include <stdio.h>
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

#include <imndihdr.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMNDIHDR_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const IMenuDrawItemHandler::DrawFlag
  IMenuDrawItemHandler::drewChecked   ( 0, IDIF_CHECKED ),
  IMenuDrawItemHandler::drewDisabled  ( 0, IDIF_DISABLED ),
  IMenuDrawItemHandler::drewSelected  ( 0, IDIF_SELECTED );
#pragma data_seg()

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::dispatchHandlerEvent                                   |
------------------------------------------------------------------------------*/
Boolean IMenuDrawItemHandler::dispatchHandlerEvent(IEvent& evt)
{
   IMenuDrawItemEvent mndievt(evt);
   switch (evt.eventId()) {
   case WM_MEASUREITEM:
   {
      ISize siz(1,1);
      if (setSize(mndievt, siz) ) {
#ifdef IC_WIN
         ((LPMEASUREITEMSTRUCT)(void*)evt.parameter2())->itemWidth =
                                                           siz.width();
         ((LPMEASUREITEMSTRUCT)(void*)evt.parameter2())->itemHeight =
                                                           siz.height();
#else
         ((POWNERITEM)(void*)evt.parameter2())->rclItem.xRight = siz.width();
         ((POWNERITEM)(void*)evt.parameter2())->rclItem.yTop = siz.height();
#endif
         return true;
      } /* endif */
      break;
   } /* endcase */
   case WM_DRAWITEM:
      if (mndievt.isDrawItem()) {
         IMenuDrawItemHandler::DrawFlag flag = 0;

         if (draw(mndievt, flag) ) {
            evt.setResult(true);

#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
            flag & IMenuDrawItemHandler::drewChecked ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                        ~MIA_CHECKED : 0;

            flag & IMenuDrawItemHandler::drewSelected ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                         ~MIA_HILITED : 0;

            flag & IMenuDrawItemHandler::drewDisabled ?
             ((POWNERITEM)(void*)evt.parameter2())->fsAttribute =
             ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                        ~MIA_DISABLED : 0;
#endif
            return true;
         } /* endif */
      }
      else
         if (mndievt.isSetHighlight()) {
            if(highlight(mndievt) ) {
               evt.setResult(true);
#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
               ((POWNERITEM)(void*)evt.parameter2())->fsAttribute &=
                                                                  ~MIA_HILITED;
               ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                                                  ~MIA_HILITED;
#endif
               return true;
            }
         }
         else
           if (mndievt.isRemoveHighlight()) {
              if(unhighlight(mndievt) ) {
                 evt.setResult(true);
#ifndef IC_WIN
// In Windows, the ownerdraw data is passed for information purposes
// only and does not require modification to complete drawing actions
                 ((POWNERITEM)(void*)evt.parameter2())->fsAttribute &=
                                                                  ~MIA_HILITED;
                 ((POWNERITEM)(void*)evt.parameter2())->fsAttributeOld &=
                                                                  ~MIA_HILITED;
#endif
                 return true;
              }
           }

      break;
   } /* endswitch */

   return false;
}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::IMenuDrawItemHandler                                   |
------------------------------------------------------------------------------*/
IMenuDrawItemHandler::IMenuDrawItemHandler()
{}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::~IMenuDrawItemHandler                                  |
------------------------------------------------------------------------------*/
IMenuDrawItemHandler::~IMenuDrawItemHandler()
{}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::setSize                                                |
------------------------------------------------------------------------------*/
Boolean IMenuDrawItemHandler::setSize(IMenuDrawItemEvent& evt, ISize& newSize)
{ return false; }

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::draw                                                   |
------------------------------------------------------------------------------*/
Boolean IMenuDrawItemHandler::draw(IMenuDrawItemEvent& evt, DrawFlag& flag)
{return false;}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::highlight                                              |
------------------------------------------------------------------------------*/
Boolean IMenuDrawItemHandler::highlight(IMenuDrawItemEvent& evt)
{return false;}

/*------------------------------------------------------------------------------
| IMenuDrawItemHandler::unhighlight                                            |
------------------------------------------------------------------------------*/
Boolean IMenuDrawItemHandler::unhighlight(IMenuDrawItemEvent& evt)
{return false;}
