/*******************************************************************************
* FILE NAME: imphdr.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imphdr.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <imphdr.hpp>
#include <iwindow.hpp>
#include <imousevt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMPHDR_CPP_
  #include <ipagetun.h>
#endif

/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
IMousePointerHandler :: IMousePointerHandler ( )
{}

/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IMousePointerHandler :: ~IMousePointerHandler ( )
{}

/*********************************************************************/
/* IMousePointerHandler::dispatchHanlderEvent                        */
/*                                                                   */
/* Return true if sent a query to determine if the window this       */
/* handler is attached to is an aggregate control (spin button,      */
/* combo box).                                                       */
/* This is needed so that the fly over help handler can determine    */
/* if the child windows need to be subclassed in order to provide    */
/* flyover help for a controls child windows.                        */
/*********************************************************************/
Boolean IMousePointerHandler::dispatchHandlerEvent( IEvent& event )
{
#ifdef IC_WIN
  if (event.eventId() == IC_UM_IS_AGGREGATE_CTRL)
  {
    event.setResult(true);
    return true;
  }
#endif
  return Inherited::dispatchHandlerEvent( event );
}

/*********************************************************************/
/* IMousePointerHandler::changeMousePointer                          */
/*                                                                   */
/* Pass message to the onwer.                                        */
/*********************************************************************/
Boolean IMousePointerHandler::mousePointerChange( IMousePointerEvent& event )
{
  IWindow *powner = event.window()->owner();
  if ( powner && event.window() != powner
              && event.window() == event.dispatchingWindow() )
  {
    event.setResult( powner->sendEvent( event ));
    return true;
  }
  return false;
}

/*******************************************************************/
/* Class to handle destruction of the static IMousePointerHandler  */
/* object.                                                         */
/*******************************************************************/
class IMousePointerHandlerStatics
{
public:
~IMousePointerHandlerStatics () {
   if(pDefHandler)
      delete pDefHandler; }

IMousePointerHandler
 *pDefHandler;
};   // IMousePointerHandlerStatics

#pragma data_seg(ICLNonConst)
static IMousePointerHandlerStatics IDefaultMPStatics;
#pragma data_seg()

/*******************************************************************/
/* IMousePointerHandler::defaultHandler                            */
/*******************************************************************/
IMousePointerHandler* IMousePointerHandler::defaultHandler ( )
{
   if(!IDefaultMPStatics.pDefHandler)
      IDefaultMPStatics.pDefHandler = new IMousePointerHandler();
   return IDefaultMPStatics.pDefHandler;
}
