/*******************************************************************************
* FILE NAME: inotebk1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in inotebnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSTDBOOK
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifdef IC_WIN
  #ifndef _ICLNOTEBOOKW_
  #include <iclnbw.h>
  #endif
#endif

#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif

#ifndef _INOTEBNH_
  #include <inotebnh.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEBK1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class INotebookNotifyStatics
{
public:

  ~INotebookNotifyStatics ()
  {
    delete notebookNotifyHandler;
    notebookNotifyHandler = 0;
  }

static INotebookNotifyHandler
  &notificationHandler ( );

static INotebookNotifyHandler
  *notebookNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static INotebookNotifyStatics notebookStatics;
INotificationId const INotebook::pageBackgroundColorId
                                       = "INotebook::pageBackgroundColor";
INotificationId const INotebook::majorTabBackgroundColorId
                                       = "INotebook::majorTabBackgroundColor";
INotificationId const INotebook::minorTabBackgroundColorId
                                       = "INotebook::minorTabBackgroundColor";
INotificationId const INotebook::majorTabForegroundColorId
                                       = "INotebook::majorTabForegroundColor";
INotificationId const INotebook::minorTabForegroundColorId
                                       = "INotebook::minorTabForegroundColor";
INotificationId const INotebook::orientationId
                                       = "INotebook::orientation";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
INotebookNotifyHandler*
  INotebookNotifyStatics::notebookNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| INotebookNotifyStatics::notificationHandler                                  |
|                                                                              |
| Static function to create and return a notification handler for INotebook.   |
------------------------------------------------------------------------------*/
INotebookNotifyHandler& INotebookNotifyStatics::notificationHandler ()
{
  if (!(INotebookNotifyStatics::notebookNotifyHandler))
  {
    INotebookNotifyStatics::notebookNotifyHandler =
                                   new INotebookNotifyHandler();
  }
  return(*INotebookNotifyStatics::notebookNotifyHandler);
}

/*------------------------------------------------------------------------------
| INotebook::enableNotification                                                |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| INotebook, does not send notifications to observer objects until it is       |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IWindow& INotebook::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(INotebookNotifyStatics::notificationHandler()));
  }
  return(IWindow::enableNotification(enabled));
}

/*------------------------------------------------------------------------------
| INotebookNotifyHandler::INotebookNotifyHandler                               |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
INotebookNotifyHandler::INotebookNotifyHandler()
{}

/*------------------------------------------------------------------------------
| INotebookNotifyHandler::~INotebookNotifyHandler                              |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
INotebookNotifyHandler::~INotebookNotifyHandler()
{}

/*------------------------------------------------------------------------------
| INotebookNotifyHandler::dispatchHandlerEvent                                 |
|                                                                              |
| This notifies INotebook Windows of the following events:                     |
|                                                                              |
| pageBackgroundColor(Id)                                                      |
| majorTabBackgroundColor(Id)                                                  |
| minorTabBackgroundColor(Id)                                                  |
| majorTabForegroundColor(Id)                                                  |
| minorTabForegroundColor(Id)                                                  |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean INotebookNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case BKM_SETNOTEBOOKCOLORS:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the selection state has been modified                      */
        /* Put color identifier into parameter 1 for later use        */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter2() ),
                               IEventParameter2( IC_NOTDEL_BOOK_COLOR ));
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* is used to determine which notification id to send           */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_BOOK_COLOR)
        {
          switch (anEvent.parameter1())
          {
            case BKA_BACKGROUNDPAGECOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::pageBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_BACKGROUNDMAJORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::majorTabBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_BACKGROUNDMINORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::minorTabBackgroundColorId,
                                          *thisWindow));
            break;

            case BKA_FOREGROUNDMAJORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::majorTabForegroundColorId,
                                          *thisWindow));
            break;

            case BKA_FOREGROUNDMINORCOLOR:
              thisWindow->notifyObservers(INotificationEvent(
                                          INotebook::minorTabForegroundColorId,
                                          *thisWindow));
            break;
          }
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

