/*******************************************************************************
* FILE NAME: inotehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     INotebookHandler                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINFRAMEMGR
//------------------------------------------------------------------------------  
// DEFECT 23265 (cloned from PORTAPAK)
//------------------------------------------------------------------------------  
  #define INCL_WININPUT             // WM_CHAR, WinGetKeyState, WinQueryFocus
//------------------------------------------------------------------------------  
  #define INCL_WINSTDBOOK
  #define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
  #include <iwindefs.h>

#ifdef IC_WIN
  #ifndef _WIN32
    #define _WIN32
  #endif
  #include <commctrl.h>
#endif
}

#include <inotehdr.hpp>

//------------------------------------------------------------------------------  
// DEFECT 23265 (cloned from PORTAPAK)
//------------------------------------------------------------------------------  
#ifdef IC_PMWIN
  #include <icanvas.hpp>
  #include <iframe.hpp>
#endif
//------------------------------------------------------------------------------  

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifdef IC_WIN
  #ifndef _ICLNOTEBOOKW_
    #include <iclnbw.h>
  #endif
  #ifndef _IWCNAME_
    #include <iwcname.hpp>
  #endif
  #ifndef _IKEYEVT_
    #include <ikeyevt.hpp>
  #endif
#endif

#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif
#ifndef _IPLATFRM_
  #include <iplatfrm.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEHDR_CPP_
  #include <ipagetun.h>
#endif

static unsigned long ulCurrentPageId = 0;


/*------------------------------------------------------------------------------
| INotebookHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean INotebookHandler :: dispatchHandlerEvent ( IEvent &event )
{
  Boolean stopProcessingEvent = false;

  switch ( event.eventId() )
  {
#ifndef IC_WIN
    case WM_HELP:
      {
        if ( !event.window()->hasFocus() )
        {          // Focus is not on the notebook (and hence, not a tab).
          IWindow* parent = event.window()->parent();
          if (parent)
          {
            /******************************************************************/
            /* The user must have requested contextual help from a            */
            /* page window.  We will avoid the notebook's processing          */
            /* of WM_HELP, which is geared for its tabs.  If the help         */
            /* request gets here, the page window must not be a frame         */
            /* (it is probably a canvas).  By routing the requests to         */
            /* the notebook's parent, these contextual helps can be           */
            /* resolved through the help subtable of the notebook's           */
            /* frame window.                                                  */
            /******************************************************************/
            parent->handle().postEvent( event.eventId(),
                                        event.parameter1(),
                                        event.parameter2() );
            stopProcessingEvent = true;
          }
        }
      }
      break;
#endif //not IC_WIN

#ifdef IC_WIN
    case WM_HELP:
      {
        /**********************************************************************/
        /* Get a pointer to the notebook object.                              */
        /**********************************************************************/
        INotebook* pNotebk = (INotebook *)event.controlWindow();
        IWindow* parent = event.window()->parent();
        HELPINFO helpInfo=*((LPHELPINFO)(void *)(event.parameter2()));

        /**********************************************************************/
        /* If focus is on a tab control's tab, (native and/or ccl)            */
        /*   then post BKN_HELP (processed by IPageHandler::help().           */
        /**********************************************************************/
        if (( pNotebk ) &&
            ( event.handle()==helpInfo.hItemHandle ) )
        {
          /********************************************************************/
          /* Windows tab control processing.  Get the top page handle.        */
          /********************************************************************/
          IPageHandle topPageHandle(0);
          if (!pNotebk->isEmpty())
            topPageHandle = pNotebk->topPage();

          if (parent)
          {
            /******************************************************************/
            /* The user must have requested contextual help on a tab          */
            /* control's tab.  Simulate the tab help notification for the     */
            /* CUA '91 notebook control by posting the page help event        */
            /* to the parent of the notebook control.                         */
            /******************************************************************/
            parent->handle().postEvent(
                               WM_COMMAND,
                               IEventParameter1( (unsigned short)pNotebk->id(),
                                                 BKN_HELP ),
                               IEventParameter2( topPageHandle ) );

            stopProcessingEvent = true;
          }
        }
        else
        {
          if (parent)
          {
            /******************************************************************/
            /* The user must have requested contextual help from a            */
            /* page window.  We will avoid the notebook's processing          */
            /* of WM_HELP, which is geared for its tabs.  If the help         */
            /* request gets here, the page window must not be a frame         */
            /* (it is probably a canvas).  By routing the requests to         */
            /* the notebook's parent, these contextual helps can be           */
            /* resolved through the help subtable of the notebook's           */
            /* frame window.  Please note that we must send the help message  */
            /* rather than post it since parameter2 is a pointer to the       */
            /* HELPINFO structure.                                            */
            /******************************************************************/
            parent->handle().sendEvent( event.eventId(),
                                        event.parameter1(),
                                        event.parameter2() );

            stopProcessingEvent = true;
          }
        }
      } //WM_HELP
      break;

    case WM_SIZE:
    case IC_UM_TCM_SIZE:
      {
        /**********************************************************************/
        /* Only process Windows tab control sizing event.                     */
        /**********************************************************************/
        IWindowHandle hwndControl( event.controlHandle() );
        if ( !(hwndControl &&
             ( IWindowClassName( hwndControl ) == WC_TABCONTROL )) )
          break;

        /**********************************************************************/
        /* Obtain pointer to notebook object.                                 */
        /**********************************************************************/
        INotebook*
          pNotebk = (INotebook*)IWindow::windowWithHandle( hwndControl );

        /**********************************************************************/
        /* If all tabs are to be shown (i.e. multi rows), we have to size the */
        /* tab control prior to calculating the display rectangle, to prevent */
        /* the application page window from clipping the bottom row of tabs.  */
        /**********************************************************************/
        ISize newSize( event.parameter2().lowNumber(),
                       event.parameter2().highNumber() );

        if ( pNotebk->areAllTabsVisible() )
        {
          if ( event.eventId() ==  WM_SIZE )
          {
            SetWindowPos( pNotebk->handle(),
                          NULL, 0, 0,
                          (int)newSize.width(),
                          (int)newSize.height(),
                          SWP_NOMOVE | SWP_NOZORDER );

            event.controlHandle().postEvent( IC_UM_TCM_SIZE,
                                             event.parameter1(),
                                             event.parameter2() );
          }
          else
          {
            pNotebk->tabControlResize( newSize );

            stopProcessingEvent = true;
          }
        }
        else
        {
          pNotebk->tabControlResize( newSize );
        }
      } //WM_SIZE & IC_UM_TCM_SIZE
      break;

    case WM_SYSKEYDOWN:
      {
        /**********************************************************************/
        /* Only process Windows tab control keyboard events of interest.      */
        /**********************************************************************/
        IWindowHandle hwndControl( event.controlHandle() );
        if ( !(hwndControl &&
             ( IWindowClassName( hwndControl ) == WC_TABCONTROL )) )
          break;

        /**********************************************************************/
        /* Process key events for the Windows tab control.                    */
        /**********************************************************************/
        IKeyboardEvent keyevt( event );
        if ( !keyevt.isVirtual() || !keyevt.isAltDown() ||
             keyevt.isShiftDown() || keyevt.isCtrlDown() )
        {
          break;
        }

        /**********************************************************************/
        /* Obtain pointer to notebook object.                                 */
        /**********************************************************************/
        INotebook*
          pNotebk = (INotebook*)IWindow::windowWithHandle( hwndControl );

        /**********************************************************************/
        /* Alt+Down:  set the focus to the application's page window          */
        /**********************************************************************/
        if ( keyevt.virtualKey() == IKeyboardEvent::down )
        {
          /********************************************************************/
          /* Get the top page handle if it exists.                            */
          /********************************************************************/
          IPageHandle topPageHandle = pNotebk->topPage();
          if (topPageHandle)
          {
            /******************************************************************/
            /* Get the application page window if it exists and set the       */
            /* focus to it.                                                   */
            /******************************************************************/
            IWindow* pageWindow = pNotebk->window( topPageHandle );
            if (pageWindow)
            {
              pageWindow->setFocus();
              stopProcessingEvent = true;
            }
          }
        }
        /**********************************************************************/
        /* Alt+PageUp:  bring the previous page to the top                    */
        /**********************************************************************/
        else if ( keyevt.virtualKey() == IKeyboardEvent::pageUp )
        {
          IPageHandle topPageHandle = pNotebk->topPage();
          if (topPageHandle)
          {
            IPageHandle prevPageHandle = pNotebk->previousPage( topPageHandle );
            if (prevPageHandle)
            {
              pNotebk->turnToPage( prevPageHandle );
              stopProcessingEvent = true;
            }
          }
        }
        /**********************************************************************/
        /* Alt+PageDown:  bring the next page to the top                      */
        /**********************************************************************/
        else if ( keyevt.virtualKey() == IKeyboardEvent::pageDown )
        {
          IPageHandle topPageHandle = pNotebk->topPage();
          if (topPageHandle)
          {
            IPageHandle nextPageHandle = pNotebk->nextPage( topPageHandle );
            if (nextPageHandle)
            {
              pNotebk->turnToPage( nextPageHandle );
              stopProcessingEvent = true;
            }
          }
        }
      } //WM_SYSKEYDOWN
      break;

    case WM_NOTIFY:
      {
        /**********************************************************************/
        /* Process the Windows tab ctrl's notifications for selection events  */
        /**********************************************************************/
        LPNMHDR lpnmhdr = (LPNMHDR)event.parameter2().asUnsignedLong();

        IWindowHandle hwndControl( lpnmhdr->hwndFrom );

        /**********************************************************************/
        /* Verify that we are processing a Windows tab control                */
        /**********************************************************************/
        if ( hwndControl  &&
             (IWindowClassName( hwndControl ) == WC_TABCONTROL) )
        {
          if ( (lpnmhdr->code == TCN_SELCHANGE) ||
               (lpnmhdr->code == TCN_SELCHANGING) )
          {
            INotebook*
              pNotebk = (INotebook*)IWindow::windowWithHandle( hwndControl );

            /******************************************************************/
            /* Fill in the PAGESELECTNOTIFY structure that we use in Windows  */
            /******************************************************************/
            PAGESELECTNOTIFY pageSelectNotify;
            pageSelectNotify.hwndBook = hwndControl;

            if ( lpnmhdr->code == TCN_SELCHANGE )
            {
              /****************************************************************/
              /* Use value for previously selected page that we stored during */
              /* processing of TCN_SELCHANGING.                               */
              /****************************************************************/
              pageSelectNotify.ulPageIdNew =
                   (DWORD)pNotebk->topPage().asUnsigned();
              pageSelectNotify.ulPageIdCur = (DWORD)ulCurrentPageId;

              /****************************************************************/
              /* The tab selection has already occurred and the new tab is    */
              /* now the top page in the tab control.  Do our required        */
              /* processing and issue the selection notification.             */
              /****************************************************************/
              stopProcessingEvent =
                pNotebk->processTabSelect( &pageSelectNotify, false );
            }
            else
            {
              /****************************************************************/
              /* The tab selection for a new tab is pending.                  */
              /* ------------------------------------------------------------ */
              /* We have no easy way of determining which new tab is selected.*/
              /* Therefore, we set both values to the top tab page.           */
              /****************************************************************/
              pageSelectNotify.ulPageIdNew =
                   (DWORD)pNotebk->topPage().asUnsigned();
              pageSelectNotify.ulPageIdCur = pageSelectNotify.ulPageIdNew;
              ulCurrentPageId = pageSelectNotify.ulPageIdCur;

              /****************************************************************/
              /* The tab selection is now in progress (pending).  Do our      */
              /* required processing and issue the selection pending          */
              /* notification.                                                */
              /****************************************************************/
              if ( pNotebk->processTabSelect( &pageSelectNotify, true ) )
                event.setResult( true );
            }
          }
          // DEFECT 23737 : duplicate of portapak fix
          // If the owner of this tab control is another tab control, we can't
          // forward any of the notification messages to the owner tab
          // control.  Doing so may result in a trap in the common controls dll.
          // Since we don't need to send these notification messages
          // up the owner chain anyway (we don't look for them in any of our
          // handlers), just return true for all of the WM_NOTIFY messages.
          stopProcessingEvent = true;
        }
        break;
      } //WM_NOTIFY

    case WM_SYSCOMMAND:
      {
        /**********************************************************************/
        /* Process the Windows tab control's WM_SYSCOMMAND/SC_CLOSE event     */
        /* for Windows95 to work around a Windows95 bug.                      */
        /**********************************************************************/
        IWindowHandle hwndControl( event.controlHandle() );
        if ( !(hwndControl &&
             ( IWindowClassName( hwndControl ) == WC_TABCONTROL )  &&
               IPlatform::isWin95()) )
        {
          break;
        }

        if ( (((unsigned long)event.parameter1()) & 0xFFF0) == SC_CLOSE )
        {
          INotebook*
            pNotebk = (INotebook*)IWindow::windowWithHandle( hwndControl );

          /********************************************************************/
          /* Return true if a tab control to circumvent Windows95 bug.        */
          /********************************************************************/
          if ( pNotebk && !pNotebk->isPMCompatible() )
          {
            return( true );
          }
        }
      } //WM_SYSCOMMAND
      break;

    case WM_SETFONT:
      {
        /**********************************************************************/
        /* Process the Windows tab control's WM_SETFONT event to resize the   */
        /* clipping window properly and not clip tabs                         */
        /**********************************************************************/
        IWindowHandle hwndControl( event.controlHandle() );
        if ( hwndControl && (IWindowClassName(hwndControl) == WC_TABCONTROL) )
        {
          // Get the notebook object
          INotebook* pnb = (INotebook*)IWindow::windowWithHandle(hwndControl);

          // Allow the native control to process the message
          this->defaultProcedure( event );

          // Event handled so don't route on.
          event.setResult( true );

          // If we found the object
          if ( pnb )
          {
            // Resize the window based on the new page window size
            IWindow* clip = pnb->pageClippingWindow();
            if (clip)
              clip->sizeTo( pnb->pageSize() );
          }
          return( true );
        }
      } //WM_SETFONT
      break;

#endif //IC_WIN

    /******************************************************************/
    /* Store the colors that are being set and then let this message  */
    /* pass on to the notebook control.                               */
    /******************************************************************/
    case BKM_SETNOTEBOOKCOLORS:
      ((INotebook*)event.window())
          ->storeNotebookColors( event.parameter1(), event.parameter2() );
      break;

//------------------------------------------------------------------------------  
// DEFECT 23265 (cloned from PORTAPAK)
//------------------------------------------------------------------------------  
    case WM_CHAR:
    {
      // Handle tabbing for the notebook.

      IKeyboardEvent keyEvent( event );
      if (keyEvent.isVirtual())
      {
         IKeyboardEvent::VirtualKey vkKey = keyEvent.virtualKey();

         if ((vkKey == IKeyboardEvent::tab ||
              vkKey == IKeyboardEvent::backTab) &&
             !keyEvent.isUpTransition())
            stopProcessingEvent = handleTab( keyEvent );
      }
      break;
    }

    case WM_SETFOCUS:
    {
      // If the notebook has received focus, check to determine whether
      // it was traversed to via a back tab.  If so, determine whether
      // the focus was previously on a descendant (ie, control in the page
      // window).  If so, then allow the notebook to receive focus.  If not,
      // set the focus on the page window if possible.
      
      unsigned long ulKeyState =
         IGETKEYSTATE( IWindow::desktopWindow()->handle(), VK_TAB ) &
         IGETKEYSTATE( IWindow::desktopWindow()->handle(), VK_SHIFT );
    
      if (ulKeyState & 0x8000)
      {
         // Set focus on the page window if it is tabbable.
         INotebook* notebook = (INotebook *)event.controlWindow();
         if ( !notebook->tabFromPage() )
            stopProcessingEvent = gotFocus( event );
         else
            notebook->tabFromPage(false);
      }
      break;
    }
//------------------------------------------------------------------------------  

    default:
      break;
  } //end switch

  return stopProcessingEvent;
}


/*------------------------------------------------------------------------------
| INotebookHandler::INotebookHandler                                           |
------------------------------------------------------------------------------*/
INotebookHandler :: INotebookHandler ( )
{
}

/*------------------------------------------------------------------------------
| INotebookHandler::~INotebookHandler                                          |
------------------------------------------------------------------------------*/
INotebookHandler :: ~INotebookHandler ( )
{
}

//------------------------------------------------------------------------------  
// DEFECT 23265 (cloned from PORTAPAK)
//------------------------------------------------------------------------------  
#ifdef IC_WIN
/*------------------------------------------------------------------------------
| INotebookHandler::handleTab                                                  |
|                                                                              |
| Process tabbing for the notebook.                                            |
------------------------------------------------------------------------------*/
Boolean INotebookHandler::handleTab ( IKeyboardEvent& keyEvent )
{
  Boolean stopProcessingEvent = false;

  IKeyboardEvent::VirtualKey vkKey = keyEvent.virtualKey();
  IWindowHandle hwndFocus = 
     IQUERYFOCUS(IWindow::desktopWindow()->handle());
  INotebook *notebook = (INotebook*)(keyEvent.dispatchingWindow());
  IWindowHandle hwndNotebook = notebook->handle();
  IPageHandle topPage = notebook->topPage();
  IWindow *pageWindow = (topPage) ? notebook->window( topPage ) : 0;

  // If focus is on the notebook itself, then set focus to the
  // page window if it is tabbable.
  if (hwndFocus == hwndNotebook &&
      pageWindow &&
      vkKey == IKeyboardEvent::tab)
  {
     if (pageWindow->isFrameWindow())
     {
        IWindowHandle
           hwndNextControl = GetNextDlgTabItem( pageWindow->handle(),
                                                NULL,
                                                FALSE );
        if (hwndNextControl)
        {
           pageWindow->postEvent( WM_NEXTDLGCTL,
                                  IEventParameter1( hwndNextControl ),
                                  IEventParameter2( true ));
           stopProcessingEvent = true;
        }
     }
     else
     {
        if (pageWindow->isTabStop())
        {
           pageWindow->setFocus();
           stopProcessingEvent = true;
        }
     }
  }
  else
  if (hwndFocus != hwndNotebook &&
      vkKey == IKeyboardEvent::backTab)
  {
     // The focus must be on a descendant of the page window.  Since we see
     // the event at this level, we must be tabbing out of the page window.
     // Set the focus to the notebook only if it is a back tab.  Also, set
     // flag to indicate that we are the one that set focus on the notebook.
     {
        notebook->tabFromPage(true);
        notebook->setFocus();
        stopProcessingEvent = true;
     }
  }
  return stopProcessingEvent;
}


/*------------------------------------------------------------------------------
| INotebookHandler::gotFocus                                                   |
|                                                                              |
| Handle a focus request for the notebook.  We only need to handle the focus   |
| request when the notebook has been traversed to via a back tab.              |
------------------------------------------------------------------------------*/
Boolean INotebookHandler::gotFocus ( IEvent& event )
{
  Boolean stopProcessingEvent = false;
  INotebook *notebook = (INotebook*)(event.dispatchingWindow());
  IWindowHandle hwndNotebook = notebook->handle();
  IPageHandle topPage = notebook->topPage();
  IWindow *pageWindow = (topPage) ? notebook->window( topPage ) : 0;

  // If the top page window is either a tabbable control or contains a tabbable
  // control, traverse into the page window.
  if (pageWindow)
  {
     if (pageWindow->isFrameWindow())
     {
        // Note:  There is a problem with the GetNextDlgTabItem returning
        //        NULL when the direction flag is TRUE (search previous)
        //        and the starting handle is NULL.  So we just check for
        //        the existence of any tabbable control in the dialog and
        //        let the frame window determine which control gets focus.
        IWindowHandle
           hwndNextControl = GetNextDlgTabItem( pageWindow->handle(),
                                                NULL,
                                                FALSE );
        if (hwndNextControl)
        {
           pageWindow->setFocus();
           stopProcessingEvent = true;
        }
     }
     else
     {
        if (pageWindow->isTabStop())
        {
           pageWindow->setFocus();
           stopProcessingEvent = true;
        }
     }
  }
  return stopProcessingEvent;
}
#endif // IC_WIN
//------------------------------------------------------------------------------  

