/*******************************************************************************
* FILE NAME: ipushbut.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipushbut.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINERRORS
  #define INCL_WINFRAMEMGR        // WM_QUERYFOCUSCHAIN
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR       // QWL_xxx
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#ifndef _IPUSHBUT_
  #include <ipushbut.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IFONT_
    #include <ifont.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif
#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif
#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
  #include <ihelpsta.hpp>
#endif


#ifdef IC_PAGETUNE
  #define _IPUSHBUT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Public push button styles.                                                   |
------------------------------------------------------------------------------*/

#pragma data_seg(ICLStaticConst)
const IPushButton::Style
#ifdef IC_PM
  IPushButton::help              = BS_HELP,
  IPushButton::systemCommand     = BS_SYSCOMMAND,
  IPushButton::defaultButton     = BS_DEFAULT,
  IPushButton::noBorder          = BS_NOBORDER,
#endif
#ifdef IC_WIN
  IPushButton::help          (0, IBS_HELP       ),
  IPushButton::systemCommand (0, IBS_SYSCOMMAND ),
  IPushButton::defaultButton (0, IBS_DEFAULT    ),
  IPushButton::noBorder          = 0,  // can this be done??
#endif
  IPushButton::classDefaultStyle = WS_VISIBLE;
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
  IPushButton::Style IPushButton::currentDefaultStyle = WS_VISIBLE;
#pragma data_seg()


#ifdef IC_WIN

class IPushButtonHandler : public IHandler {
typedef IHandler
  Inherited;
/*******************************************************************************
* The IPushButtonHandler class is used in Windows to support the generation    *
* of non standard events as a result of a button press.  The events this       *
* handler generates are:                                                       *
*    WM_COMMAND    - when neither IPushButton::systemCommand or                *
*                    IPushButton::help styles are specified.                   *
*    WM_SYSCOMMAND - when the IPushButton::systemCommand style is specified.   *
*    WM_HELP       - when the IPushButton::help style is specified.            *
* An instance of this handler is attached to all IPushButtons as a default     *
* handler.                                                                     *
*******************************************************************************/
public:
  IPushButtonHandler  ( );
virtual
  ~IPushButtonHandler ( );

/*------------------------------ Event Handling --------------------------------
|   dispatchHandlerEvent is overridden and performs all the processing for     |
|   this class.                                                                |
------------------------------------------------------------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

#ifdef IC_WIN
private:
HANDLE
  lastFocusHwnd;
#endif

}; //class IPushButtonHandler

IPushButtonHandler::IPushButtonHandler ( ) :
  IHandler( )
#ifdef IC_WIN
, lastFocusHwnd( 0 )
#endif
{ }

IPushButtonHandler::~IPushButtonHandler ( )
{ }

IBase::Boolean IPushButtonHandler::dispatchHandlerEvent ( IEvent& event )
{
   if (event.eventId() == WM_CONTROL)
      {
      IMODTRACE_ALL("IPushButtonHandler::dispatchHandlerEvent");
      ITRACE_ALL( IString("wparam=") +
                  IString( event.parameter1().asUnsignedLong() ).d2x() +
                  IString(" lparam=") +
                  IString( event.parameter2().asUnsignedLong() ).d2x() );
      unsigned long ulMsg = event.parameter1().number2();
      if (ulMsg == BN_CLICKED)
         {
         IControlEvent ctlevt(event);
         // cast safe since we only add this handler to IPushButtons
         IPushButton* button = (IPushButton*)ctlevt.controlWindow();
         // generate the needed events due to a button click
         if (button)
            {
            ITRACE_ALL( IString("button=") +
                        IString( (unsigned long)((void*)button) ).d2x() );
            unsigned long newMsg = 0;
            IEventData    wparam(ctlevt.controlId(), 0);
            IEventData    lparam(0);
            IWindow* postTo = button->owner();
            if      ( button->isHelp() )
               {
#ifdef IC_WIN
               newMsg = IC_UM_CONTEXT_HELP;
               wparam = 0;
               if (lastFocusHwnd && !button->allowsMouseClickFocus())
                 lparam = lastFocusHwnd;
               else
                 lparam = (HWND) button->handle();
               postTo = (IWindow *)ihelpwindowSearchChain( lparam, 0 );
#else
               newMsg = WM_HELP;
#endif
               }
            else if ( button->isSystemCommand() )
               {
               // note that lparam is 0, implying that syscommand was
               // selected with keyboard.  Win puts mouse coordinates
               // in lparam if chose with mouse
               newMsg = WM_SYSCOMMAND;
               }
            else
               {
               newMsg = WM_COMMAND;
               wparam = IEventData(ctlevt.controlId(), CMDSRC_PUSHBUTTON );
               }

            if ((postTo == 0) || (postTo == IWindow::desktopWindow()) )
               {
               postTo = button->parent();
               }
            if (postTo)
               {
               ITRACE_ALL( IString("postTo=") +
                           IString( (unsigned long)((void*)owner) ).d2x() +
                           IString(" hwnd=") +
                           IString( owner->handle().asUnsigned() ).d2x() );
               postTo->postEvent( newMsg,
                                 wparam,
                                 lparam );
               } // if owner
            }   // if button
         }      // if BN_CLICKED
      }         // if WM_CONTROL
#ifdef IC_WIN
   if (event.eventId() == WM_SETFOCUS)
      {
         IPushButton* button = (IPushButton*)event.window();
         // save handle of window that last had focus for
         // help button context help processing.
         if (button && button->isHelp())
            {
              lastFocusHwnd = event.parameter1();
            }
      }
   //------------------------------------------------
   // DEFECT 23913 : notify owner window, so default
   //                pushbutton can be adjusted...
   //------------------------------------------------
   if (event.eventId() == WM_KILLFOCUS)
      {
         IPushButton* button = (IPushButton*)event.window();
         if ( button && button->isDefault() ) {
            HWND hWndNext = (HWND)event.parameter1().asUnsignedLong();
            IWindow* postTo = button->owner();
            if ( postTo )
               postTo->postEvent( IC_UM_FOCUSCHANGE, IEventData(hWndNext) );
         }
      }
   //------------------------------------------------
#endif

   return false;        // pass all events on
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointed               |
| to object when the wrapper is destructed.                                    |
------------------------------------------------------------------------------*/
static struct DefaultHandler
{
  operator IPushButtonHandler* ( );
IPushButtonHandler
 *operator -> ( )
  {
    return( *this );
  }

  ~DefaultHandler ( )
   {
     if (ptr)
     {
       delete ptr;
       ptr = 0;
     }
   }

IPushButtonHandler
 *ptr;

} defaultHandler;

DefaultHandler :: operator IPushButtonHandler* ( )
{
  if (!ptr)
    ptr = new IPushButtonHandler;
  return( ptr );
}
#endif // IC_WIN

/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to create a push button control on a standard window.            |
------------------------------------------------------------------------------*/
IPushButton :: IPushButton ( unsigned long ulId,
                             IWindow* parent,
                             IWindow* pwndOwner,
                             const IRectangle& rectInit,
                             const Style& pbsStyle )
  : fPushButtonData( 0 )
{
  IASSERTPARM(parent!=0);

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | pbsStyle.asExtendedUnsignedLong() );

  IWindowHandle ownerHandle = IWindowHandle(0);
  if (pwndOwner)
     ownerHandle = pwndOwner->handle();
  IWindowHandle whPushButton =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( pbsStyle ),
                      WC_BUTTON,
                      parent->handle(),
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( pbsStyle, true ) );

  startHandlingEventsFor(whPushButton);
#ifdef IC_WIN
  addHandler(defaultHandler);
#endif
}


/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
IPushButton :: IPushButton ( unsigned long ulId,
                             IWindow* pdlgwndParent )
  : fPushButtonData( 0 )
{
  setAutoDestroyWindow  ( false );
  startHandlingEventsFor( ulId,
                          pdlgwndParent );
#ifdef IC_WIN
  // if the given button is a default then set extended style
  unsigned long ulStyle = style() & BS_PRIMARYSTYLES;
  if ( ulStyle == BS_DEFPUSHBUTTON )
  {
    setExtendedStyle( extendedStyle() |
                      defaultButton.asExtendedUnsignedLong() );
  }
  addHandler(defaultHandler);
#endif
}


/*------------------------------------------------------------------------------
| IPushButton::IPushButton                                                     |
|                                                                              |
| Constructor to instantiate from an existing push button control.             |
------------------------------------------------------------------------------*/
IPushButton :: IPushButton ( const IWindowHandle& wh )
  : fPushButtonData( 0 )
{
  setAutoDestroyWindow  ( false );
  startHandlingEventsFor( wh );
#ifdef IC_WIN
  // if the given button is a default then set extended style
  unsigned long ulStyle = style() & BS_PRIMARYSTYLES;
  if ( ulStyle == BS_DEFPUSHBUTTON )
  {
    setExtendedStyle( extendedStyle() |
                      defaultButton.asExtendedUnsignedLong() );
  }
  addHandler(defaultHandler);
#endif
}

/*------------------------------------------------------------------------------
| IPushButton::~IPushButton                                                    |
------------------------------------------------------------------------------*/
IPushButton :: ~IPushButton ( )
{
#ifdef IC_WIN
  removeHandler(defaultHandler);
#endif
  // delete fPushButtonData;
}


/*------------------------------------------------------------------------------
| IPushButton::defaultStyle                                                    |
------------------------------------------------------------------------------*/
IPushButton::Style  IPushButton :: defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::setDefaultStyle                                                 |
------------------------------------------------------------------------------*/
void  IPushButton :: setDefaultStyle( const Style& pbsStyle )
{
  currentDefaultStyle = pbsStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IPushButton::convertToGUIStyle( const IBitFlag& guiStyle,
                                              Boolean bExtOnly ) const
{
  // Obtain the style from the class (IButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // Set the GUI pushbutton style

#ifdef IC_WIN
    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTPUSH;

    // Handle overlapping styles
    if (ulTmp & IBS_DEFAULT)
      ulStyle |= BS_DEFPUSHBUTTON | WS_CHILD;
    else
#endif

      ulStyle |= BS_PUSHBUTTON
#ifdef IC_WIN
                 | WS_CHILD
#endif
      ;
  }

  return ulStyle;
}


/*------------------------------------------------------------------------------
| IPushButton::enableDefault                                                   |
|                                                                              |
| Sets the push button to be the default button.                               |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: enableDefault ( Boolean enable )
{
  // Give the button the default style, as appropriate.
#ifdef IC_WIN
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;
  unsigned long ulExtStyle = extendedStyle();
  if (enable)
  {
    ulStyle &= ~BS_PRIMARYSTYLES;
    ulStyle |= BS_DEFPUSHBUTTON;

    ulExtStyle |= defaultButton.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle &= ~BS_PRIMARYSTYLES;
    ulStyle |= BS_PUSHBUTTON;

    ulExtStyle &= ~defaultButton.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                //Force refresh

    // Update extended style ...
    setExtendedStyle(ulExtStyle);
  }

  //---------------------------------------------------------
  // DEFECT 23913 : store the current default pushbutton,
  //                same as done for PM, for later reference
  //---------------------------------------------------------
  // Update the default push button maintained by the frame.
  IWindowHandle hwndButton = this->handle();
  IEventResult mr =          // Find the frame window.
            hwndButton.sendEvent( WM_QUERYFOCUSCHAIN,
                                  IEventParameter1( QFC_FRAME ),
                                  IEventParameter2( 0 ));
  IWindowHandle hwndFrame( (HWND)mr.asUnsignedLong() );
  if ( hwndFrame.isValid() )
  {                          // Found the frame window.
     // Update frame's default, as appropriate.
     if (enable)
     {                       // Store this default button.
        ISETWINDOWULONG( hwndFrame, IGWL_DEFBUTTON, hwndButton.asUnsigned() );
     }
     else if ( (HWND)IQUERYWINDOWULONG( hwndFrame, IGWL_DEFBUTTON ) == hwndButton)
     {                       // Remove this default from frame.
        ISETWINDOWULONG( hwndFrame, IGWL_DEFBUTTON, 0 );
     }
     // (Can't use IWindow::windowULong or IWindow::setWindowData,
     // because they are protected.)
  }
  //---------------------------------------------------------
#endif

#ifdef IC_PM
  IWindowHandle hwndButton = this->handle();
  unsigned long ulStyle = style();
  if (enable)
  {
    if (!(ulStyle & defaultButton.asUnsignedLong()))
    {
      hwndButton.sendEvent( BM_SETDEFAULT, IEventParameter1( TRUE ),
                                           IEventParameter2( 0 ));
    }
  }
  else if (ulStyle & defaultButton.asUnsignedLong())
  {
    hwndButton.sendEvent( BM_SETDEFAULT, IEventParameter1( FALSE ),
                                         IEventParameter2( 0 ));
  }

  // Update the default push button maintained by the frame.
  IEventResult mr =          // Find the frame window.
            hwndButton.sendEvent( WM_QUERYFOCUSCHAIN,
                                  IEventParameter1( QFC_FRAME ),
                                  IEventParameter2( 0 ));
  IWindowHandle hwndFrame( mr.asUnsignedLong() );
  if (hwndFrame)
  {                          // Found the frame window.
     // Update frame's default, as appropriate.
     if (enable)
     {                       // Store this default button.
        ISETWINDOWULONG( hwndFrame, QWL_DEFBUTTON, hwndButton );
     }
     else if (IQUERYWINDOWULONG( hwndFrame, QWL_DEFBUTTON )
                                                 == hwndButton)
     {                       // Remove this default from frame.
        ISETWINDOWULONG( hwndFrame, QWL_DEFBUTTON, 0 );
     }
     // (Can't use IWindow::windowULong or IWindow::setWindowData,
     // because they are protected.)
  }
#endif  // IC_PM

  return *this;
}


/*------------------------------------------------------------------------------
| IPushButton::disableDefault                                                  |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: disableDefault()
{
  enableDefault(false);
  return *this;
}


/*------------------------------------------------------------------------------
| IPushButton::isDefault                                                       |
------------------------------------------------------------------------------*/
Boolean  IPushButton :: isDefault() const
{
#ifndef IC_WIN
  return( (style() & defaultButton.asUnsignedLong()) ? true : false );
#endif
#ifdef IC_WIN
  return( ((style() & BS_PRIMARYSTYLES) == BS_DEFPUSHBUTTON ) ? true : false );
#endif
}

/*------------------------------------------------------------------------------
| IPushButton::enableHelp                                                      |
|                                                                              |
| Sets the help style which causes a help event to generated when the push     |
| button is pressed.                                                           |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: enableHelp(Boolean bTurnOn)
{
#ifndef IC_WIN
  unsigned long ulStyle = style();
  unsigned long helpStyle = help.asUnsignedLong();
#endif
#ifdef IC_WIN
  unsigned long ulStyle = extendedStyle();
  unsigned long helpStyle = help.asExtendedUnsignedLong();
#endif
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle |= helpStyle;
  }
  else
  {
    ulStyle &= ~helpStyle;
  }

  if (ulStyle != ulOldStyle)
  {
#ifndef IC_WIN
    setStyle(ulStyle);
#endif
#ifdef IC_WIN
    setExtendedStyle(ulStyle);
#endif

  }
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::disableHelp                                                     |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: disableHelp()
{
  enableHelp(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::isHelp                                                          |
------------------------------------------------------------------------------*/
Boolean  IPushButton :: isHelp() const
{
#ifndef IC_WIN
  return( (style() & help.asUnsignedLong()) ? true : false );
#endif
#ifdef IC_WIN
  return( (extendedStyle() & help.asExtendedUnsignedLong()) ? true : false );
#endif
}

/*------------------------------------------------------------------------------
| IPushButton::enableSystemCommand                                             |
|                                                                              |
| Sets the system command style which causes a system command to be generated  |
| when the push button is pressed.                                             |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: enableSystemCommand(Boolean bTurnOn)
{
#ifndef IC_WIN
  unsigned long ulStyle = style();
  unsigned long newStyle = systemCommand.asUnsignedLong();
#endif
#ifdef IC_WIN
  unsigned long ulStyle = extendedStyle();
  unsigned long newStyle = systemCommand.asExtendedUnsignedLong();
#endif
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle |= newStyle;
  }
  else
  {
    ulStyle &= ~newStyle;
  }

  if (ulStyle != ulOldStyle)
  {
#ifndef IC_WIN
    setStyle(ulStyle);
#endif
#ifdef IC_WIN
    setExtendedStyle(ulStyle);
#endif
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::disableSystemCommand                                            |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: disableSystemCommand()
{
  enableSystemCommand(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::isSystemCommand                                                 |
------------------------------------------------------------------------------*/
Boolean  IPushButton :: isSystemCommand() const
{
#ifndef IC_WIN
  return( (style() & systemCommand.asUnsignedLong()) ? true : false );
#endif
#ifdef IC_WIN
  return( (extendedStyle() & systemCommand.asExtendedUnsignedLong()) ? true
                                                                     : false );
#endif
}

/*------------------------------------------------------------------------------
| IPushButton::addBorder                                                       |
|                                                                              |
| Sets the noBorder style indicating the presence of a border around the       |
| push button.                                                                 |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: addBorder( Boolean add )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if ( add )
  {
    ulStyle &= ~noBorder.asUnsignedLong();
  }
  else
  {
    ulStyle |= noBorder.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                           // Force refresh of control window
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::removeBorder                                                    |
------------------------------------------------------------------------------*/
IPushButton&  IPushButton :: removeBorder( )
{
  addBorder(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IPushButton::hasBorder                                                       |
------------------------------------------------------------------------------*/
Boolean  IPushButton :: hasBorder ( ) const
{
  return( (style() & noBorder.asUnsignedLong()) ? false : true );
}


/*------------------------------------------------------------------------------
| IPushButton::calcMinimumSize                                                 |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The height is      |
| based on the font height and width is based on the average font width        |
| multiplied by the width of the push button text.                             |
------------------------------------------------------------------------------*/
ISize  IPushButton :: calcMinimumSize() const
{
  ISize size = displaySize(text());
  unsigned long ulWidth = size.width();
  unsigned long i=textLength();

  if (i==0)
     ulWidth += (2 + (9 * IQUERYSYSVALUE( SV_CXBORDER )));
  else
     ulWidth += (ulWidth/i  + 2 + (9 * IQUERYSYSVALUE( SV_CXBORDER )));
  unsigned long ulHeight = size.height() + (9 * IQUERYSYSVALUE( SV_CYBORDER ));
  return ISize(ulWidth, ulHeight);
}

