/*******************************************************************************
* FILE NAME: isetbut1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isetbnhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINBUTTONS

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _ISETBUT_
  #include <isetbut.hpp>
#endif

#ifndef _ISETBNHD_
  #include <isetbnhd.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISETBUT1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class ISettingButtonNotifyStatics
{
public:

~ISettingButtonNotifyStatics ()
{
  delete (settingButtonNotifyHandler);
  settingButtonNotifyHandler = 0;
}
static ISettingButtonNotifyHandler
 &notificationHandler ( );

static ISettingButtonNotifyHandler
  *settingButtonNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static ISettingButtonNotifyStatics settingButtonStatics;
INotificationId const ISettingButton::selectId = "ISettingButton::select";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ISettingButtonNotifyHandler*
      ISettingButtonNotifyStatics::settingButtonNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| ISettingButtonNotifyStatics::notificationHandler                             |
|                                                                              |
| Static function to create and return a notification handler for              |
| ISettingButton.                                                              |
------------------------------------------------------------------------------*/
ISettingButtonNotifyHandler& ISettingButtonNotifyStatics::notificationHandler()
{
  if (!(ISettingButtonNotifyStatics::settingButtonNotifyHandler))
  {
    ISettingButtonNotifyStatics::settingButtonNotifyHandler =
                                   new ISettingButtonNotifyHandler();
  }
  return (*ISettingButtonNotifyStatics::settingButtonNotifyHandler);
}

/*------------------------------------------------------------------------------
| ISettingButton::enableNotification                                           |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ISettingButton, does not send notifications to observer objects until it is  |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ISettingButton& ISettingButton::enableNotification ( Boolean enabled )
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(ISettingButtonNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::ISettingButtonNotifyHandler                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ISettingButtonNotifyHandler::ISettingButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::~ISettingButtonNotifyHandler                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ISettingButtonNotifyHandler::~ISettingButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ISettingButtonNotifyHandler::dispatchHandlerEvent                            |
|                                                                              |
| This notifies ISettingButton Windows of the following events:                |
|                                                                              |
| select(Id)                                                                   |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean ISettingButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case BM_SETCHECK:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the selection state has been modified                      */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_SETBT_SELECT ));
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_SETBT_SELECT)
        {
          thisWindow->notifyObservers(INotificationEvent(
                                ISettingButton::selectId,
                                *thisWindow, true,
                                (void*)anEvent.parameter1()));
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

