/*******************************************************************************
* FILE NAME: ispbtnn1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispbtnnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSTDSPIN

extern "C" {
   #include <iwindefs.h>
}

#include <ispinnum.hpp>
#include <ispbtnnh.hpp>
#include <inotifev.hpp>
#include <ictlevt.hpp>

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
#include <iclspbw.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPBTNN1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class INumericSpinButtonNotifyStatics
{
public:

  ~INumericSpinButtonNotifyStatics ()
  {
    delete numericSpinButtonNotifyHandler;
    numericSpinButtonNotifyHandler = 0;
  }

static INumericSpinButtonNotifyHandler
  &notificationHandler ( );

static INumericSpinButtonNotifyHandler
  *numericSpinButtonNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static INumericSpinButtonNotifyStatics numericSpinButtonStatics;
INotificationId const IBaseSpinButton::valueId = "IBaseSpinButton::value";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
INumericSpinButtonNotifyHandler*
  INumericSpinButtonNotifyStatics::numericSpinButtonNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| INumericSpinButtonNotifyStatics::notificationHandler                         |
|                                                                              |
| Static function to create and return a notification handler for              |
| INumericSpinButton.                                                          |
------------------------------------------------------------------------------*/
INumericSpinButtonNotifyHandler& INumericSpinButtonNotifyStatics::
                                                      notificationHandler ()
{
  if (!(INumericSpinButtonNotifyStatics::numericSpinButtonNotifyHandler))
  {
    INumericSpinButtonNotifyStatics::numericSpinButtonNotifyHandler =
                                   new INumericSpinButtonNotifyHandler();
  }
  return( *INumericSpinButtonNotifyStatics::numericSpinButtonNotifyHandler );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::enableNotification                                       |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| INumericSpinButton, does not send notifications to observer objects until it |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(INumericSpinButtonNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButtonNotifyHandler::INumericSpinButtonNotifyHandler             |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
INumericSpinButtonNotifyHandler::INumericSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| INumericSpinButtonNotifyHandler::~INumericSpinButtonNotifyHandler            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
INumericSpinButtonNotifyHandler::~INumericSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| INumericSpinButtonNotifyHandler::dispatchHandlerEvent                        |
|                                                                              |
| This notifies INumericSpinButton Windows of the following events:            |
|                                                                              |
| value(Id)                                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean INumericSpinButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
#ifdef IC_WIN
        if ((className == WC_SPINBUTTON) || (className == WC_NATIVESPINBUTTON))
#else
        if (className == WC_SPINBUTTON)
#endif
        {
          if (ulMsg == SPBN_CHANGE)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseSpinButton::valueId,
                                        *thisWindow));
          }
#ifdef IC_WIN
          else if (ulMsg == SPBN_SETFOCUS || ulMsg == SPBN_KILLFOCUS)
          {
            thisWindow->notifyObservers( INotificationEvent(
              IBaseSpinButton::focusId, *thisWindow, false,
              (ulMsg == SPBN_SETFOCUS) ? (void*)true : (void*)false ));
          }
#endif
        }
      }
    break;
    } /* end WM_CONTROL */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

