/*******************************************************************************
* FILE NAME: ispbttn1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispbttnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSTDSPIN

extern "C" {
   #include <iwindefs.h>
}

#include <ispintxt.hpp>
#include <ispbttnh.hpp>
#include <inotifev.hpp>
#include <ictlevt.hpp>

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
#include <iclspbw.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPBTTN1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class ITextSpinButtonNotifyStatics
{
public:

  ~ITextSpinButtonNotifyStatics ()
  {
    delete textSpinButtonNotifyHandler;
    textSpinButtonNotifyHandler = 0;
  }

static ITextSpinButtonNotifyHandler
  &notificationHandler ( );

static ITextSpinButtonNotifyHandler
  *textSpinButtonNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static ITextSpinButtonNotifyStatics textSpinButtonStatics;
INotificationId const IBaseSpinButton::textId = "IBaseSpinButton::text";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ITextSpinButtonNotifyHandler*
  ITextSpinButtonNotifyStatics::textSpinButtonNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyStatics::notificationHandler                            |
|                                                                              |
| Static function to create and return a notification handler for              |
| ITextSpinButton.                                                             |
------------------------------------------------------------------------------*/
ITextSpinButtonNotifyHandler& ITextSpinButtonNotifyStatics::
                                                      notificationHandler ()
{
  if (!(ITextSpinButtonNotifyStatics::textSpinButtonNotifyHandler))
  {
    ITextSpinButtonNotifyStatics::textSpinButtonNotifyHandler =
                                   new ITextSpinButtonNotifyHandler();
  }
  return( *ITextSpinButtonNotifyStatics::textSpinButtonNotifyHandler );
}

/*------------------------------------------------------------------------------
| ITextSpinButton::enableNotification                                          |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ITextSpinButton, does not send notifications to observer objects until it    |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ITextSpinButton& ITextSpinButton::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(ITextSpinButtonNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::ITextSpinButtonNotifyHandler                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITextSpinButtonNotifyHandler::ITextSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::~ITextSpinButtonNotifyHandler                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITextSpinButtonNotifyHandler::~ITextSpinButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextSpinButtonNotifyHandler::dispatchHandlerEvent                           |
|                                                                              |
| This notifies ITextSpinButton Windows of the following events:               |
|                                                                              |
| text(Id)                                                                     |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean ITextSpinButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
#ifdef IC_WIN
        if ((className == WC_SPINBUTTON) || (className == WC_NATIVESPINBUTTON))
#else
        if (className == WC_SPINBUTTON)
#endif
        {
          if (ulMsg == SPBN_CHANGE)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IBaseSpinButton::textId,
                                        *thisWindow));
          }
#ifdef IC_WIN
          else if (ulMsg == SPBN_SETFOCUS || ulMsg == SPBN_KILLFOCUS)
          {
            thisWindow->notifyObservers( INotificationEvent(
              IBaseSpinButton::focusId, *thisWindow, false,
              (ulMsg == SPBN_SETFOCUS) ? (void*)true : (void*)false ));
          }
#endif
        }
      }
    break;
    } /* end WM_CONTROL */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

