/*******************************************************************************
* FILE NAME: ispinbas.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinbas.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINSTDSPIN
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#ifdef IC_WIN
  #include <iclspbw.h>
  #include <commctrl.h>
  #include <icctlsta.hpp>         // Statics to load dll.
#endif
#include <ispinbas.hpp>
#include <ibidiset.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <icmnfun.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <inotifev.hpp>
#include <irect.hpp>
#include <ispinszh.hpp>
#include <itrace.hpp>
#include <imphdr.hpp>
#ifdef IC_WIN
  #include <iplatfrm.hpp>
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPINBAS_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public spin button styles.                                                   |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IBaseSpinButton::Style
  IBaseSpinButton::master            ( 0, ISPBS_MASTER ),
  IBaseSpinButton::servant           ( 0, ISPBS_SERVANT ),
  IBaseSpinButton::readOnly          ( 0, ISPBS_READONLY),
  IBaseSpinButton::leftAlign         ( 0, ISPBS_JUSTLEFT),
  IBaseSpinButton::centerAlign       ( 0, ISPBS_CENTERALIGN ),
  IBaseSpinButton::rightAlign        ( 0, ISPBS_JUSTRIGHT),
  IBaseSpinButton::noBorder          ( 0, ISPBS_NOBORDER),
  IBaseSpinButton::pmCompatible      ( 0, ISPBS_PMCOMPATIBILITY),
  IBaseSpinButton::fastSpin          ( 0, ISPBS_FASTSPIN),
#ifdef IC_WIN
  IBaseSpinButton::border3D          ( 0, WS_EX_CLIENTEDGE );
#endif
#ifndef IC_WIN
  IBaseSpinButton::border3D          ( 0, 0 );
#endif
#pragma data_seg()

/*------------------------------------------------------------------------------
| The static object that IBaseSpinButton uses to properly change its size.     |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
static ISpinButtonStatics SpinButtonStatics;
#pragma data_seg()

/*------------------------------------------------------------------------------
| The spin button private data class.                                          |
------------------------------------------------------------------------------*/
class IBaseSpinButtonData
{
public:
void
  setLimit ( unsigned long ulLimit)
    { textLimit = ulLimit;}
unsigned long
  limit ( ) const
    { return textLimit; }
unsigned long
  textLimit;
}; // IBaseSpinButtonData


/*------------------------------------------------------------------------------
| IBaseSpinButton::IBaseSpinButton                                             |
|                                                                              |
| Constructor to create a spin button control.                                 |
------------------------------------------------------------------------------*/
IBaseSpinButton :: IBaseSpinButton ( )
{
   fBaseSpinButtonData = new IBaseSpinButtonData();
   fBaseSpinButtonData->setLimit(255);
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::~IBaseSpinButton                                            |
------------------------------------------------------------------------------*/
IBaseSpinButton::~IBaseSpinButton ( )
{
   delete fBaseSpinButtonData;
   SpinButtonStatics.baseSpinHandler().stopHandlingEventsFor(this);
   IMousePointerHandler::defaultHandler()->stopHandlingEventsFor(this);
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::initialize                                                  |
------------------------------------------------------------------------------*/
void IBaseSpinButton :: initialize ( unsigned long        windowId,
                                     const IWindowHandle& parent,
                                     const IWindowHandle& owner,
                                     unsigned long        ulStyle,
                                     const IRectangle&    initial )
{
  IASSERTPARM(parent!=0);
  IWindowHandle spb(0);
#ifdef IC_WIN
  if ( this->isPMCompatible() )
  {
    // If needed, load the DLL containing the control code.
    IControlStatics::loadControlDLL();
#endif
    spb =
        this -> create( windowId,
                        0,
                        ulStyle,
                        WC_SPINBUTTON,
                        parent,
                        owner,
                        initial,
                        0,
                        0,
                        defaultOrdering(),
                        convertToGUIStyle(        // convert extended styles
                          IBaseSpinButton::Style(0, extendedStyle() ),
                          true)     );
#ifdef IC_WIN
  } // if pmCompatible
  else
  {
    // Native Up/Down control

    // Create the dummy outer control of class WC_NATIVESPINBUTTON

    HINSTANCE  hinst = GetModuleHandle(0);
    WNDCLASS   wndClass;
    // Check for it already registered.  Need this in some static
    // linking scenarios.
    Boolean    registered = GetClassInfo( hinst, WC_NATIVESPINBUTTON, &wndClass);

    if (!registered)
    {
      InitCommonControls();     // make sure common controls loaded

      wndClass.style         = CS_HREDRAW | CS_VREDRAW;
      wndClass.cbClsExtra    = 0;
      wndClass.cbWndExtra    = NATIVESPINBUTTONEXTRA;
      wndClass.hCursor       = LoadCursor(0,IDC_ARROW);
      wndClass.hInstance     = GetModuleHandle(0);
      wndClass.lpfnWndProc   = (WNDPROC)IDEFWINDOWPROC;
      wndClass.hIcon         = 0;
      wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
      wndClass.lpszMenuName  = 0;
      wndClass.lpszClassName = WC_NATIVESPINBUTTON;

      if (!RegisterClass(&wndClass))
        ITHROWGUIERROR( "RegisterClass" );
    }
    spb =
        this -> create( windowId,
                        0,
                        ulStyle,
                        WC_NATIVESPINBUTTON,
                        parent,
                        owner,
                        IRectangle(initial.minXMinY(), ISize(0,0) ),
                        0,
                        0,
                        defaultOrdering(),
                        convertToGUIStyle(        // convert extended styles
                          IBaseSpinButton::Style(0, extendedStyle() ),
                          true)     );


    // Check for a bidi system.  If so, add in some extended styles for
    // alignment when creating the buddy edit field.
    unsigned long exStyleBuddy(0);
    if (IBidiSettings::isBidiSupported())
    {
       IBidiSettings
         bidiSettings( spb );
       if ( bidiSettings.windowLayout() == IBidiSettings::layoutRightToLeft )
       {
          exStyleBuddy |= WS_EX_RIGHT;
       }
       if ( bidiSettings.textOrientation() == IBidiSettings::textRightToLeft )
       {
          exStyleBuddy |= WS_EX_RTLREADING;
       }
    }

    // Create the real buddy control (Edit control ).  It uses the same Id
    // as the dummy wrapper.
    IWindowHandle buddy =
       ICREATEWINDOW(
          spb,                                  // parent
          WC_ENTRYFIELD,                        // class
          (ulStyle & INSPBS_NUMERICONLY) ? "0" : "",  //Window text
          ISpinButtonStatics::convertToBuddyStyle( extendedStyle() ),
          0,                                    // x
          0,                                    // y
          0,
          0,
          spb,                                  // owner
          0,                                    // behind (unused)
          windowId,                             // Id
          0,                                    // control parms
          exStyleBuddy,                         // extended styles
          hinst );                              // instance
    if ( buddy == 0)  // Error: throw exception.
      ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId)+
                     IString(" Class=")+IString(WC_ENTRYFIELD));

    WNDPROC defproc = (WNDPROC)ISUBCLASSWINDOW( buddy, fnwpSpinButtonEditProc );
    ISETBUDDYPROC( spb, defproc );
    ISETACTIVEBUDDY( spb, buddy.asUnsigned() );

    // If this is a master spinbutton, we need to create the updown control,
    // a fake buddy so that it works correctly, and setup fast spin.
    // For a servant, we will be using the updown control of the master
    // so we skip creation here.
    if (ulStyle & INSPBS_MASTER)
    {
       IWindowHandle fakebuddy =
          ICREATEWINDOW(
             spb,                               // parent
             WC_ENTRYFIELD,                     // class
             (ulStyle & INSPBS_NUMERICONLY) ? "0" : "",  //Window text
             WS_CHILD,                          // style
             0,                                 // x
             0,                                 // y
             0,                                 // width
             0,                                 // height
             spb,                               // owner
             0,                                 // behind (unused)
             ((windowId-2) & 0x0000ffff),       // id
             0,                                 // control parms
             0,                                 // extended styles
             hinst );                           // instance

       if ( fakebuddy == 0)  // Error: throw exception.
         ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId-1)+
                        IString(" Class=")+IString(WC_ENTRYFIELD));

       // Create the up/down control.  Its id is one less than the
       // dummy wrapper.
       unsigned long udStyle =
          WS_CHILD | WS_BORDER | WS_VISIBLE |
          UDS_AUTOBUDDY | UDS_SETBUDDYINT | UDS_NOTHOUSANDS |
          UDS_WRAP | UDS_ARROWKEYS;
       IWindowHandle updown =
          ICREATEWINDOW(
             spb,                               // parent
             UPDOWN_CLASS,                      // class
             0,                                 // Window text
             udStyle,                           // style
             0,
             0,                                 // y
             0,
             0,
             spb,                               // owner
             0,                                 // behind (unused)
             ((windowId-1) & 0x0000ffff),       // id
             0,                                 // control parms
             0,                                 // extended styles
             hinst );                           // instance

       if ( updown == 0)  // Error: throw exception.
         ITHROWGUIERROR(IString("CreateWindowEx: Id=")+IString(windowId-1)+
                        IString(" Class=")+IString(UPDOWN_CLASS));

       // Set the handle of the updown control into the extra window word
       // of the wrapper control.  The active buddy is initially this one.
       ISETUPDOWNHANDLE( spb, updown.asUnsigned() );

       // Set the initial range to be 0,0
       updown.sendEvent( UDM_SETRANGE,
                         IEventParameter1(0),
                         IEventParameter2(0, 0) );

       // Set up table needed for fast spin
       if (extendedStyle() & ISPBS_FASTSPIN)
       {
          updown.sendEvent( UDM_SETACCEL,
                   IEventParameter1(ISpinButtonStatics::fastSpinTableSize()),
                   IEventParameter2(ISpinButtonStatics::fastSpinTable()) );
       }
       else
       {
          updown.sendEvent( UDM_SETACCEL,
                   IEventParameter1(1),
                   IEventParameter2(ISpinButtonStatics::normalSpinTable()));
       }
    }   // master
    else
    {
       // servant.  Insure updown handle value is 0
       ISETUPDOWNHANDLE( spb, 0 );
    }
    // Initialize the rest of the window words
   ISETRANGE(spb, 0);
   ISETLASTPOS(spb, 0);

  }
#endif
  initialize();
  startHandlingEventsFor(spb);

#ifdef IC_WIN
  if ( (!this->isPMCompatible() ) &&
       ( initial.size() != ISize(0,0) ) )
  {
    // If the initial size specified is not 0, resize now.  This is done
    // to let our handler position the controls
    this->sizeTo( initial.size() );
  }
#endif
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::initialize                                                  |
------------------------------------------------------------------------------*/
void IBaseSpinButton :: initialize ( )
{
   SpinButtonStatics.baseSpinHandler().handleEventsFor(this);
   IMousePointerHandler::defaultHandler()->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::convertToGUIStyle                                           |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IBaseSpinButton :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   Boolean bExtOnly) const
{
  // Obtain the style from the class that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & ISPBS_EXTMASK;
#ifdef IC_WIN
    if (guiStyle.asExtendedUnsignedLong() &
        ( IBaseSpinButton::noBorder.asExtendedUnsignedLong() |
          IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() ))
    {
      // if noBorder and\or pmCompatible is specified then remove border3D
      ulStyle &= ~WS_EX_CLIENTEDGE;
    }
#endif
  }
  else
  {
    unsigned long guiExtUL = guiStyle.asExtendedUnsignedLong();
#ifdef IC_WIN
    if ( guiExtUL & IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() )
#endif
       {
       // PM Compatible spinbutton
       // Don't need to handle servant, SPBS_ALLCHARACTERS since these are 0
       if ( guiExtUL & IBaseSpinButton::master.asExtendedUnsignedLong()      )
          ulStyle |= SPBS_MASTER;
       if ( guiExtUL & IBaseSpinButton::readOnly.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_READONLY;
       if ( guiExtUL & IBaseSpinButton::leftAlign.asExtendedUnsignedLong()   )
          ulStyle |= SPBS_JUSTLEFT;
       if ( guiExtUL & IBaseSpinButton::centerAlign.asExtendedUnsignedLong() )
          ulStyle |= SPBS_JUSTCENTER;
       if ( guiExtUL & IBaseSpinButton::rightAlign.asExtendedUnsignedLong()  )
          ulStyle |= SPBS_JUSTRIGHT;
       if ( guiExtUL & IBaseSpinButton::noBorder.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_NOBORDER;
       if ( guiExtUL & IBaseSpinButton::fastSpin.asExtendedUnsignedLong()    )
          ulStyle |= SPBS_FASTSPIN;
       }
#ifdef IC_WIN
    else
       {
       ulStyle |= WS_BORDER;
       // For the native Windows up/down control, this is the style
       // of the wrapper.
       if ( guiExtUL & IBaseSpinButton::master.asExtendedUnsignedLong()   )
          ulStyle |= INSPBS_MASTER;
       if ( guiExtUL & IBaseSpinButton::noBorder.asExtendedUnsignedLong() )
          ulStyle &= (unsigned long)~WS_BORDER;
       }

    // For Windows it is always a child window
    ulStyle |= WS_CHILD;
#endif
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IBaseSpinButton::enableDataUpdate                                            |
|                                                                              |
| Set or remove the readOnly style on the spinbutton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::enableDataUpdate( Boolean writeable )
{
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;

  if (writeable)
  {
    ulStyle &= ~readOnly.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle |= readOnly.asExtendedUnsignedLong();;
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
#ifdef IC_WIN
    else
    {
      // Native Up/Down control
      IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
      ef.sendEvent(EM_SETREADONLY,
                   IEventParameter1(writeable ? false : true),
                   IEventParameter2(0));
    }
#endif
    refresh();                      //force refresh
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::disableDataUpdate                                           |
|                                                                              |
| Remove the readOnly style on the spinbutton.                                 |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::disableDataUpdate ( )
{
   return (enableDataUpdate(false));
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::enableFastSpin                                              |
|                                                                              |
| Set or remove the fastSpin style on the the spinbutton.                      |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::enableFastSpin (Boolean fast)
{
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;
  if (fast)
  {
    ulStyle |= fastSpin.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle &= ~fastSpin.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
#ifdef IC_WIN
    else
    {
      // Native Up/Down control
      IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
      if (updown.asUnsigned())  // in servant case could be empty
      {
        if (fast)
          {
          if ( !updown.sendEvent(
                   UDM_SETACCEL,
                   IEventParameter1(ISpinButtonStatics::fastSpinTableSize()),
                   IEventParameter2(ISpinButtonStatics::fastSpinTable()) )
                .asUnsignedLong()  )
             ITHROWGUIERROR2("UDM_SETACCEL",
                             IBaseErrorInfo::invalidParameter,
                             IException::recoverable);
          }
        else
          {
          if ( !updown.sendEvent(
                   UDM_SETACCEL,
                   IEventParameter1(1),
                   IEventParameter2(ISpinButtonStatics::normalSpinTable()) )
                .asUnsignedLong() )
             ITHROWGUIERROR2("UDM_SETACCEL",
                             IBaseErrorInfo::invalidParameter,
                             IException::recoverable);
          }
      }  // updown set

    }
#endif
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::disableFastSpin                                             |
|                                                                              |
| Remove the fastSpin style on the the spinbutton.                             |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::disableFastSpin ()
{
   return (enableFastSpin(false));
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setAlignment                                                |
|                                                                              |
| Change the spin field text alignment.                                        |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::setAlignment(Alignment alignment)
{
#ifndef IC_WIN  // changing on the fly not supported in Windows
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;
  ulStyle &= ~(leftAlign.asExtendedUnsignedLong() |
               centerAlign.asExtendedUnsignedLong() |
               rightAlign.asExtendedUnsignedLong() );
  switch (alignment)
  {
    case left:
      ulStyle |= leftAlign.asExtendedUnsignedLong();
    break;

    case right:
      ulStyle |= rightAlign.asExtendedUnsignedLong();
    break;

    case center:
      ulStyle |= centerAlign.asExtendedUnsignedLong();
    break;
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    if (this->isPMCompatible())
    {
      IBaseSpinButton::Style newStyle( style(), ulStyle);
      setStyle( convertToGUIStyle(newStyle, false) );     //Change style
    }
   refresh();                                          //Force refresh
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::addBorder                                                   |
|                                                                              |
| Add or remove the border on the spinbutton.                                  |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton::addBorder (Boolean add)
{
  unsigned long ulStyle = extendedStyle();
  unsigned long ulOldStyle = ulStyle;

  if (add)
  {
    ulStyle &= ~noBorder.asExtendedUnsignedLong();
  }
  else
  {
    ulStyle |= noBorder.asExtendedUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setExtendedStyle( ulStyle );
    IBaseSpinButton::Style newStyle( style(), ulStyle);
    setStyle( convertToGUIStyle(newStyle, false) );     //Change style
#ifdef IC_WIN
    if ( !this->isPMCompatible() )
       {
       long extGUI = GetWindowLong( this->handle(), GWL_EXSTYLE);
       if (add)
          extGUI |= WS_EX_CLIENTEDGE;
       else
          extGUI &= ~WS_EX_CLIENTEDGE;
       SetWindowLong( this->handle(), GWL_EXSTYLE, extGUI );

       // Need to force a resizing event to get border change to show

       RECT wrect;
       GetWindowRect( this->handle(), &wrect );
       MapWindowPoints( HWND_DESKTOP,
                        IPARENTOF( this->handle() ),
                        (POINT*)(&wrect),
                        2 );
       SetWindowPos( this->handle(), 0,
                     (int)wrect.left,
                     (int)wrect.top,
                     (int)(wrect.right - wrect.left),
                     (int)(wrect.bottom - wrect.top),
                     SWP_FRAMECHANGED | SWP_NOZORDER | SWP_NOACTIVATE );
       }
#endif
    refresh();                                         // force refresh
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::removeBorder                                                |
|                                                                              |
| Remove the spin field border.                                                |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: removeBorder ( )
{
   return addBorder(false);
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isMaster                                                    |
|                                                                              |
| Is the spin button a master?                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton :: isMaster ( ) const
{
  return (extendedStyle() & master.asExtendedUnsignedLong()) ? true : false;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isServant                                                   |
|                                                                              |
| Is the spin button a servant?                                                |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton :: isServant  ( ) const
{
   return (!isMaster());
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isWriteable                                                 |
|                                                                              |
| Is the spin field read only?                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton :: isWriteable ( ) const
{
   return (extendedStyle() & readOnly.asExtendedUnsignedLong()) ? false : true;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isFastSpinEnabled                                           |
|                                                                              |
| Is fast spin activated?                                                      |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton :: isFastSpinEnabled ( ) const
{
   return (extendedStyle() & fastSpin.asExtendedUnsignedLong()) ? true : false;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::hasBorder                                                   |
|                                                                              |
| Does the spin field have a border?                                           |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton :: hasBorder   ( ) const
{
   return (extendedStyle() & noBorder.asExtendedUnsignedLong()) ? false : true;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::hasFocus                                                    |
------------------------------------------------------------------------------*/
Boolean IBaseSpinButton::hasFocus() const
{
  HWND hwndFocus = IQUERYFOCUS(HWND_DESKTOP);
  if (handle() == IPARENTOF( hwndFocus ))
  {
     IWindowClassName wcName( hwndFocus );
     if (wcName == WC_ENTRYFIELD)
        return true;
  }
  return false;
}
#endif

/*------------------------------------------------------------------------------
| IBaseSpinButton::alignment                                                   |
|                                                                              |
| Returns the spin field alignment.                                            |
------------------------------------------------------------------------------*/
IBaseSpinButton::Alignment IBaseSpinButton :: alignment  ( ) const
{
   unsigned long ulStyle = extendedStyle();
   if ( ulStyle & leftAlign.asExtendedUnsignedLong())
      return left;
   else if (ulStyle & rightAlign.asExtendedUnsignedLong())
      return right;
   else
      return center;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::limit                                                       |
|                                                                              |
| Returns the number of characters allowed.                                    |
------------------------------------------------------------------------------*/
unsigned long IBaseSpinButton::limit ( ) const
{
   return this->fBaseSpinButtonData->limit();
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setLimit                                                    |
|                                                                              |
| Sets the number of characters permitted in a spin field.                     |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: setLimit ( unsigned long aNumber )
{
   this->fBaseSpinButtonData->setLimit(aNumber);
   if (this->isPMCompatible())
   {

     unsigned long ulOk = handle().sendEvent(SPBM_SETTEXTLIMIT,
//                                           IEventParameter1((USHORT)(aNumber)),
                                             IEventParameter1(aNumber),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SETTEXTLIMIT",
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
     ef.sendEvent( EM_SETTEXTLIMIT,
                   IEventParameter1( aNumber ),
                   IEventParameter2( 0 ));

     // Force an update of the text (for canvas layout)
//   unsigned long ulClBufLen = (unsigned long)IQUERYWINDOWTEXTLENGTH(ef) + 1;
//   char* pszClBuf = new char[ulClBufLen];
//   IQUERYWINDOWTEXT( ef, ulClBufLen, pszClBuf );
//   ISETWINDOWTEXT( ef, pszClBuf );
//   delete [] pszClBuf;
   }
#endif

   setLayoutDistorted(IWindow::minimumSizeChanged, 0);
   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setMaster                                                   |
|                                                                              |
| Assign the master for this servant spin button.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: setMaster ( IBaseSpinButton& master )
{
   IASSERTSTATE(isServant());
   IASSERTPARM(master.isMaster());

   if (this->isPMCompatible())
   {
     unsigned long ulOk =
             handle().sendEvent(SPBM_SETMASTER,
                                IEventParameter1(master.handle()),
                                IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SETMASTER",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle updown = IQUERYUPDOWNHANDLE( master.handle() );
     if ( updown.asUnsigned() )
        ISETUPDOWNHANDLE( this->handle(), updown.asUnsigned() );
     else
        ITHROWLIBRARYERROR(IC_INVALIDHANDLE,
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
   }
#endif

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::spinDown                                                    |
|                                                                              |
| Spin the button down (backward) the specified amount of times.               |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: spinDown ( unsigned long spinBy )
{
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_SPINDOWN,
                                             IEventParameter1(spinBy),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SPINDOWN",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle spinBt = this->handle();
     IWindowHandle updown = IQUERYUPDOWNHANDLE( spinBt );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) !=
           ISpinButtonStatics::buddyHandle( spinBt ) ) )
        updown = 0;

     // Try to gather info from the updown control.  If that fails,
     // use stored information to handle the servant case.
     IEventResult pos, range;
     if (updown.asUnsigned())
     {
        pos = updown.sendEvent( UDM_GETPOS );
        if ( pos.highNumber() )
           ITHROWGUIERROR2("UDM_GETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        range = updown.sendEvent( UDM_GETRANGE );
     }
     else
     {
        pos = IQUERYLASTPOS( spinBt );
        range = IQUERYRANGE( spinBt );
     }
     int newpos = (short)pos.lowNumber() - (int)spinBy;
     // lowNumber is maximum, highNumber is minimum in the event result
     if ((short)range.highNumber() > newpos)
     {
        newpos += (short)range.lowNumber() - (short)range.highNumber() + 1;
        if ((short)range.highNumber() > newpos)
        {
           // Still out of range ... error
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        }
     }

     if (updown.asUnsigned())
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2(newpos));

     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)newpos ),
                      IEventParameter2( updown.asUnsigned() ) );

     // Generate the spin up/down event expected by handlers
     IWindowHandle parent = IPARENTOF( spinBt );
     parent.sendEvent( WM_COMMAND,
                       IEventParameter1( (short)IIDOF( spinBt ),
                                         (short)SPBN_DOWNARROW ),
                       IEventParameter2( spinBt.asUnsigned()) );
   }
#endif

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::spinUp                                                      |
|                                                                              |
| Spin the button up (forward) the specified amount of times.                  |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: spinUp   ( unsigned long spinBy )
{
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_SPINUP,
                                             IEventParameter1(spinBy),
                                             IEventParameter2(0));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_SPINUP",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IWindowHandle spinBt = this->handle();
     IWindowHandle updown = IQUERYUPDOWNHANDLE( spinBt );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) !=
           ISpinButtonStatics::buddyHandle( spinBt ) ) )
        updown = 0;

     // Try to gather info from the updown control.  If that fails,
     // use stored information to handle the servant case.
     IEventResult pos, range;
     if (updown.asUnsigned())
     {
        pos = updown.sendEvent( UDM_GETPOS );
        if ( pos.highNumber() )
           ITHROWGUIERROR2("UDM_GETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        range = updown.sendEvent( UDM_GETRANGE );
     }
     else
     {
        pos = IQUERYLASTPOS( spinBt );
        range = IQUERYRANGE( spinBt );
     }
     int  newpos = (short)pos.lowNumber() + (int)spinBy;
     // lowNumber is maximum, highNumber is minimum in the event result
     if ((short)range.lowNumber() < newpos)
     {
        newpos -= (short)range.lowNumber() - (short)range.highNumber() + 1;
        if ((short)range.lowNumber() < newpos)
        {
           // Still out of range ... error
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidRequest,
                           IException::recoverable);
        }
     }
     if (updown.asUnsigned())
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2(newpos));

     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)newpos ),
                      IEventParameter2( updown.asUnsigned() ) );

     // Generate the spin up/down event expected by handlers
     IWindowHandle parent = IPARENTOF( spinBt );
     parent.sendEvent( WM_COMMAND,
                       IEventParameter1( (short)IIDOF( spinBt ),
                                         (short)SPBN_UPARROW ),
                       IEventParameter2( spinBt.asUnsigned()) );
   }
#endif

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::enable                                                      |
|                                                                              |
| Enables the window to accept keyboard and mouse input.                       |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: enable ( Boolean enableControl )
{
   IWindow::enable( enableControl);
#ifdef IC_WIN
   // If this is a native windows spin button, we need to notify the
   // entry field about this call (so that text is normal or grayed properly).
   if (!this->isPMCompatible())
   {
     // Get the entry field handle and enable/disable it.
     IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
     ef.sendEvent( WM_ENABLE,
                   IEventParameter1(enableControl),
                   IEventParameter2(0));
   }
#endif
   refresh();
   return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::isPMCompatible                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseSpinButton::isPMCompatible ( ) const
{
#ifdef IC_WIN
   return ((extendedStyle() & pmCompatible.asExtendedUnsignedLong()) ?
             true : false );
#else
   return true;
#endif
}



#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::foregroundColor                                             |
|                                                                              |
| Returns the foreground color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IColor IBaseSpinButton::foregroundColor () const
{
  IGUIColor guiColor(IGUIColor::windowText);

  if (!isWriteable())
  {
    guiColor = IGUIColor(IGUIColor::outputText);
  }
#ifdef IC_PM
  return (IWindow::color(PP_FOREGROUNDCOLOR, guiColor));
#else
  {
    if (this->isPMCompatible())
    {
      unsigned char ucRed,
                    ucGreen,
                    ucBlue;
      unsigned long ulResult =
         handle().sendEvent(SPBM_QUERYSPINCOLORS,
                            IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                            IEventParameter2((unsigned long) NULL));
      ucBlue = (unsigned char)((ulResult & 0x00ff0000) >> 16);
      ucGreen = (unsigned char)((ulResult & 0x0000ff00) >> 8);
      ucRed = (unsigned char)(ulResult & 0x000000ff);
      return (IColor(ucRed, ucGreen, ucBlue));
    }
  else
    {
      return (IWindow::color(PP_FOREGROUNDCOLOR, guiColor));
    }
  }
#endif
}
#endif

/*------------------------------------------------------------------------------
| IBaseSpinButton::calcMinimumSize                                             |
|                                                                              |
| Return the minimum size of the spin button based on the font and the text    |
| limit.                                                                       |
------------------------------------------------------------------------------*/
ISize IBaseSpinButton :: calcMinimumSize() const
{
  unsigned long ulClTextLimit = limit();

  IFont font( this );
  unsigned long ulWidth = font.avgCharWidth()   // Average char width
                            * ulClTextLimit;
  unsigned long ulHeight = font.maxCharHeight();

  /*****************************************************************/
  /* Use a step function to reflect the fact that you are more     */
  /* likely to exceed an average character width with an           */
  /* individual character, but a long string will likely be ok     */
  /* with this average value.                                      */
  /*****************************************************************/
  // DEFECT 8842 : earlier algorithm gave uneven length increases;
  // simplify to a smooth curve of lengths over text limit range.
  if (ulClTextLimit <= 2)
     ulWidth = (unsigned long)(ulWidth * 3);
  else if (ulClTextLimit <=16)
     ulWidth = (unsigned long)(ulWidth + 5);
  else if (ulClTextLimit <=24)
     ulWidth = (unsigned long)(ulWidth + 4);
  else if (ulClTextLimit <=32)
     ulWidth = (unsigned long)(ulWidth + 3);
  else if (ulClTextLimit <=40)
     ulWidth = (unsigned long)(ulWidth + 2);
  else if (ulClTextLimit <=48)
     ulWidth = (unsigned long)(ulWidth + 1);

  /*****************************************************************/
  /* Add room for the spin arrows, but still be sure that the size */
  /* does not exceed 35 avg chars                                  */
  /*                                                               */
  /* Note: Spin arrows are 17 pels wide.                           */
  /*****************************************************************/
  if (this->isPMCompatible())
  {
    ulWidth += 17;
  }
#ifdef IC_WIN
  else
  {
    ulWidth += (unsigned long) GetSystemMetrics( SM_CXVSCROLL );
  }
#endif
  unsigned long maxWidth = font.avgCharWidth() * 35;
  if ( ulWidth > maxWidth )
  {
     ulWidth = maxWidth;
  }

  if (hasBorder())
  {                     // Entry field has border.
     ulWidth += (3 * IQUERYSYSVALUE(SV_CXBORDER));
#ifdef IC_PM
     ulHeight += font.avgLowercase();
#endif
#ifdef IC_WIN
     // Use 1/2 a character since no conversion for lowercase api
     ulHeight += font.maxCharHeight()/2;
     // Oldshell needs a bit more.
     if (IPlatform::isWinNT() && !IPlatform::isNTNewShell())
        ulHeight += 2;
     if ( ( extendedStyle() & border3D.asExtendedUnsignedLong() ) &&
         !( extendedStyle() & pmCompatible.asExtendedUnsignedLong()) &&
          ( IPlatform::isWin95() || IPlatform::isNTNewShell() ))
     {
       // look for api's to get this value programmatically.
       ulWidth += 4;
       ulHeight += 4;
     }
#endif
  }

  return ISize(ulWidth, ulHeight);
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IBaseSpinButton::setForegroundColor                                          |
|                                                                              |
| Updates the foreground color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton::setForegroundColor ( const IColor &color)
{
  if (this->isPMCompatible())
  {
    unsigned long ulResult =
       handle().sendEvent(SPBM_SETSPINCOLORS,
                          IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                          IEventParameter2((unsigned long) color.asRGBLong()));
    if (ulResult)
    {
      setLayoutDistorted(IWindow::colorChanged, 0);
      this->refresh( );
    }
    else
    {
      ITHROWGUIERROR2("SPBM_SETSPINCOLORS",
                      IBaseErrorInfo::invalidParameter,
                      IException::recoverable);
    } /* endif */
  }
  else
  {
    IWindow::setColor(PP_FOREGROUNDCOLOR, color);
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::backgroundColor                                             |
|                                                                              |
| Returns the background color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IColor IBaseSpinButton::backgroundColor () const
{
  unsigned char ucRed,
                ucGreen,
                ucBlue;
  if (this->isPMCompatible())
  {
    unsigned long ulResult =
       handle().sendEvent(SPBM_QUERYSPINCOLORS,
                          IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
                          IEventParameter2((unsigned long) NULL));
    ucBlue = (unsigned char)((ulResult & 0x00ff0000) >> 16);
    ucGreen = (unsigned char)((ulResult & 0x0000ff00) >> 8);
    ucRed = (unsigned char)(ulResult & 0x000000ff);
    return (IColor(ucRed, ucGreen, ucBlue));
  }
  else
  {
    return (IWindow::color(PP_BACKGROUNDCOLOR, IGUIColor(IGUIColor::entryFieldBgnd)));
  }
}

/*------------------------------------------------------------------------------
| IBaseSpinButton::setBackgroundColor                                          |
|                                                                              |
| Updates the background color of the spinbutton.                              |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton::setBackgroundColor ( const IColor &color)
{
  if (this->isPMCompatible())
  {
    unsigned long ulResult =
       handle().sendEvent(SPBM_SETSPINCOLORS,
       IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
       IEventParameter2(NEARESTPALETTECOLOR((unsigned long) color.asRGBLong())));
    if (ulResult)
    {
      setLayoutDistorted(IWindow::colorChanged, 0);
      this->refresh( );
    }
    else
    {
      ITHROWGUIERROR2("SPBM_SETSPINCOLORS",
                      IBaseErrorInfo::invalidParameter,
                      IException::recoverable);
    } /* endif */
  }
  else
  {
    IWindow::setColor(PP_BACKGROUNDCOLOR, color);
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton :: resetForegroundColor                                      |
|                                                                              |
| Resets the foreground color of the IBaseSpinButton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton& IBaseSpinButton :: resetForegroundColor ( )
{
  if (this->isPMCompatible())
  {
    handle().sendEvent( SPBM_RESETSPINCOLORS,
                        IEventParameter1((unsigned long)SPBA_FOREGROUNDCOLOR),
                        IEventParameter2((unsigned long) NULL));
    this->refresh( );
  }
  else
  {
    IWindow::resetColor(PP_FOREGROUNDCOLOR);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseSpinButton :: resetBackgroundColor                                      |
|                                                                              |
| Resets the background color of the IBaseSpinButton.                          |
------------------------------------------------------------------------------*/
IBaseSpinButton & IBaseSpinButton :: resetBackgroundColor ( )
{
  if (this->isPMCompatible())
  {
    handle().sendEvent( SPBM_RESETSPINCOLORS,
                        IEventParameter1((unsigned long)SPBA_BACKGROUNDCOLOR),
                        IEventParameter2((unsigned long) NULL));
    this->refresh( );
  }
  else
  {
    IWindow::resetColor(PP_BACKGROUNDCOLOR);
  }
  return *this;
}
#endif  // IC_WIN

