/*******************************************************************************
* FILE NAME: itbar.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbar.hpp.                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINFRAMEMGR
  #define INCL_WININPUT
  #define INCL_WINTRACKRECT
  #define INCL_WINPOINTERS
  #define INCL_WINWINDOWMGR
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <itbar.hpp>

#include <icmdhdr.hpp>
#include <icmnfun.hpp>
#include <icolor.hpp>
#include <icvhdr.hpp>
#include <idmhndlr.hpp>
#include <iexcept.hpp>
#include <igbundle.hpp>
#include <igpyline.hpp>
#include <igrafctx.hpp>
#include <inotifev.hpp>
#include <iplatfrm.hpp>
#include <ipoint.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <isysmenu.hpp>
#include <ititle.hpp>
#include <itbarbut.hpp>
#include <itbarcnr.hpp>
#include <itrace.hpp>
#include <itbarfrm.hpp>
#include <itimer.hpp>
#include <itbpriv.hpp>
#include <imcevt.hpp>
#include <icoordsy.hpp>
#include <igrect.hpp>
#include <iobjwin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBAR_CPP_
  #include <ipagetun.h>
#endif

#ifndef _ISEQ_H
  #include <iseq.h>               // for ISequence
#endif

/*------------------------------------------------------------------------------
| Public styles.                                                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IToolBar::Style
  IToolBar::filterMisfits              ( 0, ICNV_HIDEMISFITS ),
  IToolBar::buttonBitmapVisible        ( 0, ICNV_BTNBMPVISIBLE ),
  IToolBar::buttonTextVisible          ( 0, ICNV_BTNTEXTVISIBLE ),
  IToolBar::buttonBitmapAndTextVisible ( 0, ICNV_BTNBMPANDTEXTVISIBLE ),
  IToolBar::noDragDrop                 ( 0, ICNV_NODRAGDROP ),
  IToolBar::classDefaultStyle          ( WS_VISIBLE, (ICNV_HIDEMISFITS |
                                                      ICNV_BTNBMPVISIBLE) );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IToolBar::Style
  IToolBar::fgCurrentDefaultStyle ( WS_VISIBLE, (ICNV_HIDEMISFITS |
                                                 ICNV_BTNBMPVISIBLE) );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Class:   IToolBarPaintHandler                                                |
|                                                                              |
| Purpose: Draws the border of the tool bar.                                   |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarPaintHandler : public ICanvasHandler {
typedef ICanvasHandler
  Inherited;
public:
  IToolBarPaintHandler ();
 ~IToolBarPaintHandler ();
protected:
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );
};

IToolBarPaintHandler::IToolBarPaintHandler ()
{
}

IToolBarPaintHandler::~IToolBarPaintHandler ()
{
}

/*------------------------------------------------------------------------------
| Class:   IToolBarObject                                                      |
|                                                                              |
| Purpose: Points to a window object that has been added to the toolbar.       |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarObject : public IBase {
typedef IBase
  Inherited;
public:
  IToolBarObject ( IWindow *childWindow );

void
  setVisible        ( Boolean visibility );

Boolean
  isVisible         ( ) const { return fVisible; }

IWindow
 *window            ( ) const { return pwinCl; }

private:
IWindow
 *pwinCl;
Boolean
  fVisible;
};

IToolBarObject::IToolBarObject ( IWindow* childWindow )
              : pwinCl(childWindow),
                fVisible(true)
{
}

void IToolBarObject::setVisible ( Boolean visibility )
{
  fVisible = visibility;
}

/*------------------------------------------------------------------------------
| IToolBarObjectList                                                           |
|                                                                              |
| Implementation class.                                                        |
------------------------------------------------------------------------------*/
class IToolBarObjectList : public ISequence < IToolBarObject* >
{
public:
  IToolBarObjectList ( );
 ~IToolBarObjectList ( );
};

/*------------------------------------------------------------------------------
| IToolBarObjectList::IToolBarObjectList                                       |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IToolBarObjectList :: IToolBarObjectList ( )
  : ISequence < IToolBarObject* > ( )
{ }

/*------------------------------------------------------------------------------
| IToolBarObjectList::~IToolBarObjectList                                      |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IToolBarObjectList :: ~IToolBarObjectList ( )
{ }

/*------------------------------------------------------------------------------
| IToolBarList class                                                           |
|                                                                              |
| Implementation class.                                                        |
------------------------------------------------------------------------------*/
class IToolBarList : public ISequence<IToolBar*>
{
public:
  IToolBarList ();
 ~IToolBarList ();
};

IToolBarList::IToolBarList ()
{
}

IToolBarList::~IToolBarList ()
{
}

/*------------------------------------------------------------------------------
| Class:   IToolBarFloatingHandler                                             |
|                                                                              |
| Purpose: Hides the floating frame window when the user requests that it be   |
|          closed.                                                             |
------------------------------------------------------------------------------*/
class IToolBarFloatingHandler : public ICommandHandler {
typedef ICommandHandler
  Inherited;
public:
  IToolBarFloatingHandler ();
 ~IToolBarFloatingHandler ();
protected:
virtual Boolean
  systemCommand ( ICommandEvent& event );
};

IToolBarFloatingHandler::IToolBarFloatingHandler ()
{
}

IToolBarFloatingHandler::~IToolBarFloatingHandler ()
{
}

/*------------------------------------------------------------------------------
| IToolBarFloatingHandler::systemCommand                                       |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBarFloatingHandler::systemCommand ( ICommandEvent& event )
{
#ifdef IC_WIN
  switch  ( event.commandId() & 0xFFFFFFF0 )
#else
  switch  ( event.commandId() )
#endif
  {
     case SC_CLOSE:
     {
       // Set the toolbar back to an hidden location (this will cause
       // the floating frame to be deleted)

       ((IFrameWindow*)event.window())->client()->postEvent
                                   (IC_UM_TOOLBAR, IC_UM_TBAR_HIDDEN);

       // Set the focus to the owning frame (so that activation is handled
       // properly and this is the currently active window)

       if (event.window()->owner())
         event.window()->owner()->setFocus();

       return true;
     }
  }  // switch
  return false;
}

/*------------------------------------------------------------------------------
| Class:   IToolBarTrackHandler                                                |
|                                                                              |
| Purpose: This handler is attached to the floating frame of a toolbar to      |
|          handle move requests and allow the user to "drag" the toolbar back  |
|          to the owning frame as a frame extension.                           |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarTrackHandler : public IHandler {
typedef IHandler
  Inherited;
public:
  IToolBarTrackHandler ( IToolBar* tbar, IToolBarData* tbardata );
 ~IToolBarTrackHandler ( );
void
  checkCursor ();

#ifdef IC_WIN
IToolBarTrackHandler
 &DrawTrackRect (IGraphicContext &gc, LPRECT lprc);
Boolean
  WinTrackRect (IWindowHandle hwnd, void* , PTRACKINFO lpTrackInfo);
#endif

protected:
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

private:
IToolBar
 *toolBar;
IToolBarData
 *fToolBarData;
IPointerHandle
  pointer;
IPointerHandle
  tbarLeft;
IPointerHandle
  tbarRight;
IPointerHandle
  tbarBottom;
IPointerHandle
  tbarTop;
IPointerHandle
  tbarLeft2;
IPointerHandle
  tbarRight2;
IPointerHandle
  tbarBottom2;
IPointerHandle
  tbarTop2;
IRectangle
  frameRect;
IToolBar::Location
  newLocation;
IToolBar
 *targetTbar;
IToolBarContainer
 *targetCnr;
IWindowHandle
  lastHandle;
};

IToolBarTrackHandler::IToolBarTrackHandler ( IToolBar* tbar, IToolBarData* tbardata )
                     : toolBar(tbar), fToolBarData(tbardata), pointer(0),
                       tbarLeft(0), tbarRight(0), tbarBottom(0), tbarTop(0),
                       tbarLeft2(0), tbarRight2(0), tbarBottom2(0), tbarTop2(0),
                       targetTbar(0), targetCnr(0)
{
}

IToolBarTrackHandler::~IToolBarTrackHandler ()
{
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IToolBarHandler::WinTrackRect                                                |
| This function lifted from cclnt\cnr\cnrs3p1.c.  It is an incomplete          |
| implementation of the PM function of the same name.  This version is         |
| designed specifically for use in the toolbar track handler.                  |
------------------------------------------------------------------------------*/
Boolean IToolBarTrackHandler::WinTrackRect (IWindowHandle hwnd,
                                            void* ,
                                            PTRACKINFO lpTrackInfo)
{
  IMODTRACE_DEVELOP("IToolBarTrackHandler::WinTrackRect");

  Boolean       fTrack = TRUE;
  IWindowHandle hwndFocus = GetFocus();
  RECT          rcTrack, rcBoundary;
  POINT         ptCursor;
  int           xLast, yLast;
  MSG           msg;
  WORD          wQuitMessage;


  // Use an object window to capture the mouse. The hwnd that is passed in may
  // be the desktop which can not be used to capture the mouse so we need a
  // dummy window for the capture.
  IObjectWindow objWindow;

  HDC hdc = GetWindowDC( HWND_DESKTOP );
  if ( hdc == NULL )
    return false;

  IPresSpaceHandle ps(hdc);

  IGraphicContext gc( ps );

  gc.setMixMode( IGraphicBundle::xor );
  gc.setDrawOperation( IGraphicBundle::frame );
  gc.setPenColor( IColor::white );
  gc.setPenWidth( lpTrackInfo->cxBorder );

  /*-----------------------------------------------------------*/
  /* figure out which button will signal the end of the drag;  */
  /* if more than one button is pressed, the left takes        */
  /* precedence, followed by the right, followed by the middle */
  /*-----------------------------------------------------------*/
  wQuitMessage = (GetKeyState (VK_LBUTTON) < 0) ? WM_LBUTTONUP :
                 (GetKeyState (VK_RBUTTON) < 0) ? WM_RBUTTONUP :
                                                  WM_MBUTTONUP;

  /*------------------------------------------------------*/
  /* get the current mouse position in window coordinates */
  /* (we're overloading rcTrack here, but it saves stack) */
  /*------------------------------------------------------*/
//  GetCursorPos (&ptCursor);
  DWORD messagePos = GetMessagePos();
  ptCursor.x = LOWORD(messagePos);
  ptCursor.y = HIWORD(messagePos);

  GetWindowRect (hwnd, &rcTrack);
  xLast = (int)(ptCursor.x - rcTrack.left);
  yLast = (int)(ptCursor.y - rcTrack.top);

  ITRACE_DEVELOP("Cursor pos is " + IPoint(ptCursor).asString() +
                 "Window rect is " + IRectangle(rcTrack).asString() );

  /*------------------------------------------------*/
  /* make a working copy of the tracking rectangles */
  /*------------------------------------------------*/
  rcTrack    = lpTrackInfo->rclTrack;

  /*-----------------------------------------------------------------*/
  /* make sure the mouse doesn't move outside the boundary rectangle */
  /*-----------------------------------------------------------------*/
  ITRACE_DEVELOP(IString("lpTrackInfo->rclBoundary=(") +
             IString( lpTrackInfo->rclBoundary.left) + IString(",") +
             IString( lpTrackInfo->rclBoundary.top) + IString(",") +
             IString( lpTrackInfo->rclBoundary.right) + IString(",") +
             IString( lpTrackInfo->rclBoundary.bottom) + IString(")") );
  // We add 1 to the right/bottom to prevent oscillation of the tracking
  // rectangle by 1 pixel.  Seems that if limits of the boundary are ==
  // the mouse position oscillates around the point.
  rcBoundary.left   = ptCursor.x -
                      (rcTrack.left - lpTrackInfo->rclBoundary.left );
  rcBoundary.right  = ptCursor.x +
                      (lpTrackInfo->rclBoundary.right - rcTrack.right ) + 1 ;
  rcBoundary.top    = ptCursor.y -
                      (rcTrack.top  - lpTrackInfo->rclBoundary.top );
  rcBoundary.bottom = ptCursor.y +
                      (lpTrackInfo->rclBoundary.bottom - rcTrack.bottom ) + 1;
  ITRACE_DEVELOP(IString("rcBoundary=(") +
             IString( rcBoundary.left) + IString(",") +
             IString( rcBoundary.top) + IString(",") +
             IString( rcBoundary.right) + IString(",") +
             IString( rcBoundary.bottom) + IString(")") );

  SetCapture ( objWindow.handle() );
//  ClipCursor (&rcBoundary);

  DrawTrackRect (gc, &rcTrack);

  while ( fTrack &&
          GetMessage (&msg, 0, 0, 0))
  {
    if (msg.message == wQuitMessage)
      fTrack = FALSE;

    else
    {
      switch (msg.message)
      {
        /*-----------------------*/
        /* eat keyboard messages */
        /*-----------------------*/
        case WM_KEYUP:
        case WM_SYSKEYUP:
          break;
        case WM_CHAR:
        case WM_KEYDOWN:
        case WM_SYSCHAR:
        case WM_SYSKEYDOWN:
        {
          switch (msg.wParam )
          {
            case VK_ESCAPE:
            {
              fTrack = FALSE;
              break;
            }

// You are supposed to be able to move the tracking rectangle with the keyboard
// but the setCursor call did not work. Until I can get the setCursor to work
// this code must be commented out.

//            case VK_LEFT:
//            {
//              ITRACE_DEVELOP("Left");
//              /*-------------------*/
//              /* erase, move, draw */
//              /*-------------------*/
//              this->DrawTrackRect (gc, &rcTrack);

//              int  xDelta = -lpTrackInfo->cxKeyboard;
//              int  yDelta = 0;
//              OffsetRect (&rcTrack, xDelta, yDelta);
//              this->DrawTrackRect (gc, &rcTrack);
//              ITRACE_DEVELOP(IString("rcTrack=(") +
//                      IString( rcTrack.left) + IString(",") +
//                      IString( rcTrack.top) + IString(",") +
//                      IString( rcTrack.right) + IString(",") +
//                      IString( rcTrack.bottom) + IString(")") );
//              xLast += xDelta;
//              yLast += yDelta;
//              SetCursorPos( xLast, yLast );
//              break;
//            }

//            case VK_UP:
//            {
//              ITRACE_DEVELOP("Up");
//              /*-------------------*/
//              /* erase, move, draw */
//              /*-------------------*/
//              this->DrawTrackRect (gc, &rcTrack);

//              int  xDelta = 0;
//              int  yDelta = -lpTrackInfo->cyKeyboard;
//              OffsetRect (&rcTrack, xDelta, yDelta);
//              this->DrawTrackRect (gc, &rcTrack);
//              ITRACE_DEVELOP(IString("rcTrack=(") +
//                      IString( rcTrack.left) + IString(",") +
//                      IString( rcTrack.top) + IString(",") +
//                      IString( rcTrack.right) + IString(",") +
//                      IString( rcTrack.bottom) + IString(")") );
//              xLast += xDelta;
//              yLast += yDelta;
//              SetCursorPos( xLast, yLast );
//              break;
//            }
//            case VK_RIGHT:
//            {
//              ITRACE_DEVELOP("Right");
//              /*-------------------*/
//              /* erase, move, draw */
//              /*-------------------*/
//              this->DrawTrackRect (gc, &rcTrack);

//              int  xDelta = lpTrackInfo->cxKeyboard;
//              int  yDelta = 0;
//              OffsetRect (&rcTrack, xDelta, yDelta);
//              this->DrawTrackRect (gc, &rcTrack);
//              ITRACE_DEVELOP(IString("rcTrack=(") +
//                      IString( rcTrack.left) + IString(",") +
//                      IString( rcTrack.top) + IString(",") +
//                      IString( rcTrack.right) + IString(",") +
//                      IString( rcTrack.bottom) + IString(")") );
//              xLast += xDelta;
//              yLast += yDelta;
//              SetCursorPos( xLast, yLast );
//              break;
//            }
//            case VK_DOWN:
//            {
//              ITRACE_DEVELOP("Down");
//              /*-------------------*/
//              /* erase, move, draw */
//              /*-------------------*/
//              this->DrawTrackRect (gc, &rcTrack);

//              int  xDelta = 0;
//              int  yDelta = lpTrackInfo->cyKeyboard;
//              OffsetRect (&rcTrack, xDelta, yDelta);
//              this->DrawTrackRect (gc, &rcTrack);
//              ITRACE_DEVELOP(IString("rcTrack=(") +
//                     IString( rcTrack.left) + IString(",") +
//                     IString( rcTrack.top) + IString(",") +
//                      IString( rcTrack.right) + IString(",") +
//                      IString( rcTrack.bottom) + IString(")") );
//              xLast += xDelta;
//              yLast += yDelta;
//              SetCursorPos( xLast, yLast );
//              break;
//            }
          }
        }
        break;

        case WM_MOUSEMOVE:
        {
          POINT pt;
          GetCursorPos( &pt );

          int  xDelta = (int)(pt.x - xLast);
          int  yDelta = (int)(pt.y - yLast);

          /*-------------------*/
          /* erase, move, draw */
          /*-------------------*/
          this->DrawTrackRect (gc, &rcTrack);
          OffsetRect (&rcTrack, xDelta, yDelta);
          this->DrawTrackRect (gc, &rcTrack);
          xLast += xDelta;
          yLast += yDelta;
        }
        break;

        /*---------------------------------*/
        /* eat other mouse button messages */
        /*---------------------------------*/
        case WM_LBUTTONUP:
        case WM_MBUTTONUP:
        case WM_RBUTTONUP:
        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
        case WM_RBUTTONDOWN:
        case WM_LBUTTONDBLCLK:
        case WM_MBUTTONDBLCLK:
        case WM_RBUTTONDBLCLK:
          break;

        default:
          DispatchMessage (&msg);
          break;
      }
    }
  }

  /*----------------------------------------------------------------------*/
  /* if we fell out of the loop because GetMessage returned FALSE         */
  /* (i.e. PostQuitMessage was called), put the WM_QUIT back on the queue */
  /*----------------------------------------------------------------------*/
  if (fTrack && (GetFocus() == hwndFocus) && (GetCapture() == hwnd))
  {
    PostQuitMessage (msg.wParam);
  }

  this->DrawTrackRect (gc, &rcTrack);
  ClipCursor (NULL);
  ReleaseCapture ();

  ReleaseDC( NULL, hdc );

  /*---------------------------------------*/
  /* if the user pressed ESC, return FALSE */
  /*---------------------------------------*/
  if ((msg.message == WM_KEYDOWN) && (msg.wParam == VK_ESCAPE))
  {
    return (FALSE);
  }
  /*---------------------------------------*/
  /* otherwise return the drop coordinates */
  /*---------------------------------------*/
  else
  {
    lpTrackInfo->rclTrack = rcTrack;
    return (TRUE);
  }
}

IToolBarTrackHandler &IToolBarTrackHandler::DrawTrackRect( IGraphicContext &gc,
                                                           LPRECT lprc)
{
  IRectangle theBox( IPoint(lprc->left, lprc->top),
                     IPoint(lprc->right, lprc->bottom) );
  IGRectangle myBox( theBox  );
  myBox.drawOn( gc );
  return *this;
}
#endif

/*------------------------------------------------------------------------------
| Class:   IToolBarData                                                        |
|                                                                              |
| Purpose: Store private data used by the class.                               |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarData : public IBase
{
public:
  IToolBarData ( IFrameWindow* frame );
 ~IToolBarData();

IFrameWindow
 *owningFrame;
IToolBarContainer
 *toolBarContainer;
IString
  toolBarName;
IToolBarPaintHandler
  paintHandler;
Boolean
  layoutOn;
static long
  currentDefaultPad;
static unsigned long
  currentDefaultGroupPad,
  currentDefaultMisfitWidth;
static ISize
  currentDefaultMargin;
IToolBarObjectList
 *objectList;
IToolBar
 *precedingToolBar;
IToolBarFrameWindow
 *floatingFrame;
IToolBarFloatingHandler
 *floatingHandler;
IToolBarTrackHandler
 *trackHandler;
IPoint
  floatingPosition;
Boolean
  isInitialized;
IToolBar::Location
  currentLocation;
ISize
  margin;
unsigned long
  misfitWidth;
Boolean
  isExpanded;
unsigned long
  windowId;
#ifdef IC_WIN
Boolean
  bCheckingDragStart;
IPoint
  pointLButtonDown;
IEventData
  parameter1,
  parameter2;
#endif
};

IToolBarData::IToolBarData ( IFrameWindow* frame )
             : owningFrame(frame)
             , toolBarContainer(0)
             , toolBarName("")
             , paintHandler()
             , layoutOn(true)
             , objectList(new IToolBarObjectList)
             , precedingToolBar(0)
             , floatingFrame(0)
             , floatingHandler(0)
             , trackHandler(0)
             , floatingPosition(IPoint(0,0))
             , isInitialized(false)
             , currentLocation(IToolBar::hidden)
             , isExpanded(true)
             , windowId(0)
#ifdef IC_WIN
             , bCheckingDragStart( false)
#endif
{
}

IToolBarData::~IToolBarData ()
{
}


#pragma data_seg(ICLNonConst)
long          IToolBarData::currentDefaultPad = -1;
unsigned long IToolBarData::currentDefaultGroupPad = 8;
unsigned long IToolBarData::currentDefaultMisfitWidth = 100;
ISize         IToolBarData::currentDefaultMargin = ISize(7,4);
#pragma data_seg()

/*------------------------------------------------------------------------------
| IToolBarTrackHandler::checkCursor                                            |
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBarTrackHandler::checkCursor ()
{
  IMODTRACE_DEVELOP("IToolBarTrackHandler::checkCursor");

  POINTL pnt;
  IQUERYPOINTERPOSITION(pnt.x,pnt.y);

  IWindowHandle testHandle = IWINDOWFROMPOINT
                             ( IWindow::desktopWindow()->handle(), &pnt, true );

  // Be sure we have all the toolbar pointers loaded
  if ( !tbarLeft )
  {
    try
    {
      tbarLeft = IApplication::current().resourceLibrary().loadPointer
                 (IC_TOOLBAR_POINTER_LEFT);
      tbarRight = IApplication::current().resourceLibrary().loadPointer
                  (IC_TOOLBAR_POINTER_RIGHT);
      tbarTop = IApplication::current().resourceLibrary().loadPointer
                (IC_TOOLBAR_POINTER_TOP);
      tbarBottom = IApplication::current().resourceLibrary().loadPointer
                   (IC_TOOLBAR_POINTER_BOTTOM);
      tbarLeft2 = IApplication::current().resourceLibrary().loadPointer
                  (IC_TOOLBAR_POINTER_LEFT_2);
      tbarRight2 = IApplication::current().resourceLibrary().loadPointer
                   (IC_TOOLBAR_POINTER_RIGHT_2);
      tbarTop2 = IApplication::current().resourceLibrary().loadPointer
                 (IC_TOOLBAR_POINTER_TOP_2);
      tbarBottom2 = IApplication::current().resourceLibrary().loadPointer
                    (IC_TOOLBAR_POINTER_BOTTOM_2);
    }
    catch ( IException& )
    {
      // Could not load pointers from our DLL so use system app pointer instead
      tbarLeft = ISystemPointerHandle ( ISystemPointerHandle::standardApplication );
      tbarRight = ISystemPointerHandle ( ISystemPointerHandle::standardApplication );
      tbarTop = ISystemPointerHandle ( ISystemPointerHandle::standardApplication );
      tbarBottom = ISystemPointerHandle ( ISystemPointerHandle::standardApplication );
      tbarLeft2 = tbarLeft;
      tbarRight2 = tbarRight;
      tbarTop2 = tbarTop;
      tbarBottom2 = tbarBottom;
    }
  }

  if ( testHandle == fToolBarData->owningFrame->handle() )
  {
    if ( !pointer )
    {
      // Save old pointer (so we can restore it later)
      pointer = IQUERYPOINTER;
    }

    targetTbar = 0;
    targetCnr = 0;

    IPoint pt( pnt );

    if ( ICoordinateSystem::nativeOrientation() !=
             ICoordinateSystem::originLowerLeft )
      // The logic to determine which border we are over is written based on the
      // point, which is native, being relative to the Lower Left. If the native
      // orientation is not lower left then the point needs to be converted.
      pt.setY( IWindow::desktopWindow()->size().height() - pt.y() );

    // Figure out which border side we are over
    long cxLeft   = pt.x() - frameRect.left();
    long cxRight  = frameRect.right() - pt.x();
    long cyTop    = frameRect.top() - pt.y();
    long cyBottom = pt.y() - frameRect.bottom();
    if (( cxRight < cxLeft ) && ( cxRight < cyTop ) && ( cxRight < cyBottom ))
    {
      ISETPOINTER(tbarRight2);
      newLocation = IToolBar::rightOfClient;
    }
    else
    {
      if (( cxLeft < cyTop ) && ( cxLeft < cyBottom ))
      {
        ISETPOINTER(tbarLeft2);
        newLocation = IToolBar::leftOfClient;
      }
      else
      {
        if ( cyBottom < cyTop )
        {
          ISETPOINTER(tbarBottom2);
          newLocation = IToolBar::belowClient;
        }
        else
        {
          ISETPOINTER(tbarTop2);
          newLocation = IToolBar::aboveClient;
        }
      }
    }
  }
  else
  {
    // If we are over the source tool bar, then we cannot drop and it
    // must be floating (so just reset pointer if needed)
    if ( toolBar->handle() == testHandle )
    {
      if ( pointer )
        ISETPOINTER ( pointer );
      pointer = 0;
      return;
    }

    // Assume that the tool bar will be floating
    newLocation = IToolBar::floating;

    // Check if cursor is over another toolbar or toolbar container
    targetTbar = 0;
    targetCnr = 0;
    IToolBar::FrameCursor cursor(fToolBarData->owningFrame);
    for ( cursor.setToFirst();
        ( cursor.isValid() && ( newLocation == IToolBar::floating ));
          cursor.setToNext() )
    {
      IToolBar*          tbarTest = IToolBar::toolBarAt(cursor);
      IToolBarContainer* cnr = 0;

      if ( tbarTest )
        cnr = tbarTest->toolBarContainer();

      if ( tbarTest && cnr && ( tbarTest->handle() == testHandle ))
      {
        targetTbar = tbarTest;
        newLocation = targetTbar->location();
      }
      else
      {
        if ( cnr && ( cnr->handle() == testHandle ))
        {
          targetCnr = cnr;
          newLocation = tbarTest->location();
        }
      }
    }

    if ( newLocation == IToolBar::floating )
    {
       // Reset the pointer if needed
      if ( pointer )
        ISETPOINTER ( pointer );
      pointer = 0;
    }
    else
    {
      // We are not floating, so save the system pointer handle
      if ( pointer == 0 )
        pointer = IQUERYPOINTER;

      // Set the new pointer based on the new location
      switch ( newLocation )
      {
         case IToolBar::aboveClient:
            if ( targetTbar )
              ISETPOINTER(tbarTop);
            else
              ISETPOINTER(tbarTop2);
            break;
         case IToolBar::belowClient:
            if ( targetTbar )
              ISETPOINTER(tbarBottom);
            else
              ISETPOINTER(tbarBottom2);
            break;
         case IToolBar::leftOfClient:
            if ( targetTbar )
              ISETPOINTER(tbarLeft);
            else
              ISETPOINTER(tbarLeft2);
            break;
         case IToolBar::rightOfClient:
            if ( targetTbar )
              ISETPOINTER(tbarRight);
            else
              ISETPOINTER(tbarRight2);
            break;
      }
    }
  }
}

/*------------------------------------------------------------------------------
| IToolBarTrackHandler::dispatchHandlerEvent                                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBarTrackHandler::dispatchHandlerEvent ( IEvent& event )
{
  // Check for move events
  if (( event.eventId() == WM_TRACKFRAME ) &&
      ( event.parameter1().asUnsignedLong() & TF_MOVE ) &&
      ( toolBar->allowsDragDrop() ))
  {
    Boolean   trackOk;
    IPoint    startPos;
    TRACKINFO trackInformation;

#ifdef IC_PM
    trackOk = event.window()->sendEvent ( WM_QUERYTRACKINFO, event.parameter1(),
                                          &trackInformation );
#endif
#ifdef IC_WIN
    {
       trackInformation.cxBorder   = 2;
       trackInformation.cyBorder   = 2;
       trackInformation.cxGrid     = 1;
       trackInformation.cyGrid     = 1;
       trackInformation.cxKeyboard = 8;
       trackInformation.cyKeyboard = 8;

       RECT rect;
       IQUERYWINDOWRECT( event.handle(), &rect );

       trackInformation.rclTrack = rect;

       trackInformation.rclBoundary.top    = 0;
       trackInformation.rclBoundary.left   = 0;
       trackInformation.rclBoundary.bottom = IWindow::desktopWindow()->size().height();
       trackInformation.rclBoundary.right  = IWindow::desktopWindow()->size().width();
       // We are not changing size
       trackInformation.ptlMinTrackSize.x =
          trackInformation.rclTrack.right - trackInformation.rclTrack.left;
       trackInformation.ptlMinTrackSize.y =
          trackInformation.rclTrack.bottom - trackInformation.rclTrack.top;
       trackInformation.ptlMaxTrackSize = trackInformation.ptlMinTrackSize;

       trackInformation.fs = TF_MOVE;
       trackOk = true;
    }
#endif

    if ( trackOk )
    {
      // Set up information to use during tracking
      // We always want the frameRect in lowerLeft coordinates. This is so we don't have to rewrite
      // the logic that determines which edge we are over.
      frameRect = fToolBarData->owningFrame->rect();
      if ( ICoordinateSystem::applicationOrientation() != ICoordinateSystem::originLowerLeft )
      {
        IWindow *frameParent = fToolBarData->owningFrame->parent();
        int parentHeight;
        if ( frameParent == NULL )
          parentHeight = IWindow::desktopWindow()->size().height();
        else
          parentHeight = frameParent->size().height();
        frameRect.moveTo( IPoint( frameRect.minX(), parentHeight - frameRect.maxY()));
      }

      newLocation = IToolBar::floating;

      lastHandle = 0;
      startPos = IRectangle(trackInformation.rclTrack).minXMinY();
      ITimer timer(new ITimerMemberFn0<IToolBarTrackHandler>(*this,checkCursor),
                   100);
      trackOk = WinTrackRect ( IWindow::desktopWindow()->handle(), 0,
                               &trackInformation );
      timer.stop();
    }

    if ( pointer )
    {
      ISETPOINTER( pointer );
      pointer = 0;
    }

    if ( trackOk )
    {
      // The trackInformation rectangle is native, convert to lowerLeft if needed.

      IRectangle trackRectangle(trackInformation.rclTrack);

      if ( ICoordinateSystem::nativeOrientation() != ICoordinateSystem::applicationOrientation() )
      {
        trackRectangle.moveTo( IPoint(trackRectangle.minX(),
                               IWindow::desktopWindow()->size().height() -
                                 trackRectangle.maxY() ));
      }

      IPoint newPos = trackRectangle.minXMinY();

      event.setResult(true);

      // Check if the toolbar should be a frame extension
      if ( newLocation != IToolBar::floating )
      {
        toolBar->postEvent(IC_UM_TOOLBAR, IC_UM_TBAR_MOVE, newLocation);
        if ( targetTbar )
        {
          IPoint pt = IWindow::mapPoint(IWindow::pointerPosition(),
                                        IWindow::desktopWindow()->handle(),
                                        targetTbar->handle());

          if ( pt.x() < (targetTbar->size().width()/2) )
          {
            toolBar->postEvent(IC_UM_TOOLBAR, IC_UM_TBAR_BEFORE,
                               (unsigned long) (void*) targetTbar);
          }
          else
          {
            toolBar->postEvent(IC_UM_TOOLBAR, IC_UM_TBAR_AFTER,
                               targetTbar->handle().asUnsigned() );
          }
        }
        else
          toolBar->postEvent(IC_UM_TOOLBAR, IC_UM_TBAR_AFTER, 0);
        return true;
      }

      // Check for a frame extension being dragged off the frame
      if ( fToolBarData->currentLocation != IToolBar::floating )
      {
         toolBar->setFloatingPosition(IWindow::mapPoint(newPos,
                                      IWindow::desktopWindow()->handle(),
                                      fToolBarData->owningFrame->handle()));
         toolBar->setLocation(IToolBar::floating);
         return true;
      }

      // Normal processing (move of an existing floating toolbar)
      if ( newPos != startPos )
      {
        event.window()->moveTo(newPos);
      }
      event.window()->setFocus();
    }

    targetTbar = 0;
    targetCnr = 0;
    return true;
  }

#ifdef IC_WIN
  // These two messages show up before we actually move the tool bar and they
  // cause us to wipe out the targetTbar value. Eat them!
  if ( event.eventId() == WM_WINDOWPOSCHANGING ||
       event.eventId() == WM_WINDOWPOSCHANGED )
    return false;
#endif

  targetTbar = 0;
  targetCnr = 0;

  // This is not a move event, so do default processing
  return false;
}

/*------------------------------------------------------------------------------
| IToolBarPaintHandler::dispatchHandlerEvent                                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBarPaintHandler::dispatchHandlerEvent ( IEvent& event )
{
  IMODTRACE_DEVELOP("TBarPaintHdr::dispatchHandlerEvent");

#ifdef IC_PMWIN
  // To solve a BIDI problem, we are going to also attach this handler to
  // the owning frame.  This means that we will look for the BIDI changed
  // message and then return false for all other frame events.
  if ( event.window()->isFrameWindow() )
  {
     if (( event.eventId() == IC_UM_TOOLBAR ) &&
         ( event.parameter1().asUnsignedLong() == IC_UM_TBAR_BIDI_CHANGE ))
     {
        // The frame BIDI attributes changed, so we need to update all of
        // the floating (or hidden) toolbars
        IBidiSettings bidi(*(event.window()));
        IToolBar::FrameCursor cursor( (IFrameWindow*) event.window() );
        for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
        {
          IToolBar* toolBar = IToolBar::toolBarAt(cursor);
          if ( toolBar )
          {
             if ( toolBar->location() == IToolBar::floating )
             {
               bidi.apply(*toolBar);
               bidi.apply(*(toolBar->floatingFrame()), false );
             }
             if ( toolBar->location() == IToolBar::hidden )
             {
               bidi.apply(*toolBar);
             }
          }
        }
        return true;
     }
     return false;
  }
#endif

  ITRACE_DEVELOP(event.window()->rect().asString());

  // Check for events
  switch (event.eventId())
  {
    case WM_BEGINDRAG:
    {
      IToolBar* toolBar = (IToolBar*)(event.window());
      if ( toolBar->fToolBarData->floatingFrame )
      {
        // The toolbar is already floating, so just track the frame window
        toolBar->fToolBarData->floatingFrame->sendEvent(WM_TRACKFRAME,TF_MOVE);
      }
      else
      {
        // The toolbar is in a frame extension, so create a dummy frame that
        // can be used for tracking.  Note that if the toolbar does not allow
        // drag/drop then nothing should happen
        if ( toolBar->allowsDragDrop() )
        {
          IFrameWindow trackFrame ( IC_DEFAULT_FRAME_ID,
                                    IWindow::desktopWindow(),
                                    toolBar,
                                    IRectangle(IWindow::mapPoint(IPoint(0,0),
                                               toolBar->handle(),
                                               IWindow::desktopWindow()->handle()),
                                               toolBar->size()),
                                    IFrameWindow::border );
          toolBar->fToolBarData->trackHandler->handleEventsFor(&trackFrame);
          trackFrame.sendEvent(WM_TRACKFRAME,TF_MOVE);
          toolBar->fToolBarData->trackHandler->stopHandlingEventsFor(&trackFrame);
        }
      }
    }
    return true;

    case WM_PAINT:
    {
      // Call default painting
      Inherited::dispatchHandlerEvent(event);

      IToolBar* toolBar = (IToolBar*)(event.window());
      ISize     size    = toolBar->size();

      IGraphicContext gc(toolBar->handle());
      IGraphicBundle  bundle;

      // Draw a border around the tool bar.  The default is a 3d border.
      // We make it a simple box, however, if one of the lines matches
      // the toolbar background color.  This is the case with the NT 3.51
      // default color scheme.  Otherwise, it looks partially painted.
      bundle.setPenColor( IColor(IColor::buttonLight) );
      if ( toolBar->backgroundColor() == bundle.penColor() )
         bundle.setPenColor( IColor(IColor::buttonDark) );
      bundle.setPenWidth( 1 );
      IPointArray anArray(3);
#ifdef IC_ORIGINLOWERLEFT
      anArray[0] = IPoint(1,1);
      anArray[1] = IPoint(1,size.height()-2);
      anArray[2] = IPoint(size.width()-1,size.height()-2);
#endif
#ifdef IC_ORIGINUPPERLEFT
      anArray[0] = IPoint(1,size.height()-1);
      anArray[1] = IPoint(1,1);
      anArray[2] = IPoint(size.width()-1, 1);
#endif
      IGPolyline polyline(anArray);
      polyline.setGraphicBundle(bundle);
      polyline.drawOn(gc);
      bundle.setPenColor( IColor(IColor::buttonDark) );
      if ( toolBar->backgroundColor() == bundle.penColor() )
         bundle.setPenColor( IColor(IColor::buttonLight) );
#ifdef IC_ORIGINLOWERLEFT
      anArray[1] = IPoint(size.width()-1,1);
#endif
#ifdef IC_ORIGINUPPERLEFT
      anArray[1] = IPoint(size.width()-1,size.height()-1);
#endif
      IGPolyline polyline2(anArray);
      polyline2.setGraphicBundle(bundle);
      polyline2.drawOn(gc);

    }
    return true;

    case IC_UM_TOOLBAR:
    {
      IToolBar* toolbar = (IToolBar*) event.window();
      switch ( event.parameter1().asUnsignedLong() )
      {
        case IC_UM_TBAR_HIDDEN:
          toolbar->setLocation(IToolBar::hidden);
          break;
        case IC_UM_TBAR_SET_LOCATION:
          toolbar->fToolBarData->currentLocation =
                  (IToolBar::Location) event.parameter2().asUnsignedLong();
          break;
        case IC_UM_TBAR_MOVE:
          toolbar->setLocation(
                  (IToolBar::Location) event.parameter2().asUnsignedLong());
          break;
        case IC_UM_TBAR_BEFORE:
          {
            IToolBar* targetTBar = (IToolBar*) event.parameter2().asUnsignedLong();
            toolbar->positionOnSiblings();
            toolbar->positionBehindSibling(targetTBar->handle());
            targetTBar->positionBehindSibling(toolbar->handle());
            if (targetTBar->isGroup())
            {
              toolbar->enableGroup();
              targetTBar->disableGroup();
            }
            else
            {
              toolbar->disableGroup();
            }
            toolbar->toolBarContainer()->setLayoutDistorted
                    (IWindow::layoutChanged | IWindow::immediateUpdate,0);
          }
          break;
        case IC_UM_TBAR_AFTER:
          if ( event.parameter2().asUnsignedLong() == 0 )
          {
            toolbar->enableGroup();
            toolbar->positionBehindSiblings();
            toolbar->toolBarContainer()->setLayoutDistorted
                    (IWindow::layoutChanged | IWindow::immediateUpdate,0);
          }
          else
          {
            toolbar->disableGroup();
            toolbar->positionOnSiblings();
            toolbar->positionBehindSibling(
                    (HWND)event.parameter2());
            toolbar->toolBarContainer()->setLayoutDistorted
                    (IWindow::layoutChanged | IWindow::immediateUpdate,0);
          }
          break;
        case IC_UM_TBAR_TARGET_ENTER:
          event.setResult( true );
          return true;
      }
    }
    break;

#ifdef IC_WIN
    case WM_LBUTTONDOWN:
    {
      IToolBar* toolbar = (IToolBar*) event.window();
      IMouseClickEvent mevt( event );

      // We need to use ChildWindowFromPoint instead of WindowFromPoint
      // which IMouseEvent::windowUnderPointer uses to find disabled window
      POINT pt = { (short)mevt.parameter2().number1(),
                   (short)mevt.parameter2().number2() };
      IWindowHandle childHwnd (ChildWindowFromPoint( mevt.handle(), pt));

      if ( ( mevt.mouseButton() == IMouseClickEvent::button1 ) &&
           (childHwnd) && !(ISTYLEOF( childHwnd ) & WS_DISABLED) )
      {
        // May need checking to see if this is a button or the
        // toolbar.
        // In Windows 95, the system apparently captures the pointer itself.
        // during the down event and releases it in the up event.  If we
        // mess with this incorrectly it causes no WM_COMMANDs for the
        // button clicks to be sent.
        IWindowHandle captureHandle = GetCapture( );
        if (( captureHandle != NULL ) && (captureHandle != event.handle()) )
        {
          // This means that someone else has set a capture.
          // We don't want to override it.
//          SetCapture( captureHandle );
          ITRACE_DEVELOP(IString("Pointer already captured hwnd=") +
                         IString(captureHandle.asUnsigned()).d2x() +
                         IString(" event.handle()=")+
                         IString(event.handle().asUnsigned()).d2x());
        }
        else
        {
          if (captureHandle == NULL)
             {
             ITRACE_DEVELOP(IString("capturing pointer hwnd=") +
                            IString(event.handle().asUnsigned()).d2x());
             SetCapture( event.handle() );
             }
          toolbar->fToolBarData->bCheckingDragStart = True;
          toolbar->fToolBarData->pointLButtonDown =
              mevt.mousePosition();
          toolbar->fToolBarData->parameter1 =
              event.parameter1().asUnsignedLong();
          toolbar->fToolBarData->parameter2 =
              event.parameter2().asUnsignedLong();
        }
      }
    }
    break;

    case WM_LBUTTONUP:
    {
      IToolBar* toolbar = (IToolBar*) event.window();
      ITRACE_DEVELOP("Got a button up event");
      IMouseClickEvent mevt( event );
      if ( mevt.mouseButton() == IMouseClickEvent::button1 )
      {
        if ( toolbar->fToolBarData->bCheckingDragStart )
        {
          if (!( IPlatform::isWin95() || IPlatform::isNTNewShell() ))
          {
            // Win95 appears to release the capture when processing buttonup
            // If we release it we get no WM_COMMAND for button clicks.
            ITRACE_DEVELOP(IString("Releasing capture GetCapture=") +
                           IString((unsigned long)GetCapture()).d2x()  );
            SetCapture( NULL );
          }
          else
          {
            // On 95, if captured by us, and not already released by WM_MOUSEMOVE
            // processing, (for example user clicks and releases button, without moving
            // mouse) release it here.
            IWindowHandle captureHandle = GetCapture( );    //asm
            if (( captureHandle != NULL ) && (captureHandle == event.handle()) )
            {
              ITRACE_DEVELOP(IString("asm Releasing capture GetCapture=") +
                             IString((unsigned long)GetCapture()).d2x()  );
              SetCapture( NULL );
            }
          }
          toolbar->fToolBarData->bCheckingDragStart = false;
        }
        toolbar->fToolBarData->pointLButtonDown = IPoint(0,0);
      }
      break;
    }
    case WM_MOUSEMOVE:
    {
      IToolBar* toolbar = (IToolBar*) event.window();
      if ( toolbar->fToolBarData->bCheckingDragStart)
      {
        IMouseClickEvent mevt( event );
        IPoint mPoint( toolbar->fToolBarData->pointLButtonDown );
        if ( abs(mevt.mousePosition().x() - mPoint.x()) >
                 2 /*DD_DEFDRAGMINDIST*/ ||
             abs(mevt.mousePosition().y() - mPoint.y()) >
                 2 /*DD_DEFDRAGMINDIST*/ )
        {
          SetCapture( NULL );
          toolbar->fToolBarData->bCheckingDragStart = false;
          // Use the original point that started the drag, not the point
          // associated with the move.
          toolbar->sendEvent( WM_BEGINDRAG,
                              IEventParameter1( mPoint.x(), mPoint.y()),
                              IEventParameter2( 0 ));
        }
      }
    }
    break;
#ifdef IC_TRACE_DEVELOP
    // Windows 95 only
    case WM_CAPTURECHANGED:
    {
       ITRACE_DEVELOP(IString("WM_CAPTURECHANGED new capture=") +
                      IString(event.parameter2().asUnsignedLong()).d2x() +
                      IString(" event.handle()=")+
                      IString(event.handle().asUnsigned()).d2x());
    }
    break;
#endif
#endif

  }

  return false;
}

/*------------------------------------------------------------------------------
| IToolBar::setLocation                                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::setLocation( IToolBar::Location location )
{
  IMODTRACE_DEVELOP("IToolBar::setLocation");

  // Before we monkey with the toolbar parent and owner, set the
  // focus to the owning frame.  We need to do this so that we do
  // not cause focus loop problems later on.
  if ( IISCHILD ( IQUERYFOCUS ( IWindow::desktopWindow()->handle() ), handle() ))
  {
    fToolBarData->owningFrame->setFocus();
  }

  if (( location != fToolBarData->currentLocation ) ||
      ( fToolBarData->isInitialized == false ))
  {
    // Change the parent and owner to frame
    setParent(objectWindow());
    // In Windows setOwner does nothing for top level windows.  That is
    // not a problem here since the toolbar is a child window.
    setOwner(fToolBarData->owningFrame);
    setId(fToolBarData->windowId);

    // If there is a floating frame, delete it
    if ( fToolBarData->floatingFrame )
    {
#ifdef IC_WIN
      fToolBarData->owningFrame->setFocus();
#endif
      // Save the old floating position first
      fToolBarData->floatingPosition = floatingPosition();
      delete fToolBarData->floatingHandler;
      delete fToolBarData->floatingFrame;
      fToolBarData->floatingFrame = 0;
      fToolBarData->floatingHandler = 0;
    }

    // If there is a container, check if we need to delete it
    if ( fToolBarData->toolBarContainer )
    {
      // Check if the container still has a child window
      Boolean child = false;
      {
        IWindow::ChildCursor cursor(*(fToolBarData->toolBarContainer));
        cursor.setToFirst();
        if ( cursor.isValid() )
          child = true;
      }
      // Delete the container if no child windows
      if ( child == false )
      {
        // Before removing the extension, see if we will create another
        // one and if so, disable update to prevent flashing
        if (( location == IToolBar::aboveClient )  ||
            ( location == IToolBar::belowClient )  ||
            ( location == IToolBar::leftOfClient ) ||
            ( location == IToolBar::rightOfClient ))
        {
#ifdef IC_PM
          fToolBarData->owningFrame->disableUpdate();
#endif
          fToolBarData->owningFrame->removeExtension
                       (fToolBarData->toolBarContainer);
#ifdef IC_PM
          fToolBarData->owningFrame->enableUpdate();
#endif
        }
        else
          fToolBarData->owningFrame->removeExtension
                       (fToolBarData->toolBarContainer);

        delete fToolBarData->toolBarContainer;
      }
      else
      {
        // Call layout on the container (since we did not delete it)
        fToolBarData->toolBarContainer->setLayoutDistorted
                                       (IWindow::layoutChanged, 0);
      }
      fToolBarData->toolBarContainer = 0;
    }

    // Show toolbar and reset layout for new location
    fToolBarData->currentLocation = location;
    setLayoutDistorted(IWindow::layoutChanged, 0);

    // Be sure that the toolbar is not collapsed
    if ( !fToolBarData->isExpanded )
      fToolBarData->isExpanded = true;

    // Handle floating toolbar
    if ( location == floating )
    {
      // Get a new floating frame
      fToolBarData->floatingFrame = createFloatingFrame
                                   ( fToolBarData->owningFrame );

      // Set the state of the floating toolbar
      // expand/collapse button.
      fToolBarData->floatingFrame->sendEvent( IC_UM_TOOLBAR,
                                             IC_UM_TBAR_SETEXPAND_BTN,
                                             isExpanded() );

      // Add handlers to frame
      fToolBarData->floatingHandler = new IToolBarFloatingHandler;
      fToolBarData->floatingHandler->handleEventsFor(fToolBarData->floatingFrame);
      fToolBarData->trackHandler->handleEventsFor(fToolBarData->floatingFrame);

      // Make the toolbar the client area of the frame
      setParent(fToolBarData->floatingFrame);
      // In Windows setOwner does nothing for top level windows.  That is
      // not a problem here since the toolbar is a child window.
      setOwner(fToolBarData->floatingFrame);
      fToolBarData->floatingFrame->setClient(this);

      // Set the frame title text
      fToolBarData->floatingFrame->setTitleText( fToolBarData->toolBarName );

      // Set the frame position and size
      IPoint newPos = mapPoint(fToolBarData->floatingPosition,
                               fToolBarData->owningFrame->handle(),
                               desktopWindow()->handle());
      newPos.setX(newPos.x()+1);
      newPos.setY(newPos.y()+1);

      // Check if collapsed
      ISize minSize = calcMinimumSize();
      if (!isExpanded())
      {
        minSize = Inherited::calcMinimumSize();
        minSize.setHeight(0);
      }
      fToolBarData->floatingFrame->moveSizeToClient(IRectangle(newPos,minSize));

      // Show the new frame
      fToolBarData->floatingFrame->show();
#ifdef IC_WIN
      if (isExpanded())
      {
        // We need to show maximized
        fToolBarData->floatingFrame->maximize();
      }
#endif

      // Put the floating frame on top of sibling windows if the owning frame
      // has the focus (otherwise let the z-order be the default)
      if ( fToolBarData->owningFrame->hasFocus() )
      {
        fToolBarData->floatingFrame->positionOnSiblings();
      }

    }

    // Handle frame extension toolbar
    if (( location != floating ) && ( location != hidden ))
    {
      IFrameWindow::Location frameLocation;
      switch ( location )
      {
        case aboveClient:
          frameLocation = IFrameWindow::aboveClient;
          setDeckOrientation(ISetCanvas::horizontal);
          setMargin(ISize(fToolBarData->margin.height(),
                          fToolBarData->margin.width()));
          break;
        case belowClient:
          frameLocation = IFrameWindow::belowClient;
          setDeckOrientation(ISetCanvas::horizontal);
          setMargin(ISize(fToolBarData->margin.height(),
                          fToolBarData->margin.width()));
          break;
        case leftOfClient:
          frameLocation = IFrameWindow::leftOfClient;
          setDeckOrientation(ISetCanvas::vertical);
          setMargin(fToolBarData->margin);
          break;
        case rightOfClient:
          frameLocation = IFrameWindow::rightOfClient;
          setDeckOrientation(ISetCanvas::vertical);
          setMargin(fToolBarData->margin);
          break;
      }

      // Check for misfits
      filter();

      // Get existing container or create a new one
      fToolBarData->toolBarContainer = IToolBarContainer::frameToolBarContainer
                                      (fToolBarData->owningFrame, frameLocation);

      if (!fToolBarData->toolBarContainer)
      {
        fToolBarData->toolBarContainer = createToolBarContainer
                                       (fToolBarData->owningFrame);
      }
      // Add the toolbar to the container
      setParent(fToolBarData->toolBarContainer);
      fToolBarData->toolBarContainer->setLayoutDistorted
                   (IWindow::layoutChanged | IWindow::immediateUpdate, 0);
      positionBehindSiblings();
    }

  }

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::location                                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::Location IToolBar::location ( ) const
{
  return fToolBarData->currentLocation;
}

/*------------------------------------------------------------------------------
| IToolBar::IToolBar                                                           |
|                                                                              |
| Constructor to create a tool bar using a tool bar area.  Here, we create     |
| the tool bar as a child of the object window and later re-parent it          |
| to the appropriate container or floating frame.                              |
|                                                                              |
| Note:  We have a potential portability problem here.  If the tool bar        |
|        is a floating tool bar, its parent is the frame window containing     |
|        it.  If the tool bar is a frame extension, its parent is the          |
|        tool bar container in the frame extension area.  To accomplish        |
|        this, we need to re-parent the tool bar.  We also need to re-parent   |
|        the tool bar to move it from a floating tool bar to a frame           |
|        extension and vice versa.                                             |
------------------------------------------------------------------------------*/
IToolBar::IToolBar ( unsigned long id,
                     IFrameWindow* owner,
                     Location      frameLocation,
                     Boolean       groupWithPreceding,
                     const Style&  style)
         :ISetCanvas(id,IWindow::objectWindow(),owner),
          fToolBarData(new IToolBarData(owner))
{
  IMODTRACE_DEVELOP("TBar::TBar(location)");

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  // Copy correct bidi settings (from the owning frame window)
  if (IBidiSettings::isBidiSupported() )
  {
    IBidiSettings ownerSettings(*owner);
    ownerSettings.apply(*this);
  }
#endif

  // Create a track handler for this toolbar
  fToolBarData->trackHandler = new IToolBarTrackHandler ( this, fToolBarData );

  // Save default values
  setPad(ISize(defaultPad(), defaultPad()));
  setGroupPad(defaultGroupPad());
  fToolBarData->margin = defaultMargin();
  fToolBarData->misfitWidth = defaultMisfitWidth();
  fToolBarData->windowId = id;

  // Save the style & location
  unsigned long ulStyle = extendedStyle() & ICNV_EXTTBAR ;
  unsigned long ulOldStyle = ulStyle;

  // Convert redundant styles
  if (((ulStyle & buttonTextVisible.asExtendedUnsignedLong()) &&
      (ulStyle & buttonBitmapVisible.asExtendedUnsignedLong())) ||
      (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong()))
  {
    ulStyle &= ~(buttonTextVisible.asExtendedUnsignedLong() |
                 buttonBitmapVisible.asExtendedUnsignedLong());
    ulStyle |= buttonBitmapAndTextVisible.asExtendedUnsignedLong();
  }

  // Reject mutually exclusive styles
  if (((ulStyle & buttonTextVisible.asExtendedUnsignedLong()) &&
       (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong())) ||
      ((ulStyle & buttonBitmapVisible.asExtendedUnsignedLong())  &&
       (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong())))
  {
     ITHROWLIBRARYERROR(IC_INVALIDSTYLE,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
  }

  // Update if redundant style was eliminated
  if (ulStyle != ulOldStyle)
    setExtendedStyle( ulStyle );

  fToolBarData->currentLocation = frameLocation;

  if(!groupWithPreceding)
    enableGroup();
  else
    enableGroup(false);

  setMargin(defaultMargin());
}

/*------------------------------------------------------------------------------
| IToolBar::IToolBar                                                           |
|                                                                              |
| Constructor to create a tool bar on a tool bar area.                         |
------------------------------------------------------------------------------*/
IToolBar::IToolBar ( unsigned long id,
                     IToolBar*     precedingToolBar,
                     Boolean       groupWithPreceding,
                     const Style&  style)
         :ISetCanvas(id, IWindow::objectWindow(),
                    precedingToolBar->fToolBarData->owningFrame),
          fToolBarData(new IToolBarData
                     (precedingToolBar->fToolBarData->owningFrame))
{
  IMODTRACE_DEVELOP("TBar::TBar(Tbar)");

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_PMWIN
  // Copy correct bidi settings (from the owning frame window)
  if ( IBidiSettings::isBidiSupported() )
  {
    IBidiSettings ownerSettings(*(precedingToolBar->fToolBarData->owningFrame));
    ownerSettings.apply(*this);
  }
#endif

  // Create a track handler for this toolbar
  fToolBarData->trackHandler = new IToolBarTrackHandler ( this, fToolBarData );

  // Save default values
  setPad(ISize(defaultPad(), defaultPad()));
  setGroupPad(defaultGroupPad());
  fToolBarData->margin = defaultMargin();
  fToolBarData->misfitWidth = defaultMisfitWidth();
  fToolBarData->windowId = id;

  // Save the style & location
  unsigned long ulStyle = extendedStyle() & ICNV_EXTTBAR;
  unsigned long ulOldStyle = ulStyle;

  // Convert redundant styles
  if (((ulStyle & buttonTextVisible.asExtendedUnsignedLong()) &&
      (ulStyle & buttonBitmapVisible.asExtendedUnsignedLong())) ||
      (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong()))
  {
    ulStyle &= ~(buttonTextVisible.asExtendedUnsignedLong() |
                 buttonBitmapVisible.asExtendedUnsignedLong());
    ulStyle |= buttonBitmapAndTextVisible.asExtendedUnsignedLong();
  }

  // Reject mutually exclusive styles
  if (((ulStyle & buttonTextVisible.asExtendedUnsignedLong()) &&
       (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong())) ||
      ((ulStyle & buttonBitmapVisible.asExtendedUnsignedLong())  &&
       (ulStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong())))
  {
     ITHROWLIBRARYERROR(IC_INVALIDSTYLE,
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
  }

  // Update if redundant style was eliminated
  if (ulStyle != ulOldStyle)
    setExtendedStyle( ulStyle );

  fToolBarData->currentLocation = precedingToolBar->fToolBarData->currentLocation;
  fToolBarData->precedingToolBar = precedingToolBar;

  if(!groupWithPreceding)
    enableGroup();
  else
    enableGroup(false);

  setMargin(defaultMargin());
}

/*------------------------------------------------------------------------------
| IToolBar::~IToolBar                                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::~IToolBar ( )
{
  IMODTRACE_DEVELOP("TBar::~TBar");

  // Remove ourselves from the frames list of tool bars and delete
  // the list if we are the last one in it.

  Boolean sharedLocation = false;
  Boolean removed = false;

  IToolBarList* tbarList = fToolBarData->owningFrame->toolBarList();
  if (tbarList != 0)
  {
    IToolBarList::Cursor cursor(*tbarList);
    cursor.setToFirst();
    while (cursor.isValid() && !removed)
    {
      IToolBar* currentToolBar = tbarList->elementAt(cursor);
      if (this == currentToolBar)
      {
         tbarList->removeAt(cursor);
         removed = true;
      }
      else
        cursor.setToNext();
    }

    // Delete the list if it is now empty
    if(tbarList->numberOfElements() == 0)
    {
      delete tbarList;
      fToolBarData->owningFrame->setToolBarList(0);
      tbarList = 0;
    }
  }

  // Check if any other toolbars have the same location (if so then
  // they must also be sharing the same toolbar container)

  if (tbarList != 0)
  {
    IToolBarList::Cursor cursor(*tbarList);
    for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    {
      IToolBar* currentToolBar = tbarList->elementAt(cursor);
      if ( currentToolBar->location() == location() )
        sharedLocation = true;
    }
  }

  // Delete tracking handler (if needed)
  if ( fToolBarData->trackHandler )
  {
    delete fToolBarData->trackHandler;
  }

  // Delete the floating frame (if needed)
  if ( fToolBarData->floatingFrame )
  {
    delete fToolBarData->floatingHandler;
    delete fToolBarData->floatingFrame;
  }

  // Delete the toolbar container (if needed)
  if ( fToolBarData->toolBarContainer )
  {
    // Delete the container if no other toolbars share the same location
    // Note: The toolbar container frame extension should only be removed
    //       if the frame is a valid PM window (this is needed to prevent
    //       the removeExtension from throwing an exception)
    if ( sharedLocation == false )
    {
      if ( fToolBarData->owningFrame->isValid() )
        fToolBarData->owningFrame->removeExtension(fToolBarData->toolBarContainer);
      delete fToolBarData->toolBarContainer;
    }
    else
    {
      // Call layout on the container (since we did not delete it)
      if ( fToolBarData->toolBarContainer->isValid() )
        fToolBarData->toolBarContainer->setLayoutDistorted
                                       (IWindow::layoutChanged, 0);
    }
  }

  // Delete any IToolBarObjects still in the list
  {
    IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    {
      IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
      delete winObj;
    }
    delete fToolBarData->objectList;
  }

  // Delete our local data
  delete fToolBarData;
}

/*------------------------------------------------------------------------------
| IToolBar::initialize                                                         |
|                                                                              |
| Setup the initial state of the tool bar.  This includes:                     |
| 1) creating the frames tool bar list if necessary                            |
| 2) storing the tool bar in the frames list of tool bars                      |
| 3) setting the tool bar location                                             |
| 4) establishing the initial characteristics of the tool bar.                 |
| 5) enabling any handlers we are using                                        |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::initialize (  )
{
  IMODTRACE_DEVELOP("TBar::initialize");

  // set the default color
  setBackgroundColor(IGUIColor(IGUIColor::menuBgnd));

  // Set the location of the tool bar
  setLocation(fToolBarData->currentLocation);

  // Setup Orientation based on the parent container
  setAlignment(ISetCanvas::centerCenter);

  // Add our paint handler to paint a different border
  fToolBarData->paintHandler.handleEventsFor(this);

#ifdef IC_PMWIN
  // Add the paint handler to the frame as well so that we can pick up
  // any BIDI changes
  if ( IBidiSettings::isBidiSupported() )
    fToolBarData->paintHandler.handleEventsFor((ICanvas*)fToolBarData->owningFrame);
#endif

  // Add our drag drop handler, unless D&D has been disabled
  if (allowsDragDrop())
    {
    IDMHandler::enableDropOn(this);
    }
  // Store the tool bar in the frames list
  IToolBarList* tbarList = fToolBarData->owningFrame->toolBarList();

  // Allocate the tool bar list if necessary and store it back
  // into the frame
  if (tbarList == 0)
  {
    tbarList = new IToolBarList;
    fToolBarData->owningFrame->setToolBarList(tbarList);
  }

  // Add ourselves to the list
  tbarList->addAsLast(this);

  fToolBarData->isInitialized = true;
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::toolBarContainer                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarContainer* IToolBar::toolBarContainer( ) const
{
  IMODTRACE_DEVELOP("TBar::toolBarContainer");
  if(!fToolBarData->isInitialized)
  {
    ((IToolBar*)this)->initialize();
  }
  return fToolBarData->toolBarContainer;
}

/*------------------------------------------------------------------------------
| IToolBar::floatingFrame                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow* IToolBar::floatingFrame( )
{
  IMODTRACE_DEVELOP("TBar::floatingFrame");
  if(!fToolBarData->isInitialized)
  {
    initialize();
  }
  return fToolBarData->floatingFrame;
}

/*------------------------------------------------------------------------------
| IToolBar::createToolBarContainer                                             |
|                                                                              |
| Create a container to be used to house tool bars in one of the frame         |
| extensions around the client.  The current location of the toolbar is used   |
| when creating the container.                                                 |
------------------------------------------------------------------------------*/
IToolBarContainer* IToolBar::createToolBarContainer ( IFrameWindow* frame )
{
  IMODTRACE_DEVELOP("TBar::createToolBarContainer");

  IToolBarContainer* cnr = 0;

  switch ( fToolBarData->currentLocation )
  {
    case aboveClient:
      cnr = new IToolBarContainer((unsigned long) fToolBarData->currentLocation,
                                  frame,
                                  IFrameWindow::aboveClient);
      break;
    case belowClient:
      cnr = new IToolBarContainer((unsigned long) fToolBarData->currentLocation,
                                  frame,
                                  IFrameWindow::belowClient);
      break;
    case leftOfClient:
      cnr = new IToolBarContainer((unsigned long) fToolBarData->currentLocation,
                                  frame,
                                  IFrameWindow::leftOfClient);
      break;
    case rightOfClient:
      cnr = new IToolBarContainer((unsigned long) fToolBarData->currentLocation,
                                  frame,
                                  IFrameWindow::rightOfClient);
      break;
  }

#ifdef IC_PMWIN
  if ( cnr && IBidiSettings::isBidiSupported() )
  {
     IBidiSettings bidi(*(fToolBarData->owningFrame));
     bidi.apply(*cnr);
  }
#endif

  return cnr;
}

/*------------------------------------------------------------------------------
| IToolBar::createFloatingFrame                                                |
|                                                                              |
| Create the frame window for a floating tool bar.  This includes establishing |
| its initial style and removing any unnecessary items from the system menu.   |
------------------------------------------------------------------------------*/
IToolBarFrameWindow* IToolBar::createFloatingFrame ( IFrameWindow* owner )
{
  IMODTRACE_DEVELOP("TBar::createFloatingFrame");

  // Floating tool bars need a frame window.
  IToolBarFrameWindow* floater = new IToolBarFrameWindow( IC_DEFAULT_FRAME_ID,
                                                          owner );

#ifdef IC_PMWIN
  if ( floater && IBidiSettings::isBidiSupported() )
  {
     IBidiSettings bidi(*(fToolBarData->owningFrame));
     bidi.apply(*floater);
  }
#endif

  return floater;
}

/*------------------------------------------------------------------------------
| IToolBar::addControl                                                         |
|                                                                              |
| Private function to handle common code when adding a new control to the      |
| tool bar.                                                                    |
------------------------------------------------------------------------------*/
void IToolBar::addControl ( IControl* control,
                            Boolean   newGroup )
{

  if ( newGroup )
  {
    // This is the start of a new group, so just enable group style
    control->enableGroup();
  }
  else
  {
    // Turn of group style (we will turn it back on if needed)
    if (control->isGroup())
      control->disableGroup();

    // Find the next control
    IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));
    Boolean found = false;
    Boolean first = true;

    for ( cursor.setToFirst();
        ( cursor.isValid() && !found );
          cursor.setToNext())
    {
      IToolBarObject* obj = fToolBarData->objectList->elementAt(cursor);
      if ( obj->window() == control )
        found = true;
      else
        first = false;
    }

    if ( found && cursor.isValid())
    {
      // If the next control has group style, move it to the new control
      IToolBarObject* nextObj = fToolBarData->objectList->elementAt(cursor);
      IControl* nextControl = (IControl*) nextObj->window();
      if ( nextControl->isGroup() )
      {
         control->enableGroup();
         nextControl->disableGroup();
      }
    }
    else
    {
      // There is no "next" control so check if this is the first control
      if (first)
      {
        control->enableGroup();
      }
    }
  }

  // Get the toolbar style
  unsigned long ulExtStyle = extendedStyle();

  // Ensure the correct button state
  unsigned long message = IC_UM_BMP_VIEW;
  if (ulExtStyle & buttonTextVisible.asExtendedUnsignedLong())
     message = IC_UM_TXT_VIEW;
  else if (ulExtStyle & buttonBitmapAndTextVisible.asExtendedUnsignedLong())
     message = IC_UM_BMP_TXT_VIEW;
  control->sendEvent(message);

  /****************************************************************/
  /* Drag/Drop Enablement                                         */
  /*                                                              */
  /* Note that we are sending a user defined message to allow the */
  /* window being added to enable drag/drop support.  When the    */
  /* compiler supports RTTI, we can directly call the drag/drop   */
  /* enablement function from here, and let the compiler take     */
  /* care of the rest.                                            */
  /*                                                              */
  /* Do not dispatch this event if D&D have been disabled for the */
  /* tool bar.                                                    */
  /****************************************************************/
  if (allowsDragDrop())
    control->sendEvent( IC_UM_TOOLBAR, IC_UM_TBAR_WIN_DRAG_ENABLE );

  // Check for misfits
  filter();

  // Force layout
  setLayoutDistorted(IWindow::layoutChanged | IWindow::minimumSizeChanged |
                     IWindow::immediateUpdate, 0);
}

/*------------------------------------------------------------------------------
| IToolBar::addAsFirst                                                         |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::addAsFirst( IWindow* window,
                                Boolean  startNewGroup )
{
  IMODTRACE_DEVELOP("TBar::addAsFirst");

  // Check if we still need to initialize the tool bar
  if(!fToolBarData->isInitialized)
  {
    // Turn layout off during initialization
    fToolBarData->layoutOn = false;
    initialize();
    fToolBarData->layoutOn = true;
  }

  // Make sure the window is a child of the tool bar
  window->setParent(this);

  // Position the window
  window->positionOnSiblings ();

  // Create the object and add to set
  IToolBarObject* newObject = new IToolBarObject(window);
  fToolBarData->objectList->addAsFirst(newObject);

  // Finish adding the control
  addControl((IControl*)window,startNewGroup);

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::addAsNext                                                          |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::addAsNext( IWindow* window,
                               IWindow* referenceWindow,
                               Boolean  startNewGroup )
{
  IMODTRACE_DEVELOP("TBar::addAsNext");

  ITRACE_DEVELOP(IString("startNewGroup =")+IString(startNewGroup));

  // Check if we still need to initialize the tool bar
  if(!fToolBarData->isInitialized)
  {
    // Turn layout off during initialization
    fToolBarData->layoutOn = false;
    initialize();
    fToolBarData->layoutOn = true;
  }

  // Make sure the window is a child of the tool bar
  window->setParent(this);

  // Create the object
  IToolBarObject* newObject = new IToolBarObject(window);

  // Find the last visible window (before the add location) and add object
  IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));
  Boolean done = false;
  IWindow* lastVisible = 0;
  for (cursor.setToFirst();(cursor.isValid() && !done); cursor.setToNext())
  {
    IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
    if ( winObj->isVisible() )
      lastVisible = winObj->window();
    if ( winObj->window() == referenceWindow )
    {
      fToolBarData->objectList->addAsNext(newObject,cursor);
      done = true;
    }
  }

  if (!done)
    {
    fToolBarData->objectList->addAsFirst(newObject);
    lastVisible = 0;
    }

  // Position window
  window->positionOnSiblings();
  if ( lastVisible )
    window->positionBehindSibling ( lastVisible->handle() );

  // Finish adding the control
  addControl((IControl*)window,true);

  // Handle grouping (since addControl tries to group with the next window
  // and we need to group with the previous window)
  if (!startNewGroup)
  {
    ((IControl*)window)->disableGroup();
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::addAsPrevious                                                      |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::addAsPrevious( IWindow* window,
                                   IWindow* referenceWindow,
                                   Boolean  startNewGroup )
{
  IMODTRACE_DEVELOP("TBar::addAsPrevious");

  // Check if we still need to initialize the tool bar
  if(!fToolBarData->isInitialized)
  {
    // Turn layout off during initialization
    fToolBarData->layoutOn = false;
    initialize();
    fToolBarData->layoutOn = true;
  }

  // Make sure the window is a child of the tool bar
  window->setParent(this);

  // Create the object
  IToolBarObject* newObject = new IToolBarObject(window);

  // Plan to add the object at the beginning (if we can't find a previous)
  IToolBarObject* prevObject = 0;

  // Find the last visible window (before the add location)
  IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));
  Boolean done = false;
  IWindow* lastVisible = 0;
  for (cursor.setToFirst();(cursor.isValid() && !done); cursor.setToNext())
  {
    IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
    if ( winObj->window() == referenceWindow )
    {
      done = true;
    }
    else
    {
      prevObject = winObj;
      if ( winObj->isVisible() )
        lastVisible = winObj->window();
    }
  }

  // Add the object (after prevObject or at beginning)
  if ( done && prevObject )
  {
    cursor.setToFirst();
    while (prevObject != fToolBarData->objectList->elementAt(cursor))
      cursor.setToNext();
    fToolBarData->objectList->addAsNext(newObject,cursor);
  }
  else
  {
    fToolBarData->objectList->addAsFirst(newObject);
    lastVisible = 0;
  }

  // Position window
  window->positionOnSiblings();
  if ( lastVisible )
    window->positionBehindSibling ( lastVisible->handle() );

  // Finish adding the control
  addControl((IControl*)window,startNewGroup);

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::addAsLast                                                          |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::addAsLast( IWindow* window,
                               Boolean  startNewGroup )
{
  IMODTRACE_DEVELOP("TBar::addAsLast");

  // Check if we still need to initialize the tool bar
  if(!fToolBarData->isInitialized)
  {
    // Turn layout off during initialization
    fToolBarData->layoutOn = false;
    initialize();
    fToolBarData->layoutOn = true;
  }

  // Make sure the window is a child of the tool bar
  window->setParent(this);

  // Position the window
  window->positionBehindSiblings ();

  // Create the object and add to set
  IToolBarObject* newObject = new IToolBarObject(window);
  fToolBarData->objectList->addAsLast(newObject);

  // Finish adding the control
  addControl((IControl*)window,startNewGroup);

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::moveToFirst                                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::moveToFirst ( IWindow* window,
                                  Boolean  startNewGroup )
{
  remove(window);
  addAsFirst(window,startNewGroup);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::moveAfter                                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::moveAfter ( IWindow* window,
                                IWindow* referenceWindow,
                                Boolean  startNewGroup )
{
  IASSERTPARM( window != referenceWindow );
  remove(window);
  addAsNext(window,referenceWindow,startNewGroup);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::moveBefore                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::moveBefore ( IWindow* window,
                                 IWindow* referenceWindow,
                                 Boolean  startNewGroup )
{
  IASSERTPARM( window != referenceWindow );
  remove(window);
  addAsPrevious(window,referenceWindow,startNewGroup);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::moveToLast                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::moveToLast ( IWindow* window,
                                 Boolean  startNewGroup )
{
  remove(window);
  addAsLast(window,startNewGroup);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::remove                                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::remove ( IWindow* window )
{
  Boolean removed = false;
  window->setParent(objectWindow());

  IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));

  // Check if we need to move the group style to the next control (if it exists)
  if (((IControl*)window)->isGroup())
  {
    Boolean found = false;
    for (cursor.setToFirst();(cursor.isValid() && !found); cursor.setToNext())
    {
      IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
      if ( winObj->window() == window )
         found = true;
    }
    if (cursor.isValid())
    {
      IToolBarObject* nextObj = fToolBarData->objectList->elementAt(cursor);
      ((IControl*)(nextObj->window()))->enableGroup();
    }
  }

  // Remove the object
  cursor.setToFirst();
  while (cursor.isValid() && !removed)
  {
    IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
    if ( winObj->window() == window )
    {
      fToolBarData->objectList->removeAt(cursor);
      removed = true;

      // Check if we need to refresh display
      if ( winObj->isVisible() )
      {
        setLayoutDistorted (IWindow::childWindowDestroyed |
                            IWindow::immediateUpdate, 0);
      }
      delete winObj;
    }
    else
      cursor.setToNext();
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::setButtonView                                                      |
|                                                                              |
| Tells all the tool bar buttons to show the view.                             |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::setButtonView ( IToolBarButton::View buttonView )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;
  unsigned long ulMask = ~(buttonTextVisible.asExtendedUnsignedLong()   |
                           buttonBitmapVisible.asExtendedUnsignedLong() |
                           buttonBitmapAndTextVisible.asExtendedUnsignedLong());

  unsigned long message = 0;

  if (buttonView == IToolBarButton::textView)
  {
     message = IC_UM_TXT_VIEW;
     ulExtStyle = (ulExtStyle & ulMask) |
                                  buttonTextVisible.asExtendedUnsignedLong();
  }
  else if (buttonView == IToolBarButton::bitmapAndTextView)
  {
     message = IC_UM_BMP_TXT_VIEW;
     ulExtStyle = (ulExtStyle & ulMask) |
                  buttonBitmapAndTextVisible.asExtendedUnsignedLong();
  }
  else if (buttonView == IToolBarButton::bitmapView)
  {
     message = IC_UM_BMP_VIEW;
     ulExtStyle = (ulExtStyle & ulMask) |
                                  buttonBitmapVisible.asExtendedUnsignedLong();
  }

  // Update if style changed
  if (ulExtStyle != ulOldExtStyle)
  {
    setExtendedStyle( ulExtStyle );

    IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    {
      IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
      winObj->window()->sendEvent(message);
    }

    filter();

    setLayoutDistorted(IWindow::childMinimumSizeChanged |
                       IWindow::immediateUpdate, 0);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::buttonView                                                         |
|                                                                              |
| Tells all the tool bar buttons to show their bitmaps.                        |
------------------------------------------------------------------------------*/
IToolBarButton::View IToolBar::buttonView ( ) const
{
  unsigned long ulExtStyle = extendedStyle();

  if (ulExtStyle & buttonBitmapVisible.asExtendedUnsignedLong())
     return IToolBarButton::bitmapView;
  else if (ulExtStyle & buttonTextVisible.asExtendedUnsignedLong())
     return IToolBarButton::textView;
  return IToolBarButton::bitmapAndTextView;
}

/*------------------------------------------------------------------------------
| IToolBar::enableMisfitFiltering                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::enableMisfitFiltering ( Boolean enable )
{
  if ( isMisfitFilteringEnabled() != enable )
  {
    // Change the extended style
    unsigned long ulExtStyle = extendedStyle();
    if (enable)
      ulExtStyle |= filterMisfits.asExtendedUnsignedLong();
    else
      ulExtStyle &= ~filterMisfits.asExtendedUnsignedLong();

    setExtendedStyle(ulExtStyle);

    // Check for misfit filtering (in or out) if vertical
    if ( deckOrientation() == vertical )
    {
      filter();
      setLayoutDistorted(IWindow::childMinimumSizeChanged |
                         IWindow::immediateUpdate, 0);
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::disableMisfitFiltering                                             |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::disableMisfitFiltering ( )
{
  enableMisfitFiltering(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::isMisfitFilteringEnabled                                           |
------------------------------------------------------------------------------*/
Boolean IToolBar::isMisfitFilteringEnabled ( ) const
{
  return( (extendedStyle() & filterMisfits.asExtendedUnsignedLong()) ? true
                                                                     : false );
}

/*------------------------------------------------------------------------------
| IToolBar::defaultStyle                                                       |
|                                                                              |
| Returns the current default style.                                           |
| Notes: Can not inline this since it is in the constructor declaration.       |
------------------------------------------------------------------------------*/
IToolBar::Style  IToolBar::defaultStyle ( )
{
  return fgCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IToolBar::setDefaultStyle                                                    |
|                                                                              |
| Sets the current default style.                                              |
------------------------------------------------------------------------------*/
void IToolBar::setDefaultStyle ( const Style& style )
{
  fgCurrentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IToolBar::convertToGUIStyle                                                  |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IToolBar::convertToGUIStyle(const IBitFlag& guiStyle,
                                          Boolean bExtOnly) const
{
  // Obtain the style from the class (ISetCanvas) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // All the styles for the toolbar are extended styles, so we do not mask.
    ulStyle |= guiStyle.asUnsignedLong();
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IToolBar::setFloatingTitle                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::setFloatingTitle ( const char* pszText )
{
  fToolBarData->toolBarName = pszText;
  if ( fToolBarData->floatingFrame )
  {
    fToolBarData->floatingFrame->setTitleText(fToolBarData->toolBarName);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::setFloatingTitle                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar &IToolBar::setFloatingTitle ( const IResourceId& residText)
{
  IString aString = residText.resourceLibrary().loadString(residText);
  return setFloatingTitle((char*)aString);
}

/*------------------------------------------------------------------------------
| IToolBar::floatingTitle                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IString  IToolBar::floatingTitle() const
{
  return fToolBarData->toolBarName;
}

/*------------------------------------------------------------------------------
| IToolBar::setFloatingPosition                                                |
|                                                                              |
| Set the Frame relative tool bar postition.                                   |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::setFloatingPosition ( const IPoint& frameRelativePosition)
{
  fToolBarData->floatingPosition = frameRelativePosition;
  if ( fToolBarData->floatingFrame )
  {
    fToolBarData->floatingFrame->moveTo(mapPoint(frameRelativePosition,
                                       fToolBarData->owningFrame->handle(),
                                       desktopWindow()->handle()));
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::floatingPosition                                                   |
|                                                                              |
| Returns the frame relative tool bar position.                                |
------------------------------------------------------------------------------*/
IPoint IToolBar::floatingPosition ( ) const
{
  // If we are floating get the current floating position
  // Otherwise return our stored value

  if ( fToolBarData->floatingFrame )
  {
    fToolBarData->floatingPosition = mapPoint
                                  ( fToolBarData->floatingFrame->position(),
                                    desktopWindow()->handle(),
                                    fToolBarData->owningFrame->handle());
  }
  return fToolBarData->floatingPosition;
}

/*------------------------------------------------------------------------------
| IToolBar::setDefaultPad                                                      |
|                                                                              |
| Sets the current default pad.                                                |
------------------------------------------------------------------------------*/
void IToolBar::setDefaultPad ( long pixelsOfPad)
{
  IToolBarData::currentDefaultPad = pixelsOfPad;
}

/*------------------------------------------------------------------------------
| IToolBar::setDefaultGroupPad                                                 |
|                                                                              |
| Sets the current default group pad.                                          |
------------------------------------------------------------------------------*/
void IToolBar::setDefaultGroupPad ( unsigned long pixelsOfGroupPad)
{
  IToolBarData::currentDefaultGroupPad = pixelsOfGroupPad;
}

/*------------------------------------------------------------------------------
| IToolBar::setDefaultMisfitWidth                                              |
|                                                                              |
| Sets the default misfit width.                                               |
------------------------------------------------------------------------------*/
void IToolBar::setDefaultMisfitWidth ( unsigned long  maximumWidth )
{
  IToolBarData::currentDefaultMisfitWidth = maximumWidth;
}

/*------------------------------------------------------------------------------
| IToolBar::setDefaultMargin                                                   |
|                                                                              |
| Sets the current default margin.                                             |
------------------------------------------------------------------------------*/
void IToolBar::setDefaultMargin ( const ISize& sizeOfMargin)
{
  IToolBarData::currentDefaultMargin = sizeOfMargin;
}

/*------------------------------------------------------------------------------
| IToolBar::defaultPad                                                         |
|                                                                              |
| Returns the current pad.                                                     |
------------------------------------------------------------------------------*/
long IToolBar::defaultPad ( )
{
  return IToolBarData::currentDefaultPad;
}

/*------------------------------------------------------------------------------
| IToolBar::defaultGroupPad                                                    |
|                                                                              |
| Returns the current group pad.                                               |
------------------------------------------------------------------------------*/
unsigned long IToolBar::defaultGroupPad ( )
{
  return IToolBarData::currentDefaultGroupPad;
}

/*------------------------------------------------------------------------------
| IToolBar::defaultMisfitWidth                                                 |
|                                                                              |
| Returns the current misfit width.                                            |
------------------------------------------------------------------------------*/
unsigned long IToolBar::defaultMisfitWidth ( )
{
  return IToolBarData::currentDefaultMisfitWidth;
}


/*------------------------------------------------------------------------------
| IToolBar::defaultMargin                                                      |
|                                                                              |
| Returns the current default margin.                                          |
------------------------------------------------------------------------------*/
ISize IToolBar::defaultMargin ( )
{
  return IToolBarData::currentDefaultMargin;
}

/*------------------------------------------------------------------------------
| IToolBar::setLayoutDistorted                                                 |
|                                                                              |
| Provide the response specific for a tool bar to various distortion           |
| notifications.  Some will cause the canvas to need to update how it has      |
| layed out its child windows.                                                 |
------------------------------------------------------------------------------*/
IToolBar&  IToolBar::setLayoutDistorted ( unsigned long layoutAttributeOn,
                                          unsigned long layoutAttributeOff )
{
  IMODTRACE_DEVELOP("ITBar::setLayoutDistorted");
  ITRACE_DEVELOP(IString("Layout Attribute is: ")+
                 IString(layoutAttributeOn).d2x());

  // Ensure that changes to the children of the tool bar cause a change
  // to the tool bar container so that it can resize its frame when floating

  if (( fToolBarData->currentLocation == floating) &&
      ( layoutAttributeOn & childMinimumSizeChanged ))
  {
    layoutAttributeOn |= IWindow::layoutChanged;
  }

  if ( fToolBarData->layoutOn || ( layoutAttributeOn != IWindow::layoutChanged ))
  {
    ITRACE_DEVELOP("Called Inherited::setLayoutDistorted");
    Inherited::setLayoutDistorted(layoutAttributeOn, layoutAttributeOff);

    // Need to adjust the size of a floating tool bar
    if (( fToolBarData->floatingFrame ) &&
        ( layoutAttributeOn & minimumSizeChanged ))
    {
      IPoint newPos(fToolBarData->floatingFrame->position());
      ISize  minimum(minimumSize());

      // Check if the toolbar is empty or has been collapsed.
      // If so make sure the floating toolbar frame retains
      // its width.
      if (minimum == ISize(0,0))
      {
        minimum.setWidth(size().width());
        minimum.setHeight(1);                   // 1 pel height
      }


      // Position the frame window such that the title bar remains
      // in the same location regardless of changes to the toolbar.

      IRectangle frameRect(
        fToolBarData->floatingFrame->frameRectFor(IRectangle( newPos, minimum )));

      if (ICoordinateSystem::applicationOrientation() ==
          ICoordinateSystem::originLowerLeft)
         {
         // need to move position to adjust for changes in the size of
         // the client area.
         IPoint topLeft(fToolBarData->floatingFrame->rect().topLeft());

         frameRect.moveTo( IPoint( topLeft.x(),
                                   topLeft.y() - frameRect.size().height() ));
         }
      else
         {
         frameRect.moveTo( newPos );
         }

      fToolBarData->floatingFrame->moveSizeTo(frameRect);
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::expand                                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::expand( Boolean expand )
{
  if ( fToolBarData->isExpanded != expand )
  {
    fToolBarData->isExpanded = expand;
    setLayoutDistorted(IWindow::layoutChanged | IWindow::minimumSizeChanged |
                       IWindow::immediateUpdate, 0);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::collapse                                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::collapse ( )
{
  expand ( false );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::isExpanded                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::isExpanded( ) const
{
  return fToolBarData->isExpanded;
}

/*------------------------------------------------------------------------------
| IToolBar::enableDragDrop                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar :: enableDragDrop ( Boolean bTurnOn )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  if (bTurnOn)
    ulExtStyle &= ~noDragDrop.asExtendedUnsignedLong();
  else
    ulExtStyle |= noDragDrop.asExtendedUnsignedLong();

  if (ulExtStyle != ulOldExtStyle)
    setExtendedStyle( ulExtStyle );

  return( *this );
}

/*------------------------------------------------------------------------------
| IToolBar::disableDragDrop                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar :: disableDragDrop ( )
{
  enableDragDrop( false );
  return( *this );
}


/*------------------------------------------------------------------------------
| IToolBar::allowsDragDrop                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar :: allowsDragDrop ( ) const
{
  return( (extendedStyle() & noDragDrop.asExtendedUnsignedLong()) ? false
                                                                  : true );
}

/*------------------------------------------------------------------------------
| IToolBar::filter                                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar& IToolBar::filter ( )
{
  IToolBarObjectList::Cursor cursor(*(fToolBarData->objectList));

  IToolBarObject* lastObj = 0;
  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
    IToolBarObject* winObj = fToolBarData->objectList->elementAt(cursor);
    if (( deckOrientation() == vertical ) &&
        ( extendedStyle() & filterMisfits.asExtendedUnsignedLong() ))
    {
      // Check if any windows need to be hidden
      if ( winObj->isVisible() &&
         ( winObj->window()->minimumSize().width() >
           fToolBarData->misfitWidth ))
      {
        winObj->window()->setParent(IWindow::objectWindow());
        winObj->setVisible(false);
      }

      // Check if any hidden windows need to be reshown
      if ( ( winObj->isVisible() == false ) &&
           ( winObj->window()->minimumSize().width() <
             fToolBarData->misfitWidth ))
      {
        winObj->window()->setParent(this);
        winObj->setVisible(true);
        winObj->window()->positionOnSiblings();
        if ( lastObj )
          winObj->window()->positionBehindSibling(lastObj->window()->handle());
      }

    }
    else
    {
      // Check if any hidden windows need to be reshown
      if ( winObj->isVisible() == false )
      {
        winObj->window()->setParent(this);
        winObj->setVisible(true);
        winObj->window()->positionOnSiblings();
        if ( lastObj )
          winObj->window()->positionBehindSibling(lastObj->window()->handle());
      }
    }

    if ( winObj->isVisible() )
      lastObj = winObj;
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IToolBar::calcMinimumSize                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
ISize IToolBar::calcMinimumSize( ) const
{
  if (fToolBarData->isExpanded)
  {
    ISize minimum;
    IWindow::ChildCursor cursor(*((IToolBar*)this));
    cursor.setToFirst();
    if ( cursor.isValid() )
    {
      minimum = Inherited::calcMinimumSize();
    }
    else
    {
      // Use standard button size for default minimum size of an empty toolbar
      unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
      unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);
      minimum.setWidth  ( cxBorder * 5 + 10 +
                          IToolBarButton::standardBitmapSize().width());
      minimum.setHeight ( cyBorder * 5 + 16 +
                          IToolBarButton::standardBitmapSize().height());
    }
    ITRACE_ALL(IString("IToolBar::calcMinimumSize =") + minimum.asString() );
    return minimum;
  }
  else
  {
    ITRACE_ALL(IString("IToolBar::calcMinimumSize =") + ISize(0,0).asString() );
    return ISize(0,0);
  }
}

/*------------------------------------------------------------------------------
| Class:   IToolBarFrameCursorData                                             |
|                                                                              |
| Purpose: Data for an IToolBar::FrameCursor.                                  |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarFrameCursorData : public IBase
{
public:
IFrameWindow
  *frame;
IToolBarList
  *list;
IToolBarList::Cursor
  *cursor;
};

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::FrameCursor                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::FrameCursor::FrameCursor ( const IFrameWindow* frame )
{
   fToolBarFrameCursorData = new IToolBarFrameCursorData;
   fToolBarFrameCursorData->frame = (IFrameWindow*) frame;
   fToolBarFrameCursorData->list = 0;
   fToolBarFrameCursorData->cursor = 0;
}

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::~FrameCursor                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::FrameCursor::~FrameCursor ()
{
  delete fToolBarFrameCursorData->cursor;
  delete fToolBarFrameCursorData;
}

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::setToFirst                                            |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::FrameCursor::setToFirst ()
{
  if ( fToolBarFrameCursorData->list !=
       fToolBarFrameCursorData->frame->toolBarList() )
  {
    delete fToolBarFrameCursorData->cursor;
    fToolBarFrameCursorData->cursor = 0;
    fToolBarFrameCursorData->list = fToolBarFrameCursorData->frame->toolBarList();
  }

  if ( fToolBarFrameCursorData->cursor )
  {
    return fToolBarFrameCursorData->cursor->setToFirst();
  }
  else
  {
    if ( fToolBarFrameCursorData->list )
    {
      fToolBarFrameCursorData->cursor = new IToolBarList::Cursor
                                        (*(fToolBarFrameCursorData->list));
      return fToolBarFrameCursorData->cursor->setToFirst();
    }
  }
  return false;
}

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::setToNext                                             |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::FrameCursor::setToNext ()
{
  if ( fToolBarFrameCursorData->list !=
       fToolBarFrameCursorData->frame->toolBarList() )
  {
    delete fToolBarFrameCursorData->cursor;
    fToolBarFrameCursorData->cursor = 0;
    fToolBarFrameCursorData->list = fToolBarFrameCursorData->frame->toolBarList();
  }

  if ( fToolBarFrameCursorData->cursor )
  {
    return fToolBarFrameCursorData->cursor->setToNext();
  }
  else
  {
    if ( fToolBarFrameCursorData->list )
    {
      fToolBarFrameCursorData->cursor = new IToolBarList::Cursor
                                        (*(fToolBarFrameCursorData->list));
      return fToolBarFrameCursorData->cursor->setToNext();
    }
  }
  return false;
}

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::isValid                                               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::FrameCursor::isValid () const
{
  if ( fToolBarFrameCursorData->list !=
       fToolBarFrameCursorData->frame->toolBarList() )
  {
    delete fToolBarFrameCursorData->cursor;
    fToolBarFrameCursorData->cursor = 0;
    fToolBarFrameCursorData->list = fToolBarFrameCursorData->frame->toolBarList();
  }

  if ( fToolBarFrameCursorData->cursor )
  {
    return fToolBarFrameCursorData->cursor->isValid();
  }
  return false;
}

/*------------------------------------------------------------------------------
| IToolBar::FrameCursor::invalidate                                            |
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBar::FrameCursor::invalidate ()
{
  if ( fToolBarFrameCursorData->cursor )
  {
    delete fToolBarFrameCursorData->cursor;
    fToolBarFrameCursorData->cursor = 0;
  }
}

/*------------------------------------------------------------------------------
| IToolBar::toolBarAt                                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar* IToolBar::toolBarAt ( const FrameCursor& cursor )
{
  if ( cursor.fToolBarFrameCursorData->cursor )
  {
    return cursor.fToolBarFrameCursorData->list->elementAt
                  (*(cursor.fToolBarFrameCursorData->cursor));
  }
  // The cursor is invalid so throw exception
  ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                     IBaseErrorInfo::invalidParameter,
                     IException::recoverable);
  return (0);
}

/*------------------------------------------------------------------------------
| Class:   IToolBarWindowCursorData                                            |
|                                                                              |
| Purpose: Data for an IToolBar::WindowCursor.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarWindowCursorData : public IBase
{
public:
  IToolBarWindowCursorData (IToolBarObjectList* objectList);
  ~IToolBarWindowCursorData ( );
IToolBarObjectList::Cursor
  cursor;
};

IToolBarWindowCursorData::IToolBarWindowCursorData (IToolBarObjectList* objectList)
                         :cursor(*objectList)
{
}

IToolBarWindowCursorData::~IToolBarWindowCursorData ()
{
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::WindowCursor                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::WindowCursor::WindowCursor ( const IToolBar* toolbar )
{
  fToolBarWindowCursorData = new IToolBarWindowCursorData
                             (toolbar->fToolBarData->objectList);
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::~WindowCursor                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBar::WindowCursor::~WindowCursor ()
{
  delete fToolBarWindowCursorData;
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::setToFirst                                           |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::WindowCursor::setToFirst ()
{
  return fToolBarWindowCursorData->cursor.setToFirst();
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::setToNext                                            |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::WindowCursor::setToNext ()
{
  return fToolBarWindowCursorData->cursor.setToNext();
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::isValid                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IToolBar::WindowCursor::isValid () const
{
  return fToolBarWindowCursorData->cursor.isValid();
}

/*------------------------------------------------------------------------------
| IToolBar::WindowCursor::invalidate                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBar::WindowCursor::invalidate ()
{
  fToolBarWindowCursorData->cursor.invalidate();
}

/*------------------------------------------------------------------------------
| IToolBar::windowAt                                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IWindow* IToolBar::windowAt ( const WindowCursor& cursor ) const
{
  if ( cursor.isValid() )
  {
    IToolBarObject* winObj = fToolBarData->objectList->elementAt
                             (cursor.fToolBarWindowCursorData->cursor);
    return winObj->window();
  }
  // The cursor is invalid so throw exception
  ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                     IBaseErrorInfo::invalidParameter,
                     IException::recoverable);
  return (0);
}

