/*******************************************************************************
* FILE NAME: ititle1.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ititlenh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSYS
extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _ITITLE_
  #include <ititle.hpp>
#endif

#ifndef _ITITLENH_
  #include <ititlenh.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITITLE1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class ITitleNotifyStatics
{
public:

  ~ITitleNotifyStatics ()
  {
    delete titleNotifyHandler;
    titleNotifyHandler = 0;
  }

static ITitleNotifyHandler
  &notificationHandler ( );

static ITitleNotifyHandler
  *titleNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static ITitleNotifyStatics titleStatics;
INotificationId const ITitle::activeTextForegroundColorId =
                                         "ITitle::activeTextForegroundColor";
INotificationId const ITitle::activeTextBackgroundColorId =
                                         "ITitle::activeTextBackgroundColor";
INotificationId const ITitle::inactiveTextForegroundColorId =
                                         "ITitle::inactiveTextForegroundColor";
INotificationId const ITitle::inactiveTextBackgroundColorId =
                                         "ITitle::inactiveTextBackgroundColor";
INotificationId const ITitle::objectTextId = "ITitle::objectText";
INotificationId const ITitle::viewTextId   = "ITitle::viewText";
INotificationId const ITitle::viewNumberId = "ITitle::viewNumber";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ITitleNotifyHandler* ITitleNotifyStatics::titleNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| ITitleNotifyStatics::notificationHandler                                     |
|                                                                              |
| Static function to create and return a notification handler for ITitle.      |
------------------------------------------------------------------------------*/
ITitleNotifyHandler& ITitleNotifyStatics::notificationHandler ()
{
  if (!(ITitleNotifyStatics::titleNotifyHandler))
  {
    ITitleNotifyStatics::titleNotifyHandler =
                                   new ITitleNotifyHandler();
  }
  return(*ITitleNotifyStatics::titleNotifyHandler);
}

/*------------------------------------------------------------------------------
| ITitle::enableNotification                                                   |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| ITitle, does not send notifications to observer objects until it is          |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
ITitle& ITitle::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(ITitleNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::ITitleNotifyHandler                                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITitleNotifyHandler::ITitleNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::~ITitleNotifyHandler                                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITitleNotifyHandler::~ITitleNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITitleNotifyHandler::dispatchHandlerEvent                                    |
|                                                                              |
| This notifies ITitle Windows of the following events:                        |
|                                                                              |
| activeTextForegroundColor(Id)                                                |
| activeTextBackgroundColor(Id)                                                |
| inactiveTextForegroundColor(Id)                                              |
| inactiveTextBackgroundColor(Id)                                              |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean ITitleNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PM
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_PRESPARAMCHANGED:

        switch (anEvent.parameter1())
        {
          case PP_ACTIVETEXTFGNDCOLOR:
          case PP_ACTIVETEXTBGNDCOLOR:
          case PP_INACTIVETEXTFGNDCOLOR:
          case PP_INACTIVETEXTBGNDCOLOR:
            /**********************************************************/
            /* Post private message to delay notification until after */
            /* the selection state has been modified                  */
            /* Put color identifier into parameter 1 for later use    */
            /**********************************************************/
            thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                             IEventParameter1( anEvent.parameter1() ),
                             IEventParameter2( IC_NOTDEL_TITLE_COLOR ));
            break;

          default:
            break;
        }
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* is used to determine which notification id to send           */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_TITLE_COLOR)
        {
          switch (anEvent.parameter1())
          {
            case PP_ACTIVETEXTFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::activeTextForegroundColorId,
                                      *thisWindow));
              break;

            case PP_ACTIVETEXTBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::activeTextBackgroundColorId,
                                      *thisWindow));
              break;

            case PP_INACTIVETEXTFGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::inactiveTextForegroundColorId,
                                      *thisWindow));
              break;

            case PP_INACTIVETEXTBGNDCOLOR:

              thisWindow->notifyObservers(INotificationEvent(
                                      ITitle::inactiveTextBackgroundColorId,
                                      *thisWindow));
              break;
          }
        }
        break;

      default:
        break;
    }  /* end switch */
  }
#endif  // IC_PM
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

