/*******************************************************************************
* FILE NAME: itrnsfrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itrnsfrm.cpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #define INCL_WINWINDOWMGR
  #define INCL_GPITRANSFORMS
  #include <iwindefs.h>
  #include <math.h>
}

#include <itrnsfrm.hpp>
#include <itrnpriv.hpp>
#include <ihandle.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITRNSFRM_CPP_
  #include <ipagetun.h>
#endif

#define      DBL2FIXED(x)     ((FIXED)  ((x)*65536) )
#define      FIXED2DBL(x)     ((double) (((double)x)/(double)65536) )
#define      DEGREES_TO_RADS  57.29577951
#ifdef IC_WIN
  #define    UNITY            1.0
#endif

#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData()
  : fElement11(UNITY), fElement12(0), fElement21(0), fElement22(UNITY),
    fElement31(0), fElement32(0)
{}

/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData( FIXED f11, FIXED f12, FIXED f21, FIXED f22,
                                long l31, long l32 )
  : fElement11(f11), fElement12(f12), fElement21(f21),
    fElement22(f22), fElement31(l31), fElement32(l32)
{}

/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData( const ITransformData& tData )
  : fElement11(tData.fElement11), fElement12(tData.fElement12),
    fElement21(tData.fElement21), fElement22(tData.fElement22),
    fElement31(tData.fElement31), fElement32(tData.fElement32)
{}

#else

/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData( )
  : fElement11(1.0), fElement12(0), fElement21(0),
    fElement22(1.0), fElement31(0), fElement32(0)
{}

/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData( float f11, float f12, float f21,
                                float f22, float l31, float l32 )
  : fElement11(f11), fElement12(f12), fElement21(f21),
    fElement22(f22), fElement31(l31), fElement32(l32)
{}

/*------------------------------------------------------------------------------
| ITransformData::ITransformData                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData::ITransformData( const ITransformData& tData )
  : fElement11(tData.fElement11), fElement12(tData.fElement12),
    fElement21(tData.fElement21), fElement22(tData.fElement22),
    fElement31(tData.fElement31), fElement32(tData.fElement32)
{}

/*------------------------------------------------------------------------------
| ITransformData::operator*=                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData ITransformData::operator* ( const ITransformData& transformMatrix )
{
  ITransformData temp;

  temp.fElement11 = this->fElement11 * transformMatrix.fElement11 +
                    this->fElement12 * transformMatrix.fElement21;

  temp.fElement12 = this->fElement11 * transformMatrix.fElement12 +
                    this->fElement12 * transformMatrix.fElement22;

  temp.fElement21 = this->fElement21 * transformMatrix.fElement11 +
                    this->fElement22 * transformMatrix.fElement21;

  temp.fElement22 = this->fElement21 * transformMatrix.fElement12 +
                    this->fElement22 * transformMatrix.fElement22;

  temp.fElement31 = this->fElement31 * transformMatrix.fElement11 +
                    this->fElement32 * transformMatrix.fElement21 +
                    transformMatrix.fElement31;

  temp.fElement32 = this->fElement31 * transformMatrix.fElement12 +
                    this->fElement32 * transformMatrix.fElement22 +
                    transformMatrix.fElement32;

  return temp;
}

#endif

/*------------------------------------------------------------------------------
| ITransformData::operator=                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformData& ITransformData::operator=( const ITransformData& transfromData )
{
  if ( this == &transfromData )
    return *this;
  fElement11 = transfromData.fElement11;
  fElement12 = transfromData.fElement12;
  fElement21 = transfromData.fElement21;
  fElement22 = transfromData.fElement22;
  fElement31 = transfromData.fElement31;
  fElement32 = transfromData.fElement32;
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformData::operator==                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITransformData::operator==( const ITransformData& transfromData ) const
{
  return ( fElement11 == transfromData.fElement11 &&
           fElement12 == transfromData.fElement12 &&
           fElement21 == transfromData.fElement21 &&
           fElement22 == transfromData.fElement22 &&
           fElement31 == transfromData.fElement31 &&
           fElement32 == transfromData.fElement32 );
}

/*------------------------------------------------------------------------------
| ITransformData::operator!=                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITransformData::operator!=( const ITransformData& transfromData ) const
{
  return !( *this == transfromData );
}

/*------------------------------------------------------------------------------
| ITransformMatrix::ITransformMatrix                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::ITransformMatrix()
  : fTransformData(new ITransformData)
{ }

/*------------------------------------------------------------------------------
| ITransformMatrix::ITransformMatrix                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::ITransformMatrix( double element11, double element12,
                                    double element21, double element22,
                                    long   element31, long   element32 )
#ifdef IC_PM
  : fTransformData( new ITransformData( DBL2FIXED(element11), DBL2FIXED(element12),
                                        DBL2FIXED(element21), DBL2FIXED(element22),
                                        element31, element32 ))
#else
  : fTransformData( new ITransformData( (float)element11, (float)element12,
                                        (float)element21, (float)element22,
                                        (float)element31, (float)element32 ))
#endif
{ }

#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITransformMatrix::ITransformMatrix                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::ITransformMatrix( const struct _MATRIXLF &matrixlf )
  : fTransformData( new ITransformData( matrixlf.fxM11, matrixlf.fxM12,
                                        matrixlf.fxM12, matrixlf.fxM22,
                                        matrixlf.lM31,  matrixlf.lM32 ))
{}
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITransformMatrix::ITransformMatrix                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::ITransformMatrix( const struct tagXFORM& tagxform )
  : fTransformData( new ITransformData( tagxform.eM11, tagxform.eM12,
                                        tagxform.eM21, tagxform.eM22,
                                        tagxform.eDx,  tagxform.eDy ))
{}
#endif


/*------------------------------------------------------------------------------
| ITransformMatrix::ITransformMatrix                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::ITransformMatrix( const ITransformMatrix& transformMatrix )
  : fTransformData( new ITransformData(*transformMatrix.fTransformData))
{ }

/*------------------------------------------------------------------------------
| ITransformMatrix::~ITransformMatrix                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix::~ITransformMatrix()
{
  delete fTransformData;
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITransformMatrix::asMATRIXLF                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
struct _MATRIXLF ITransformMatrix::asMATRIXLF() const
{
  MATRIXLF mat;
  mat.fxM11 = fTransformData->fElement11;
  mat.fxM12 = fTransformData->fElement12;
  mat.lM13  = 0;
  mat.fxM21 = fTransformData->fElement21;
  mat.fxM22 = fTransformData->fElement22;
  mat.lM23  = 0;
  mat.lM31  = fTransformData->fElement31;
  mat.lM32  = fTransformData->fElement32;
  mat.lM33  = 1;
  return mat;
}
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITransformMatrix::asTagXFORM                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
struct tagXFORM ITransformMatrix::asTagXFORM() const
{
  tagXFORM mat;
  mat.eM11  = fTransformData->fElement11;
  mat.eM12  = fTransformData->fElement12;
  mat.eM21  = fTransformData->fElement21;
  mat.eM22  = fTransformData->fElement22;
  mat.eDx   = fTransformData->fElement31;
  mat.eDy   = fTransformData->fElement32;
  return mat;
}
#endif

/*------------------------------------------------------------------------------
| ITransformMatrix::setElement11                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement11( double value )
{
#ifdef IC_PM
  fTransformData->fElement11 = DBL2FIXED( value );
#else
  fTransformData->fElement11 = (float) value ;
#endif
  return *this;
}
/*------------------------------------------------------------------------------
| ITransformMatrix::setElement12                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement12( double value )
{
#ifdef IC_PM
  fTransformData->fElement12 = DBL2FIXED( value );
#else
  fTransformData->fElement12 = (float) value ;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::setElement21                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement21( double value )
{
#ifdef IC_PM
  fTransformData->fElement21 = DBL2FIXED( value );
#else
  fTransformData->fElement21 = (float) value ;
#endif
  return *this;
}
/*------------------------------------------------------------------------------
| ITransformMatrix::setElement22                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement22( double value )
{
#ifdef IC_PM
  fTransformData->fElement22 = DBL2FIXED( value );
#else
  fTransformData->fElement22 = (float) value ;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::setElement31                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement31( long value )
{
#ifdef IC_PM
  fTransformData->fElement31 = value;
#else
  fTransformData->fElement31 = (float) value ;
#endif
  return *this;
}
/*------------------------------------------------------------------------------
| ITransformMatrix::setElement32                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setElement32( long value )
{
#ifdef IC_PM
  fTransformData->fElement32 = value;
#else
  fTransformData->fElement32 = (float) value ;
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::setToIdentity                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::setToIdentity()
{
  fTransformData->fElement11 = UNITY;
  fTransformData->fElement12 = 0;
  fTransformData->fElement21 = 0;
  fTransformData->fElement22 = UNITY;
  fTransformData->fElement31 = 0;
  fTransformData->fElement32 = 0;
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element11                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double ITransformMatrix::element11() const
{
#ifdef IC_PM
  return FIXED2DBL(fTransformData->fElement11);
#else
  return (double)fTransformData->fElement11;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element12                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double ITransformMatrix::element12() const
{
#ifdef IC_PM
  return FIXED2DBL(fTransformData->fElement12);
#else
  return (double)fTransformData->fElement12;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element21                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double ITransformMatrix::element21() const
{
#ifdef IC_PM
  return FIXED2DBL(fTransformData->fElement21);
#else
  return (double)fTransformData->fElement21;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element22                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
double ITransformMatrix::element22() const
{
#ifdef IC_PM
  return FIXED2DBL(fTransformData->fElement22);
#else
  return (double)fTransformData->fElement22;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element31                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
long ITransformMatrix::element31() const
{
#ifdef IC_PM
  return fTransformData->fElement31;
#else
  return (double)fTransformData->fElement31;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::element32                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
long ITransformMatrix::element32() const
{
#ifdef IC_PM
  return fTransformData->fElement32;
#else
  return (double)fTransformData->fElement32;
#endif
}

/*------------------------------------------------------------------------------
| ITransformMatrix::scaleBy                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::scaleBy( double xScale, double yScale,
                                             const IPoint& pt )
{
#ifdef IC_PM
  FIXED afixed[2] = { DBL2FIXED( xScale ), DBL2FIXED( yScale ) };
  POINTL ptl(pt.asPOINTL());
  MATRIXLF mat(asMATRIXLF());

  IPresSpaceHandle ps = WinGetPS( HWND_DESKTOP );
  if (!GpiScale(ps, &mat, TRANSFORM_ADD, afixed, &ptl ))
  {
    ITHROWGUIERROR("GpiScale");
  }
  else
  {
    fTransformData->fElement11 = mat.fxM11;
    fTransformData->fElement12 = mat.fxM12;
    fTransformData->fElement21 = mat.fxM21;
    fTransformData->fElement22 = mat.fxM22;
    fTransformData->fElement31 = mat.lM31;
    fTransformData->fElement32 = mat.lM32;
  }
  WinReleasePS( ps );
#else
  ITransformData xfrmSource;

  xfrmSource.fElement11 = xScale;
  xfrmSource.fElement22 = yScale;

  fTransformData->fElement31 -= (float)pt.x();
  fTransformData->fElement32 -= (float)pt.y();

  *fTransformData = *fTransformData * xfrmSource;

  fTransformData->fElement31 += (float)pt.x();
  fTransformData->fElement32 += (float)pt.y();

#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::rotateBy                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::rotateBy( double angle, const IPoint& pt )
{
#ifdef IC_PM
  POINTL ptl(pt.asPOINTL());
  MATRIXLF mat(asMATRIXLF());

  IPresSpaceHandle ps = WinGetPS( HWND_DESKTOP );
  if (!GpiRotate(ps, &mat, TRANSFORM_ADD, DBL2FIXED(angle), &ptl ))
  {
    ITHROWGUIERROR("GpiRotate");
  }
  else
  {
    fTransformData->fElement11 = mat.fxM11;
    fTransformData->fElement12 = mat.fxM12;
    fTransformData->fElement21 = mat.fxM21;
    fTransformData->fElement22 = mat.fxM22;
    fTransformData->fElement31 = mat.lM31;
    fTransformData->fElement32 = mat.lM32;
  }
  WinReleasePS( ps );
#else

  float originalAngle;
  double fsin;
  double fcos;

  if (angle < 0)
  {
    angle = -angle;
    angle = 360 - fmod( angle, 360.0 );
  }
  else if (angle > 360.0)
  {
    angle = fmod( angle, 360.0 );
  } /* endif */

  // convert angle to quadrant zero

  originalAngle = angle;

  if (angle > 90.0)
  {
    if (angle > 270.0)
    {
      angle = 360.0 - angle;
    }
    else if (angle > 180.0)
    {
      angle = angle - 180.0;
    }
    else
    {
      angle = 180.0 - angle;
    }
  }

  fsin = sin((double)angle/(double)DEGREES_TO_RADS );
  fcos = cos((double)angle/(double)DEGREES_TO_RADS );

  // determine quadrant based on the original angle

  if ((originalAngle > 90.0) && (originalAngle < 270.0))
  {
    fcos = -fcos;
  }

  if (originalAngle > 180.0)
  {
    fsin = -fsin;
  }

  // translate center to origin
  fTransformData->fElement31 -= (float)pt.x();
  fTransformData->fElement32 -= (float)pt.y();

  ITransformData xfrmSource;

  xfrmSource.fElement11 = fcos;
  xfrmSource.fElement12 = fsin;
  xfrmSource.fElement21 = -fsin;
  xfrmSource.fElement22 = fcos;

  *fTransformData = *fTransformData * xfrmSource;

  fTransformData->fElement31 += (float)pt.x();
  fTransformData->fElement32 += (float)pt.y();

#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::translateBy                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::translateBy( const IPoint& pt )
{
#ifdef IC_PM
  POINTL ptl(pt.asPOINTL());
  MATRIXLF mat(asMATRIXLF());

  IPresSpaceHandle ps = WinGetPS( HWND_DESKTOP );
  if (!GpiTranslate(ps, &mat, TRANSFORM_ADD, &ptl ))
  {
    ITHROWGUIERROR("GpiTranslate");
  }
  else
  {
    fTransformData->fElement11 = mat.fxM11;
    fTransformData->fElement12 = mat.fxM12;
    fTransformData->fElement21 = mat.fxM21;
    fTransformData->fElement22 = mat.fxM22;
    fTransformData->fElement31 = mat.lM31;
    fTransformData->fElement32 = mat.lM32;
  }
  WinReleasePS( ps );
#else
  fTransformData->fElement31 += (float)pt.x();
  fTransformData->fElement32 += (float)pt.y();
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::operator=                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITransformMatrix& ITransformMatrix::operator= (
                                       const ITransformMatrix& transformMatrix )
{
  if ( this == &transformMatrix )
    return *this;
  *fTransformData = *transformMatrix.fTransformData;
  return *this;
}

/*------------------------------------------------------------------------------
| ITransformMatrix::operator==                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean ITransformMatrix::operator== ( const ITransformMatrix& transformMatrix ) const
{
  return ( *fTransformData == *transformMatrix.fTransformData );
}

/*------------------------------------------------------------------------------
| ITransformMatrix::operator!=                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean ITransformMatrix::operator!= ( const ITransformMatrix& transformMatrix ) const
{
  return !( *this == transformMatrix );
}
