#ifndef _IWINDOW_
#define _IWINDOW_
/*******************************************************************************
* FILE NAME: iwindow.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IWindow                                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibitflag.hpp>
#include <ievent.hpp>
#include <ihandle.hpp>
#include <inotify.hpp>
#include <ipoint.hpp>
#ifdef IC_PMWIN
#include <ibidiset.hpp>
#endif

#ifdef IC_MOTIF
union  _XEvent;
struct _WidgetRec;

extern "C" {
  typedef _WidgetRec *  (*IXmCreateFunction) ( _WidgetRec*,
                                               char*,
                                               void*,
                                               unsigned int );
}
#endif //IC_MOTIF
#ifdef IC_WIN
extern "C" {
  extern void* __stdcall _pfnwpICWinProc ( void*,
                                           unsigned long,
                                           void*,
                                           void* );
}
#endif
#ifdef IC_PM
extern "C" {
  extern void* _System _pfnwpICWinProc ( unsigned long,
                                         unsigned long,
                                         void*,
                                         void* );
}
#endif

class IAcceleratorTable;
class IAccelTblHandle;
class IColor;
class IDMItemProvider;
class IException;
class IFont;
class IHandler;
class INotificationEvent;
class IRectangle;
class IString;
class IChildCursorData;
class IWindowBidiData;
class IWindowData;
class IWindowNotifyHandler;

#pragma pack(4)

class IWindow : public INotifier {
typedef INotifier
  Inherited;

public:
/*------------------------------- Constructors -------------------------------*/
  IWindow ( unsigned long        identifier,
            IWindow*             parent );
  IWindow ( const IWindowHandle& handle );

virtual
 ~IWindow ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF0( Style, IWindow );
static const Style
  IC_IMPORTU noStyle,
  IC_IMPORTU visible,
  IC_IMPORTU disabled,
  IC_IMPORTU clipChildren,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU clipToParent,
  IC_IMPORTU saveBits,
  IC_IMPORTU synchPaint,
#endif
  IC_IMPORTU clipSiblings;

#ifndef IC_MOTIF_FLAGNOP
virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;
#endif

/*---------------------------- Window Positioning ----------------------------*/
virtual IWindow
 &moveTo     ( const IPoint&     newPosition ),
 &sizeTo     ( const ISize&      newSize ),
 &moveSizeTo ( const IRectangle& newSizeAndPosition );

virtual IPoint
  position   ( ) const;

virtual ISize
#ifdef IC_PMWIN
  parentSize ( ) const,
#endif
  size       ( ) const;

virtual IRectangle
#ifdef IC_PMWIN
  nativeRect ( ) const,
#endif
  rect       ( ) const;

#ifdef IC_PMWIN
static ISize
  parentSize ( const IWindowHandle& windowHandle );
#endif

/*-------------------------------- Attributes --------------------------------*/
virtual IWindow
 &setOwner                  ( const IWindow* newOwner ),
#ifndef IC_MOTIF_FLAGNOP
 &setParent                 ( const IWindow* newParent ),
 &setId                     ( unsigned long  newIdentifier ),
#endif
 &enable                    ( Boolean        enableWindow = true ),
 &disable                   ( ),
 &setFocus                  ( );

IWindow
 &setHelpId                 ( unsigned long  helpTopicId );

IWindow
 *parent                    ( ) const;

virtual IWindow
 *owner                     ( ) const;

Boolean
  isEnabled                 ( ) const;

virtual Boolean
  hasFocus                  ( ) const,
  isValid                   ( ) const,
  isGroup                   ( ) const,
  isTabStop                 ( ) const,
  isFrameWindow             ( ) const;

static IWindow
 *desktopWindow             ( );

static IWindow
 *objectWindow              ( );

virtual IWindowHandle
  handle                    ( ) const;

static IWindow
 *windowWithHandle          ( const IWindowHandle& windowHandle,
                              Boolean              allThreads = true ),
 *windowWithOwner           ( unsigned long        identifier,
                              const IWindow*       owner,
                              Boolean              allThreads = true ),
 *windowWithParent          ( unsigned long        identifier,
                              const IWindow*       parent,
                              Boolean              allThreads = true );

static IWindowHandle
  handleWithParent          ( unsigned long        identifier,
                              const IWindowHandle& parent );

#ifdef IC_PMWIN
static Boolean
  isWindowValid             ( const IWindow*       window );

static IWindowHandle
  handleWithPointerCaptured ( );
#endif

virtual unsigned long
  id                        ( ) const;

unsigned long
  helpId                    ( ) const;

#ifdef IC_PM
IMessageQueueHandle
  messageQueue              ( ) const;
#endif

/*------------------------- Capturing Pointer Events -------------------------*/
#ifdef IC_PMWIN
virtual IWindow
 &capturePointer     ( Boolean capture = true ),
 &releasePointer     ( );

virtual Boolean
  hasPointerCaptured ( ) const;
#endif

/*--------------------------- Pointer Positioning ----------------------------*/
#ifdef IC_PMWIN
static void
  movePointerTo   ( const IPoint& position );

static IPoint
  pointerPosition ( );
#endif

/*------------------------------ Sibling Order -------------------------------*/
enum SiblingOrder {
  onTopOfSiblings,
  behindSiblings
  };

virtual IWindow
 &positionBehindSiblings ( ),
 &positionOnSiblings     ( ),
 &positionBehindSibling  ( const IWindowHandle& siblingWindow );

static void
 setDefaultOrdering      ( SiblingOrder         order );
static SiblingOrder
 defaultOrdering         ( );

/*--------------------------- Event Send and Post ----------------------------*/
enum EventType {
  command=1,
  systemCommand,
  control,
  help,
  character
  };

virtual const IWindow
 &postEvent ( unsigned long           eventId,
              const IEventParameter1& parm1 = 0,
              const IEventParameter2& parm2 = 0 ) const,
 &postEvent ( const IEvent&           event )     const,
 &postEvent ( EventType               eventType,
              const IEventParameter1& parm1 = 0,
              const IEventParameter2& parm2 = 0 ) const;

virtual IEventResult
  sendEvent ( unsigned long           eventId,
              const IEventParameter1& parm1 = 0,
              const IEventParameter2& parm2 = 0 ) const,
  sendEvent ( const IEvent&           event )     const,
  sendEvent ( EventType               eventType,
              const IEventParameter1& parm1 = 0,
              const IEventParameter2& parm2 = 0 ) const;

/*----------------------------- Window Painting ------------------------------*/
enum RefreshType {
  paintAll,
  immediate,
  paintAllImmediate
  };

virtual IWindow
 &show               ( Boolean           showWindow = true ),
 &hide               ( ),
 &enableUpdate       ( Boolean           enableWindow = true ),
 &disableUpdate      ( ),
 &refresh            ( RefreshType       type=paintAll ),
 &refresh            ( const IRectangle& invalidRectangle,
                       Boolean           immediate=false ),
 &showSourceEmphasis ( Boolean           show = true ),
 &hideSourceEmphasis ( );

Boolean
  isVisible          ( ) const,
  isShowing          ( ) const;

virtual IPresSpaceHandle
  presSpace          ( ) const;

virtual void
  releasePresSpace   ( const IPresSpaceHandle& presentationSpaceHandle ) const;

#ifdef IC_PMWIN
/*------------------------- Accelerator Key Support --------------------------*/
IWindow
 &setAcceleratorTable  ( const IAcceleratorTable* acceleratorTable ),
 &setAcceleratorHandle ( const IAccelTblHandle&   handle ),
 &setAcceleratorHandle ( const IAccelTblHandle&   handle,
                         Boolean handleDelete );

IAcceleratorTable
  acceleratorTable     ( ) const;
IAccelTblHandle
  acceleratorHandle    ( ) const;
#endif

/*---------------------------- Compound Controls -----------------------------*/
#ifdef IC_MOTIF
virtual IWindowHandle
  topHandle ( ) const;
#endif

/*----------------------------- Object Deletion ------------------------------*/
IWindow
 &setAutoDeleteObject  ( Boolean autoDelete = true ),
 &setAutoDestroyWindow ( Boolean autoDestroy = false );

Boolean
  isAutoDeleteObject   ( ) const,
  isAutoDestroyWindow  ( ) const;

/*-------------------------- Drag and Drop Support ---------------------------*/
IDMItemProvider
 *itemProvider    ( ) const;

IWindow
 &setItemProvider ( IDMItemProvider* dragProvider );


class ChildCursor : public IVBase {
typedef IVBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  ChildCursor ( IWindow& parent );
virtual
 ~ChildCursor ( );

/*----------------------------- Cursor Functions -----------------------------*/
virtual Boolean
  setToFirst ( ),
  setToNext  ( ),
  isValid    ( ) const;

virtual void
  invalidate ( );

private:
/*----------------------------- Hidden Functions -----------------------------*/
  ChildCursor ( const ChildCursor& cursor );
ChildCursor
 &operator =  ( const ChildCursor& cursor );

/*--------------------------------- Private ----------------------------------*/
friend class IWindow;

IChildCursorData
 *pCursorData;

IEnumHandle
  enumHandle;

#ifdef IC_MOTIF
_WidgetRec
**children;

unsigned int
  num_children;

int
  index;
#endif //IC_MOTIF

IWindowHandle
  hwnd,
  hwndParent;
}; // IWindow::ChildCursor

/*--------------------------- Child Cursor Support ---------------------------*/
IWindowHandle
  childAt ( const ChildCursor& cursor ) const;

/*------------------------------ Canvas Support ------------------------------*/
virtual IWindowHandle
  defaultPushButton  ( ) const,
  matchForMnemonic   ( unsigned short character ) const;

enum Layout {
  windowCreated=1,
  colorChanged=2,
  sizeChanged=4,
  minimumSizeChanged=8,
  childMinimumSizeChanged=16,
  fontChanged=32,
  fontPropogated=64,
  layoutChanged=128,
  immediateUpdate=256,
  childWindowCreated=512,
  windowDestroyed=1024,
  childWindowDestroyed=2048
  };

virtual IWindow
 &setLayoutDistorted ( unsigned long layoutAttributesOn,
                       unsigned long layoutAttributesOff );

virtual Boolean
  isLayoutDistorted  ( unsigned long layoutAttribute ) const;

IWindow
 &resetMinimumSize   ( ),
 &setMinimumSize     ( const ISize& size );

ISize
  minimumSize        ( Boolean windowCalculatedSize = false ) const;

virtual IRectangle
  layoutAdjustment   ( ) const,
  visibleRectangle   ( ) const;

#ifdef IC_PMWIN
IWindow
 &enableMinimumSizeCaching    ( Boolean enableCaching = true ),
 &disableMinimumSizeCaching   ( );
Boolean
  isMinimumSizeCachingEnabled ( ) const;

#endif
/*------------------------------ Miscellaneous -------------------------------*/
static IPoint
  mapPoint ( const IPoint&        point,
             const IWindowHandle& from,
             const IWindowHandle& to );

/*------------------------------- Diagnostics --------------------------------*/
virtual IString
  asString    ( ) const,
  asDebugInfo ( ) const;

class ExceptionFn : public IVBase {
typedef IVBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
virtual
 ~ExceptionFn ( );

/*--------------------------- Exception Processing ---------------------------*/
virtual Boolean
  handleException ( IException& dispatcherException,
                    IEvent&     exceptionEvent ) = 0;
}; // IWindow::ExceptionFn


/*--------------------------- Exception Processing ---------------------------*/
static IWindow::ExceptionFn
 *setExceptionFunction ( IWindow::ExceptionFn* exceptionFunction ),
 *exceptionFunction    ( );

virtual Boolean
  handleException      ( IException& dispatcherException,
                         IEvent&     exceptionEvent );

/*----------------------------- Color Functions ------------------------------*/
virtual IColor
  foregroundColor              ( ) const,
  backgroundColor              ( ) const;
#ifndef IC_WIN_FLAGNOP
virtual IColor
  borderColor                  ( ) const;
#endif

virtual IWindow
 &setForegroundColor           ( const IColor& color ),
 &setBackgroundColor           ( const IColor& color );
#ifndef IC_WIN_FLAGNOP
virtual IWindow
 &setBorderColor               ( const IColor& color );
#endif

#ifndef IC_MOTIFWIN_FLAGNOP
virtual IColor
  shadowColor                  ( ) const,
  disabledForegroundColor      ( ) const,
  disabledBackgroundColor      ( ) const,
  hiliteForegroundColor        ( ) const,
  hiliteBackgroundColor        ( ) const,
  activeColor                  ( ) const,
  inactiveColor                ( ) const;

virtual IWindow
 &setShadowColor               ( const IColor& color ),
 &setDisabledForegroundColor   ( const IColor& color ),
 &setDisabledBackgroundColor   ( const IColor& color ),
 &setHiliteForegroundColor     ( const IColor& color ),
 &setHiliteBackgroundColor     ( const IColor& color ),
 &setActiveColor               ( const IColor& color ),
 &setInactiveColor             ( const IColor& color );
#endif // !IC_MOTIFWIN_FLAGNOP
#ifndef IC_MOTIF_FLAGNOP
virtual IWindow
 &resetForegroundColor         ( ),
 &resetBackgroundColor         ( );
#endif // !IC_MOTIF_FLAGNOP
#ifndef IC_MOTIFWIN_FLAGNOP
virtual IWindow
 &resetBorderColor             ( ),
 &resetShadowColor             ( ),
 &resetDisabledForegroundColor ( ),
 &resetDisabledBackgroundColor ( ),
 &resetHiliteForegroundColor   ( ),
 &resetHiliteBackgroundColor   ( ),
 &resetActiveColor             ( ),
 &resetInactiveColor           ( );
#endif //!IC_MOTIFWIN_FLAGNOP

/*------------------------------ Font Functions ------------------------------*/
virtual IWindow
 &setFont       ( const IFont& font );

#ifdef IC_PMWIN
virtual IWindow
 &resetFont     ( );
#endif

virtual IFont
  font          ( ) const;

ISize
  characterSize ( ) const;

/*-------------------------- Observer Notification ---------------------------*/
virtual IWindow
 &enableNotification       ( Boolean enable = true ),
 &disableNotification      ( );

virtual Boolean
  isEnabledForNotification ( ) const;

virtual IWindow
 &notifyObservers          ( const INotificationEvent& event );

/*------------------------------ Notifications -------------------------------*/
static INotificationId const
  IC_IMPORTU sizeId,
  IC_IMPORTU positionId,
  IC_IMPORTU visibleId,
  IC_IMPORTU enableId,
  IC_IMPORTU deleteId,
  IC_IMPORTU focusId,
  IC_IMPORTU foregroundColorId,
  IC_IMPORTU backgroundColorId,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU borderColorId,
  IC_IMPORTU shadowColorId,
  IC_IMPORTU disabledForegroundColorId,
  IC_IMPORTU disabledBackgroundColorId,
  IC_IMPORTU activeColorId,
  IC_IMPORTU inactiveColorId,
  IC_IMPORTU hiliteForegroundColorId,
  IC_IMPORTU hiliteBackgroundColorId,
#endif
  IC_IMPORTU fontId,
  IC_IMPORTU commandId,
  IC_IMPORTU systemCommandId;

#ifdef IC_PMWIN
/*---------------------- Event Handling Implementation -----------------------*/
Boolean
  dispatchRemainingHandlers ( IEvent& event,
                              Boolean callDefProc = true );
#endif
static void
   enableFastWindowWithHandle   ( Boolean enable = true );
static Boolean
   isFastWindowWithHandleEnabled( );

protected:
/*------------------------------ Implementation ------------------------------*/
#ifdef IC_PMWIN
virtual IWindowHandle
 create            ( unsigned long         id,
                     const char*           text,
                     unsigned long         style,
                     const char*           windowClass,
                     const IWindowHandle&  parent,
                     const IWindowHandle&  owner,
                     const IRectangle&     initRect,
                     const void*           ctlData,
                     const void*           presParams,
                     IWindow::SiblingOrder ordering = defaultOrdering(),
                     unsigned long         extendedStyle = 0 );
#endif

#ifdef IC_MOTIF
virtual IWindowHandle
  create           ( unsigned long         id,
                     const char*           text,
                     unsigned long         style,
                     IXmCreateFunction     createFunction,
                     const IWindowHandle&  parent,
                     const IWindowHandle&  owner,
                     const IRectangle&     initRect,
                     const void*           callerArgList,
                     const unsigned int    callerNumberArguments,
                     IWindow::SiblingOrder ordering = defaultOrdering() );

virtual IWindow
 &prepareForUse    ( const IWindowHandle& windowHandle );
#endif  //IC_MOTIF

IWindow
 &addHandler       ( IHandler* newHandler ),
 &removeHandler    ( IHandler* oldHandler );

Boolean
  isPrimaryWindow  ( ) const;

#ifdef IC_PMWIN
IWindow
 &setColor         ( unsigned long colorArea,
                     const IColor& color ),
 &resetColor       ( unsigned long colorArea );
#endif //IC_PMWIN

#ifndef IC_MOTIF_FLAGNOP
virtual IWindow
 &setStyle         ( unsigned long style ),
 &setExtendedStyle ( unsigned long extendedStyle );
#endif

#ifndef IC_MOTIF_FLAGNOP
virtual unsigned long
  style            ( ) const,
  extendedStyle    ( ) const;
#endif

#ifdef IC_PMWIN
IColor
  color            ( unsigned long colorArea ) const,
  color            ( unsigned long colorArea,
                     const IColor& defaultColor ) const;
unsigned short
  windowUShort     ( long index ) const;

unsigned long
  windowULong      ( long index ) const;

IWindow
 &setWindowData    ( long           index,
                     unsigned short dataValue ),
 &setWindowData    ( long           index,
                     unsigned long  dataValue );
#endif // IC_PMWIN

#ifdef IC_MOTIF
/*--------------------- Compound Control Implementation ----------------------*/
virtual Boolean
  isRelatedHandle ( const IWindowHandle& windowHandle ) const;
#endif

/*-------------------------- Protected Constructors --------------------------*/
  IWindow ( );

/*---------------------------------- Layout ----------------------------------*/
virtual ISize
  calcMinimumSize  ( ) const;
#ifdef IC_PMWIN
IWindow
 &saveMinimumSize  ( const ISize& size );
ISize
  savedMinimumSize ( ) const;
#endif

/*---------------------- Event Handling Implementation -----------------------*/
IWindow
 &startHandlingEventsFor ( const IWindowHandle& windowHandle ),
 &startHandlingEventsFor ( unsigned long        identifier,
                           IWindow*             parent );
static void
  addToWindowSet         ( IWindow*             window,
                           const IWindowHandle& windowHandle ),
  removeFromWindowSet    ( IWindow*             window );

#ifdef IC_MOTIF
static void
  addRelatedHandleToWindowSet      ( IWindow*             window,
                                     const IWindowHandle& windowHandle ),
  removeRelatedHandleFromWindowSet ( IWindow*             window,
                                     const IWindowHandle& windowHandle );
#endif

Boolean
  dispatch               ( IEvent& event );

IWindow
 &defaultProcedure       ( IEvent& event );

Boolean
  deleteIsInProcess      ( ) const;

#ifdef IC_MOTIF
virtual void
  registerCallbacks      ( ),
  unregisterCallbacks    ( );

virtual Boolean
  passEventToOwner       ( IEvent &event );
#endif  //IC_MOTIF

/*------------------- Observer Notification Implementation -------------------*/
virtual IWindow
 &addObserver            ( IObserver&             observer,
                           const IEventData&      userData = IEventData(0) ),
 &removeObserver         ( IObserver&             observer ),
 &removeAllObservers     ( );

IObserverList
 &observerList           ( ) const;

IWindow
 &setNotificationHandler ( IWindowNotifyHandler*  notifyHandler );

IWindowNotifyHandler
 *notificationHandler    ( ) const;

virtual IWindow
 &notifyObservers        ( const INotificationId& notification );

/*-------------------------- Drag and Drop Support ---------------------------*/
#ifdef IC_WIN
virtual Boolean
  isDragStarting ( IEvent &event );
#endif

private:
/*----------------------------- Hidden Functions -----------------------------*/
IWindow     ( const IWindow& window );
IWindow
 &operator= ( const IWindow& window );

/*--------------------------------- Private ----------------------------------*/
friend class IHandle;
friend class IHandler;
friend class IFileDialog;
friend class IFontDialog;

#ifdef IC_MOTIF
friend class IApplication;
friend class IThreadData;
friend class IWindowData;
#endif

#ifdef IC_WIN
friend class IFont;
friend class IDMDragHandler;
friend class IMessageHandler;
#endif

enum State {
  autoDeleteObject=1,
  autoDestroyWindow=2,
  deleteInProcess=4,
  primaryWindow=8,
  addedToList=16,
  dispatchInProcess=32,
  needsDelete=64,
  updateRHtInProgress=128,
  neverBeenShown=256,
  forgetBitGravity=512,
  fontWasSet=1024,
  bkgndColorWasSet=2048,
  frgndColorWasSet=4096,
  cacheMinimumSize=8192
  };

friend unsigned long const
 &key             ( IWindow* const& window );

#ifdef IC_PM
friend void* _System
  _pfnwpICWinProc ( unsigned long hwnd,
                    unsigned long ulMsg,
                    void*         mp1,
                    void*         mp2 );
#endif

#ifdef IC_WIN
friend void* __stdcall
  _pfnwpICWinProc ( void*         hwnd,
                    unsigned long ulMsg,
                    void*         mp1,
                    void*         mp2 );
#endif

#ifdef   IC_MOTIF
friend void _System
  iwindowXEventCallback               ( _WidgetRec* w,
                                        void*       client_data,
                                        _XEvent*    event,
                                        char*       continue_to_dispatch ),
  iwindowMotifCallback                ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  iwindowSashCallback                 ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  iwindowDestroyCallback              ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  iwindowRelatedHandleDestroyCallback ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  ientryfmleMotifCallback             ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  imenuMotifCallback                  ( _WidgetRec* w,
                                        void*       client_data,
                                        void*       call_data ),
  imenuXEventCallback                 ( _WidgetRec* w,
                                        void*       client_data,
                                        _XEvent*    event,
                                        char*       continue_to_dispatch );
friend class IDMMotif;
#endif //IC_MOTIF

static void
  cleanUpHandler          ( IHandler& handler );

#ifdef IC_MOTIF
static IWindow
 *windowWithRelatedHandle ( const IWindowHandle& windowHandle );
#endif

#ifdef IC_WIN
static Boolean
  useRealCoordinates      ( const IWindow* window );
#endif

IWindow
 &setDeleteInProcess      ( ),
 &cleanUp                 ( );

#ifdef IC_PMWIN
Boolean
  dispatchCleanup         ( IEvent& event,
                            Boolean nested );
#endif

IWinProc
 *setDefaultProcedure     ( IWinProc* newDefProc );

IWindowData
 *pWindowData;

unsigned long
  hwnd;

/*----------------------------- Obsolete Members -----------------------------*/
#ifdef IC_PMWIN
#if (IC_OBSOLETE <= IC_OBSOLETE_2)
public:
class BidiSettings;

Boolean
  isBidiSupported ( ) const;

enum BidiTextType {
  visual,
  implicit
  };

enum BidiTextOrientation {
  textLeftToRight,
  textRightToLeft,
  textContextual
  };

enum BidiLayout {
  layoutLeftToRight,
  layoutRightToLeft
  };

enum BidiNumeralType {
  arabic,
  asStored,
  national,
  contextual
  };

enum BidiTextShape {
  displayShaped,
  saveShaped,
  nominalShape,
  initialShape,
  middleShape,
  finalShape,
  isolatedShape
  };

virtual IWindow
 &applyBidiSettings ( const BidiSettings& settings,
                      Boolean             childInherit = true,
                      Boolean             refresh      = true );

class BidiSettings : public IVBase {
typedef IVBase
  Inherited;
public:
  BidiSettings ( const IWindow& window );

virtual
 ~BidiSettings ( );

virtual BidiSettings
 &setTextType                   ( BidiTextType        textType ),
 &setTextOrientation            ( BidiTextOrientation textOrientation ),
 &setWindowLayout               ( BidiLayout          windowLayout ),
 &setNumeralDisplay             ( BidiNumeralType     numeralDisplay ),
 &enableSymmetricSwapping       ( Boolean             enable = true ),
 &disableSymmetricSwapping      ( ),
 &enableWordByWordReordering    ( Boolean             enable = true ),
 &disableWordByWordReordering   ( ),
 &setTextShape                  ( BidiTextShape       textShape );

BidiTextType
  textType                      ( ) const;
BidiTextOrientation
  textOrientation               ( ) const;
BidiLayout
  windowLayout                  ( ) const;
BidiNumeralType
  numeralDisplay                ( ) const;
Boolean
  isSymmetricSwappingEnabled    ( ) const;
Boolean
  isWordByWordReorderingEnabled ( ) const;
BidiTextShape
  textShape                     ( ) const;

private:
friend class IWindow;
  BidiSettings ( const BidiSettings& settings );
BidiSettings
 &operator=    ( const BidiSettings& settings );
IBidiSettings
 *fBidiSettings;
}; // IWindow::BidiSettings

private:
friend class IWindow::BidiSettings;
#endif // IC_OBSOLETE
#endif // IC_PMWIN

}; // IWindow

#pragma pack()

#ifndef I_NO_INLINES
  #include <iwindow.inl>
#endif

#endif /* _IWINDOW_ */
